///////////////////////////////////////////////////////////////////////////////
// Apocalypse Weekend Single Player game info
// Copyright 2003 Running With Scissors, Inc.  All Rights Reserved.
//
// Game info that drives the add on pack.
//
///////////////////////////////////////////////////////////////////////////////
class AW7GameSP extends AW7InvGameSP;

var class<GameState> GameStateClass;	// class of gamestate to be used
var globalconfig int FinStop;
var globalconfig int MultCast;
var globalconfig int Tunneling;

const FinStopPath=	"EAFX.AW7GameSP FinStop"; // ini path
const MultCastPath=	"EAFX.AW7GameSP MultCast"; // ini path
const TunnelingPath="EAFX.AW7GameSP Tunneling"; // ini path

const CAST_MAX	= 42;

///////////////////////////////////////////////////////////////////////////////
// Change day, but only in value, don't warp him or anything
///////////////////////////////////////////////////////////////////////////////
function IncrementDay()
{
	// See if there's any more days
	if (TheGameState.CurrentDay + 1 < Days.length)
	{
		TheGameState.bChangeDayPostTravel = false;
		TheGameState.NextDay = TheGameState.CurrentDay + 1;
		TheGameState.CurrentDay = TheGameState.NextDay;
		//log(self$" increment day, now "$TheGameState.CurrentDay);
	}
}

///////////////////////////////////////////////////////////////////////////////
// Don't allow reminder hints in add-on. No map here.
///////////////////////////////////////////////////////////////////////////////
function bool AllowReminderHints()
{
	return false;
}

///////////////////////////////////////////////////////////////////////////////
//	This had to be copied from P2GameInfoSingle because the GameState
// class was explicitly specified. Bad form! We've made it a class here(not
// that it really matters *now*).
//
// Called from P2Player.TravelPostAccept(), which is after the player pawn
// has traveled to (or been created in) a new level.
//
// This ultimately gets called in three basic situations, shown here along
// with how to differentiate them:
//
//  Loaded game: TheGameState != None
//	New game:    TheGameState == None and player's inventory does NOT contain GameState
//  New level:   TheGameState == None and player's inventory contains GameState
//
///////////////////////////////////////////////////////////////////////////////
event PostTravel(P2Pawn PlayerPawn)
{
	if (TheGameState != None)
	{
		// We loaded a saved game
		bLoadedSavedGame = true;
		Log(self$" PostTravel(): Loaded a saved game (using the loaded GameState)");
	}
	else
	{
		// Try to get GameState from player's inventory
		TheGameState = GameState(PlayerPawn.FindInventoryType(GameStateClass));
		if (TheGameState != None)
		{
			PlayerPawn.DeleteInventory(TheGameState);
			Log(self$" PostTravel(): Continuing existing game (got GameState from player inventory)");
		}
		else
		{
			Log(self$" PostTravel(): Starting new game (creating new GameState)");
			TheGameState = spawn(GameStateClass);

			// We just created a new GameState, so if we're NOT on one of the
			// "pre game" maps then we must be testing a map by loading it directly.
			if (IsMainMenuMap() || IsIntroMap())
			{
				TheGameState.bPreGameMode = true;
				SetupDifficultyOnce();
			}
			else
			{
				// Start new game for testing
				bTesting = true;
				TheGameState.bChangeDayPostTravel = true;
				TheGameState.NextDay = 0;
				P2RootWindow(GetPlayer().Player.InteractionMaster.BaseMenu).StartingGame();
				MostRecentGameSlot = -1; // This is unknown when testing
				SetupDifficultyOnce();
			}
		}

		// Check if we need to change the day (includes starting a new game)
		if (TheGameState.bChangeDayPostTravel)
		{
			// Check if starting a new game
			if (NextDay() == 0)
			{
				TheGameState.bFirstLevelOfGame = true;
				TheGameState.bPreGameMode = false;

				// Check for valid errand goals (only done once, reports errors to log)
				CheckForValidErrandGoals();
			}
			else
			{
				TheGameState.bFirstLevelOfGame = false;

				// Remove inventory player can't leave the current day with
				Days[TheGameState.CurrentDay].TakeInventoryFromPlayer(PlayerPawn);
			}

			// Change the day
			TheGameState.CurrentDay = NextDay();
			TheGameState.bFirstLevelOfDay = true;
			TheGameState.ErrandsCompletedToday = 0;
			TheGameState.bChangeDayPostTravel = false;
			TheGameState.bChangeDayForDebug = false;
		}
		else
		{
			// Same day, so clear flag
			TheGameState.bFirstLevelOfDay = false;
		}

		// GameInfo is now valid
		GameInfoIsNowValid();

		// Tell GameState the level has changed
		TheGameState.PostLevelChange(PlayerPawn, ParseLevelName(Level.GetLocalURL()));
		
		// Get most recent slot
		MostRecentGameSlot = TheGameState.MostRecentGameSlot;

		// Restore day and its errands
		RestoreDayAndErrands();
		
		// Restore pawns that teleported with the player
		TheGameState.RestoreAllTeleportedPawns(PlayerPawn);

		// Handle changing things because we're going to the first level of a new day
		if (TheGameState.bFirstLevelOfDay)
			// Check if we need to give the player the starting inventory for this day
			Days[TheGameState.CurrentDay].AddInStartingInventory(PlayerPawn);

		// Decide whether auto-save should be done (although it's not actually
		// done here).  We force an autosave at the start of the game's first level
		// even if autosave is disabled because we need something to go back to
		// when the player dies and wants to to restart (we can't assume he'll save
		// on his own).  We also force an autosave at the start of each day because
		// most players won't realize they'd have to go back to a previous day if
		// they don't manually save on a new day.
		bDoAutoSave = false;
		if (!Level.IsDemoBuild() && !IsFinishedDayMap())
		{
			if ((bUseAutoSave && !IsPreGame()) || TheGameState.bFirstLevelOfDay)
			{
				bDoAutoSave = true;
				AutoSaveSlot = AUTOSAVE_SLOT;
				if (TheGameState.bFirstLevelOfDay)
				bForcedAutoSave = true;
			}
		}
	}
}

///////////////////////////////////////////////////////////////////////////////
// Reach into entry and put a new reference to every chameleon texture/meshes in this
// level.
///////////////////////////////////////////////////////////////////////////////
function StoreChams()
{
	local LevelInfo lev;
	local TextureLoader texl;
	local MeshLoader mesher;
	local Actor checkme;
	local PersonPawn checkpawn;

	lev = GetPlayer().GetEntryLevel();

	foreach lev.AllActors(class'Actor', checkme)
	{
		if(TextureLoader(checkme) != None)
			texl = TextureLoader(checkme);
		else if(MeshLoader(checkme) != None)
			mesher = MeshLoader(checkme);
	}

	if(texl != None
		&& mesher != None)
	{
		// Clear the lists
		texl.ClearArray();
		mesher.ClearArray();
		// Now go through each pawn and put their textures/meshes into the list
		foreach DynamicActors(class'PersonPawn', checkpawn)
		{
			texl.AddTexture(checkpawn.Skins[0]);
			mesher.AddMesh(checkpawn.Mesh);
			if(checkpawn.MyHead != None)
			{
				texl.AddTexture(checkpawn.MyHead.Skins[0]);
				mesher.AddMesh(checkpawn.MyHead.Mesh);
			}
		}
	}
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
function bool VerifyGH()
{
	return (DoMP1(true)
			&& DoMP2(true)
			&& DoMP3(true)
			&& FinallyOver());
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
function bool ChangedMP1()
{
	local int oldval;
	const MP1 = 5;
	oldval = int(ConsoleCommand("get "@MultCastPath));
	//log(Self$" cheattest old val, oldval "$oldval$" MultCast "$MultCast);
	return (oldval < MP1
		&& MultCast > MP1 && MultCast <= 2*MP1);
}
function bool DoMP1(optional bool bNoWrite)
{
	local int oldval;
	const MP1A = 5;
	//log(self$" cheattest do mp1, multcast "$MultCast);
	if(MultCast > MP1A && MultCast <= 2*MP1A)
	{
		if(!bNoWrite)
		{
			//log(self$" setting ");
			ConsoleCommand("set "@MultCastPath@MultCast);
		}
		return true;
	}
	else
	{
		oldval = int(ConsoleCommand("get "@MultCastPath));
		//log(Self$" getting old val "$oldval);
		if(oldval > MP1A && oldval <= 2*MP1A)
		{
			return true;
		}
		else
			return false;
	}
}
function bool ChangedMP2()
{
	local int oldval;
	const MP2 = 23;
	oldval = int(ConsoleCommand("get "@FinStopPath));
	log(Self$" cheattest old val, oldval "$oldval$" FinStop "$FinStop);
	return (oldval < MP2
		&& FinStop > MP2 && FinStop <= 2*MP2);
}
function bool DoMP2(optional bool bNoWrite)
{
	local int oldval;
	const MP2A = 23;
	//log(self$" cheattest do mp2, finstop "$finstop);
	if(FinStop > MP2A && FinStop <= 2*MP2A)
	{
		if(!bNoWrite)
		{
			//log(self$" setting ");
			ConsoleCommand("set "@FinStopPath@FinStop);
		}
		return true;
	}
	else
	{
		oldval = int(ConsoleCommand("get "@FinStopPath));
		//log(self$" getting it, "$oldval);
		if(oldval > MP2A && oldval <= 2*MP2A)
		{
			return true;
		}
		else
			return false;
	}
}
function bool ChangedMP3()
{
	local int oldval;
	const MP3 = 18;
	oldval = int(ConsoleCommand("get "@TunnelingPath));
	return (oldval < MP3
		&& Tunneling > MP3 && Tunneling <= 2*MP3);
}
function bool DoMP3(optional bool bNoWrite)
{
	local int oldval;
	const MP3A = 23;
	if(Tunneling > MP3A && Tunneling <= 2*MP3A)
	{
		if(!bNoWrite)
		{
			ConsoleCommand("set "@TunnelingPath@Tunneling);
		}
		return true;
	}
	else
	{
		oldval = int(ConsoleCommand("get "@TunnelingPath));
		if(oldval > MP3A && oldval <= 2*MP3A)
		{
			return true;
		}
		else
			return false;
	}
}

///////////////////////////////////////////////////////////////////////////////
// Something wants to set the game speed, make sure the player isn't in catnip mode
// before something else resets it early, or something like that.
///////////////////////////////////////////////////////////////////////////////
function PossibleSetGameSpeed(float newspeed)
{
	local AWPlayer adude;
	
	adude = AWPlayer(GetPlayer());

	// Catnip mode isn't controlling things, so it's okay
	if(adude.CatnipUseTime == 0)
	{
		if(P2GameInfoSingle(Level.Game) != None)
			P2GameInfoSingle(Level.Game).SetGameSpeedNoSave(newspeed);
	}
}

defaultproperties
{
     GameStateClass=Class'AWGame.AWGameState'
     Begin Object Class=DayBase Name=DayBase0
         Description="Saturday"
         UniqueName="DAY_A"
         ExcludeDays(0)="DEMO"
         LoadTex="aw_textures.loading_sat"
         DudeStartComment="DudeDialog.dude_map_exit1"
         Errands(0)=ErrandBase'EAFX.ErrandBase0'
         PlayerInvList(0)=(InvClassName="Inventory.MoneyInv",NeededAmount=50)
         PlayerInvList(1)=(InvClassName="Inventory.HandsWeapon")
         TakeFromPlayerList(0)=(InvClassName="Inventory.NewspaperInv")
         TakeFromPlayerList(1)=(InvClassName="Inventory.GimpClothesInv")
         TakeFromPlayerList(2)=(InvClassName="Inventory.DudeClothesInv")
         Name="DayBase0"
     End Object
     Days(0)=DayBase'EAFX.DayBase0'
     Begin Object Class=DayBase Name=DayBase1
         Description="Sunday"
         UniqueName="DAY_B"
         ExcludeDays(0)="DEMO"
         LoadTex="aw_textures.loading_sun"
         DudeStartComment="DudeDialog.dude_map_exit1"
         Errands(0)=ErrandBase'EAFX.ErrandBase1'
         TakeFromPlayerList(0)=(InvClassName="Inventory.NewspaperInv")
         TakeFromPlayerList(1)=(InvClassName="Inventory.GimpClothesInv")
         TakeFromPlayerList(2)=(InvClassName="Inventory.DudeClothesInv")
         Name="DayBase1"
     End Object
     Days(1)=DayBase'EAFX.DayBase1'
     IntroURL="MovieIntro.fuk"
     StartFirstDayURL="custest.fuk"
     StartNextDayURL="custest.fuk"
     MainMenuURL="AWStartup.fuk"
     MatSwaps(0)=(OrigMat=Texture'Josh-textures.signs.game_banner_2',NewMat=Texture'Josh-textures.signs.game_banner_5')
     MatSwaps(1)=(OrigMat=Texture'Timb.arcade.Game_FagHunter',NewMat=Texture'Timb.arcade.Game_BastardFish')
     DefaultPlayerClassName="AWPawns.AWDude"
     HUDType="AWGame.AWHUD"
     DefaultPlayerName="TheDude"
     GameName="Postal2 Single Player"
     PlayerControllerClassName="AWGame.AWPlayer"
}
