///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
class EAEarlyGameInfo extends AW7GameSP;

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
const MAX_LIMBS = 30; // Any more than this and we'll start killing old limbs

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
var globalconfig bool bCartoonyEffects;
var globalconfig bool bShinyWeapons;

var globalconfig bool bHeadFix;
var globalconfig bool bLimbFix;

var Limb DeadLimbs[MAX_LIMBS]; // Track limbs here to prevent too many from crashing the game
var int LastLimb;

var() Texture HandsIcon;
var() Texture PeeIcon;
var() Vector PisserAttachPos;
///////////////////////////////////////////////////////////////////////////////
// AddLimb
// This limb just got created, so let us know and kill off old limbs if needed
// Works in a "first in, first out" manner (FIFO)
///////////////////////////////////////////////////////////////////////////////
function AddLimb(Limb NewLimb)
{
	local int i;

	if (LastLimb >= MAX_LIMBS)
		LastLimb = 0;
	else
		LastLimb++;

	if (DeadLimbs[LastLimb] != None)
		DeadLimbs[LastLimb].Destroy();

	DeadLimbs[LastLimb] = NewLimb;
}

function bool IsWeekend()
{
	return false;
}

function bool WeekendOnlyGame()
{
	return false;
}

function bool WeekendGame()
{
	return true;
}

///////////////////////////////////////////////////////////////////////////////
// Kamek add to allow pickups to properly trigger AWTriggers.
// Called when pawn has a chance to pick Item up (i.e. when
// the pawn touches a weapon pickup). Should return true if
// he wants to pick it up, false if he does not want it.
///////////////////////////////////////////////////////////////////////////////
function bool PickupQuery( Pawn Other, Pickup item )
{
	local byte bAllowPickup;
	local bool Q;

	if ( (GameRulesModifiers != None) && GameRulesModifiers.OverridePickupQuery(Other, item, bAllowPickup) )
		return (bAllowPickup == 1);

	if ( Other.Inventory == None )
		return true;
	else
	{
		Q = Other.Inventory.HandlePickupQuery(Item);

		// If this is TRUE, then the inventory handles the pickup and the pickup will not cause a trigger.
		// So, check to see if the item got picked up (bDeleteMe) and then have it trigger its event.
		if (Q && item.bDeleteMe)
			item.TriggerEvent(item.Event, item, Other);

		return !Q;
	}
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
event InitGame( out string Options, out string Error )
{
	Super.InitGame(Options, Error);

	// Kamek: redirect the stock Postal 2's Hands and Urethra icons to the new ED icons
	class'HandsWeapon'.default.Texture = HandsIcon;
	class'UrethraWeapon'.default.Texture = PeeIcon;
	Level.ConsoleCommand("set urethraattachment relativelocation (x=-3.5,y=-30)");
}

defaultproperties
{
     bShinyWeapons=True
     Begin Object Class=DayBase Name=DayBase6
         Description="Saturday"
         UniqueName="DAY_A"
         ExcludeDays(0)="DEMO"
         LoadTex="aw_textures.loading_sat"
         DudeStartComment="DudeDialog.dude_map_exit1"
         Errands(0)=ErrandBase'EAFX.ErrandBase0'
         PlayerInvList(0)=(InvClassName="Inventory.MoneyInv",NeededAmount=50)
         PlayerInvList(1)=(InvClassName="Inventory.HandsWeapon")
         TakeFromPlayerList(0)=(InvClassName="Inventory.NewspaperInv")
         TakeFromPlayerList(1)=(InvClassName="EAInventory.EAGimpClothesInv")
         TakeFromPlayerList(2)=(InvClassName="EAInventory.EADudeClothesInv")
         Name="DayBase6"
     End Object
     Days(0)=DayBase'EAFX.DayBase6'
     Begin Object Class=DayBase Name=DayBase7
         Description="Sunday"
         UniqueName="DAY_B"
         ExcludeDays(0)="DEMO"
         LoadTex="aw_textures.loading_sun"
         DudeStartComment="DudeDialog.dude_map_exit1"
         Errands(0)=ErrandBase'EAFX.ErrandBase1'
         TakeFromPlayerList(0)=(InvClassName="Inventory.NewspaperInv")
         TakeFromPlayerList(1)=(InvClassName="EAInventory.EAGimpClothesInv")
         TakeFromPlayerList(2)=(InvClassName="EAInventory.EADudeClothesInv")
         Name="DayBase7"
     End Object
     Days(1)=DayBase'EAFX.DayBase7'
     HandsIcon=Texture'ED_Hud.HUDempty'
     PeeIcon=Texture'ED_Hud.HUD_zipper'
     bShinyWeapons=True
     MainMenuURL="EA_Startup.fuk"
     IntroURL="EA_Intro.fuk"
     JailURL="EA_Police.fuk#cell"
     StartFirstDayURL="EA_Suburbs-3.fuk"
     StartNextDayURL="EA_Suburbs-3.fuk"
     FinishedDayURL="EA_HomeAtNight.fuk"
     DifficultyNames(10)="Biohazardous_Snark is a fucking Ginger!"
     MatSwaps(0)=(OrigMat=Texture'Josh-textures.signs.game_banner_2',NewMat=Texture'Josh-textures.signs.game_banner_5')
     MatSwaps(1)=(OrigMat=Texture'Timb.arcade.Game_FagHunter',NewMat=Texture'Timb.arcade.Game_BastardFish')
     DefaultPlayerClassName="EAPawns.EAPostalDude"
     PlayerControllerClassName="EAGame.EADudePlayer"
     ChameleonClass=Class'EAGame.EAChameleon'
}
