class EAGameState extends AWGameState;

const MAX_BONUS = 5;

///////////////////////////////////////////////////////////////////////////////
// vars
///////////////////////////////////////////////////////////////////////////////
var travel bool bCopKilla;				// if the cop killer cheat is on
var travel bool bCharisma;				// if people love me
var travel bool bPlayaHata;				// if people hate me
var travel bool bSlowRockets;
var travel bool bFastRockets;
var travel byte bBonusMission[MAX_BONUS];
var travel int RedTurtleDay;			// day number 0-4 that people can see the red turtle

var travel private bool bCheated;		// if player used any cheat codes
var travel float TimeElapsed;			// time elapsed in current game
var travel int InnocentsKilled;			// number of innocents killed
var travel int BaseballHeads;			// number of heads batted off with the baseball bat
var travel int KatanaImpaleCount;		// number of people impaled with katana

var localized string ShotgunRanking;	// ranking for exploding lots of heads
var localized string BaseballRanking;	// ranking for batting lots of heads
var localized string KatanaRanking;		// ranking for impaling people with katana
var localized string ZombieRanking;		// ranking for killing lots of zombies

var localized string SpeedRanking;		// ranking for completing the game quickly
var localized string SuperSpeedRanking;	// ranking for completing the game quickly
var localized string UltraSpeedRanking;	// ranking for completing the game quickly

const SPEED_RUN = 10800.00;				// complete all seven days in under 3 hours
const SUPER_SPEED_RUN = 9000.00;		// complete all seven days in under 2 and a half hours
const ULTRA_SPEED_RUN = 5400.00;		// complete all seven days in under one and a half hours
const MAX_NEWPICK_LENGTH = 10;			// maximum new pickup/weapon travel inf

/* FOR REFERENCE ONLY, DO NOT UNCOMMENT
// Stats for AW
var travel int ZombiesKilledOverall;	// Total zombies killed in game
var travel int ZombiesResurrected;		// total number of zombies brought back to 'life'
var travel int BryanSurvived;			// If RWS Bryan doesn't get killed in the Vince House level
										// defaults to 1.0, which says he's alive. When he dies
										// he tells the player about it so he can count it up.
var travel int KillElephantsScythe;		// If all the elephants were killed with the scythe
										// When an elephant is killed by anything other than the scythe
										// this is set to 0.0
var travel int FanaticsKilled;
var travel int ArmyKilled;
var travel int LimbsHacked;			// Any limb (no heads) cut off
var travel int HeadsLopped;			// Any heads cut off (not exploded)
var travel int LostSledgeInCow;		// Number of sledges lost in the butts of cows

// Stats for P2
var travel int PeopleKilled;		// total people player killed in game
var travel int CopsKilled;			// total cops player killed in game
var travel int ElephantsKilled;		// total elephants player killed in game
var travel int DogsKilled;			// total dogs player killed in game
var travel int CatsKilled;			// total cats player killed in game (includes ones used on guns)
var travel int PistolHeadShot;		// people that you snuck up behind to get a silent pistol head shot on
var travel int ShotgunHeadShot;		// people you shotgunned in the face, blowing up their heads
var travel int RifleHeadShot;		// people you got a one-shot rifle kill on
var travel int CatsUsed;			// cats you violated with your gun
var travel int MoneySpent;			// money the player spent
var travel int PeeTotal;			// 'gallons' of piss you peed. (divide by 10-ish)
var travel int DoorsKicked;			// number of times you kicked a door in a game
var travel int TimesArrested;		// number of times you've been arrested
var travel int DressedAsCop;		// number of times you impersonated a cop (dressed like him)
var travel int DogsTrained;			// number of dogs (non-unique) you trained
var travel int PeopleRoasted;		// number of people killed by fire
var travel int CopsLuredByDonuts;	// Number of cops you've lured by dropping donuts
*/

///////////////////////////////////////////////////////////////////////////////
// Returns true if the player used any cheat codes during the game.
///////////////////////////////////////////////////////////////////////////////
function bool DidPlayerCheat()
{
	return bCheated;
}

///////////////////////////////////////////////////////////////////////////////
// Called by the individual cheats to scar the player's stats screen.
///////////////////////////////////////////////////////////////////////////////
function PlayerCheated()
{
	bCheated = True;
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
function SetBonusMission(int Mission, bool bDone)
{
	if (bDone)
		bBonusMission[Mission] = 1;
	else
		bBonusMission[Mission] = 0;
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
function bool IsBonus(int Mission)
{
	return (bBonusMission[Mission] == 1);
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
function string HoursPlayed()
{
	local int Hours;
	local string S;

	Hours = Int(TimeElapsed/3600);
	S = string(Hours);

	if (Hours < 10)
		S = "0" $ S;

	return S;
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
function string MinutesPlayed()
{
	local int Hours;
	local int Minutes;
	local string S;

	Hours = Int(TimeElapsed/3600);
	Minutes = Int((TimeElapsed - float(Hours*3600)) / 60);
	S = string(Minutes);

	if (Minutes < 10)
		S = "0" $ S;

	return S;
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
function string SecondsPlayed()
{
	local int Hours;
	local int Minutes;
	local int Seconds;
	local string S;

	Hours = Int(TimeElapsed/3600);
	Minutes = Int((TimeElapsed - float(Hours*3600)) / 60);
	Seconds = Int((TimeElapsed - float(Hours*3600) - float(Minutes*60)));
	S = string(Seconds);

	if (Seconds < 10)
		S = "0" $ S;

	return S;
}
/*
///////////////////////////////////////////////////////////////////////////////
// Record a pickup for this level, so it won't be remade when you come back
// to this level
///////////////////////////////////////////////////////////////////////////////
function RecordPickup(name Pname)
{
	local int i;

	i = GottenPickups.Length;
	GottenPickups.Insert(i, 1);
	GottenPickups[i].PickupName = Pname;
	GottenPickups[i].LevelName = P2GameInfo(Level.Game).ParseLevelName(Level.GetLocalURL());
	log(self$" recording "$pname$", for level "$GottenPickups[i].LevelName$" num now "$i+1);
}
*/

function RecordPickup(name Pname)
{
	local EAEarlyGameInfo UseGame;

	UseGame = EAEarlyGameInfo(Level.Game);

	// Don't record pickups during the weekend
	if (!UseGame.IsWeekend())
		Super(GameState).RecordPickup(Pname);
}

///////////////////////////////////////////////////////////////////////////////
// Get the name of the way the player has been playing based on the game stats.
///////////////////////////////////////////////////////////////////////////////
function string GetPlayerRanking()
{
	local EAEarlyGameInfo UseGame;

	UseGame = EAEarlyGameInfo(Level.Game);

	// Speed Run rankings only valid in 7-day game

	if (!UseGame.WeekendOnlyGame() && UseGame.WeekendGame())
	{
		// If you beat the game extremely quickly
		if(TimeElapsed <= ULTRA_SPEED_RUN)
			return UltraSpeedRanking;

		// If you beat the game very quickly
		if(TimeElapsed <= SUPER_SPEED_RUN)
			return SuperSpeedRanking;

		// If you beat the game quickly
		if(TimeElapsed <= SPEED_RUN)
			return SpeedRanking;
	}

	// If you cut off lots and lots of limbs
	if(LimbsHacked > LimbCutMin
		&& LimbsHacked > (ZombiesKilledOverall + PeopleKilled) * LimbCutRatio)
		return GhoulRanking;

	// If 3/4 the people you kill were cops, make you a cop killer
	if(CopsKilled > PeopleKilled*0.75)
		return CopKillerRanking;

	// If you killed more animals than people
	if((ElephantsKilled + DogsKilled + CatsKilled) > (PeopleKilled + ZombiesKilledOverall))
		return AnimalKillerRanking;

	// If you killed at least 1.5x more zombies than people
	if (1.5*ZombiesKilledOverall > PeopleKilled)
		return ZombieRanking;

	// If the number of katanas impaled over half the people you killed in the game
	if(KatanaImpaleCount > PeopleKilled*0.5)
		return KatanaRanking;

	// If the number of baseball head shots over half the people you killed in the game
	if(BaseballHeads > PeopleKilled*0.5)
		return BaseballRanking;

	// If the number of shotgun head shots over half the people/zombies you killed in the game
	if(ShotgunHeadShot > (ZombiesKilledOverall + PeopleKilled)*0.5)
		return ShotgunRanking;

	// If the number of rifle head shots over half the people you killed in the game
	if(RifleHeadShot > PeopleKilled*0.5)
		return RifleKillerRanking;

	// If the number of people killed by fire is over half the people you killed in the game
	if(PeopleRoasted > PeopleKilled*0.5)
		return FireKillerRanking;

	// If you lose your sledge in a few too many cows, you may need to seek therapy
	if(LostSledgeInCow > SledgesLostMax)
		return CowSledgeRanking;

	// If you stick guns up waaay too many cats butts (most of the cats in the game)
	if(CatsUsed > CAT_USED_TOTAL)
		return CatSexRanking;

	// If you pissed more in 'gallons' than killed people in numbers
	if(float(PeeTotal) * 0.1 > PeopleKilled
		// Make sure you killed a few people too though, so you can get the Jesus ranking easier.
		&& PeopleKilled >= Killed3)
		return PeeRanking;

	// rankings for number of people killed

	// Jesus ranking only achievable in a M-F game
	if(PeopleKilled + CatsKilled + ElephantsKilled + DogsKilled + ZombiesKilledOverall == 0)
		return Killed0Ranking;
	else if(PeopleKilled <= Killed1)
		return Killed1Ranking;
	else if(PeopleKilled <= Killed2)
		return Killed2Ranking;
	else if(PeopleKilled <= Killed3)
		return Killed3Ranking;
	else if(PeopleKilled <= Killed4)
		return Killed4Ranking;
	else if(PeopleKilled <= Killed5)
		return Killed5Ranking;
	else if(PeopleKilled <= Killed6)
		return Killed6Ranking;
	else if(PeopleKilled <= Killed7)
		return Killed7Ranking;
	else if(PeopleKilled <= Killed8)
		return Killed8Ranking;
	else if(PeopleKilled <= Killed9)
		return Killed9Ranking;
	else if(PeopleKilled <= Killed10)
		return Killed10Ranking;
	else if(PeopleKilled > Killed10)
		return Killed11Ranking;

}

///////////////////////////////////////////////////////////////////////////////
// Save objects before a level change
///////////////////////////////////////////////////////////////////////////////
function PreLevelChange(P2Pawn PlayerPawn, String LevelName)
{
	local int i, j;
	local FPSPawn p;
	local P2WeaponPickup wp;
	local P2PowerupPickup pp;
	local PawnTravelInv pawninv;
	local WeaponTravelInvNew weaponinv;
	local PowerupTravelInvNew powerupinv;
	local PickupTravelInv pickupinv;
	local Inventory inv;
	local P2Player p2p;

	// If we're sending him to jail, set that he'll be in a cell
	if(bSendingPlayerToJail)
		bPlayerInCell=true;
	else // If not, then clear that he'll be in a cell
		bPlayerInCell=false;

	// Grab some state info out of the dude and set it in the gamestate
	p2p = P2Player(PlayerPawn.Controller);
	if(p2p != None)
	{
		// Save our toggle weapons
		LastWeaponGroupPee = p2p.LastWeaponGroupPee;
		LastWeaponOffsetPee = p2p.LastWeaponOffsetPee;
		LastWeaponGroupHands = p2p.LastWeaponGroupHands;
		LastWeaponOffsetHands = p2p.LastWeaponOffsetHands;

		// Save the inventory item we have selected now
		if(PlayerPawn.SelectedItem != None)
		{
			LastSelectedInventoryGroup = PlayerPawn.SelectedItem.InventoryGroup;
			LastSelectedInventoryOffset = PlayerPawn.SelectedItem.GroupOffset;
		}
		else
		{
			LastSelectedInventoryGroup = -1;
		}

		// Save our current clothes if we're not going to jail.
		// If we are, just set our clothes to the defaults, so we arrive in jail in our
		// normal clothes, even if we were arrested in cop clothes.
		if(!bPlayerInCell)
			CurrentClothes = p2p.CurrentClothes;
		else
			CurrentClothes = p2p.DefaultClothes;

		// Save the status of your map reminders for the player
		TimeSinceErrandCheck = p2p.TimeSinceErrandCheck;
		MapReminderCount = p2p.MapReminderCount;

		// Remember god mode
		bCheatGod = p2p.bGodMode;

		// Save our catnip time
		CatnipUseTime = p2p.CatnipUseTime;

		// Save our armor class type
		HudArmorClass = p2p.HudArmorClass;
	}

	// Find all persistent dead pawns and save them
	foreach DynamicActors(class 'FPSPawn', p)
		{
		if (p.bPersistent 
			&& p.Health <= 0
			&& p.Tag != 'None')
			{
			// Add them to the player's inventory
			i = PawnsArr.Length;
			PawnsArr.Insert(i, 1);
			// If we have an original name, use it
			if(p.OrigLevelName != "")
				PawnsArr[i].LevelName = p.OrigLevelName;
			else
				PawnsArr[i].LevelName = LevelName;
			PawnsArr[i].Tag = p.Tag;
			}
		}

	// Find all persistent weapon pickups and save them
	foreach DynamicActors(class 'P2WeaponPickup', wp)
		{
		if (wp.bPersistent)
			{
			i = WeaponsArr.Length;
			WeaponsArr.Insert(i, 1);
			WeaponsArr[i].LevelName = LevelName;
			wp.PersistentSave(WeaponsArr[i]);
			}
		}

	// Find all persistent powerup pickups and save them
	foreach DynamicActors(class 'P2PowerupPickup', pp)
		{
		if (pp.bPersistent)
			{
			i = PowerupsArr.Length;
			PowerupsArr.Insert(i, 1);
			PowerupsArr[i].LevelName = LevelName;
			pp.PersistentSave(PowerupsArr[i]);
			}
		}

	// This array is added to level during the game so it doesn't need to be created like above.
	// Move all the info about pickups that have been gotten during a day in the game (in any level)
	// and move it for transfer to the players inventory
	// To just slightly limit the number of inventory items sent with this, each inventory of this
	// type can actually count up a certain number of these pickups and record them in an array.
	// This can't be too long though, because travel only supports a certain amount of data
	// per variables (arrays included)
	if(!bChangeDayPostTravel)
	{
		for(i=0; i< GottenPickups.Length; i++)
		{
			if(pickupinv == None)
			{
				pickupinv = spawn(class'PickupTravelInv');
				j=0;	// reset internal array count
			}
			pickupinv.info.Insert(j, 1);
			pickupinv.info[j]= GottenPickups[i];
			j++;
			// Enough in array packet.. send it to the inventory
			if(j == MAX_PICKUP_PACKET)
			{
				PlayerPawn.AddInventory(pickupinv);
				pickupinv=None;
			}
		}
		// Add any residual pickups that didn't fill an entire packet
		if(pickupinv != None)
		{
			PlayerPawn.AddInventory(pickupinv);
			pickupinv=None;
		}
	}
	else //if(bFirstLevelOfDay)
		// If we're on our way to a new day, then don't readd things, just
		// wipe the array clean.
	{
		log(self$" Prechange Forgetting Persistence");
		RemovePersistanceForNewDay(PlayerPawn);
	}

	// Go through the dynamic arrays in GameState that keep persistent values
	// and transfer those to the player's inventory--for travelling between levels.
	for(i=0; i< PawnsArr.Length; i++)
	{
		pawninv = spawn(class'PawnTravelInv');
		pawninv.info = PawnsArr[i];
		PlayerPawn.AddInventory(pawninv);
	}

	// Kamek edit
	// Use "new" WeaponTravelInv and PowerupTravelInv, to stop the infinite recursion glitches.
	log(self@"entering new weapontravelinv and poweruptravelinv section");
	// reset index
	j = 0;
	for(i=0; i< WeaponsArr.Length; i++)
	{
		// make a new weaponinv
		if (weaponinv == none)
			weaponinv = spawn(class'WeaponTravelInvNew');

		// add this weapon to the array
		weaponinv.info.Insert(j,1);
		weaponinv.info[j] = WeaponsArr[i];
		log(self@"inserting weapons info"@WeaponsArr[i].ClassName@WeaponsArr[i].LevelName@"in slot"@j@"of weaponinv"@weaponinv);
		j++;

		// it's full, so add it to the inventory
		if(j >= MAX_NEWPICK_LENGTH)
		{
			log(self@"weaponinv"@weaponinv@"is full, adding to player inventory");
			PlayerPawn.AddInventory(weaponinv);
			weaponinv = None;
			j = 0;
		}
	}
	// add back in any leftovers
	if (Weaponinv != None)
	{
		log(self@"all weapons accounted for, adding in"@weaponinv);
		PlayerPawn.AddInventory(weaponinv);
	}

	// reset index
	j = 0;
	for(i=0; i< PowerupsArr.Length; i++)
	{
		// make a new powerupinv
		if (powerupinv == None)
			powerupinv = spawn(class'PowerupTravelInvNew');

		// add this powerup
		powerupinv.info.Insert(j,1);
		powerupinv.info[j] = PowerupsArr[i];
		log(self@"inserting powerup info"@PowerupsArr[i].ClassName@PowerupsArr[i].LevelName@"in slot"@j@"of powerupinvinv"@powerupinv);
		j++;

		// if it's full, add it in
		if(j >= MAX_NEWPICK_LENGTH)
		{
			log(self@"powerupinv"@powerupinv@"is full, adding to player inventory");
			PlayerPawn.AddInventory(powerupinv);
			powerupinv = None;
			j = 0;
		}
	}
	// add back in any leftovers
	if (PowerupInv != None)
	{
		log(self@"all powerups accounted for, adding in"@powerupinv);
		PlayerPawn.AddInventory(PowerupInv);
	}

	log(self$" Synchronizing and writing new difficulty to ini, was "$P2GameInfo(Level.Game).GameDifficulty$" is now "$GameDifficulty);
	// Use the game difficulty from the game state (this was the saved diff)
	P2GameInfo(Level.Game).GameDifficulty = GameDifficulty;
	// Make sure to write the difficulty to the ini, or it won't be carried to the next
	// level correctly.
	ConsoleCommand("set "@DifficultyPath@P2GameInfo(Level.Game).GameDifficulty);
}

///////////////////////////////////////////////////////////////////////////////
// Restore objects after a level change
///////////////////////////////////////////////////////////////////////////////
function PostLevelChange(P2Pawn PlayerPawn, String LevelName)
	{
	local int i, j;
	local P2Pawn ppawn;
	local FPSPawn fpawn;
	local Inventory inv, oldinv;
	local P2Player p2p;
	local PickupTravelInv ptinv;
	local WeaponTravelInvNew wtinv;
	local PowerupTravelInvNew powtinv;

	log(self$" PostLevelChange");
	// Go back through the inventory of the player, and for each inventory
	// item that stored info about persistant objects, remove it
	// and give it back to the gamestate.
	inv = PlayerPawn.Inventory;
	while(inv != None)
	{
		oldinv = inv.Inventory;
		if(PawnTravelInv(inv) != None)
		{
			// Remember the guy we travelled
			i = PawnsArr.Length;
			PawnsArr.Insert(i, 1);
			PawnsArr[i] = PawnTravelInv(inv).info;
			// Now, delete him from our inventory
			PlayerPawn.DeleteInventory(inv);
			inv = PlayerPawn.Inventory;
		}
		else if(WeaponTravelInvNew(inv) != None)
		{
			wtinv = WeaponTravelInvNew(inv);
			for (j=0; j<wtinv.info.Length; j++)
			{
				i = WeaponsArr.Length;
				WeaponsArr.Insert(i, 1);
				WeaponsArr[i] = wtinv.info[j];
			}
			PlayerPawn.DeleteInventory(inv);
			inv = PlayerPawn.Inventory;
		}
		else if(PowerupTravelInvNew(inv) != None)
		{
			powtinv = PowerupTravelInvNew(inv);
			for (j=0; j<powtinv.info.Length; j++)
			{
				i = PowerupsArr.Length;
				PowerupsArr.Insert(i, 1);
				PowerupsArr[i] = powtinv.info[j];
			}
			PlayerPawn.DeleteInventory(inv);
			inv = PlayerPawn.Inventory;
		}
		else if(PickupTravelInv(inv) != None)
		{
			ptinv = PickupTravelInv(inv);
			for(j=0; j< ptinv.info.Length; j++)
			{
				i = GottenPickups.Length;
				GottenPickups.Insert(i, 1);
				GottenPickups[i] = ptinv.info[j];
			}
			PlayerPawn.DeleteInventory(inv);
			inv = PlayerPawn.Inventory;
		}
		else
			inv = inv.Inventory;
	}

	// Go through you're current inventory and mark any powerups or weapons mentioned
	// in the Inactive lists as not wanting to show their hints anymore.
	inv = PlayerPawn.Inventory;
	while(inv != None)
	{
		if(P2Weapon(inv) != None)
		{
			if(FoundInvHintInactive(P2Weapon(inv).GetRank()))
				P2Weapon(inv).bAllowHints=false;
		}
		else if(P2PowerupInv(inv) != None)
		{
			if(FoundInvHintInactive(P2PowerupInv(inv).GetRank()))
				P2PowerupInv(inv).bAllowHints=false;
		}

		inv = inv.Inventory;
	}

	// If we're starting a new day, yet somehow gottenpickups magically got filled (through some
	// odd debug level changing) wipe it away
	if(bFirstLevelOfDay)
	{
		log(self$" Postchange Forgetting Persistence");
		RemovePersistanceForNewDay(PlayerPawn);
	}

	// Kill any pawns that are supposed to be dead on this level.
	// Note that we never remove the pawns from the list because they should
	// remain dead for the duration of the game.
	for (i = 0; i < PawnsArr.Length; i++)
		{
		if (PawnsArr[i].LevelName ~= LevelName)
			{
			foreach DynamicActors(class'FPSPawn', fpawn, PawnsArr[i].Tag)
				{
				// Post-release fix: Fixes the terrible 1000000 iteration preppawnslider
				// crash bug. Seems to be because the default pawn code doesn't correctly
				// clean up the controller when it's destroyed. So we explicitly 
				// destroy it now here.
				if(fpawn.Controller != None)
					fpawn.Controller.Destroy();
				// End

				fpawn.Destroy();
				}
			}
		}

	// Restore all persistent weapon pickups that belong on this level.
	// Start at end of array and work towards front to make removals easier/faster.
	for (i = WeaponsArr.Length - 1; i >= 0; i--)
		{
		if (WeaponsArr[i].LevelName ~= LevelName)
			{
			class'P2WeaponPickup'.Static.PersistentRestore(WeaponsArr[i], PlayerPawn.Level);
			WeaponsArr.remove(i, 1);
			}
		}

	// Restore all persistent powerup paickups that belong on this level.
	// Start at end of array and work towards front to make removals easier/faster.
	for (i = PowerupsArr.Length - 1; i >= 0; i--)
		{
		if (PowerupsArr[i].LevelName ~= LevelName)
			{
			class'P2PowerupPickup'.Static.PersistentRestore(PowerupsArr[i], PlayerPawn.Level);
			PowerupsArr.remove(i, 1);
			}
		}

	// Find all the pawns in the level and set any that are supposed to now hate
	// the player to hating him.
	for (i = 0; i < CurrentHaters.Length; i++)
		{
			foreach DynamicActors(class'P2Pawn', ppawn)
			{
				if(ppawn.IsA(CurrentHaters[i].ClassName))
				{
					ppawn.bPlayerIsEnemy = true;
					ppawn.bPlayerHater = true;
				}
			}
		}

	// Now that we're back in our level, start decrementing our timer again
	SetRadioTimer();

	// Get the dude specific stuff back out of gamestate
	p2p = P2Player(PlayerPawn.Controller);
	if(p2p != None)
		{
		p2p.LastWeaponGroupPee = LastWeaponGroupPee;
		p2p.LastWeaponOffsetPee = LastWeaponOffsetPee;
		p2p.LastWeaponGroupHands = LastWeaponGroupHands;
		p2p.LastWeaponOffsetHands = LastWeaponOffsetHands;

		// Restore our clothes
		if(!bPreGameMode && !bFirstLevelOfDay)
			{
			p2p.CurrentClothes = CurrentClothes;
			p2p.SetClothes(p2p.CurrentClothes);
			}

		// Save the status of your map reminders for the player
		p2p.TimeSinceErrandCheck = TimeSinceErrandCheck;
		p2p.MapReminderCount = MapReminderCount;

		// Restore god mode if necessary
		if (bCheatGod)
			p2p.bGodMode = true;

		// Re-init cat useage
		p2p.SetupCatnipUseage(CatnipUseTime);

		// Re-init hud armor type (inventory classes not defined yet -- hence this ugly code)
		p2p.HudArmorClass = HudArmorClass;
		if(HudArmorClass != None)
			{
			inv = spawn(HudArmorClass);
			p2p.HudArmorIcon = Texture(inv.Icon);
			inv.Destroy();
			}
		}
	}

defaultproperties
{
     RedTurtleDay=-1
     ShotgunRanking="Shotgun Ninja"
     BaseballRanking="Head-Batting Major Leaguer"
     KatanaRanking="Katana-Wielding Ninja of Death"
     ZombieRanking="Zombie-Killing Holy Warrior"
     SpeedRanking="Speed Runner"
     SuperSpeedRanking="Seasoned Speed Runner"
     UltraSpeedRanking="Hardcore Speed Runner"
     Killed2=50
     Killed3=100
     Killed4=300
     Killed5=600
     Killed6=800
     Killed7=1000
     Killed8=1500
     Killed9=2000
     Killed10=4000
     SledgesLostMax=5
     GhoulRanking="Limb-Hacking Postal Ghoul"
     Killed0Ranking="Jesus Freak"
     Killed8Ranking="Society-Shunning Grade-A Psycho"
     Killed10Ranking="Disciple of Hitler"
     Killed11Ranking="Satanist"
     AnimalKillerRanking="PETA-Hating Animal Murderer"
}
