///////////////////////////////////////////////////////////////////////////////
// EADudePlayer
// By: Kamek (kamek@postalleague.com)
// For: EA
//
// Dude controller for EA. Custom cheat manager plus necessary functions for
// Random Wardrobe.
///////////////////////////////////////////////////////////////////////////////
class EADudePlayer extends AWDudePlayer;

     var Sound RadarClickSound;
     var Sound RadarBuzzSound;
     var Sound RadarClickSoundoff;

///////////////////////////////////////////////////////////////////////////////
// Structs
///////////////////////////////////////////////////////////////////////////////
struct ClothesAndFriends
{
	var class<P2Pawn> PawnClass; // class of pawns who are friends with me in this outfit
	var class<P2Pawn> HaterClass; // class of pawns who hate me in this outfit
	var class<Inventory> ClothesClass; // outfit class
};

///////////////////////////////////////////////////////////////////////////////
// Structs (merge with existing)
///////////////////////////////////////////////////////////////////////////////
struct OldPawnSkins
{
	var Pawn Pawn;								// Pawn who used these skins
	var BodyPart Head;								// head I had
	var bool bWasUnlit;							// False for most pawns
	var bool bHeadUnlit;						// False for most heads
	var array<Material> PawnSkins;				// Skins for the pawn
	var array<Material> HeadSkins;				// Head skins for the pawn
};

///////////////////////////////////////////////////////////////////////////////
// Consts
///////////////////////////////////////////////////////////////////////////////
const STATIC_ALPHA = 64;

///////////////////////////////////////////////////////////////////////////////
// Public vars
///////////////////////////////////////////////////////////////////////////////
//var() float HeatVisionRadiusMin;				// Max radius to draw the full heat vision pawn
//var() float HeatVisionRadiusMax;				// Radius at which we stop drawing the heat vision pawn
var() float HeatVisionRadius;					// Max radius to draw the heat vision pawn
var() Material HeatTex;							// Texture for heat vision
var() Material ColdTex;							// Texture for cold actors (zombies)
var() Material OverlayTex;						// Overlay for static effects
var() Material DefHands;
var() CameraEffect CatnipEffect;

var() array<Texture> Crosshairs;
//var int CrosshairNum;

// Dude comments for changing into different clothing
var() sound GimpSound;
var() sound BandSound;
var() sound BookSound;
var() sound BumSound;
var() sound ButcherSound;
var() sound FanaticSound;
var() sound GangSound;
var() sound Hippysound;
var() sound HomoSound;
var() sound MallSound;
var() sound FoodSound;
var() sound VeganSound;
var() sound PriestSound;
var() sound RedneckSound;
var() sound RWSSound;
var() sound ScientistSound;
var() sound ZealotSound;

// How other pawns treat the dude depending on how he's dressed
var() array<ClothesAndFriends> DudeRelations;

///////////////////////////////////////////////////////////////////////////////
// Internal vars
///////////////////////////////////////////////////////////////////////////////
var array<OldPawnSkins> OldSkins;				// Remember what skins were on what pawns
var bool bHeatVisionActive;						// Whether heat vision is active or not.
var bool bWallHackActive;						// Whether "wall hack" is active or not

// Kamek
var bool bCountDown;
var float CountdownStart;
var float CountdownLength;
var name CountdownEvent;
var rotator LastRocketRotation;					// To tell whether we're actively controlling the rocket or not
var DayBase FakeSaturday;
var DayBase FakeSunday;

function RequestStats();

///////////////////////////////////////////////////////////////////////////////
// Here's a hack to get the proper day showing
///////////////////////////////////////////////////////////////////////////////
function DisplayLoad(DayBase day, String URL)
{
	const MAP_EAWEEKENDSTART = "EA_Intro.fuk";
	const MAP_AWSTART = "ed_ss_basement.fuk";

	local DayBase UseDay;

	UseDay = day;

	// If we're starting a new weekend-only game, use the Saturday loading screen instead
	// Uses a "fake" DayBase, otherwise the gameinfo will use Monday's daybase and loading screen
	if (Left(URL, Len(MAP_EAWEEKENDSTART)) == MAP_EAWEEKENDSTART
		|| Left(URL, Len(MAP_AWSTART)) == MAP_AWSTART)
		UseDay = FakeSaturday;

	// Only do this for single player games
	if (P2GameInfoSingle(Level.Game) != None)
	{
		if (!MyLoadScreen.IsRunning())
		{
			MyLoadScreen.Show(UseDay, URL);
			CurrentScreen = MyLoadScreen;
			SetMyOldState();
			GotoState('WaitScreen');
		}
	}
	else
		Warn("DisplayLoad() should not be called in multiplayer game.");
}

///////////////////////////////////////////////////////////////////////////////
// Setup the catnip with this time
///////////////////////////////////////////////////////////////////////////////
function SetupCatnipUseage(float starttime)
{
	if(starttime > 0)
	{
		CatnipUseTime = starttime;
		// Leave a log here just so we know when this was used.
		//log(self$" Setting catnip time--slomo: "$starttime);
		if(P2GameInfoSingle(Level.Game) != None)
			P2GameInfoSingle(Level.Game).SetGameSpeedNoSave(1/CATNIP_SPEED);

//		AddCameraEffect(CatnipEffect);
	}
	else // reset things
	{
		CatnipUseTime = 0;
		//log(self$" Resetting catnip time: "$starttime);
		// Reset effects
		if(P2GameInfoSingle(Level.Game) != None)
			P2GameInfoSingle(Level.Game).SetGameSpeedNoSave(1.0);

//		RemoveCameraEffect(CatnipEffect);
	}
}

///////////////////////////////////////////////////////////////////////////////
// Get current reticle index
///////////////////////////////////////////////////////////////////////////////
simulated function Texture GetReticleTexture()
	{
	if (bEnableReticle)
		return Crosshairs[ReticleNum];
	return None;
	}

///////////////////////////////////////////////////////////////////////////////
// Say something funny when someone dies (if you're not already talking)
// Only in single player
///////////////////////////////////////////////////////////////////////////////
function SomeoneDied(P2Pawn DeadGuy, P2Pawn Killer, class<DamageType> DeadDamageType)
{
	local bool bValidQuickKilling;
	local bool bNoSuper;

	if(Killer == MyPawn
		&& Level.Game != None
		&& FPSGameInfo(Level.Game).bIsSinglePlayer)
	{
		// You're killing people in quick succession. If you kill the next person
		// within QUICK_KILL_TIME of killing the last one, he'll say something
		// else and give you something.

		// Only check real bystanders for this--innocents. (they may have weapons though).
		bValidQuickKilling = (ValidQuickKill(DeadGuy) && bQuickKilling);
		if(bValidQuickKilling)
		{
			//log(self$" just killed "$DeadGuy$" time "$Level.TimeSeconds$" time section "$(Level.TimeSeconds - QuickKillLastTime));
			if((Level.TimeSeconds - QuickKillLastTime) < (QUICK_KILL_TIME - QuickKillSoundIndex))
			{
				QuickKillLastTime=Level.TimeSeconds;
				DelayedSayLine = QuickKillLine;
				bWaitingToTalk=true;
				SayTime = WAIT_FOR_DEATH_COMMENT_GUN;
				DeadGuy.PickQuickKillPrize(QuickKillSoundIndex);
			}
			else	// You've waited too long... not into the
			{
				//Pawn.PlaySound(QuickKillBadEndSound, SLOT_Misc);
				bQuickKilling=false;
			}
			if(SayTime > 0
				|| bWaitingToTalk)
			{
				SetTimer(SayTime, false);
				bStillTalking=true;
			}
		}

		if(!bStillTalking
			&& !bValidQuickKilling)
		{
			SayTime = 0;

			if(FRand() <= SayThingsOnGuyDeathFreq)
			{
				if(ClassIsChildOf(DeadDamageType , class'MacheteDamage')
					|| ClassIsChildOf(DeadDamageType , class'ScytheDamage'))
				{
					DelayedSaySound = DudeBladeKill[Rand(DudeBladeKill.Length)];
					SayTime = WAIT_FOR_DEATH_COMMENT_MELEE;
					bNoSuper=true;
					bWaitingToTalk=true;
				}
			}

			if(SayTime > 0
				|| bWaitingToTalk)
			{
				SetTimer(SayTime, false);
				bStillTalking=true;
			}
			else if(DeadDamageType == class'OnFireDamage'
				|| ClassIsChildOf(DeadDamageType, class'BurnedDamage'))
			{
				if(FRand() <= SayThingsOnGuyBurningDeathFreq)
					SayTime = MyPawn.Say(MyPawn.myDialog.lDude_BurningPeople) + 0.5;
			}
			else if(DudeIsCop())
			{	
				if(FRand() <= SayThingsOnGuyDeathFreq)
					SayTime = MyPawn.Say(MyPawn.myDialog.lDude_AttackAsCop) + 0.5;
			}
			else if(FRand() <= SayThingsOnGuyDeathFreq)
			{
				if(ClassIsChildOf(DeadDamageType , class'BulletDamage'))
				{
					// Randomly (not very often) start allowing the player to kill people
					// in quick succession (only with guns). Doesn't count sniper rifle
					// head shot kills in this.
					if(!bQuickKilling
						&& FRand() < QUICK_KILL_FREQ
						&& ValidQuickKill(DeadGuy))
					{
						bQuickKilling=true;
						QuickKillSoundIndex=0;
						QuickKillLine = MyPawn.myDialog.lDude_QuickKills;
						QuickKillLastTime=Level.TimeSeconds;
					}
					else // Otherwise, make general comments about killing them with a gun
					{
						if(FRand() < COMMENT_ON_RACE
							&& (P2MocapPawn(DeadGuy).bIsBlack
								|| P2MocapPawn(DeadGuy).bIsMexican
								|| P2MocapPawn(DeadGuy).bIsHindu
								|| P2MocapPawn(DeadGuy).bIsAsian))
							DelayedSayLine = MyPawn.myDialog.lDude_ShootMinorities;
						else
							DelayedSayLine = MyPawn.myDialog.lDude_KillWithGun;
						bWaitingToTalk=true;
						SayTime = WAIT_FOR_DEATH_COMMENT_GUN;
					}
				}
				else if(ClassIsChildOf(DeadDamageType , class'BludgeonDamage'))
				{
					DelayedSayLine = MyPawn.myDialog.lDude_KillWithMelee;
					bWaitingToTalk=true;
					SayTime = WAIT_FOR_DEATH_COMMENT_MELEE;
				}
				else if(ClassIsChildOf(DeadDamageType , class'ExplodedDamage'))
				{
					DelayedSayLine = MyPawn.myDialog.lDude_KillWithProjectile;
					bWaitingToTalk=true;
					SayTime = WAIT_FOR_DEATH_COMMENT_PROJ;
				}
			}

			if(SayTime > 0
				|| bWaitingToTalk)
			{
				SetTimer(SayTime, false);
				bStillTalking=true;
			}
		}
	}
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
function bool IsAddictive(Inventory Inv)
{
	return (Inv.IsA('CrackInv') || Inv.IsA('MaDWeedInv'));
}


///////////////////////////////////////////////////////////////////////////////
// Turns every current non-player, bystander pawns into Gary Colemans
// It's easy to debug in a player controller (hard in a cheat manager)
///////////////////////////////////////////////////////////////////////////////
function GarySize()
{
	local FPSPawn checkpawn;
	local EAGary newgary;
	local vector useloc;
	local rotator userot;

	if(!CheatsAllowed())
		return;

	log(self$" CHEAT: Whatchutalkinbout");
	ClientMessage("Gary-sizing your bystanders--please wait.");

	foreach DynamicActors(class'FPSPawn', checkpawn)
	{
		if(!checkpawn.bPlayer
			&& !checkpawn.bSliderStasis
			&& BystanderController(checkpawn.Controller) != None
			&& CashierController(checkpawn.Controller) == None
			&& RWSController(checkpawn.Controller) == None
			&& Gary(checkpawn) == None)
		{
			useloc = checkpawn.Location;
			userot = checkpawn.Rotation;
			checkpawn.Destroy();
			newgary = spawn(class'EAGary',,,useloc,userot,GarySkins[Rand(GarySkins.Length)]);
			if (newgary != None 
				&& newgary.Controller == None
				&& newgary.Health > 0 )
			{
				// Don't put the real gary controller on these... you'l have the right
				// dialog, and the real one has a cashier controller and needs other things.
				newgary.Controller = spawn(class'BystanderController');
				if ( newgary.Controller != None )
				{
					newgary.Controller.Possess(newgary);
					newgary.CheckForAIScript();
				}
				else
					newgary.Destroy();
			}
		}
	}

	MyPawn.PlaySound(GaryTalking);
}

///////////////////////////////////////////////////////////////////////////////
// Search through the players inventory and use the most powerful
// non-addictive health he has.
// Uses smaller stuff as we get closer and closer to full
///////////////////////////////////////////////////////////////////////////////
exec function QuickHealth2()
{
	local Inventory inv;
	local P2PowerupInv pickme;
	local float healval, maxheal, mindiff;
	local float HealthRequired;

	if (Level.Pauser!=None)
		return;

	if (P2Pawn(Pawn) != None)
	{
		//mypawnfix
		inv = Pawn.Inventory;
		HealthRequired = P2Pawn(Pawn).HealthMax - Pawn.Health;
		mindiff = 0 - HealthRequired;

		while(inv != None)
		{
			if(P2PowerupInv(inv) != None)
			{
				healval = P2PowerupInv(inv).RateHealingPower();
//				log(self@"considering healing item"@inv@"heals"@healval@"health required"@healthrequired@"mindiff"@mindiff);
				if(!IsAddictive(Inv) && (healval - HealthRequired) > mindiff && (healval - HealthRequired) < 0 && healval > 0)
				{
					mindiff = healval - HealthRequired;
					pickme = P2PowerupInv(inv);
				}
			}
			inv = inv.Inventory;
		}

		// Heal the player with this item
		if(pickme != None)
		{
			// Don't let him use food when he's over the max
			if(Pawn.Health < P2Pawn(Pawn).HealthMax)
			{
				pickme.Activate();
				MyHUD.LocalizedMessage(class'PickupMessagePlus', ,,,,pickme.HealingString);
			}
		}
		else // use the lowest-healing health we have
		{
			inv = Pawn.Inventory;
			maxheal = P2Pawn(Pawn).HealthMax;

			while(inv != None)
			{
				if(P2PowerupInv(inv) != None)
				{
					healval = P2PowerupInv(inv).RateHealingPower();
					if(!IsAddictive(Inv) && healval > 0 && healval < maxheal)
					{
						maxheal = healval;
						pickme = P2PowerupInv(inv);
					}
				}
				inv = inv.Inventory;
			}

			// Heal the player with this item
			if(pickme != None)
			{
				// Don't let him use food when he's over the max
				if(Pawn.Health < P2Pawn(Pawn).HealthMax)
				{
					pickme.Activate();
					MyHUD.LocalizedMessage(class'PickupMessagePlus', ,,,,pickme.HealingString);
				}
			}
			else // Out of options; call super
			{
				QuickHealth();
			}
		}
	}
}

///////////////////////////////////////////////////////////////////////////////
//	AGAIN in order to extend this reasonably, all the class specified below
// should have been variables, so they could easily have been changed
// in the expansion. Instead, this entire function had to be copied
// over just to change the stats screen--yippee!
//
// Setup various screens.  Screens aren't saved because they're objects (and
// are markred transient).  Once they are created, the screens will exist
// throughout the game, so we always look for existing screens first, and if
// they don't exist then we create them.
///////////////////////////////////////////////////////////////////////////////
function SetupScreens()
	{
	local int i;
	local MapScreen map;
	local NewsScreen news;
	local VoteScreen vote;
	local PickScreen pick;
	local LoadScreen load;
	local ClothesScreen clothes;
	local StatsScreen stats;

	// Only do this for single player games
	if (P2GameInfoSingle(Level.Game) != None)
		{
		// Search for existing screens
		for (i = 0; i < Player.LocalInteractions.Length; i++)
			{
			map = MapScreen(Player.LocalInteractions[i]);
			if (map != None)
				MyMapScreen = map;

			news = NewsScreen(Player.LocalInteractions[i]);
			if (news != None)
				MyNewsScreen = news;

			vote = VoteScreen(Player.LocalInteractions[i]);
			if (vote != None)
				MyVoteScreen = vote;

			pick = PickScreen(Player.LocalInteractions[i]);
			if (pick != None)
				MyPickScreen = pick;

			load = LoadScreen(Player.LocalInteractions[i]);
			if (load != None)
				MyLoadScreen = load;

			clothes = ClothesScreen(Player.LocalInteractions[i]);
			if (clothes != None)
				MyClothesScreen = clothes;

			stats = StatsScreen(Player.LocalInteractions[i]);
			if (stats != None)
				MyStatsScreen = stats;
			}

		// If screens weren't found, create new ones
		if (MyMapScreen == None)
			MyMapScreen = MapScreen(Player.InteractionMaster.AddInteraction("EAGame.EAMapScreen", Player));

		if (MyNewsScreen == None)
			MyNewsScreen = NewsScreen(Player.InteractionMaster.AddInteraction("Postal2Game.NewsScreen", Player));

		if (MyVoteScreen == None)
			MyVoteScreen = VoteScreen(Player.InteractionMaster.AddInteraction("Postal2Game.VoteScreen", Player));

		if (MyPickScreen == None)
			MyPickScreen = PickScreen(Player.InteractionMaster.AddInteraction("Postal2Game.PickScreen", Player));

		if (MyLoadScreen == None)
			MyLoadScreen = LoadScreen(Player.InteractionMaster.AddInteraction("Postal2Game.LoadScreen", Player));

		if (MyClothesScreen == None)
			MyClothesScreen = ClothesScreen(Player.InteractionMaster.AddInteraction("Postal2Game.ClothesScreen", Player));

		// Change to new stats screen!
		if (MyStatsScreen == None)
			MyStatsScreen = StatsScreen(Player.InteractionMaster.AddInteraction("EAGame.EAStatsScreen", Player));
		}
	}

///////////////////////////////////////////////////////////////////////////////
// Cheat for testing heat vision. Leave in if desired
///////////////////////////////////////////////////////////////////////////////
exec function Heat()
{
	if(!CheatsAllowed())
		return;

	log(self$" CHEAT: Heat");

	if (bHeatVisionActive)
		DeactivateHeatVision();
	else
	{
		bWallHackActive = False;
		ActivateHeatVision();
	}

	CommentOnCheating();
}

///////////////////////////////////////////////////////////////////////////////
// Silly wallhack cheat
///////////////////////////////////////////////////////////////////////////////
exec function WallHack()
{
	if(!CheatsAllowed())
		return;

	log(self$" CHEAT: WallHack");

	bWallHackActive = !bWallHackActive;
	if (bWallHackActive)
		DeactivateHeatVision();

	CommentOnCheating();
}

///////////////////////////////////////////////////////////////////////////////
// Draw the blue static first, then overlay the heat vision pawns
///////////////////////////////////////////////////////////////////////////////
function RenderHeatVisionOverlay(Canvas Canvas)
{
	local Controller C;
	local Pawn P;
	local float Dist;
	local byte Alpha;
	local FireEmitter Fire;

	const Scale = 1.0;

	// Set to blue color with 1/2 transparency. Change alpha value if needed.
	Canvas.SetDrawColor(0, 0, 255, STATIC_ALPHA);
	Canvas.SetPos(0,0);

	// Set to alpha rendering style and draw in the static effect.
	Canvas.Style = ERenderStyle.STY_Alpha;
	Canvas.DrawTile(OverlayTex, Canvas.SizeX, Canvas.SizeY, (Canvas.CurX-Canvas.OrgX)*Scale, (Canvas.CurY-Canvas.OrgY)*Scale, Canvas.SizeX*Scale, Canvas.SizeY*Scale );
	// Can't use DrawPattern as it requires a Texture and not a Material
//	Canvas.DrawPattern(OverlayTex, Canvas.SizeX, Canvas.SizeY, 1.0);

	// Reset to normal drawing with full color and no alpha
	Canvas.Style = ERenderStyle.STY_Normal;
	Canvas.SetDrawColor(255, 255, 255, 255);

//	foreach DynamicActors(class'FireEmitter', Fire)
//		Canvas.DrawActor(Fire, false);

	// I used Level.ControllerList instead of foreach DynamicActors here.
	// The controller list is faster, and pawns that are dead don't get overlaid, so they appear "cold"
	for (C=Level.ControllerList; C!=None; C=C.NextController)
	{
		// This draws all non-hidden pawns within the radius, including those behind walls.
		// Remove the comments around C.Pawn.PlayerCanSeeMe() to remove the "wallhack" effect if desired.
		// Also, to make zombies appear "cold", uncomment the "C.Pawn.IsA('ZombiePawn')" and put in your zombie class
		// (or subclass of AW's Zombies)
		if (C != Self && C.Pawn != None && !C.Pawn.bHidden /* && C.Pawn.PlayerCanSeeMe() */ /* && C.Pawn.IsA('ZombiePawn') */)
		{
			P = C.Pawn;
			Dist = VSize(P.Location - Pawn.Location);
			Canvas.Style = ERenderStyle.STY_Alpha;

			// When I first tried this out, drawing every single pawn really lagged the game,
			// so I forced a "visible radius" for the heat vision. That caused the nasty effect
			// of pawns just popping into view randomly. The commented out code below was my
			// attempt to get them to fade in and out nicely instead of just "popping" into view.
			// Canvas.DrawActor doesn't seem to obey the drawstyle.

//			if (Dist < HeatVisionRadiusMin)
			if (Dist < HeatVisionRadius || C.Pawn.PlayerCanSeeMe())
				Canvas.DrawActor(P, false);
//			else if (Dist < HeatVisionRadiusMax)
//			{
//				Alpha = byte(255*(Dist-HeatVisionRadiusMin)/(HeatVisionRadiusMax-HeatVisionRadiusMin));
//				log("SetDrawColor Alpha="@Alpha);
//				Canvas.SetDrawColor(255, 255, 255, Alpha);
//				Canvas.DrawActor(P, false);
//			}
//			else
//				Canvas.DrawActor(P, true);
		}
	}
}

function RegisterHeatPawn(Pawn P)
{
	local P2MoCapPawn MoCap;
	local BodyPart H;
	local int i, j;

	j = OldSkins.Length;

//	log("===== Processing Pawn"@P);

	OldSkins.Insert(OldSkins.Length, 1);
	OldSkins[j].Pawn = P;
	OldSkins[j].bWasUnlit = P.bUnlit;
	P.bUnlit = True;

//	log("Added skin entry"@j);

	// If this is a Mocap Pawn, skin the head as well
	if (P2MoCapPawn(P) != None)
	{
//		log("This is a MoCap Pawn");
		MoCap = P2MoCapPawn(P);
		H = MoCap.MyHead;
		OldSkins[j].Head = H;
		OldSkins[j].bHeadUnlit = H.bUnlit;
		H.bUnlit = True;
//		log("Adding"@H.Skins.Length@"head skins");
		OldSkins[j].HeadSkins.Insert(0, H.Skins.Length);
		for (i=0; i<H.Skins.Length; i++)
		{
//			log("Adding Head Skin"@H.Skins[i]);
			OldSkins[j].HeadSkins[i] = H.Skins[i];

			if(AWZombie(P) != None || P.Health <= 0)
				H.Skins[i] = ColdTex;
			else
				H.Skins[i] = HeatTex;
		}
	}
//	log("Adding"@P.Skins.Length@"pawn skins");
	OldSkins[j].PawnSkins.Insert(0, P.Skins.Length);
	for (i=0; i<P.Skins.Length; i++)
	{
//		log("Adding Pawn Skin"@P.Skins[i]);
		OldSkins[j].PawnSkins[i] = P.Skins[i];
		if(AWZombie(P) != None || P.Health <= 0)
			P.Skins[i] = ColdTex;
		else
			P.Skins[i] = HeatTex;
	}
//	j++;
//	log("End processing of Pawn"@P);
}

///////////////////////////////////////////////////////////////////////////////
// Turns heat vision on.
// Call this function from the Crossbow Alt-Fire.
///////////////////////////////////////////////////////////////////////////////
function ActivateHeatVision()
{
	local Pawn P;
	local P2MoCapPawn MoCap;
	local BodyPart H;
	local int i, j;

	// fails if heat vision is already on
	if (bHeatVisionActive)
		return;

//	log("Activate Heat Vision.");

	bHeatVisionActive=True;

	j = 0;

	// Swap all pawn skins with the heat vision skins.
	foreach DynamicActors(class'Pawn', P)
	{
//		log("===== Processing Pawn"@P);

		OldSkins.Insert(OldSkins.Length, 1);
		OldSkins[j].Pawn = P;
		OldSkins[j].bWasUnlit = P.bUnlit;
		P.bUnlit = True;

//		log("Added skin entry"@j);

		// If this is a Mocap Pawn, skin the head as well
		if (P2MoCapPawn(P) != None)
		{
//			log("This is a MoCap Pawn");
			MoCap = P2MoCapPawn(P);
			H = MoCap.MyHead;
			OldSkins[j].bHeadUnlit = H.bUnlit;
			H.bUnlit = True;
//			log("Adding"@H.Skins.Length@"head skins");
			OldSkins[j].HeadSkins.Insert(0, H.Skins.Length);
			for (i=0; i<H.Skins.Length; i++)
			{
//				log("Adding Head Skin"@H.Skins[i]);
				OldSkins[j].HeadSkins[i] = H.Skins[i];

				// if(P is a Zombie)
				//	H.Skins[i] = ColdTex;
				//else
					H.Skins[i] = HeatTex;
			}
		}
//		log("Adding"@P.Skins.Length@"pawn skins");
		OldSkins[j].PawnSkins.Insert(0, P.Skins.Length);
		for (i=0; i<P.Skins.Length; i++)
		{
//			log("Adding Pawn Skin"@P.Skins[i]);
			OldSkins[j].PawnSkins[i] = P.Skins[i];
			P.Skins[i] = HeatTex;
		}
		j++;
//		log("End processing of Pawn"@P);
	}

//	foreach DynamicActors(class'WeaponAttachment', W
}

///////////////////////////////////////////////////////////////////////////////
// Deactivate heat vision.
// Call when coming out of crossbow alt-fire.
///////////////////////////////////////////////////////////////////////////////
function DeactivateHeatVision()
{
	local Pawn P;
	local P2MoCapPawn MoCap;
	local BodyPart H;
	local int i, j;

	// fails if head vision is off already
	if (!bHeatVisionActive)
		return;

//	log("Deactivate Heat Vision.");

	bHeatVisionActive=False;

	// Don't need to use ForEach this time. We have a nice list of pawns that we altered.
	for (i=0; i<OldSkins.Length; i++)
	{
		P = OldSkins[i].Pawn;

		P.bUnlit = OldSkins[i].bWasUnlit;
//		log("Found entry"@i);
		if (P2MoCapPawn(P) != None)
		{
//			log("This is a MoCap Pawn");
			MoCap = P2MoCapPawn(P);
			H = MoCap.MyHead;
			H.bUnlit = OldSkins[i].bHeadUnlit;
//			log("Restoring"@OldSkins[i].HeadSkins.Length@"head skins");
			for (j=0; j<OldSkins[i].HeadSkins.Length; j++)
			{
//				log("Restoring head skin"@OldSkins[i].HeadSkins[j]);
				H.Skins[j] = OldSkins[i].HeadSkins[j];
			}
		}
//		log("Restoring"@OldSkins[i].PawnSkins.Length@"pawn skins");
		for (j=0; j<OldSkins[i].PawnSkins.Length; j++)
		{
//			log("Restoring pawn skin"@OldSkins[i].PawnSkins[j]);
			P.Skins[j] = OldSkins[i].PawnSkins[j];
		}
	}

	// Empty out the array and clean up
	OldSkins.Remove(0, OldSkins.Length);
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
function bool CheatsAllowed()
{
	if (EAGameInfo(Level.Game) != None
		&& EAGameInfo(Level.Game).SissyEnabled())
		return true;
	else
		return false;
}

///////////////////////////////////////////////////////////////////////////////
// Disable Sissy, we use a new command. Make fun of the player.
///////////////////////////////////////////////////////////////////////////////
exec function Sissy()
{
	// Say he's a sissy
	SayTime = MyPawn.Say(EADudeDialog(MyPawn.myDialog).lDude_PlayerTriedSissy) + 0.5;
	SetTimer(SayTime, false);
	bStillTalking=true;
}

///////////////////////////////////////////////////////////////////////////////
// Toggle the cheats for single player game
///////////////////////////////////////////////////////////////////////////////
exec function ImACheatingCheaterWhoCheats()
{
	if (CheatsAllowed())
	{
		EAGameInfo(Level.Game).DisableSissy();
		ClientMessage(CheatsOffText);
	}
	else
	{
		EAGameInfo(Level.Game).EnableSissy();

		// Say he's a sissy
		SayTime = MyPawn.Say(MyPawn.myDialog.lDude_PlayerSissy) + 0.5;
		SetTimer(SayTime, false);
		bStillTalking=true;
		ClientMessage(CheatsOnText);
	}
}

///////////////////////////////////////////////////////////////////////////////
// Stub forcesissy since it's an exec
///////////////////////////////////////////////////////////////////////////////
exec function ForceSissy()
{
	Sissy();
}

///////////////////////////////////////////////////////////////////////////////
// Turn cheats on for single player game. Not a toggle
///////////////////////////////////////////////////////////////////////////////
exec function ForceCheatingCheater()
{
	if(!CheatsAllowed())
	{
		EAGameInfo(Level.Game).EnableSissy();

		// Say he's a sissy
		SayTime = MyPawn.Say(MyPawn.myDialog.lDude_PlayerSissy) + 0.5;
		SetTimer(SayTime, false);
		bStillTalking=true;
		ClientMessage(CheatsOnText);
	}
}

///////////////////////////////////////////////////////////////////////////////
// Make friends with this pawn
///////////////////////////////////////////////////////////////////////////////
function MakeFriends(P2Pawn P)
{
	P.bPlayerIsEnemy=False;
	P.bPlayerIsFriend=True;
}

///////////////////////////////////////////////////////////////////////////////
// Make enemies with this pawn
///////////////////////////////////////////////////////////////////////////////
function MakeEnemies(P2Pawn P)
{
	P.bPlayerIsEnemy=True;
	P.bPlayerIsFriend=False;
}

///////////////////////////////////////////////////////////////////////////////
// Set "normal" relations with this pawn
///////////////////////////////////////////////////////////////////////////////
function NormalRelations(P2Pawn P)
{
	P.bPlayerIsEnemy=P.bPlayerHater;
	P.bPlayerIsFriend=P.Default.bPlayerIsFriend;
}

///////////////////////////////////////////////////////////////////////////////
// After we change into these clothes, update our level haters
///////////////////////////////////////////////////////////////////////////////
function UpdateLevelHaters(class<Inventory> ClothesType)
{
	local P2Pawn P;
	local int i;
	local bool bDoneSomethingWithThisPawn;

	// only works with Random Wardrobe
	if (class<WardrobeInv>(ClothesType) == None)
		return;

	// go through each pawn on the level and update it
	ForEach DynamicActors(class'P2Pawn', P)
	{
		// bool to tell whether we've done anything with this pawn
		bDoneSomethingWithThisPawn = False;

		for (i=0; i<DudeRelations.Length; i++)
		{
			// The dude's wearing these clothes, check for haters/friends
			if (ClothesType == DudeRelations[i].ClothesClass)
			{
				if (P.Class == DudeRelations[i].PawnClass) // make friends
				{
					MakeFriends(P);
					bDoneSomethingWithThisPawn = True;
				}
				else if (P.Class == DudeRelations[i].HaterClass) // make enemies
				{
					MakeEnemies(P);
					bDoneSomethingWithThisPawn = True;
				}
			}
		}

		// if we went through all the clothing sets and couldn't find anything to do with this pawn, use normal relations
		if (!bDoneSomethingWithThisPawn)
			NormalRelations(P);
	}
}

///////////////////////////////////////////////////////////////////////////////
// Called after travel to a new level
///////////////////////////////////////////////////////////////////////////////
event TravelPostAccept()
{
	Super.TravelPostAccept();
	UpdateLevelHaters(CurrentClothes);
}

///////////////////////////////////////////////////////////////////////////////
// See if we picked up a weapon, and if we're in a disguise, change the hands
// on the weapon to match our diguise
///////////////////////////////////////////////////////////////////////////////
function NotifyAddInventory(inventory NewItem)
{
	local class<ClothesInv> UseNewClass;
	local class<WardrobeInv> UseWardClass;
	local P2Weapon p2weap;

	Super.NotifyAddInventory(NewItem);

	// Check for weapons and change hand textures to match our disguise
	p2weap = P2Weapon(NewItem);
	if(p2weap != None)
	{
		if(CurrentClothes != DefaultClothes)
		{
			UseWardClass = class<WardrobeInv>(CurrentClothes);
			if (UseWardClass != None)
				SetSleeveMesh(P2Weap, UseWardClass.Default.bShortSleeves);

			UseNewClass = class<ClothesInv>(CurrentClothes);
			p2weap.ChangeHandTexture(UseNewClass.default.HandsTexture, DefaultHandsTexture,
				UseNewClass.default.FootTexture);
		}
	}
}

///////////////////////////////////////////////////////////////////////////////
// Set the proper mesh for our weapon... short sleeves or long?
///////////////////////////////////////////////////////////////////////////////
function SetSleeveMesh(P2Weapon inv, bool bShortSleeves)
{
	local class<P2WeaponPickup> PickupClass;
	local class<Inventory> RegularType, ShortSleeveType;
	local Mesh RegularMesh, ShortSleeveMesh;

	//log("SetSleeveMesh :: Inv ="@Inv);

	PickupClass = class<P2WeaponPickup>(Inv.PickupClass);
	if (PickupClass == None)
	{
		//log("SetSleeveMesh :: No PickupClass for" @ Inv $ ", breaking out");
		return;
	}
	//log("SetSleeveMesh :: PickupClass ="@PickupClass);

	RegularType = PickupClass.Default.InventoryType;
	//log("SetSleeveMesh :: RegularType ="@RegularType);

	RegularMesh = RegularType.Default.Mesh;
	//log("SetSleeveMesh :: RegularMesh ="@RegularMesh);

	ShortSleeveType = PickupClass.Default.ShortSleeveType;
	//log("SetSleeveMesh :: ShortSleeveType ="@ShortSleeveType);

	if (ShortSleeveType != None)
		ShortSleeveMesh = ShortSleeveType.Default.Mesh;
	else
	{
		//log("SetSleeveMesh :: No ShortSleeveType, breaking out");
		return;
	}

	//log("SetSleeveMesh :: ShortSleeveMesh ="@ShortSleeveMesh);

	if (bShortSleeves)
	{
		if (Inv.Mesh != ShortSleeveMesh)
		{
			//log("SetSleeveMesh :: Using ShortSleeveMesh ="@ShortSleeveMesh);
			Inv.LinkMesh(ShortSleeveMesh);
		}
		//else
			//log("SetSleeveMesh :: No need to change mesh");
	}
	else
	{
		if (Inv.Mesh != RegularMesh)
		{
			//log("SetSleeveMesh :: Using RegularMesh ="@RegularMesh);
			Inv.LinkMesh(RegularMesh);
		}
		//else
			//log("SetSleeveMesh :: No need to change mesh");
	}
	//log("SetSleeveMesh :: Mesh is now" @ Inv.Mesh);
}

///////////////////////////////////////////////////////////////////////////////
// Go through and fix all our weapons to use the new mesh (after changing
// clothes)
///////////////////////////////////////////////////////////////////////////////
function ChangeAllWeaponMeshes(bool bShortSleeves)
{
	local Inventory inv;

	//Mypawnfix
	inv = Pawn.Inventory;

	// Do all the weapons in your inventory
	while(inv != None)
	{
		if(P2Weapon(inv) != None)
		{
			SetSleeveMesh(P2Weapon(Inv), bShortSleeves);
		}
		inv = inv.Inventory;
	}
}

///////////////////////////////////////////////////////////////////////////////
// All these functions here just make sure we have the right mesh for our
// clothes.
///////////////////////////////////////////////////////////////////////////////
exec function SwitchToBestWeapon()
{
	local P2Weapon NewWeap;

	Super.SwitchToBestWeapon();

	NewWeap = P2Weapon(Pawn.PendingWeapon);
	if (NewWeap != None && class<WardrobeInv>(CurrentClothes) != None)
		SetSleeveMesh(NewWeap, class<WardrobeInv>(CurrentClothes).Default.bShortSleeves);
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
exec function PrevWeapon()
{
	local P2Weapon NewWeap;

	Super.PrevWeapon();

	NewWeap = P2Weapon(Pawn.PendingWeapon);
	if (NewWeap != None && class<WardrobeInv>(CurrentClothes) != None)
		SetSleeveMesh(NewWeap, class<WardrobeInv>(CurrentClothes).Default.bShortSleeves);
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
exec function NextWeapon()
{
	local P2Weapon NewWeap;

	Super.NextWeapon();

	NewWeap = P2Weapon(Pawn.PendingWeapon);
	if (NewWeap != None && class<WardrobeInv>(CurrentClothes) != None)
		SetSleeveMesh(NewWeap, class<WardrobeInv>(CurrentClothes).Default.bShortSleeves);
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
exec function SwitchWeapon(byte F)
{
	local P2Weapon NewWeap;

	Super.SwitchWeapon(F);

	NewWeap = P2Weapon(Pawn.PendingWeapon);
	if (NewWeap != None && class<WardrobeInv>(CurrentClothes) != None)
		SetSleeveMesh(NewWeap, class<WardrobeInv>(CurrentClothes).Default.bShortSleeves);
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
function bool SwitchAfterOutOfAmmo()
{
	local P2Weapon NewWeap;
	local bool b;

	b = Super.SwitchAfterOutOfAmmo();

	NewWeap = P2Weapon(Pawn.PendingWeapon);
	if (NewWeap != None && class<WardrobeInv>(CurrentClothes) != None)
		SetSleeveMesh(NewWeap, class<WardrobeInv>(CurrentClothes).Default.bShortSleeves);

	return b;
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
function bool SwitchToThisWeapon(int GroupNum, int OffsetNum, optional bool bForceReady)
{
	local P2Weapon NewWeap;
	local bool b;

	b = Super.SwitchToThisWeapon(GroupNum, OffsetNum, bForceReady);

	NewWeap = P2Weapon(Pawn.PendingWeapon);
	if (NewWeap != None && class<WardrobeInv>(CurrentClothes) != None)
		SetSleeveMesh(NewWeap, class<WardrobeInv>(CurrentClothes).Default.bShortSleeves);

	return b;
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
exec function SwitchToLastWeaponInGroup(int GroupNum)
{
	local P2Weapon NewWeap;

	Super.SwitchToLastWeaponInGroup(GroupNum);

	NewWeap = P2Weapon(Pawn.PendingWeapon);
	if (NewWeap != None && class<WardrobeInv>(CurrentClothes) != None)
		SetSleeveMesh(NewWeap, class<WardrobeInv>(CurrentClothes).Default.bShortSleeves);
}

///////////////////////////////////////////////////////////////////////////////
// Change the dudes clothes from what they are now, to the this new clothing
// type, and if specified, keep the old clothes in his inventory
///////////////////////////////////////////////////////////////////////////////
function ChangeToNewClothes()
{
	local class<WardrobeInv> UseNewClass;

	UseNewClass = class<WardrobeInv>(NewClothes);

	if (UseNewClass != None)
		ChangeAllWeaponMeshes(UseNewClass.Default.bShortSleeves);
	else
		ChangeAllWeaponMeshes(False);

	Super.ChangeToNewClothes();
}

///////////////////////////////////////////////////////////////////////////////
// Just finished putting my clothes on, say something funny
///////////////////////////////////////////////////////////////////////////////
function FinishedPuttingOnClothes()
{
	UpdateLevelHaters(CurrentClothes);

	if(CurrentClothes == class'CopClothesInv')
		NowIsCop();
	else if(CurrentClothes == class'DudeClothesInv')
		NowIsDude();
	else if(CurrentClothes == class'GimpClothesInv')
		NowIsGimp();
	else if (CurrentClothes == class'wBandClothes')
		NowIsBandmember();
	else if (CurrentClothes == class'wBookClothes')
		NowIsBookProtestor();
	else if (CurrentClothes == class'wBumClothes')
		NowIsBum();
	else if (CurrentClothes == class'wButcherClothes')
		NowIsButcher();
	else if (CurrentClothes == class'wFanaticClothes')
		NowIsFanatic();
	else if (CurrentClothes == class'wGangsterClothes')
		NowIsGangster();
	else if (CurrentClothes == class'wHippyClothes')
		NowIsHippy();
	else if (CurrentClothes == class'wHomoClothes')
		NowIsHomo();
	else if (CurrentClothes == class'wMallworkerClothes')
		NowIsMallworker();
	else if (CurrentClothes == class'wMcDClothes')
		NowIsFoodworker();
	else if (CurrentClothes == class'wMeatClothes')
		NowIsVegan();
	else if (CurrentClothes == class'wPriestClothes')
		NowIsPriest();
	else if (CurrentClothes == class'wRedneckClothes')
		NowIsRedneck();
	else if (CurrentClothes == class'wRWSClothes')
		NowIsRWS();
	else if (CurrentClothes == class'wScientistClothes')
		NowIsScientist();
	else if (CurrentClothes == class'wZealotClothes')
		NowIsZealot();
}

///////////////////////////////////////////////////////////////////////////////
// I just finished dressing back as Dude, so say something
///////////////////////////////////////////////////////////////////////////////
function NowIsDude()
{
	SayTime = MyPawn.Say(MyPawn.myDialog.lDude_NowIsDude) + 0.5;

	SetTimer(SayTime, false);
	bStillTalking=true;
}

///////////////////////////////////////////////////////////////////////////////
// I just finished dressing back as Gimp, so say something
///////////////////////////////////////////////////////////////////////////////
function NowIsGimp()
{
	SayTime = GetSoundDuration(GimpSound) + 0.5;
	MyPawn.PlaySound(GimpSound, SLOT_Talk);

	SetTimer(SayTime, false);
	bStillTalking=true;
}

///////////////////////////////////////////////////////////////////////////////
// I just finished dressing back as Book Protestor, so say something
///////////////////////////////////////////////////////////////////////////////
function NowIsBookProtestor()
{
	SayTime = GetSoundDuration(BookSound) + 0.5;
	MyPawn.PlaySound(BookSound, SLOT_Talk);

	SetTimer(SayTime, false);
	bStillTalking=true;
}

///////////////////////////////////////////////////////////////////////////////
// I just finished dressing back as Bandmember, so say something
///////////////////////////////////////////////////////////////////////////////
function NowIsBandmember()
{
	SayTime = GetSoundDuration(BandSound) + 0.5;
	MyPawn.PlaySound(BandSound, SLOT_Talk);

	SetTimer(SayTime, false);
	bStillTalking=true;
}

///////////////////////////////////////////////////////////////////////////////
// I just finished dressing back as Scientist, so say something
///////////////////////////////////////////////////////////////////////////////
function NowIsScientist()
{
	SayTime = GetSoundDuration(ScientistSound) + 0.5;
	MyPawn.PlaySound(ScientistSound, SLOT_Talk);

	SetTimer(SayTime, false);
	bStillTalking=true;
}

///////////////////////////////////////////////////////////////////////////////
// I just finished dressing back as Butcher, so say something
///////////////////////////////////////////////////////////////////////////////
function NowIsButcher()
{
	SayTime = GetSoundDuration(ButcherSound) + 0.5;
	MyPawn.PlaySound(ButcherSound, SLOT_Talk);

	SetTimer(SayTime, false);
	bStillTalking=true;
}

///////////////////////////////////////////////////////////////////////////////
// I just finished dressing back as Fanatic, so say something
///////////////////////////////////////////////////////////////////////////////
function NowIsFanatic()
{
	SayTime = GetSoundDuration(FanaticSound) + 0.5;
	MyPawn.PlaySound(FanaticSound, SLOT_Talk);

	SetTimer(SayTime, false);
	bStillTalking=true;
}

///////////////////////////////////////////////////////////////////////////////
// I just finished dressing back as Gangster, so say something
///////////////////////////////////////////////////////////////////////////////
function NowIsGangster()
{
	SayTime = GetSoundDuration(GangSound) + 0.5;
	MyPawn.PlaySound(GangSound, SLOT_Talk);

	SetTimer(SayTime, false);
	bStillTalking=true;
}

///////////////////////////////////////////////////////////////////////////////
// I just finished dressing back as Hippy, so say something
///////////////////////////////////////////////////////////////////////////////
function NowIsHippy()
{
	SayTime = GetSoundDuration(HippySound) + 0.5;
	MyPawn.PlaySound(HippySound, SLOT_Talk);

	SetTimer(SayTime, false);
	bStillTalking=true;
}

///////////////////////////////////////////////////////////////////////////////
// I just finished dressing back as Homo, so say something
///////////////////////////////////////////////////////////////////////////////
function NowIsHomo()
{
	SayTime = GetSoundDuration(HomoSound) + 0.5;
	MyPawn.PlaySound(HomoSound, SLOT_Talk);

	SetTimer(SayTime, false);
	bStillTalking=true;
}

///////////////////////////////////////////////////////////////////////////////
// I just finished dressing back as Mallworker, so say something
///////////////////////////////////////////////////////////////////////////////
function NowIsFoodworker()
{
	SayTime = GetSoundDuration(MallSound) + 0.5;
	MyPawn.PlaySound(MallSound, SLOT_Talk);

	SetTimer(SayTime, false);
	bStillTalking=true;
}

///////////////////////////////////////////////////////////////////////////////
// I just finished dressing back as Mallworker, so say something
///////////////////////////////////////////////////////////////////////////////
function NowIsMallworker()
{
	SayTime = GetSoundDuration(MallSound) + 0.5;
	MyPawn.PlaySound(MallSound, SLOT_Talk);

	SetTimer(SayTime, false);
	bStillTalking=true;
}

///////////////////////////////////////////////////////////////////////////////
// I just finished dressing back as Vegan, so say something
///////////////////////////////////////////////////////////////////////////////
function NowIsVegan()
{
	SayTime = GetSoundDuration(VeganSound) + 0.5;
	MyPawn.PlaySound(VeganSound, SLOT_Talk);

	SetTimer(SayTime, false);
	bStillTalking=true;
}

///////////////////////////////////////////////////////////////////////////////
// I just finished dressing back as Priest, so say something
///////////////////////////////////////////////////////////////////////////////
function NowIsPriest()
{
	SayTime = GetSoundDuration(PriestSound) + 0.5;
	MyPawn.PlaySound(PriestSound, SLOT_Talk);

	SetTimer(SayTime, false);
	bStillTalking=true;
}

///////////////////////////////////////////////////////////////////////////////
// I just finished dressing back as RWS, so say something
///////////////////////////////////////////////////////////////////////////////
function NowIsRWS()
{
	SayTime = GetSoundDuration(RWSSound) + 0.5;
	MyPawn.PlaySound(RWSSound, SLOT_Talk);

	SetTimer(SayTime, false);
	bStillTalking=true;
}

///////////////////////////////////////////////////////////////////////////////
// I just finished dressing back as Bum, so say something
///////////////////////////////////////////////////////////////////////////////
function NowIsBum()
{
	SayTime = GetSoundDuration(BumSound) + 0.5;
	MyPawn.PlaySound(BumSound, SLOT_Talk);

	SetTimer(SayTime, false);
	bStillTalking=true;
}

///////////////////////////////////////////////////////////////////////////////
// I just finished dressing back as Redneck, so say something
///////////////////////////////////////////////////////////////////////////////
function NowIsRedneck()
{
	SayTime = GetSoundDuration(RedneckSound) + 0.5;
	MyPawn.PlaySound(RedneckSound, SLOT_Talk);

	SetTimer(SayTime, false);
	bStillTalking=true;
}

///////////////////////////////////////////////////////////////////////////////
// I just finished dressing back as Zealot, so say something
///////////////////////////////////////////////////////////////////////////////
function NowIsZealot()
{
	SayTime = GetSoundDuration(ZealotSound) + 0.5;
	MyPawn.PlaySound(ZealotSound, SLOT_Talk);

	SetTimer(SayTime, false);
	bStillTalking=true;
}

///////////////////////////////////////////////////////////////////////////////
// The player unzips his pants, and prepares his urethra for peeing.
// The Fire button makes him actually pee.
///////////////////////////////////////////////////////////////////////////////
exec function UseZipper( optional float F )
{
	if(P2Pawn(Pawn) == None
		//mypawnfix
		|| P2Pawn(Pawn).bPlayerStarting)
		return;

	// Don't allow this to unpause the game
	if ( Level.Pauser == PlayerReplicationInfo )
		return;

	//  if you're also pressing fire, then don't allow this
	if(Pawn.PressingFire())
	{
		return;
	}
	
	// Must have a urethra to go pee
	if(P2Pawn(Pawn).MyUrethra == None)
	{
		log("I'm a p2pawn and I have no urethra "$self);
		return;
	}

	//log(self$" my weapon "$MyPawn.Weapon$" urethra "$MyPawn.MyUrethra);
	// Unzip pants and prepare to pee
	if(Pawn.Weapon != P2Pawn(Pawn).MyUrethra)
	{
		if(Pawn.Weapon != None)
		{
			LastWeaponGroupPee = Pawn.Weapon.InventoryGroup;
			LastWeaponOffsetPee= Pawn.Weapon.GroupOffset;
		}
		else
		{
			LastWeaponGroupPee = 1;
			LastWeaponOffsetPee= 0;
		}

		SwitchToThisWeapon(class'UrethraWeapon'.default.InventoryGroup,
						class'UrethraWeapon'.default.GroupOffset);
	}
	// Pants are already down, so zip them back up
	else if(!P2Pawn(Pawn).MyUrethra.IsInState('NormalFire'))
	{
		bPee=0;
		// Head back to the weapon we had up before we started peeing
		SwitchToThisWeapon(LastWeaponGroupPee, LastWeaponOffsetPee);
	}
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// You've shot a rocket, and now the camera is racing around with the rocket.
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
state PlayerWatchRocket
{
	///////////////////////////////////////////////////////////////////////////////
	// We control the rocket with the mouse now
	///////////////////////////////////////////////////////////////////////////////
	function ModifyRocketMotion(out float PlayerTurnX, out float PlayerTurnY)
	{
		local float RSpeed;

		PlayerTurnX = 0;
		PlayerTurnY = 0;
		
		if (Rotation != LastRocketRotation)
		{
			ViewTarget.SetRotation(Rotation);
			RSpeed = VSize(ViewTarget.Velocity);
			ViewTarget.Velocity = Vector(Rotation) * RSpeed;
			if (LauncherSeekingProjectile(ViewTarget) != None)
				ViewTarget.Acceleration = LauncherSeekingProjectile(ViewTarget).SeekingAccelerationMag*Normal(ViewTarget.Velocity);
			else if (LauncherSeekingProjectileTrad(ViewTarget) != None)
				ViewTarget.Acceleration = LauncherSeekingProjectileTrad(ViewTarget).SeekingAccelerationMag*Normal(ViewTarget.Velocity);
			else
				ViewTarget.Acceleration = LauncherProjectile(ViewTarget).ForwardAccelerationMag*Normal(ViewTarget.Velocity);
			LastRocketRotation = Rotation;
		}
	}

	///////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////
	function CalcFirstPersonView( out vector CameraLocation, out rotator CameraRotation )
	{
		local vector x1, y1, z1;

		GetAxes(ViewTarget.Rotation, x1, y1, z1);

		// First-person view.
		// Viewing the rocket
		if(Projectile(ViewTarget) != None)
		{
			CameraLocation = CameraLocation + ShakeOffset
						+ CAMERA_ROCKET_OFFSET_X*x1 + CAMERA_ROCKET_OFFSET_Z*z1;
			// Put some extra crazy shake it
			CameraLocation = CameraLocation + VRand();
			SetRotation(ViewTarget.Rotation);
			LastRocketRotation = Rotation;
		}
		else // viewing the carnage
		{
			CameraRotation = ViewTarget.Rotation;
			CameraLocation = CameraLocation + ShakeOffset
						+ CAMERA_VICTIM_OFFSET_X*x1 + CAMERA_VICTIM_OFFSET_Z*z1;
		}
	}
}
/*
//OK Why is this not being
state PlayerFlying
{
//ignores SeePlayer, HearNoise, Bump;

	function PlayerMove(float DeltaTime)
	{

		local vector X,Y,Z;

		GetAxes(Rotation,X,Y,Z);

//dopa		Pawn.Acceleration = aForward*X + aStrafe*Y;
//		log("topss");

                Pawn.Acceleration = aForward*X + aStrafe*Y + aLookUp*Z;
		if ( VSize(Pawn.Acceleration) < 1.0 )
			Pawn.Acceleration = vect(0,0,0);
		if ( bCheatFlying && (Pawn.Acceleration == vect(0,0,0)) )
			Pawn.Velocity = vect(0,0,0);
		// Update rotation.
		UpdateRotation(DeltaTime, 2);

		if ( Role < ROLE_Authority ) // then save this move and replicate it
			ReplicateMove(DeltaTime, Pawn.Acceleration, DCLICK_None, rot(0,0,0));
		else
			ProcessMove(DeltaTime, Pawn.Acceleration, DCLICK_None, rot(0,0,0));
	}

	function BeginState()
	{
		Pawn.SetPhysics(PHYS_Flying);
	}
}
*/

// Silent-Scope - New code added for Thermal Googles.

simulated function WarmupRadar()
{
	if(P2Hud(MyHud) != None)
	{
		// Snap it up there
//		RadarBackY=P2Hud(MyHud).GetRadarYOffset();
		Pawn.PlaySound(RadarBuzzSound, SLOT_Misc, 1.0,,TransientSoundRadius);
	}
	RadarState = ERadarWarmUp;
}


simulated function BringupRadar()
{
	if(P2Hud(MyHud) != None)
	{
		Pawn.PlaySound(RadarClickSound, SLOT_Misc, 1.0,,TransientSoundRadius);
//		RadarBackY=P2Hud(MyHud).GetStartRadarY();
	}
	RadarState = ERadarBringUp;
}

simulated function DropdownRadar()
{
	Pawn.PlaySound(RadarClickSoundoff, SLOT_Misc, 1.0,,TransientSoundRadius);
//	RadarState = ERadarDropDown;
}

///////////////////////////////////////////////////////////////////////////////
// Kamek: StartCountdown
// Starts a timed countdown.
///////////////////////////////////////////////////////////////////////////////
function StartCountdown(float CountdownTime, name ExpireEvent)
{
//	log(self@"countdown initiated: length"@CountdownTime@"event"@ExpireEvent);
	if (!bCountDown)
	{
//		log(self@"setting values");
		CountdownStart = Level.TimeSeconds;
		CountdownLength = CountdownTime;
		CountdownEvent = ExpireEvent;
		bCountDown = True;
	}
}

///////////////////////////////////////////////////////////////////////////////
// Kamek: StopCountdown
// Stops a timed countdown.
///////////////////////////////////////////////////////////////////////////////
function StopCountdown(optional bool bTriggerEvent)
{
	if (bCountDown)
	{
		CountdownStart = 0;
		CountdownLength = 0;
		if (bTriggerEvent)
			TriggerEvent(CountdownEvent, Self, Pawn);
		CountdownEvent = '';
		bCountDown = False;
	}
}

///////////////////////////////////////////////////////////////////////////////
// Default properties
///////////////////////////////////////////////////////////////////////////////

defaultproperties
{
     RadarBuzzSound=Sound'ed_moresounds.Thermal.thermalgoggles'
     RadarClickSoundoff=Sound'ed_moresounds.Thermal.thermalgoggles_off'
     HeatVisionRadius=2000.000000
     HeatTex=TexEnvMap'EATex.heatvision.HeatEnvMap'
     ColdTex=Texture'EATex.heatvision.face4'
     OverlayTex=Shader'EATex.heatvision.StaticFuzz'
     CatnipEffect=MotionBlur'EAGame.MotionBlurCatnip'
     Crosshairs(0)=Texture'P2Misc.Reticle.Reticle_Crosshair_Redline'
     Crosshairs(1)=Texture'P2Misc.Reticle.Reticle_Crosshair_Circular'
     Crosshairs(2)=Texture'P2Misc.Reticle.Reticle_Crosshair_Cross'
     Crosshairs(3)=Texture'P2Misc.Reticle.Reticle_Crosshair_WhiteLine'
     Crosshairs(4)=Texture'P2Misc.Reticle.Reticle_CrosshairOpenLg'
     Crosshairs(5)=Texture'P2Misc.Reticle.Reticle_CrosshairOpen'
     Crosshairs(6)=Texture'P2Misc.Reticle.Reticle_Red4Dots'
     Crosshairs(7)=Texture'P2Misc.Reticle.Reticle_GreenDot'
     Crosshairs(8)=Texture'P2Misc.Reticle.Reticle_RedDot'
     GimpSound=Sound'DudeDialog.dude_imthedamngimp'
     BandSound=Sound'DudeDialog.dude_heyliberacecatch'
     BookSound=Sound'DudeDialog.dude_stupidbook'
     BumSound=Sound'DudeDialog.dude_gimmesomemoney'
     ButcherSound=Sound'DudeDialog.dude_buttsauce'
     FanaticSound=Sound'HabibDialog.habib_ailili'
     GangSound=Sound'DudeDialog.dude_holyshitimnot'
     Hippysound=Sound'DudeDialog.dude_nowtheflowers'
     HomoSound=Sound'DudeDialog.dude_yougogirl'
     MallSound=Sound'DudeDialog.dude_hotnumbertwo'
     FoodSound=Sound'DudeDialog.dude_hotnumbertwo'
     VeganSound=Sound'DudeDialog.dude_notright'
     PriestSound=Sound'DudeDialog.dude_sorryfolksbible'
     RedneckSound=Sound'DudeDialog.dude_grandfatherof'
     RWSSound=Sound'DudeDialog.dude_butijuststarted'
     ScientistSound=Sound'DudeDialog.dude_justcallmedr'
     ZealotSound=Sound'DudeDialog.dude_imsowrong'
     DudeRelations(0)=(PawnClass=Class'EAPawns.EABandMembers',ClothesClass=Class'EAInventory.wBandClothes')
     DudeRelations(1)=(PawnClass=Class'EAPawns.EABookProtestors',ClothesClass=Class'EAInventory.wBookClothes')
     DudeRelations(4)=(PawnClass=Class'EAPawns.EAFanatics',ClothesClass=Class'EAInventory.wFanaticClothes')
     DudeRelations(5)=(PawnClass=Class'EAPawns.EAKumquat',ClothesClass=Class'EAInventory.wFanaticClothes')
     DudeRelations(6)=(PawnClass=Class'EAPawns.EAShopkeeper_AMH',ClothesClass=Class'EAInventory.wFanaticClothes')
     DudeRelations(7)=(PawnClass=Class'EAPawns.EAGangMembers',ClothesClass=Class'EAInventory.wGangsterClothes')
     DudeRelations(8)=(PawnClass=Class'EAPawns.EAMallWorkers',ClothesClass=Class'EAInventory.wMallworkerClothes')
     DudeRelations(9)=(PawnClass=Class'EAPawns.EAMallWorkers',ClothesClass=Class'EAInventory.wMcDClothes')
     DudeRelations(10)=(PawnClass=Class'EAPawns.EAMeatProtestors')
     DudeRelations(11)=(PawnClass=Class'EAPawns.EAPriest_AMW',HaterClass=Class'EAPawns.EAFanatics',ClothesClass=Class'EAInventory.wPriestClothes')
     DudeRelations(12)=(PawnClass=Class'EAPawns.EARednecks',ClothesClass=Class'EAInventory.wRedneckClothes')
     DudeRelations(13)=(PawnClass=Class'EAPawns.EAScientist',ClothesClass=Class'EAInventory.wScientistClothes')
     DudeRelations(14)=(PawnClass=Class'EAPawns.EARWSProtestors',ClothesClass=Class'EAInventory.wRWSClothes')
     DudeRelations(15)=(HaterClass=Class'EAPawns.EAKumquat',ClothesClass=Class'EAInventory.wPriestClothes')
     DudeRelations(16)=(HaterClass=Class'EAPawns.EAShopkeeper_AMH',ClothesClass=Class'EAInventory.wPriestClothes')
     GarySkins(0)=Texture'ChameleonSkins.Special.Gary'
     GarySkins(1)=Texture'MPSkins.BlueTeam.MB__136__Mini_M_Jacket_Pants'
     GarySkins(2)=Texture'MPSkins.RedTeam.MB__137__Mini_M_Jacket_Pants'
     DefHands=Texture'MP_FPArms.LS_arms.LS_hands_dude'
     DeathHints1(0)="Hmmm... looks like you're dying pretty quickly."
     DeathHints1(1)="Instead of just standing around enjoying the pain,"
     DeathHints1(2)="try running and hiding from your aggressors."
     DeathHints1(3)="Wait and hide from them and then attack them"
     DeathHints1(4)="when they come running around the corner to find you."
     DeathHints2(0)="Make sure to conserve your health powerups for the worst"
     DeathHints2(1)="fire-fights. Try to keep and eye on your health for the"
     DeathHints2(2)="best time for a health boost. Gather up lots of powerups"
     DeathHints2(3)="before going into a big battle."
     DeathHints3(0)="Take a slower pace when you get into fire-fights."
     DeathHints3(1)="If you rush through an area of people who are trying to"
     DeathHints3(2)="kill you, get ready for a lot of damage."
     DeathHints3(3)="Try moving along slowly, letting only a few of their"
     DeathHints3(4)="buddies know about you at once. They'll be easier to handle."
     FireDeathHints1(0)="That sure looks hot. I bet it hurts too."
     FireDeathHints1(1)="Did you know there's a way to put yourself out"
     FireDeathHints1(2)="when you're on fire? Yup... there sure is."
     FireDeathHints1(3)="Try thinking with you're lower half next time."
     CheatClass=Class'EAGame.EACheatManager'
}
