class EAP2GameSingle extends GameSinglePlayer;

var() localized string GameNameShort;

///////////////////////////////////////////////////////////////////////////////
// This function is called before any other scripts (including PreBeginPlay().
///////////////////////////////////////////////////////////////////////////////
event InitGame(out string Options, out string Error)
{
	Super.InitGame(Options, Error);
	MySlotInfoMgr = spawn(class'EASlotInfoMgr');
}

///////////////////////////////////////////////////////////////////////////////
// Save game to the specified slot
///////////////////////////////////////////////////////////////////////////////
function SaveGame(int Slot, bool bShowMessage)
	{
	Log(self$" SaveGame(): Slot="$Slot$" bAutoSave="$bDoAutoSave$" bForcedAutoSave="$bForcedAutoSave$" the game state difficulty "$TheGameState.GameDifficulty);

	if (bShowMessage)
		{
		if (Slot == QUICKSAVE_SLOT)
			GetPlayer().ClientMessage(EasySaveMessageString);
		else if (Slot == AUTOSAVE_SLOT && bForcedAutoSave)
			GetPlayer().ClientMessage(ForcedSaveMessageString);
		else if (Slot == AUTOSAVE_SLOT)
			GetPlayer().ClientMessage(AutoSaveMessageString);
		else
			GetPlayer().ClientMessage(NormalSaveMessageString);
		}

	// This slot is now the most recently used slot.  We update this BEFORE the
	// actual save so that if this game is loaded, it will refer to itself as
	// the most recently used.  In other words, whenever a game is loaded, the
	// slot it loaded from becomes the most recently used slot.
	MostRecentGameSlot = Slot;

	// Do the actual save
	ConsoleCommand("SaveGame "$Slot);

	// Save info about this slot
	MySlotInfoMgr.SetInfo(
		Slot,
		GetCurrentDayBase().Description@"-"@Level.Title@"-"@GetDiffName(TheGameState.GameDifficulty)@"-"@GameNameShort,
		GetCurrentDay(),
		Level.Year$"-"$Val2Str(level.Month, 2)$"-"$Val2Str(Level.Day, 2)@":"@Val2Str(Level.Hour, 2)$":"$Val2Str(Level.Minute, 2),
		int(ConsoleCommand("GETGMTIME")));
	}

defaultproperties
{
     GameNameShort="P2"
     Begin Object Class=DayBase Name=DayBase69
         Description="Monday"
         UniqueName="DAY_A"
         ExcludeDays(0)="DEMO"
         MapTex="p2misc_full.map_day1"
         NewsTex="p2misc.newspaper_day_1"
         DudeNewsComment="DudeDialog.dude_news_monday"
         LoadTex="p2misc_full.loading1"
         DudeStartComment="DudeDialog.dude_map_exit1"
         Errands(0)=ErrandBase'GameTypes.ErrandBase0'
         Errands(1)=ErrandBase'GameTypes.ErrandBase1'
         Errands(2)=ErrandBase'GameTypes.ErrandBase39'
         PlayerInvList(0)=(InvClassName="Inventory.MoneyInv",NeededAmount=20)
         PlayerInvList(1)=(InvClassName="Inventory.MapInv")
         PlayerInvList(2)=(InvClassName="Inventory.HandsWeapon")
         PlayerInvList(3)=(InvClassName="Inventory.CopClothesInv",bEnhancedOnly=True)
         TakeFromPlayerList(0)=(InvClassName="Inventory.NewspaperInv")
         TakeFromPlayerList(1)=(InvClassName="Inventory.GimpClothesInv")
         TakeFromPlayerList(2)=(InvClassName="Inventory.DudeClothesInv")
         TakeFromPlayerList(3)=(InvClassName="Inventory.MilkInv")
         TakeFromPlayerList(4)=(InvClassName="Inventory.PaycheckInv")
         Name="DayBase69"
     End Object
     Days(0)=DayBase'EAGame.DayBase69'
     Begin Object Class=DayBase Name=DayBase70
         Description="Tuesday"
         UniqueName="DAY_B"
         ExcludeDays(0)="DEMO"
         MapTex="p2misc_full.map_day2"
         NewsTex="p2misc_full.newspaper_day_2"
         DudeNewsComment="DudeDialog.dude_news_tuesday"
         LoadTex="p2misc_full.loading2"
         DudeStartComment="DudeDialog.dude_map_exit2"
         Errands(0)=ErrandBase'GameTypes.ErrandBase3'
         Errands(1)=ErrandBase'GameTypes.ErrandBase4'
         Errands(2)=ErrandBase'GameTypes.ErrandBase5'
         Errands(3)=ErrandBase'GameTypes.ErrandBase6'
         PlayerInvList(0)=(InvClassName="Inventory.LibraryBookInv")
         PlayerInvList(1)=(InvClassName="Inventory.ClipboardWeapon")
         PlayerInvList(2)=(InvClassName="Inventory.GimpClothesInv",bEnhancedOnly=True)
         TakeFromPlayerList(0)=(InvClassName="Inventory.NewspaperInv")
         TakeFromPlayerList(1)=(InvClassName="Inventory.GimpClothesInv")
         TakeFromPlayerList(2)=(InvClassName="Inventory.DudeClothesInv")
         TakeFromPlayerList(3)=(InvClassName="Inventory.ClipboardWeapon")
         TakeFromPlayerList(4)=(InvClassName="Inventory.LibraryBookInv")
         Name="DayBase70"
     End Object
     Days(1)=DayBase'EAGame.DayBase70'
     Begin Object Class=DayBase Name=DayBase71
         Description="Wednesday"
         UniqueName="DAY_C"
         ExcludeDays(0)="DEMO"
         MapTex="p2misc_full.map_day3"
         NewsTex="p2misc_full.newspaper_day_3"
         DudeNewsComment="DudeDialog.dude_news_wednesday"
         LoadTex="p2misc_full.loading3"
         DudeStartComment="DudeDialog.dude_map_exit3"
         Errands(0)=ErrandBase'GameTypes.ErrandBase7'
         Errands(1)=ErrandBase'GameTypes.ErrandBase8'
         Errands(2)=ErrandBase'GameTypes.ErrandBase9'
         Errands(3)=ErrandBase'GameTypes.ErrandBase10'
         PlayerInvList(0)=(InvClassName="Inventory.CatnipInv",NeededAmount=5,bEnhancedOnly=True)
         TakeFromPlayerList(0)=(InvClassName="Inventory.NewspaperInv")
         TakeFromPlayerList(1)=(InvClassName="Inventory.GimpClothesInv")
         TakeFromPlayerList(2)=(InvClassName="Inventory.DudeClothesInv")
         TakeFromPlayerList(3)=(InvClassName="Inventory.TreeInv")
         Name="DayBase71"
     End Object
     Days(2)=DayBase'EAGame.DayBase71'
     Begin Object Class=DayBase Name=DayBase72
         Description="Thursday"
         UniqueName="DAY_D"
         ExcludeDays(0)="DEMO"
         MapTex="p2misc_full.map_day4"
         NewsTex="p2misc_full.newspaper_day_4"
         DudeNewsComment="DudeDialog.dude_news_thursday"
         LoadTex="p2misc_full.loading4"
         DudeStartComment="DudeDialog.dude_map_exit4"
         Errands(0)=ErrandBase'GameTypes.ErrandBase11'
         Errands(1)=ErrandBase'GameTypes.ErrandBase12'
         Errands(2)=ErrandBase'GameTypes.ErrandBase13'
         Errands(3)=ErrandBase'GameTypes.ErrandBase14'
         PlayerInvList(0)=(InvClassName="Inventory.CitationInv")
         PlayerInvList(1)=(InvClassName="Inventory.RocketCamInv",bEnhancedOnly=True)
         TakeFromPlayerList(0)=(InvClassName="Inventory.NewspaperInv")
         TakeFromPlayerList(1)=(InvClassName="Inventory.GimpClothesInv")
         TakeFromPlayerList(2)=(InvClassName="Inventory.DudeClothesInv")
         TakeFromPlayerList(3)=(InvClassName="Inventory.KrotchyInv")
         TakeFromPlayerList(4)=(InvClassName="Inventory.SteakInv")
         TakeFromPlayerList(5)=(InvClassName="Inventory.CitationInv")
         Name="DayBase72"
     End Object
     Days(3)=DayBase'EAGame.DayBase72'
     Begin Object Class=DayBase Name=DayBase73
         Description="Friday"
         UniqueName="DAY_E"
         ExcludeDays(0)="DEMO"
         MapTex="p2misc_full.map_day5"
         NewsTex="p2misc_full.newspaper_day_5"
         DudeNewsComment="DudeDialog.dude_news_friday"
         LoadTex="EA_Product.Full.Loading_Friday"
         DudeStartComment="DudeDialog.dude_map_exit5"
         Errands(0)=ErrandBase'GameTypes.ErrandBase15'
         Errands(1)=ErrandBase'GameTypes.ErrandBase16'
         Errands(2)=ErrandBase'GameTypes.ErrandBase17'
         Errands(3)=ErrandBase'GameTypes.ErrandBase18'
         PlayerInvList(0)=(InvClassName="Inventory.GiftInv")
         PlayerInvList(1)=(InvClassName="Inventory.GonorrheaAmmoInv")
         PlayerInvList(2)=(InvClassName="Inventory.CopClothesInv",bEnhancedOnly=True)
         TakeFromPlayerList(0)=(InvClassName="Inventory.NewspaperInv")
         TakeFromPlayerList(1)=(InvClassName="Inventory.GimpClothesInv")
         TakeFromPlayerList(2)=(InvClassName="Inventory.DudeClothesInv")
         TakeFromPlayerList(3)=(InvClassName="Inventory.ParcelInv")
         TakeFromPlayerList(4)=(InvClassName="Inventory.AlternatorInv")
         TakeFromPlayerList(5)=(InvClassName="Inventory.PillsInv")
         TakeFromPlayerList(6)=(InvClassName="Inventory.GonorrheaAmmoInv")
         Name="DayBase73"
     End Object
     Days(4)=DayBase'EAGame.DayBase73'
     MatSwaps(0)=(OrigMat=Texture'Josh-textures.signs.game_banner_2',NewMat=Texture'Josh-textures.signs.game_banner_5')
     MatSwaps(1)=(OrigMat=Texture'Timb.arcade.Game_FagHunter',NewMat=Texture'Timb.arcade.Game_BastardFish')
     GameName="Original Postal 2 game"
}
