///////////////////////////////////////////////////////////////////////////////
// EAStatsScreen
// Our stats screen for EA.
///////////////////////////////////////////////////////////////////////////////
class EAStatsScreen extends AWStatsScreen;

///////////////////////////////////////////////////////////////////////////////
// Vars, structs, consts, enums...
///////////////////////////////////////////////////////////////////////////////
var bool bGameOver;

// EA stuff
var localized string New_Metool;
var localized string New_Busta;
var localized string New_Fathom;
var localized string New_Matrix;
var localized string New_Final;
var localized string GameIncompleteRanking;
var localized string InnocentsKilled;
var localized string TimeElapsed;
var localized string HeadsBatted;
var localized string Cheating;
var localized string KatanasImpaled;

var bool bMetool;
var bool bBusta;
var bool bFathom;
var bool bMatrix;
var bool bFinal;

var bool bWeekendOnly;
var bool bWeekendGame;
var bool bFOver;

///////////////////////////////////////////////////////////////////////////////
// Consts
///////////////////////////////////////////////////////////////////////////////
const	INC_Y			=	0.03;
const SPEED_RUN = 10800.00;

///////////////////////////////////////////////////////////////////////////////
// FOR REFERENCE ONLY! DO NOT UNCOMMENT!
///////////////////////////////////////////////////////////////////////////////
/*
// AW stuff
var localized string ZombiesKilledOverall;
var localized string BryanSurvived;
var localized string LostSledgeInCow;
var localized string KillElephantsScythe;
var localized string FanaticsKilled;
var localized string ArmyKilled;
var localized string LimbsHacked;			// Any limb (no heads) cut off
var localized string HeadsLopped;			// Any heads cut off (not exploded)
var localized string ZombiesResurrected;
var localized string NewC1;
var localized string NewC2;
var localized string NewC3;
var localized string NewItemNow;

// P2 stuff
var localized string PeopleKilled;
var localized string CopsKilled;
var localized string PeopleRoasted;
var localized string ElephantsKilled;
var localized string DogsKilled;
var localized string CatsKilled;
var localized string PistolHeadShot;
var localized string ShotgunHeadShot;
var localized string RifleHeadShot;
var localized string CatsUsed;
var localized string MoneySpent;
var localized string PeeTotal;
var localized string DoorsKicked;
var localized string TimesArrested;
var localized string DressedAsCop;
var localized string DogsTrained;
var localized string CopsLuredByDonuts;
var localized string Ranking;
*/

///////////////////////////////////////////////////////////////////////////////
// Call this to bring up the screen
///////////////////////////////////////////////////////////////////////////////
function Show(String URLin)
{
	local EAGameInfo useinfo;

	useinfo = EAGameInfo(GetGameSingle());

	Super.Show(URLin);

	if (URLin != "")
		bGameOver = True;
	else
		PlayerRank = GameIncompleteRanking;
		
	bWeekendOnly = useinfo.WeekendOnlyGame();
	bWeekendGame = useinfo.WeekendGame();
	bFOver = useinfo.FinallyOver();
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
function DoMPupdate()
{
	const FIN_BLAST	= 23;
	const STOP_VAL = 5;
	const TOUCH_BLANK = 18;

	local EAGameInfo useinfo;
	local EAGameState usegs;

	usegs = EAGameState(gamest);
	useinfo = EAGameInfo(GetGameSingle());

	if (usegs == None || useinfo == None)
		return;

	if (usegs.DogsKilled == 0)
	{
		useinfo.EnableMetool();
		bMetool = True;
	}

	if (usegs.CopsKilled == 0 && usegs.DressedAsCop == 0)
	{
		useinfo.EnableBusta();
		bBusta = True;
	}

	if (usegs.InnocentsKilled == 0)
	{
		useinfo.EnableFathom();
		bFathom = True;
	}
	
	// The following cheats are only unlockable in the full 7-day game
	if (!useinfo.WeekendOnlyGame() && UseInfo.WeekendGame())
	{
		if (usegs.TimeElapsed <= SPEED_RUN)
		{
			useinfo.EnableMatrix();
			bMatrix = True;
		}
	}

	// These are only unlockable in a weekend game
	if (UseInfo.WeekendGame())
	{
		if (usegs.BryanSurvived != 0)
		{
			useinfo.MultCast = STOP_VAL + Rand(STOP_VAL);
		}

		if (usegs.KillElephantsScythe != 0)
		{
			useinfo.FinStop = FIN_BLAST + Rand(FIN_BLAST);
		}
	}

	if (usegs.ZombiesResurrected != 0)
	{
		useinfo.Tunneling = TOUCH_BLANK + Rand(TOUCH_BLANK);
	}

	if (useinfo.AllEnabled())
	{
		bFinal = True;
	}

	bMP1 = useinfo.ChangedMP1();
	bMP2 = useinfo.ChangedMP2();
	bMP3 = useinfo.ChangedMP3();

//	log(Self$" show 1 "$bMp1$" 2 "$bMp2$" 3 "$bMp3$" newgs "$newgs);
}

///////////////////////////////////////////////////////////////////////////////
// Render the screen
// Returns with each increment of StatNum as it slowly reveals the numbers
///////////////////////////////////////////////////////////////////////////////
function RenderScreen(canvas Canvas)
{
	local int i;
	local float sxl, sxr, sy;
	local string ustr;
	local EAGameState newgs;
	local EAGameInfo useinfo;

	newgs = EAGameState(gamest);

	if(!bEnableRender
		|| newgs == None) return;

	if(bMPupdate && bGameOver)
		DoMPupdate();

	// Let super draw background texture
	Super(P2Screen).RenderScreen(Canvas);

	useinfo = EAGameInfo(GetGameSingle());

	// Show our stats
	sxl = LEFT_START_X;
	sxr = RIGHT_START_X;
	sy = START_Y;
	if(StatNum == 0) return;

	// Total number of people killed
	MyFont.DrawTextEx(Canvas, Canvas.ClipX, sxl * Canvas.ClipX, sy * Canvas.ClipY, PeopleKilled, 0, false, EJ_Left);
	MyFont.DrawTextEx(Canvas, Canvas.ClipX, sxr * Canvas.ClipX, sy * Canvas.ClipY, ustr$newgs.PeopleKilled, 0, false, EJ_Left);
	sy += INC_Y;
	if(StatNum == 1) return;

	// Total number of innocents killed
	if (newgs.InnocentsKilled == 0 && bFathom)
	{
		MyFont.TextColor = YellowC;
		MyFont.DrawTextEx(Canvas, Canvas.ClipX, sxl * Canvas.ClipX, sy * Canvas.ClipY, InnocentsKilled@New_Fathom, 0, false, EJ_Left);
		MyFont.DrawTextEx(Canvas, Canvas.ClipX, sxr * Canvas.ClipX, sy * Canvas.ClipY, ustr$newgs.InnocentsKilled, 0, false, EJ_Left);
		MyFont.TextColor = MyFont.Default.TextColor;
	}
	else
	{
		MyFont.DrawTextEx(Canvas, Canvas.ClipX, sxl * Canvas.ClipX, sy * Canvas.ClipY, InnocentsKilled, 0, false, EJ_Left);
		MyFont.DrawTextEx(Canvas, Canvas.ClipX, sxr * Canvas.ClipX, sy * Canvas.ClipY, ustr$newgs.InnocentsKilled, 0, false, EJ_Left);
	}
	sy += INC_Y;
	if(StatNum == 2) return;

	// Number of zombies killed
	MyFont.DrawTextEx(Canvas, Canvas.ClipX, sxl * Canvas.ClipX, sy * Canvas.ClipY, ZombiesKilledOverall, 0, false, EJ_Left);
	MyFont.DrawTextEx(Canvas, Canvas.ClipX, sxr * Canvas.ClipX, sy * Canvas.ClipY, ustr$newgs.ZombiesKilledOverall, 0, false, EJ_Left);
	sy += INC_Y;
	if(StatNum == 3) return;

	// Number of zombies revived
	if(newgs.ZombiesResurrected > 0
		&& bMP3)
	{
		MyFont.TextColor=YellowC;
		MyFont.DrawTextEx(Canvas, Canvas.ClipX, sxl * Canvas.ClipX, sy * Canvas.ClipY, ZombiesResurrected$NewC3, 0, false, EJ_Left);
		MyFont.DrawTextEx(Canvas, Canvas.ClipX, sxr * Canvas.ClipX, sy * Canvas.ClipY, ustr$newgs.ZombiesResurrected, 0, false, EJ_Left);
		MyFont.TextColor=MyFont.default.TextColor;
	}
	else
	{
		MyFont.DrawTextEx(Canvas, Canvas.ClipX, sxl * Canvas.ClipX, sy * Canvas.ClipY, ZombiesResurrected, 0, false, EJ_Left);
		MyFont.DrawTextEx(Canvas, Canvas.ClipX, sxr * Canvas.ClipX, sy * Canvas.ClipY, ustr$newgs.ZombiesResurrected, 0, false, EJ_Left);
	}
	sy += INC_Y;
	if(StatNum == 4) return;

	// Number of cops killed -- M-F only
	if (!bWeekendOnly)
	{
		if (newgs.CopsKilled == 0 && bBusta)
		{
			MyFont.TextColor=YellowC;
			MyFont.DrawTextEx(Canvas, Canvas.ClipX, sxl * Canvas.ClipX, sy * Canvas.ClipY, CopsKilled@New_Busta, 0, false, EJ_Left);
			MyFont.DrawTextEx(Canvas, Canvas.ClipX, sxr * Canvas.ClipX, sy * Canvas.ClipY, ustr$newgs.CopsKilled, 0, false, EJ_Left);
			MyFont.TextColor = MyFont.Default.TextColor;
		}
		else
		{
			MyFont.DrawTextEx(Canvas, Canvas.ClipX, sxl * Canvas.ClipX, sy * Canvas.ClipY, CopsKilled, 0, false, EJ_Left);
			MyFont.DrawTextEx(Canvas, Canvas.ClipX, sxr * Canvas.ClipX, sy * Canvas.ClipY, ustr$newgs.CopsKilled, 0, false, EJ_Left);
		}
		sy += INC_Y;
	}
	if(StatNum == 5) return;

	// People roasted
	MyFont.DrawTextEx(Canvas, Canvas.ClipX, sxl * Canvas.ClipX, sy * Canvas.ClipY, PeopleRoasted, 0, false, EJ_Left);
	MyFont.DrawTextEx(Canvas, Canvas.ClipX, sxr * Canvas.ClipX, sy * Canvas.ClipY, ustr$newgs.PeopleRoasted, 0, false, EJ_Left);
	sy += INC_Y;
	if(StatNum == 6) return;

	// Killed elephants with scythe? -- Weekend only
	if (bWeekendGame)
	{
//		log(self@"bWeekendGame = true, drawing elephant crap");
		if(newgs.KillElephantsScythe != 0)
		{
			if(bMP2)
			{
				MyFont.TextColor=YellowC;
				MyFont.DrawTextEx(Canvas, Canvas.ClipX, sxl * Canvas.ClipX, sy * Canvas.ClipY, KillElephantsScythe$NewC2, 0, false, EJ_Left);
				MyFont.DrawTextEx(Canvas, Canvas.ClipX, sxr * Canvas.ClipX, sy * Canvas.ClipY, ustr$YesStr, 0, false, EJ_Left);
				MyFont.TextColor=MyFont.default.TextColor;
			}
			else
			{
				MyFont.DrawTextEx(Canvas, Canvas.ClipX, sxl * Canvas.ClipX, sy * Canvas.ClipY, KillElephantsScythe, 0, false, EJ_Left);
				MyFont.DrawTextEx(Canvas, Canvas.ClipX, sxr * Canvas.ClipX, sy * Canvas.ClipY, ustr$YesStr, 0, false, EJ_Left);
			}
		}
		else
		{
			MyFont.DrawTextEx(Canvas, Canvas.ClipX, sxl * Canvas.ClipX, sy * Canvas.ClipY, KillElephantsScythe, 0, false, EJ_Left);
			MyFont.DrawTextEx(Canvas, Canvas.ClipX, sxr * Canvas.ClipX, sy * Canvas.ClipY, ustr$NoStr, 0, false, EJ_Left);
		}
		sy += INC_Y;
	}
	if(StatNum == 7) return;

	// Dogs killed
	if (newgs.DogsKilled == 0 && bMetool)
	{
		MyFont.TextColor = YellowC;
		MyFont.DrawTextEx(Canvas, Canvas.ClipX, sxl * Canvas.ClipX, sy * Canvas.ClipY, DogsKilled@New_Metool, 0, false, EJ_Left);
		MyFont.DrawTextEx(Canvas, Canvas.ClipX, sxr * Canvas.ClipX, sy * Canvas.ClipY, ustr$newgs.DogsKilled, 0, false, EJ_Left);
		MyFont.TextColor = MyFont.Default.TextColor;
	}
	else
	{
		MyFont.DrawTextEx(Canvas, Canvas.ClipX, sxl * Canvas.ClipX, sy * Canvas.ClipY, DogsKilled, 0, false, EJ_Left);
		MyFont.DrawTextEx(Canvas, Canvas.ClipX, sxr * Canvas.ClipX, sy * Canvas.ClipY, ustr$newgs.DogsKilled, 0, false, EJ_Left);
	}
	sy += INC_Y;
	if(StatNum == 8) return;

	// Cats killed
	MyFont.DrawTextEx(Canvas, Canvas.ClipX, sxl * Canvas.ClipX, sy * Canvas.ClipY, CatsKilled, 0, false, EJ_Left);
	MyFont.DrawTextEx(Canvas, Canvas.ClipX, sxr * Canvas.ClipX, sy * Canvas.ClipY, ustr$newgs.CatsKilled, 0, false, EJ_Left);
	sy += INC_Y;
	if(StatNum == 9) return;

	// Bryan survived? -- weekend only
	if (bWeekendGame)
	{
		if(newgs.BryanSurvived != 0)
		{
			if(bMP1)
			{
				MyFont.TextColor=YellowC;
				MyFont.DrawTextEx(Canvas, Canvas.ClipX, sxl * Canvas.ClipX, sy * Canvas.ClipY, BryanSurvived$NewC1, 0, false, EJ_Left);
				MyFont.DrawTextEx(Canvas, Canvas.ClipX, sxr * Canvas.ClipX, sy * Canvas.ClipY, ustr$YesStr, 0, false, EJ_Left);
				MyFont.TextColor=MyFont.default.TextColor;
			}
			else
			{
				MyFont.DrawTextEx(Canvas, Canvas.ClipX, sxl * Canvas.ClipX, sy * Canvas.ClipY, BryanSurvived, 0, false, EJ_Left);
				MyFont.DrawTextEx(Canvas, Canvas.ClipX, sxr * Canvas.ClipX, sy * Canvas.ClipY, ustr$YesStr, 0, false, EJ_Left);
			}
		}
		else
		{
			MyFont.DrawTextEx(Canvas, Canvas.ClipX, sxl * Canvas.ClipX, sy * Canvas.ClipY, BryanSurvived, 0, false, EJ_Left);
			MyFont.DrawTextEx(Canvas, Canvas.ClipX, sxr * Canvas.ClipX, sy * Canvas.ClipY, ustr$NoStr, 0, false, EJ_Left);
		}
		sy += INC_Y;
	}
	if(StatNum == 10) return;

	// Fanatics killed
	MyFont.DrawTextEx(Canvas, Canvas.ClipX, sxl * Canvas.ClipX, sy * Canvas.ClipY, FanaticsKilled, 0, false, EJ_Left);
	MyFont.DrawTextEx(Canvas, Canvas.ClipX, sxr * Canvas.ClipX, sy * Canvas.ClipY, ustr$newgs.FanaticsKilled, 0, false, EJ_Left);
	sy += INC_Y;
	if(StatNum == 11) return;

	// Army killed
	MyFont.DrawTextEx(Canvas, Canvas.ClipX, sxl * Canvas.ClipX, sy * Canvas.ClipY, ArmyKilled, 0, false, EJ_Left);
	MyFont.DrawTextEx(Canvas, Canvas.ClipX, sxr * Canvas.ClipX, sy * Canvas.ClipY, ustr$newgs.ArmyKilled, 0, false, EJ_Left);
	sy += INC_Y;
	if(StatNum == 12) return;

	// Shotgun headshots
	MyFont.DrawTextEx(Canvas, Canvas.ClipX, sxl * Canvas.ClipX, sy * Canvas.ClipY, ShotgunHeadShot, 0, false, EJ_Left);
	MyFont.DrawTextEx(Canvas, Canvas.ClipX, sxr * Canvas.ClipX, sy * Canvas.ClipY, ustr$newgs.ShotgunHeadShot, 0, false, EJ_Left);
	sy += INC_Y;
	if(StatNum == 13) return;

	// Rifle headshots
	MyFont.DrawTextEx(Canvas, Canvas.ClipX, sxl * Canvas.ClipX, sy * Canvas.ClipY, RifleHeadShot, 0, false, EJ_Left);
	MyFont.DrawTextEx(Canvas, Canvas.ClipX, sxr * Canvas.ClipX, sy * Canvas.ClipY, ustr$newgs.RifleHeadShot, 0, false, EJ_Left);
	sy += INC_Y;
	if(StatNum == 14) return;

	// Cats violated
	MyFont.DrawTextEx(Canvas, Canvas.ClipX, sxl * Canvas.ClipX, sy * Canvas.ClipY, CatsUsed, 0, false, EJ_Left);
	MyFont.DrawTextEx(Canvas, Canvas.ClipX, sxr * Canvas.ClipX, sy * Canvas.ClipY, ustr$newgs.CatsUsed, 0, false, EJ_Left);
	sy += INC_Y;
	if(StatNum == 15) return;

	// Sledges lost in cow
	MyFont.DrawTextEx(Canvas, Canvas.ClipX, sxl * Canvas.ClipX, sy * Canvas.ClipY, LostSledgeInCow, 0, false, EJ_Left);
	MyFont.DrawTextEx(Canvas, Canvas.ClipX, sxr * Canvas.ClipX, sy * Canvas.ClipY, ustr$newgs.LostSledgeInCow, 0, false, EJ_Left);
	sy += INC_Y;
	if(StatNum == 16) return;

	// Limbs hacked
	MyFont.DrawTextEx(Canvas, Canvas.ClipX, sxl * Canvas.ClipX, sy * Canvas.ClipY, LimbsHacked, 0, false, EJ_Left);
	MyFont.DrawTextEx(Canvas, Canvas.ClipX, sxr * Canvas.ClipX, sy * Canvas.ClipY, ustr$newgs.LimbsHacked, 0, false, EJ_Left);
	sy += INC_Y;
	if(StatNum == 17) return;

	// Heads lopped
	MyFont.DrawTextEx(Canvas, Canvas.ClipX, sxl * Canvas.ClipX, sy * Canvas.ClipY, HeadsLopped, 0, false, EJ_Left);
	MyFont.DrawTextEx(Canvas, Canvas.ClipX, sxr * Canvas.ClipX, sy * Canvas.ClipY, ustr$newgs.HeadsLopped, 0, false, EJ_Left);
	sy += INC_Y;
	if(StatNum == 18) return;

	// Heads batted off
	MyFont.DrawTextEx(Canvas, Canvas.ClipX, sxl * Canvas.ClipX, sy * Canvas.ClipY, HeadsBatted, 0, false, EJ_Left);
	MyFont.DrawTextEx(Canvas, Canvas.ClipX, sxr * Canvas.ClipX, sy * Canvas.ClipY, ustr$newgs.BaseballHeads, 0, false, EJ_Left);
	sy += INC_Y;
	if(StatNum == 19) return;

	// Katanas impaled
	MyFont.DrawTextEx(Canvas, Canvas.ClipX, sxl * Canvas.ClipX, sy * Canvas.ClipY, KatanasImpaled, 0, false, EJ_Left);
	MyFont.DrawTextEx(Canvas, Canvas.ClipX, sxr * Canvas.ClipX, sy * Canvas.ClipY, ustr$newgs.KatanaImpaleCount, 0, false, EJ_Left);
	sy += INC_Y;
	if(StatNum == 20) return;

	// Money spent
	MyFont.DrawTextEx(Canvas, Canvas.ClipX, sxl * Canvas.ClipX, sy * Canvas.ClipY, MoneySpent, 0, false, EJ_Left);
	MyFont.DrawTextEx(Canvas, Canvas.ClipX, sxr * Canvas.ClipX, sy * Canvas.ClipY, ustr$newgs.MoneySpent, 0, false, EJ_Left);
	sy += INC_Y;
	if(StatNum == 21) return;

	// Pee total
	MyFont.DrawTextEx(Canvas, Canvas.ClipX, sxl * Canvas.ClipX, sy * Canvas.ClipY, PeeTotal, 0, false, EJ_Left);
	MyFont.DrawTextEx(Canvas, Canvas.ClipX, sxr * Canvas.ClipX, sy * Canvas.ClipY, ustr$(0.1*float(newgs.PeeTotal)), 0, false, EJ_Left);
	sy += INC_Y;
	if(StatNum == 22) return;

	// Doors kicked
	MyFont.DrawTextEx(Canvas, Canvas.ClipX, sxl * Canvas.ClipX, sy * Canvas.ClipY, DoorsKicked, 0, false, EJ_Left);
	MyFont.DrawTextEx(Canvas, Canvas.ClipX, sxr * Canvas.ClipX, sy * Canvas.ClipY, ustr$newgs.DoorsKicked, 0, false, EJ_Left);
	sy += INC_Y;
	if(StatNum == 23) return;

	// Times arrested -- M-F only
	if (!bWeekendOnly)
	{
		MyFont.DrawTextEx(Canvas, Canvas.ClipX, sxl * Canvas.ClipX, sy * Canvas.ClipY, TimesArrested, 0, false, EJ_Left);
		MyFont.DrawTextEx(Canvas, Canvas.ClipX, sxr * Canvas.ClipX, sy * Canvas.ClipY, ustr$newgs.TimesArrested, 0, false, EJ_Left);
		sy += INC_Y;
	}
	if(StatNum == 24) return;

	// Dressed as cop -- M-F only
	if (!bWeekendOnly)
	{
		if (newgs.DressedAsCop == 0 && bBusta)
		{
			MyFont.TextColor = YellowC;
			MyFont.DrawTextEx(Canvas, Canvas.ClipX, sxl * Canvas.ClipX, sy * Canvas.ClipY, DressedAsCop@New_Busta, 0, false, EJ_Left);
			MyFont.DrawTextEx(Canvas, Canvas.ClipX, sxr * Canvas.ClipX, sy * Canvas.ClipY, ustr$newgs.DressedAsCop, 0, false, EJ_Left);
			MyFont.TextColor = MyFont.Default.TextColor;
		}
		else
		{
			MyFont.DrawTextEx(Canvas, Canvas.ClipX, sxl * Canvas.ClipX, sy * Canvas.ClipY, DressedAsCop, 0, false, EJ_Left);
			MyFont.DrawTextEx(Canvas, Canvas.ClipX, sxr * Canvas.ClipX, sy * Canvas.ClipY, ustr$newgs.DressedAsCop, 0, false, EJ_Left);
		}
		sy += INC_Y;
	}
	if(StatNum == 25) return;

	// Time elapsed -- Full game only
	if (bWeekendGame && !bWeekendOnly)
	{
		if (newgs.TimeElapsed <= SPEED_RUN && bMatrix)
		{
			MyFont.TextColor = YellowC;
			MyFont.DrawTextEx(Canvas, Canvas.ClipX, sxl * Canvas.ClipX, sy * Canvas.ClipY, TimeElapsed@New_Matrix, 0, false, EJ_Left);
			MyFont.DrawTextEx(Canvas, Canvas.ClipX, sxr * Canvas.ClipX, sy * Canvas.ClipY, ustr$newgs.HoursPlayed()$":"$newgs.MinutesPlayed()$":"$newgs.SecondsPlayed(), 0, false, EJ_Left);
			MyFont.TextColor = MyFont.Default.TextColor;
		}
		else
		{
			MyFont.DrawTextEx(Canvas, Canvas.ClipX, sxl * Canvas.ClipX, sy * Canvas.ClipY, TimeElapsed, 0, false, EJ_Left);
			MyFont.DrawTextEx(Canvas, Canvas.ClipX, sxr * Canvas.ClipX, sy * Canvas.ClipY, ustr$newgs.HoursPlayed()$":"$newgs.MinutesPlayed()$":"$newgs.SecondsPlayed(), 0, false, EJ_Left);
		}
		sy += INC_Y;
	}
	if (StatNum < 28) return;

	// Final summary
	MyFont.DrawTextEx(Canvas, Canvas.ClipX, 0.5 * Canvas.ClipX, sy * Canvas.ClipY, Ranking, 2, false, EJ_Center);
	sy += INC_Y;

	if (newgs.DidPlayerCheat())
		MyFont.DrawTextEx(Canvas, Canvas.ClipX, 0.5 * Canvas.ClipX, sy * Canvas.ClipY, ustr$Cheating@PlayerRank, 3, false, EJ_Center);
	else
		MyFont.DrawTextEx(Canvas, Canvas.ClipX, 0.5 * Canvas.ClipX, sy * Canvas.ClipY, ustr$PlayerRank, 3, false, EJ_Center);

	sy += INC_Y;
	sy += INC_Y;
	if(useinfo != None && bFOver && !ViewportOwner.Actor.Level.IsDemoBuild())
	{
		MyFont.DrawTextEx(Canvas, Canvas.ClipX, 0.5 * Canvas.ClipX, sy * Canvas.ClipY, CheatsNow, 1, false, EJ_Center);
		if(StatNum == 29) return;
/*
		if(useinfo.DoMP1()
			&& useinfo.DoMP2()
			&& useinfo.DoMP3())
		{
			sy += INC_Y;
			MyFont.TextColor=YellowC;
			MyFont.DrawTextEx(Canvas, Canvas.ClipX, 0.5 * Canvas.ClipX, sy * Canvas.ClipY, NewItemNow, 1, false, EJ_Center);
			MyFont.TextColor=MyFont.default.TextColor;
			if(StatNum == 30) return;
		}
*/
	}
}

///////////////////////////////////////////////////////////////////////////////
// Default properties
///////////////////////////////////////////////////////////////////////////////

defaultproperties
{
     New_Metool="(New code: DogLover)"
     New_Busta="(New code: CopKilla)"
     New_Fathom="(New code: Charismatic)"
     New_Matrix="(New speed codes!)"
     New_Final="All cheats unlocked -- Final Cheats menu now available!"
     GameIncompleteRanking="Beat the damn game first!"
     InnocentsKilled="Innocents killed:"
     TimeElapsed="Time Elapsed:"
     HeadsBatted="Heads batted off:"
     Cheating="Cheating"
     KatanasImpaled="People impaled with katana:"
     StatMax=30
}
