class EAChameleon extends Chameleon;

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
const GLASSES_RATIO = 0.125;		// Percentage of people who wear glasses
const SUNGLASSES_RATIO = 0.125;		// Percentage of people who wear sunglasses
const GIVE_NORMAL_HAT = 0.125;		// How often to give a normal hat (baseball cap)
const GIVE_STUPID_HAT = 0.0625;		// How often to give a stupid hat (fez, beanie, etc.)
const BACKWARDS_HAT = 0.25;			// If given a normal hat, sometimes wear it backward
const PURSE_RATIO = 0.25;			// If a female, maybe give a purse
const HOMO_PURSE_RATIO = 0.0625;	// maybe give purses to feminine males
const BRIEFCASE_RATIO = 0.25;		// If a businessman/woman, maybe give a briefcase

const BLUE_COP_HAT = 1.0;			// How often to give blue cops a blue cop hat
const OTHER_COP_HAT = 0.25;			// How often to give Leatherman hats to other cops

const BRIEFCASE_BONE = 'MALE01 R Finger21';
const HEAD_BONE = 'NODE_Parent';

var() array<StaticMesh> CorrectiveLenses;		// Boltons for corrective glasses
var() array<StaticMesh> SunGlasses;				// Boltons for sunglasses
var() array<StaticMesh> NormalHats;				// Boltons for ballcaps, visors, etc.
var() array<StaticMesh> NormalBackwardHats;		// Boltons for backward normal hats
var() array<StaticMesh> StupidHats;				// Boltons for silly hats (fez, beanie etc.)
var() array<StaticMesh> Purses;					// Boltons for purses
var() array<StaticMesh> Briefcases;				// Boltons for briefcase
var() StaticMesh BlueCopHatMale;				// The one-and-only blue cop hat bolton for men
var() StaticMesh BlueCopHatFem;					// The one-and-only blue cop hat for women with short hair
var() StaticMesh OtherCopHat;					// The one-and-only Leatherman cop hat for men
var() Mesh FemLHMesh;							// Long-haired females can't wear hats

var() array<Material> Businessmen;				// Pawn skins that are considered to be businessmen and can carry a briefcase

///////////////////////////////////////////////////////////////////////////////
// Is it safe to overwrite this bolton record.
///////////////////////////////////////////////////////////////////////////////
function bool UnusedBolton(P2MoCapPawn.SBolton Bolt)
{
	return (Bolt.bInActive ||
		(Bolt.Mesh == None && Bolt.StaticMesh == None));
}

///////////////////////////////////////////////////////////////////////////////
// Some people don't want to have purses or things because they're on
// duty, like police officers or cashiers.
///////////////////////////////////////////////////////////////////////////////
function bool OnOfficialBusiness(P2MocapPawn P)
{
	return (
		EACashierController(P.Controller) != None
		|| P.bAuthorityFigure
		|| P.IsA('EAMarchers')
		|| P.IsA('AWZombie'));
}

///////////////////////////////////////////////////////////////////////////////
// Find the first unused bolton record
///////////////////////////////////////////////////////////////////////////////
function int GetNextBolton(P2MocapPawn P, optional int CurrBolt)
{
	while (CurrBolt < ArrayCount(P.Boltons) && !UnusedBolton(P.Boltons[CurrBolt]))
		CurrBolt++;

	if (CurrBolt >= ArrayCount(P.Boltons))
		return -1;
	else
		return CurrBolt;
}

function RandomizeScale(out float DScale)
{
	DScale = DScale + (MyRand() - 0.5)/1;
}

///////////////////////////////////////////////////////////////////////////////
// Pick a random appearance for this pawn's body.
// Add in silly hats, purses, briefcases etc.
///////////////////////////////////////////////////////////////////////////////
function Pick(P2MocapPawn P, FPSPawn.EGender ReqGender)
{
	local int NextBolton;
	local int i;
	local bool bUse;
	local bool bHasHat;
	local bool bHasGlasses;
	local vector NewScale;

//	log(self@"EAChameleon :: Pick for"@P@"gender"@ReqGender);
//	log(self@"Calling Super.Pick");
	Super.Pick(P, ReqGender);
//	log(self@"Back from Super");

/*
	// Head scale and height variance
	if (P.bRandomizeHeadScale)
	{
		log(self@"randomizing head and body scale for"@P);

		NewScale = P.DrawScale3D;
		RandomizeScale(NewScale.Y);
		log(self@"new body scale is"@NewScale);
		P.SetDrawScale3D(NewScale);

		NewScale = P.MyHead.DrawScale3D;
		RandomizeScale(NewScale.X);
		RandomizeScale(NewScale.Y);
		RandomizeScale(NewScale.Z);
		log(self@"new head scale is"@NewScale);
		P.MyHead.SetDrawScale3D(NewScale);

	}
*/

	// Certain people don't get these boltons, no matter what
	if (EAFanatics(P) != None			// Tali-banana don't care
		|| AWFanatics(P) != None		// Taliban still don't care
		|| EABum(P) != None			// Bums can't afford them
		|| EAGary(P) != None			// Gary doesn't care
		|| AWGary(P) != None			// Gary still doesn't care
		|| EAGimp_AMW(P) != None		// The gimp already looks fucking silly enough
		|| EAKrotchy(P) != None		// As does Krotchy
		|| EAKumquat(P) != None		// Taliban don't care
		|| EAPostalDude(P) != None		// He's the motherfucking Postal Dude
		|| AWRWSStaff(P) != None		// RWS don't care
		|| EAUncleDave(P) != None		// Dave doesn't care
		|| Kamek(P) != None				// Kamek don't care
		|| AWMilitary(P) != None)		// The military don't care
		return;

//	if (OnOfficialBusiness(P))
//		log(self@"This pawn IS on official business");
//	else
//		log(self@"This pawn is NOT on official business");

	NextBolton = GetNextBolton(P);
//	log(self@"Next bolton index is"@NextBolton);
	if (NextBolton == -1)
		return;

//	log(self@"Possibly give a blue hat to"@P@"bIsFat"@P.bIsFat@"head mesh"@P.HeadMesh);
	// If it's a blue cop, maybe give a blue cop hat
	if (EACopBlue(P) != None
		&& !P.bIsFat
		&& !bHasHat
		&& MyRand() <= BLUE_COP_HAT)
	{
		P.Boltons[NextBolton].Bone = HEAD_BONE;
		if (P.bIsFemale && P.HeadMesh != FemLHMesh)
			P.Boltons[NextBolton].StaticMesh = BlueCopHatFem;
		else if (!P.bIsFemale)
			P.Boltons[NextBolton].StaticMesh = BlueCopHatMale;
		P.Boltons[NextBolton].bAttachToHead = True;
		P.HeadScale.X = FMin(P.HeadScale.X, 1.00);
		P.HeadScale.Y = FMin(P.HeadScale.Y, 1.00);
		P.HeadScale.Z = FMin(P.HeadScale.Z, 1.00);
		bHasHat = True;
//		log(self@"Gave a blue hat!");
	}

	NextBolton = GetNextBolton(P);
//	log(self@"Next bolton index is"@NextBolton);
	if (NextBolton == -1)
		return;

//	log(self@"Possibly give a Leatherman to"@P@"bHasHat"@bHasHat@"bIsFat"@P.bIsFat@"bIsFemale"@P.bIsFemale);
	// If it's some other type of cop, maybe give a Leatherman
	if (EAPolice(P) != None
		&& !bHasHat
		&& !P.bIsFat
		&& !P.bIsFemale
		&& MyRand() <= OTHER_COP_HAT)
	{
		P.Boltons[NextBolton].Bone = HEAD_BONE;
		P.Boltons[NextBolton].StaticMesh = OtherCopHat;
		P.Boltons[NextBolton].bAttachToHead = True;
		P.HeadScale.X = FMin(P.HeadScale.X, 1.00);
		P.HeadScale.Y = FMin(P.HeadScale.Y, 1.00);
		P.HeadScale.Z = FMin(P.HeadScale.Z, 1.00);
		bHasHat = True;
//		log(self@"Gave a leatherman!");
	}

	NextBolton = GetNextBolton(P);
//	log(self@"Next bolton index is"@NextBolton);
	if (NextBolton == -1)
		return;

//	log(self@"possibly give a briefcase to"@P@"skin"@P.Skins[0]);
	// If it's a businessman, possibly give a briefcase
	bUse = False;
	for (i=0; i<Businessmen.Length; i++)
		if (P.Skins[0] == Businessmen[i])
			bUse = True;

	if (bUse)
	{
//		log(self@"gave a briefcase!");
		P.Boltons[NextBolton].Bone = BRIEFCASE_BONE;
		P.Boltons[NextBolton].bCanDrop = True;
		P.Boltons[NextBolton].StaticMesh = Briefcases[MyRandRange(0, Briefcases.Length)];
	}

	NextBolton = GetNextBolton(P);
//	log(self@"Next bolton index is"@NextBolton);
	if (NextBolton == -1)
		return;

//	log(self@"maybe give out corrective glasses to"@P@"bIsFat"@P.bIsFat@"bIsFemale"@P.bIsFemale);
	// Maybe give out corrective glasses
	if (!P.bIsFat
		&& MyRand() <= GLASSES_RATIO)
	{
		P.Boltons[NextBolton].Bone = HEAD_BONE;
		P.Boltons[NextBolton].bAttachToHead = True;
		P.Boltons[NextBolton].StaticMesh = CorrectiveLenses[MyRandRange(0, CorrectiveLenses.Length)];
		bHasGlasses=True;
//		log(self@"gave corrective lenses! don't give him sunglasses too!");
	}

	NextBolton = GetNextBolton(P);
//	log(self@"Next bolton index is"@NextBolton);
	if (NextBolton == -1)
		return;

//	log(self@"maybe give out sunglasses to"@P@"bIsFat"@P.bIsFat@"bIsFemale"@P.bIsFemale@"bHasGlasses"@bHasGlasses);
	// Maybe give out sunglasses
	if (!P.bIsFat
		&& !bHasGlasses
		&& MyRand() <= SUNGLASSES_RATIO)
	{
		P.Boltons[NextBolton].Bone = HEAD_BONE;
		P.Boltons[NextBolton].bAttachToHead = True;
		P.Boltons[NextBolton].StaticMesh = Sunglasses[MyRandRange(0, Sunglasses.Length)];
		bHasGlasses=True;
//		log(self@"gave sunglasses to"@P);
	}

	NextBolton = GetNextBolton(P);
//	log(self@"Next bolton index is"@NextBolton);
	if (NextBolton == -1)
		return;

//	log(self@"maybe give a baseball cap to"@P@"bIsFat"@P.bIsFat@"bIsFemale"@P.bIsFemale@"bHasHat"@bHasHat);
	// Maybe give out a baseball cap
	if (!P.bIsFat
		&& !P.bIsFemale
		&& !bHasHat
		&& !OnOfficialBusiness(P)
		&& MyRand() <= GIVE_NORMAL_HAT)
	{
		P.Boltons[NextBolton].Bone = HEAD_BONE;
		P.Boltons[NextBolton].bAttachToHead = True;
		if (MyRand() <= BACKWARDS_HAT)
			P.Boltons[NextBolton].StaticMesh = NormalBackwardHats[MyRandRange(0, NormalBackwardHats.Length)];
		else
			P.Boltons[NextBolton].StaticMesh = NormalHats[MyRandRange(0, NormalHats.Length)];
		P.HeadScale.X = FMin(P.HeadScale.X, 1.00);
		P.HeadScale.Y = FMin(P.HeadScale.Y, 1.00);
		P.HeadScale.Z = FMin(P.HeadScale.Z, 1.00);
		bHasHat=True;
//		log(self@"gave baseball cap!");
	}

	NextBolton = GetNextBolton(P);
//	log(self@"Next bolton index is"@NextBolton);
	if (NextBolton == -1)
		return;

//	log(self@"maybe give a silly hat to"@P@"bIsFat"@P.bIsFat@"bIsFemale"@P.bIsFemale@"bHasHat"@bHasHat);
	// Maybe give out a silly hat
	if (!P.bIsFat
		&& !P.bIsFemale
		&& !bHasHat
		&& !OnOfficialBusiness(P)
		&& MyRand() <= GIVE_STUPID_HAT)
	{
		P.Boltons[NextBolton].Bone = HEAD_BONE;
		P.Boltons[NextBolton].bAttachToHead = True;
		P.Boltons[NextBolton].StaticMesh = StupidHats[MyRandRange(0, StupidHats.Length)];
		P.HeadScale.X = FMin(P.HeadScale.X, 1.00);
		P.HeadScale.Y = FMin(P.HeadScale.Y, 1.00);
		P.HeadScale.Z = FMin(P.HeadScale.Z, 1.00);
		bHasHat=True;
//		log(self@"gave stupid hat!");
	}

	NextBolton = GetNextBolton(P);
//	log(self@"Next bolton index is"@NextBolton);
	if (NextBolton == -1)
		return;

//	log(self@"maybe give a purse to"@P@"bIsFemale"@P.bIsFemale@"bIsFeminine"@P.bIsFeminine);
	// Maybe give a purse
	if (
		(P.bIsFemale
			&& !OnOfficialBusiness(P)
			&& MyRand() <= PURSE_RATIO)
		||
		(!P.bIsFemale
			&& !OnOfficialBusiness(P)
			&& P.bIsFeminine
			&& MyRand() <= HOMO_PURSE_RATIO)
		)
	{
//		log(self@"gave a purse!");
		P.Boltons[NextBolton].Bone = BRIEFCASE_BONE;
		P.Boltons[NextBolton].bCanDrop = True;
		P.Boltons[NextBolton].StaticMesh = Purses[MyRandRange(0, Purses.Length)];
	}

//	log(self@"all done!");
}

///////////////////////////////////////////////////////////////////////////////
// Use MyRand() to give a pseudo-random number from Low to High
///////////////////////////////////////////////////////////////////////////////
function int MyRandRange(int Low, int High)
{
	local float f;
	local int i;

	f = MyRand() * (High - Low);
	i = Int(f);
	return i;
}

///////////////////////////////////////////////////////////////////////////////
// Default properties
///////////////////////////////////////////////////////////////////////////////

defaultproperties
{
     CorrectiveLenses(0)=StaticMesh'EAMesh.Boltons_AW7.Glasses'
     Sunglasses(0)=StaticMesh'EAMesh.Boltons_AW7.Sunglasses'
     NormalHats(0)=StaticMesh'EAMesh.Boltons_AW7.BaseballCap1_FWD_AvgMale'
     NormalHats(1)=StaticMesh'EAMesh.Boltons_AW7.BaseballCap2_FWD_AvgMale'
     NormalHats(2)=StaticMesh'EAMesh.Boltons_AW7.BaseballCap3_FWD_AvgMale'
     NormalBackwardHats(0)=StaticMesh'EAMesh.Boltons_AW7.BaseballCap1_BACK_AvgMale'
     NormalBackwardHats(1)=StaticMesh'EAMesh.Boltons_AW7.BaseballCap2_BACK_AvgMale'
     NormalBackwardHats(2)=StaticMesh'EAMesh.Boltons_AW7.BaseballCap3_BACK_AvgMale'
     StupidHats(0)=StaticMesh'EAMesh.Boltons_AW7.Dumb-Ass-Beanie_AvgMale'
     StupidHats(1)=StaticMesh'EAMesh.Boltons_AW7.Fedora_AvgMale'
     Purses(0)=StaticMesh'EAMesh.Boltons_AW7.Purse_Black'
     Purses(1)=StaticMesh'EAMesh.Boltons_AW7.Purse_Red'
     Purses(2)=StaticMesh'EAMesh.Boltons_AW7.Purse_Tan'
     Briefcases(0)=StaticMesh'EAMesh.Boltons_AW7.Briefcase'
     BlueCopHatMale=StaticMesh'EAMesh.Boltons_AW7.CopHat_AvgMale'
     BlueCopHatFem=StaticMesh'EAMesh.Boltons_AW7.CopHat_FemSH'
     OtherCopHat=StaticMesh'EAMesh.Boltons_AW7.Leatherman_AvgMale'
     FemLHMesh=SkeletalMesh'heads.FemLH'
     Businessmen(0)=Texture'AW7PawnSkins.PaletteSwap.MW__212__Avg_M_Jacket_Pants'
     Businessmen(1)=Texture'ChameleonSkins.BystandersM.MM__018__Avg_M_Jacket_Pants'
     Businessmen(2)=Texture'ChameleonSkins.BystandersM.MW__019__Avg_M_Jacket_Pants'
     Businessmen(3)=Texture'ChameleonSkins.BystandersM.MW__021__Avg_M_Jacket_Pants'
     RandSeed=16384
}
