///////////////////////////////////////////////////////////////////////////////
// EADoorMover
// Extends from AWDoorMover to make it more useful
// These can be destroyed now.
///////////////////////////////////////////////////////////////////////////////
class EADoorMover extends AWDoorMover;

///////////////////////////////////////////////////////////////////////////////
// Public vars
///////////////////////////////////////////////////////////////////////////////
var(AWDoorMover) class<TimedMarker> DestroyedMarkerMade;

///////////////////////////////////////////////////////////////////////////////
// Explosions and sledge damage now can destroy doors
// Same as super but reorient the prop broken effect so it looks better
///////////////////////////////////////////////////////////////////////////////
function TakeDamage( int Damage, Pawn instigatedBy, Vector hitlocation,
						Vector momentum, class<DamageType> damageType)
{
	local TimedMarker ADanger;

	if(Health > 0)
	{
		if(ClassIsChildOf(damageType, class'ExplodedDamage')
			|| ClassIsChildOf(damageType, class'SledgeDamage')
			|| ClassIsChildOf(damageType, class'AxeDamage'))
//			|| ClassIsChildOf(damageType, class'ChainsawDamage'))
		{
			Health -= Damage;
			if(Health <= 0)
			{
				// Throw out effects
				// Reorient the break effect so it spawns at the hitlocation instead of at our location
				if(ExplodeEffect != None)
					spawn(ExplodeEffect, self,, HitLocation, Rotation);

				// Make noise
				PlaySound(ExplodeSound,, 1.0,,1000,GetRandPitch());
				
				// "Destroy" ourself by removing collision and hiding the static mesh
				InvisDestroy();

				// Spawn a timed marker to alert other people that we're destroying their property
				if(DestroyedMarkerMade != None
					&& ClassIsChildOf(DamageType, class'SledgeDamage'))
				{
					ADanger = spawn(DestroyedMarkerMade,,,HitLocation);
					ADanger.CreatorPawn = FPSPawn(InstigatedBy);
					ADanger.OriginActor = self;
					// This will cause people to see if they noticed and decide what to do
					ADanger.NotifyAndDie();
				}
			}
		}
	}
}

///////////////////////////////////////////////////////////////////////////////
// Default properties
///////////////////////////////////////////////////////////////////////////////

defaultproperties
{
     DestroyedMarkerMade=Class'Postal2Game.PropBreakMarker'
     Health=100.000000
     ExplodeSound=Sound'LevelSoundsToo.library.woodCrash02'
     ExplodeEffect=Class'EAFX.WoodChunksDoor'
}
