///////////////////////////////////////////////////////////////////////////////
// EAGameInfo
// By: Kamek (kamek@postalleague.com)
// For: EA
//
// Since the AW game extends from P2GameInfoSingle and not GameSinglePlayer,
// we have to copy a bunch of crap.
///////////////////////////////////////////////////////////////////////////////
class EAGameInfo extends EAEarlyGameInfo;

///////////////////////////////////////////////////////////////////////////////
// Config Vars
///////////////////////////////////////////////////////////////////////////////
var globalconfig int InitVal;
var globalconfig int CornHole;
var globalconfig int UberSquire;
var globalconfig int MetoolAlloy;
var globalconfig int BustaRhymes;
var globalconfig int UnFathomable;
var globalconfig int MatrixMan;

var globalconfig bool bNewWeaponOrder;	// splits up Group 1 melee weapons between 1 and 2, moves Group 2 weapons to Group 3
var globalconfig bool bDrawTime;

var() localized string GameNameShort;

///////////////////////////////////////////////////////////////////////////////
// Consts
///////////////////////////////////////////////////////////////////////////////
const INIT_1 = 150;
const INIT_2 = 120;
const INIT_3 = 3;
const INIT_PATH = "EAGame.EAGameInfo InitVal";

const CORN_1 = 291;
const CORN_2 = 555;
const CORN_3 = 5.11;
const CORN_PATH = "EAGame.EAGameInfo CornHole";

const UBER_1 = 101;
const UBER_2 = 102;
const UBER_3 = 2;
const UBER_PATH = "EAGame.EAGameInfo UberSquire";

const METOOL_1 = 666;
const METOOL_2 = 911;
const METOOL_3 = 3.14;
const METOOL_PATH = "EAGame.EAGameInfo MetoolAlloy";

const BUSTA_1 = 222;
const BUSTA_2 = 333;
const BUSTA_3 = 1.214;
const BUSTA_PATH = "EAGame.EAGameInfo BustaRhymes";

const FATHOM_1 = 1000;
const FATHOM_2 = 1500;
const FATHOM_3 = 0.5;
const FATHOM_PATH = "EAGame.EAGameInfo UnFathomable";

const MATRIX_1 = 111;
const MATRIX_2 = 222;
const MATRIX_3 = 3.03;
const MATRIX_PATH = "EAGame.EAGameInfo MatrixMan";

const REDTURTLE_DAYNAME = "REDTURTLEDAY";

///////////////////////////////////////////////////////////////////////////////
// Public Vars
///////////////////////////////////////////////////////////////////////////////
var() string WeekendStartURL;
var() class<AWCatPawn> ApocalypseCatClass;
var() class<AWGaryHead> GaryHeadClass;
var() class<ZombieHead> ZombieHeadClass;
var string EAMutatorClassName;

///////////////////////////////////////////////////////////////////////////////
// Internal Vars
///////////////////////////////////////////////////////////////////////////////
var Material FireCorona;
var float NewCatTime;	// When we'll make a new cat
var float CatRainTime;	// When we'll start raining again
var bool bRainingCats;

///////////////////////////////////////////////////////////////////////////////
// More Consts
///////////////////////////////////////////////////////////////////////////////
const NEW_CAT_BASE_TIME	= 5;
const NEW_CAT_RAND_TIME	= 5;
const RAIN_BASE_TIME	= 60;
const RAIN_RAND_TIME	= 120;
const CAT_MAKE_RANGE_DIST=4000;
const CAT_MAKE_RANGE_XY = 1500;
const CAT_MAKE_BASE_DIST= 200;
const CAT_MAKE_Z		= 3000;
const CAT_RAIN_SPEED	= 100;
const CAT_RAIN_ACC		= -1200;

const DAY_FRIDAY = 4;
const DAY_SATURDAY = 5;

///////////////////////////////////////////////////////////////////////////////
// Returns true if we're on Saturday or Sunday
///////////////////////////////////////////////////////////////////////////////
function bool IsWeekend()
{
	return (TheGameState.CurrentDay >= DAY_SATURDAY);
}

///////////////////////////////////////////////////////////////////////////////
// Returns true if this game is only Saturday and Sunday
///////////////////////////////////////////////////////////////////////////////
function bool WeekendOnlyGame()
{
	return false;
}

///////////////////////////////////////////////////////////////////////////////
// Returns true if this game includes Saturday and Sunday
///////////////////////////////////////////////////////////////////////////////
function bool WeekendGame()
{
	return true;
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
function KillTravelInv()
{
	local Pawn P;
	local P2Player P2P;
	local TravelInv T;

	P = GetPlayer().Pawn;

	foreach AllActors(class'TravelInv',T)
	{
		P.DeleteInventory(T);
		T.Destroy();
	}
}

/*
///////////////////////////////////////////////////////////////////////////////
// Hack
///////////////////////////////////////////////////////////////////////////////
function SearchForOldControllers()
{
	foreach DynamicActors(class'AWPerson', P)
		if (P.Controller.class == class'BystanderController')
			ConvertController(P);
}

function ConvertController(AWPerson P)
{
	P.Controller.Destroy();
	P.Controller = None;
	P.Controller = spawn(class'EACivilianController');
	if(P.Controller != None )
	{
		P.Controller.Possess(P);
		AIController(P.Controller).Skill += P.SkillModifier;
		P.CheckForAIScript();
	}
}
*/

function int ReduceDamage( int Damage, pawn injured, pawn instigatedBy, vector HitLocation, out vector Momentum, class<DamageType> DamageType )
{
	if (IsCinematic() && EAPostalDude(Injured) != None)
		return 0;
	else
		return Super.ReduceDamage(Damage, Injured, InstigatedBy, HitLocation, Momentum, DamageType);
}

///////////////////////////////////////////////////////////////////////////////
// HACK -- switch weapons to shiny or non-shiny textures
///////////////////////////////////////////////////////////////////////////////
function HackMakeShiny()
{
	local MaDBloodWeapon W;
//	local P2WeaponPickup P;

	foreach DynamicActors(class'MaDBloodWeapon', W)
		W.MakeShiny();
}

///////////////////////////////////////////////////////////////////////////////
// HACK -- switch weapons to shiny or non-shiny textures
///////////////////////////////////////////////////////////////////////////////
function HackMakeUnShiny()
{
	local MaDBloodWeapon W;
//	local P2WeaponPickup P;

	foreach DynamicActors(class'MaDBloodWeapon', W)
		W.MakeUnShiny();
}

///////////////////////////////////////////////////////////////////////////////
// REALLY BAD HACK.
// The "proper" way is to subclass all these pawns. But I don't have the time...
///////////////////////////////////////////////////////////////////////////////
function HackChanger(class<AWPerson> PawnClass, bool bIsZombie, bool bIsGary)
{
	if (bIsZombie)
		PawnClass.Default.HeadClass = ZombieHeadClass;
	else if (bIsGary)
		PawnClass.Default.HeadClass = GaryHeadClass;
	else
		PawnClass.Default.HeadClass = class'EAPerson'.Default.HeadClass;

	if (!bIsGary)
	{
		PawnClass.Default.LimbClass = class'EAPerson'.Default.LimbClass;
		PawnClass.Default.StumpClass = class'EAPerson'.Default.StumpClass;
	}

	PawnClass.Default.GutsEmitterClass1 = class'EAPerson'.Default.GutsEmitterClass1;
	PawnClass.Default.GutsEmitterClass2 = class'EAPerson'.Default.GutsEmitterClass2;
}

///////////////////////////////////////////////////////////////////////////////
// REALLY BAD HACK.
// The "proper" way is to subclass all these pawns. But I don't have the time...
///////////////////////////////////////////////////////////////////////////////
function HackChangeHeadLimbAndGutsClass()
{
	HackChanger(class'AWMilitary',false,false);
	HackChanger(class'AWGary',false,true);
	HackChanger(class'AWSuperGary',false,true);
	HackChanger(class'BattleDamageGary',false,true);
	HackChanger(class'AWZombie',true,false);
	HackChanger(class'AWZombieCharger',true,false);
	HackChanger(class'AWZombieNoCharge',true,false);
	HackChanger(class'AWZombieNoSpit',true,false);
	HackChanger(class'AWZombieSpitter',true,false);
	HackChanger(class'AWZombieSuperCharger',true,false);
	HackChanger(class'AWZombieSuperSpitter',true,false);
	HackChanger(class'AWZombieTom',true,false);
	HackChanger(class'AnimalProtestors',false,false);
	HackChanger(class'AWBystanders',false,false);
	HackChanger(class'AWFanatics',false,false);
	HackChanger(class'AWSuperFanatics',false,false);
	HackChanger(class'AWRednecks',false,false);
	HackChanger(class'AWScaredGary',false,false);
	HackChanger(class'InvisiGuard',false,false);
	HackChanger(class'PhraudHogslop',false,false);
	HackChanger(class'PublisherGuard',false,false);

	if (TheyHateMeMode())
	{
		HackChangePawnSpawners();
		class'AWMilitary'.default.ControllerClass = class'EACivilianController';
		class'PublisherGuard'.default.ControllerClass = class'EACivilianController';
	}
}

function HackChangePawnSpawners()
{
	local PawnSpawner P;
	
	foreach DynamicActors(class'PawnSpawner', P)
		if (P.InitControllerClass == class'AWMilitaryController')
			P.InitControllerClass = class'EACivilianController';
}

///////////////////////////////////////////////////////////////////////////////
// Hacky, but could be worse...
///////////////////////////////////////////////////////////////////////////////
function SetWeaponOrder()
{
	const MachetePath = "AWInventory.MacheteWeapon";
	const ScythePath = "AWInventory.ScytheWeapon";
	const MeatcleaverPath = "meatcleaver.MeatCleaverWeapon";
	const PitchforkPath = "Pitchfork.PitchForkWeapon";
	const KatanaPath = "EAInventory.EAKatanaWeapon";
	const MaDBAxePath = "Axe.AxeWeapon";
//	const MaDLAxePath = "AW7MaDInventory.MaDLAxeWeapon";
	const PistolPath = "Inventory.PistolWeapon";
	const MaDSPistolPath = "EAInventory.MaDSPistolWeapon";
	const MaDColtPath = "EAInventory.MaDColtWeapon";
	const ShotgunPath = "Inventory.ShotgunWeapon";
	const SMEGPath = "EAInventory.SMGEWeapon";

	// Reorder various melee weapons. Also a hack, but unavoidable.
	class'MacheteWeapon'.Default.GroupOffset = 50;
	class'ScytheWeapon'.Default.GroupOffset = 51;

	if (bNewWeaponOrder)
	{
		Level.ConsoleCommand("set"@MachetePath@"InventoryGroup 2");
		Level.ConsoleCommand("set"@ScythePath@"InventoryGroup 2");
		Level.ConsoleCommand("set"@MeatcleaverPath@"InventoryGroup 2");
		Level.ConsoleCommand("set"@PitchforkPath@"InventoryGroup 2");
		Level.ConsoleCommand("set"@KatanaPath@"InventoryGroup 2");
		Level.ConsoleCommand("set"@MaDBAxePath@"InventoryGroup 2");
//		Level.ConsoleCommand("set"@MaDLAxePath@"InventoryGroup 2");

		Level.ConsoleCommand("set"@PistolPath@"InventoryGroup 3");
		Level.ConsoleCommand("set"@MaDSPistolPath@"InventoryGroup 3");
		Level.ConsoleCommand("set"@MaDColtPath@"InventoryGroup 3");

		Level.ConsoleCommand("set"@ShotgunPath@"GroupOffset 4");
		Level.ConsoleCommand("set"@SMEGPath@"GroupOffset 5");
	}
	else
	{
		Level.ConsoleCommand("set"@MachetePath@"InventoryGroup 1");
		Level.ConsoleCommand("set"@ScythePath@"InventoryGroup 1");
		Level.ConsoleCommand("set"@MeatcleaverPath@"InventoryGroup 1");
		Level.ConsoleCommand("set"@PitchforkPath@"InventoryGroup 1");
		Level.ConsoleCommand("set"@KatanaPath@"InventoryGroup 1");
		Level.ConsoleCommand("set"@MaDBAxePath@"InventoryGroup 1");
//		Level.ConsoleCommand("set"@MaDLAxePath@"InventoryGroup 1");

		Level.ConsoleCommand("set"@PistolPath@"InventoryGroup 2");
		Level.ConsoleCommand("set"@MaDSPistolPath@"InventoryGroup 2");
		Level.ConsoleCommand("set"@MaDColtPath@"InventoryGroup 2");

		Level.ConsoleCommand("set"@ShotgunPath@"GroupOffset 1");
		Level.ConsoleCommand("set"@SMEGPath@"GroupOffset 2");
	}
	
	if (bShinyWeapons)
		HackMakeShiny();
	else
		HackMakeUnShiny();
}



///////////////////////////////////////////////////////////////////////////////
// This function is called before any other scripts (including PreBeginPlay().
///////////////////////////////////////////////////////////////////////////////
event InitGame(out string Options, out string Error)
{
	Super.InitGame(Options, Error);
	MySlotInfoMgr = spawn(class'EASlotInfoMgr');

	HackChangeHeadLimbAndGutsClass();
	SetWeaponOrder();
//	DestroyUnusedWeapons();

	// Raise lifespan of gas trail. Also a hack.
	class'GasTrail'.Default.LifeSpan = 600;
	class'ScissorsWeapon'.Default.WeaponSpeedShoot1 = class'ScissorsWeapon'.Default.WeaponSpeedShoot1MP;
	class'ScissorsWeapon'.Default.WeaponSpeedShoot2 = class'ScissorsWeapon'.Default.WeaponSpeedShoot2MP;

	if (EAGameState(TheGameState).RedTurtleDay <= 0)
	{
		EAGameState(TheGameState).RedTurtleDay=Rand(5);
		log(self@"set red turtle day to"@EAGameState(TheGameState).RedTurtleDay$". Don't forget to remove this log message!");
	}
	else
		log(self@"red turtle day is"@EAGameState(TheGameState).RedTurtleDay$". Don't forget to remove this log message!");

	// Let telepads travel more pawns
//	class'Telepad'.Default.TravelRadius = 4800;

//	AddMutator(EAMutatorClassName);

	// Attempted crash fix DeadBodyMarker. Doesn't seem to work well.
//	class'DeadBodyMarker'.Default.NotifyTime = 2.5;

	// This didn't work the way I wanted, so stubbed
//	class'FirePuddle'.Default.LightBrightness = 128;
//	class'FirePuddle'.Default.LightHue = 255;
//	class'FirePuddle'.Default.LightSaturation = 127;
//	class'FirePuddle'.Default.LightType = LT_Flicker;
//	class'FirePuddle'.Default.bDynamicLight = True;
//	class'FirePuddle'.Default.DrawScale = 0.5;
//	class'FirePuddle'.Default.bCorona = True;
//	class'FirePuddle'.Default.Skins[0] = FireCorona;
}

///////////////////////////////////////////////////////////////////////////////
// Check to swap out the postal dude's bandaged head skin.
///////////////////////////////////////////////////////////////////////////////
function CheckDudeHeadskin()
{
	local EAPostalDude TheDude;

	foreach DynamicActors(class'EAPostalDude', TheDude)
		TheDude.DudeCheckHeadSkin();
}

///////////////////////////////////////////////////////////////////////////////
// As soon as we have a valid game state, check for the dude's bandaged head
///////////////////////////////////////////////////////////////////////////////
event PostTravel(P2Pawn PlayerPawn)
{
	Super.PostTravel(PlayerPawn);
	CheckDudeHeadSkin();
}

///////////////////////////////////////////////////////////////////////////////
// As soon as we have a valid game state, check for the dude's bandaged head
///////////////////////////////////////////////////////////////////////////////
event PostLoadGame()
{
	Super.PostLoadGame();
	CheckDudeHeadSkin();
}

///////////////////////////////////////////////////////////////////////////////
// Add a controller to our newly spawned cat
///////////////////////////////////////////////////////////////////////////////
function AddController(FPSPawn newcat)
{
	if ( newcat.Controller == None
		&& newcat.Health > 0 )
	{
		if ( (newcat.ControllerClass != None))
			newcat.Controller = spawn(newcat.ControllerClass);
		if ( newcat.Controller != None )
			newcat.Controller.Possess(newcat);
		// Check for AI Script
		newcat.CheckForAIScript();
	}
}

///////////////////////////////////////////////////////////////////////////////
// Make sure it's the final day and all of our errands are done.
// *Doesn't* check to make sure it's not the demo.
///////////////////////////////////////////////////////////////////////////////
function bool AfterFinalErrand()
{
	// Last day of the real game and you're done with your errands
	return ((TheGameState.CurrentDay == DAY_FRIDAY)
			&& TheGameState.ErrandsCompletedToday >= Days[TheGameState.CurrentDay].NumActiveErrands());
}

///////////////////////////////////////////////////////////////////////////////
// Send player to the AW intro movie
///////////////////////////////////////////////////////////////////////////////
function StartWeekend(PlayerController Player)
{
	bShowDayDuringLoad = True;
	SendPlayerTo(Player, WeekendStartURL);
}

///////////////////////////////////////////////////////////////////////////////
// Send player to the next day
///////////////////////////////////////////////////////////////////////////////
function SendPlayerToNextDay(PlayerController player)
{
	// See if there's any more days
	if (TheGameState.CurrentDay + 1 < Days.length)
	{
		TheGameState.bChangeDayPostTravel = true;
		TheGameState.NextDay = TheGameState.CurrentDay + 1;
//		log(self@"SendPlayerToNextDay: Next day is"@TheGameState.NextDay);
		if (TheGameState.NextDay == DAY_SATURDAY)
			SendPlayerTo(Player, WeekendStartURL);
		else
			SendPlayerTo(player, StartNextDayURL);
	}
	else
	{
		// This shouldn't happen because the end-of-game movie
		// should end up calling EndOfGame() instead.  If it does
		// happen we'll treat it like a quit.
		QuitGame();
	}
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
static final function string Lower(coerce string Text) {

  local int IndexChar;

  for (IndexChar = 0; IndexChar < Len(Text); IndexChar++)
    if (Mid(Text, IndexChar, 1) >= "A" &&
        Mid(Text, IndexChar, 1) <= "Z")
      Text = Left(Text, IndexChar) $ Chr(Asc(Mid(Text, IndexChar, 1)) + 32) $ Mid(Text, IndexChar + 1);

  return Text;
  }

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
function string ParseLevelName(string In)
{
	local string S;

	S = Super.ParseLevelName(In);

	if (Right(S,4) ~= ".FUK")
		S = Left(S, Len(S)-4);

	return Lower(S);
}
/*
///////////////////////////////////////////////////////////////////////////////
// Check to remove this pickup from the GottenPickups list.
///////////////////////////////////////////////////////////////////////////////
function FindInPickupList(Pickup DelMe, String Lname)
{
	local int i;

	log(self@"searching for gotten pickup"@DelMe.Name@"from level"@LName);
	for(i=0; i<TheGameState.GottenPickups.Length; i++)
	{
		log(self$" checking"@TheGameState.GottenPickups[i].PickupName@"from level"@TheGameState.GottenPickups[i].LevelName);
		log(self@TheGameState.GottenPickups[i].PickupName@"=="@DelMe.Name$":"@string(TheGameState.GottenPickups[i].PickupName == DelMe.name));
		log(self@TheGameState.GottenPickups[i].LevelName@"~="@LName$":"@string(TheGameState.GottenPickups[i].LevelName ~= LName));
		if(TheGameState.GottenPickups[i].PickupName == DelMe.name
			&& TheGameState.GottenPickups[i].LevelName ~= Lname)
		{
			log(self$"												Removing "$DelMe);
			DelMe.Destroy();
			DelMe = None;
			break;
		}
	}
}

///////////////////////////////////////////////////////////////////////////////
//
// Go through all the pickups in the level, and remove any that have been
// previously taken by the dude.
//
///////////////////////////////////////////////////////////////////////////////
function RemoveGottenPickups()
{
	local Pickup CheckP, DelMe;
	local String Lname;

	Lname = ParseLevelName(Level.GetLocalURL());

	log(self$" RemoveGottenPickups for level"@Lname$", gottenpickups length "$TheGameState.GottenPickups.Length);

	foreach AllActors(class'Pickup', CheckP)
	{
		DelMe = CheckP;
//		log(self$" RemoveGottenPickups "$DelMe);
		// If you're not allowed to be recorded, you're not allowed to be restored
		if((P2WeaponPickup(DelMe) != None
				&& P2WeaponPickup(DelMe).bRecordAfterPickup)
			|| (P2PowerupPickup(DelMe) != None
					&& P2PowerupPickup(DelMe).bRecordAfterPickup)
			|| (P2AmmoPickup(DelMe) != None
					&& P2AmmoPickup(DelMe).bRecordAfterPickup))
		{
			FindInPickupList(DelMe, Lname);
		}
	}
}
*/
///////////////////////////////////////////////////////////////////////////////
// Rounds float X to DecimalPlaces places.
///////////////////////////////////////////////////////////////////////////////
static final function float Round(float X, int DecimalPlaces)
{
	local float Y;
	local int Z;

//	log("Round" @ X @ "to" @ DecimalPlaces @ "decimal places");
	Y = X * (10^DecimalPlaces);
	Z = Int(Y);
	Y = float(Z) / (10^DecimalPlaces);
//	log("Result is" @ Y);

	return Y;
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
function int GetRefVal(float Ref, float Parm1, float Parm2, float Parm3)
{
	local int Y;

	Y = Int((Ref + Parm1) * Parm2 / (Ref ^ Parm3));
	return Y;
}

///////////////////////////////////////////////////////////////////////////////
// Make sure these are done at StartGame and every time you load, in case,
// in between, someone deletes their ini, but uses an old load
///////////////////////////////////////////////////////////////////////////////
function PrepIniStartVals()
{
	GameRefVal = int(ConsoleCommand("get "@RefPath));

	if(GameRefVal == START_GAME_REF)
	{
		// Setup the reference value
		GameRefVal=(Rand(REF_STARTUP)/2);
		GameRefVal=2*GameRefVal;
		ConsoleCommand("set "@RefPath@GameRefVal);
//		log(self$" NEW made up gameref "$GameRefVal);

		InitVal = GetRefVal(GameRefVal, INIT_1, INIT_2, INIT_3);
		SaveVals(InitVal);
	}
	else
//		log(self$" CURRENT gameref "$GameRefVal);

	if(CoolStart == 0)
	{
		CoolStart = Rand(COOL_FACTOR);
		ConsoleCommand("set "@CoolStartPath@CoolStart);
	}

	if (InitVal != GetRefVal(GameRefVal, INIT_1, INIT_2, INIT_3))
	{
//		log(self@"Invalid InitVal ("$InitVal$") -- resetting");
		ZeroAllCheats();
	}
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
function SaveVals(
	optional int NewInit,
	optional int NewCorn,
	optional int NewUber,
	optional int NewMetool,
	optional int NewBusta,
	optional int NewFathom,
	optional int NewMatrix)
{

	if (NewInit != 0)
	{
		InitVal = NewInit;
//		log (self@"NEW InitVal --"@InitVal);
	    ConsoleCommand("set "@INIT_PATH@InitVal);
    }

	if (NewCorn != 0)
	{
		CornHole = NewCorn;
//		log (self@"NEW CornHole -- "@CornHole);
	    ConsoleCommand("set "@CORN_PATH@CornHole);
	}

	if (NewUber != 0)
	{
		UberSquire = NewUber;
//		log (self@"NEW UberSquire --"@UberSquire);
	    ConsoleCommand("set "@UBER_PATH@UberSquire);
	}

	if (NewMetool != 0)
	{
		MetoolAlloy = NewMetool;
//		log (self@"NEW Metool --"@MetoolAlloy);
	    ConsoleCommand("set "@METOOL_PATH@MetoolAlloy);
	}

	if (NewBusta != 0)
	{
		BustaRhymes = NewBusta;
//		log (self@"NEW Busta --"@BustaRhymes);
	    ConsoleCommand("set "@BUSTA_PATH@BustaRhymes);
	}

	if (NewFathom != 0)
	{
		UnFathomable = NewFathom;
//		log (self@"NEW UnFathomable --"@UnFathomable);
	    ConsoleCommand("set "@FATHOM_PATH@UnFathomable);
	}

	if (NewMatrix != 0)
	{
		MatrixMan = NewMatrix;
//		log (self@"NEW Matrix --"@MatrixMan);
	    ConsoleCommand("set "@MATRIX_PATH@MatrixMan);
	}
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
function ZeroAllCheats()
{
	InitVal = GetRefVal(GameRefVal, INIT_1, INIT_2, INIT_3);
	SaveVals(InitVal, -1, -1, -1, -1, -1, -1);
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
function EnableAllCheats()
{
	InitVal = GetRefVal(GameRefVal, INIT_1, INIT_2, INIT_3);
	SaveVals(
		InitVal,
		GetRefVal(InitVal, CORN_1, CORN_2, CORN_3),
		GetRefVal(InitVal, UBER_1, UBER_2, UBER_3),
		GetRefVal(InitVal, METOOL_1, METOOL_2, METOOL_3),
		GetRefVal(InitVal, BUSTA_1, BUSTA_2, BUSTA_3),
		GetRefVal(InitVal, FATHOM_1, FATHOM_2, FATHOM_3),
		GetRefVal(InitVal, MATRIX_1, MATRIX_2, MATRIX_3)
	);
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
function EnableMetool()
{
	SaveVals(,,, GetRefVal(InitVal, METOOL_1, METOOL_2, METOOL_3));
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
function EnableBusta()
{
	SaveVals(,,,, GetRefVal(InitVal, BUSTA_1, BUSTA_2, BUSTA_3));
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
function EnableFathom()
{
	SaveVals(,,,,, GetRefVal(InitVal, FATHOM_1, FATHOM_2, FATHOM_3));
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
function EnableMatrix()
{
	SaveVals(,,,,,, GetRefVal(InitVal, MATRIX_1, MATRIX_2, MATRIX_3));
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
function bool SissyEnabled()
{
	local int Ref;
	local int OldVal;

    ref = GetRefVal(InitVal, UBER_1, UBER_2, UBER_3);
//	log(self@"CheckSissy: UberSquire ="@UberSquire@"Must match"@ref);
	if (UberSquire == Ref)
		return true;
	else
	{
		OldVal = Int(ConsoleCommand("get"@UBER_PATH));
//		log(self@"CheckSissy Take 2: OldVal ="@OldVal@"Must match"@ref);
		return (OldVal == Ref);
	}
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
function bool MatrixEnabled()
{
	local int Ref;
	local int OldVal;

    ref = GetRefVal(InitVal, MATRIX_1, MATRIX_2, MATRIX_3);
//	log(self@"CheckMatrix: MatrixMan ="@MatrixMan@"Must match"@ref);
	if (MatrixMan == Ref)
		return true;
	else
	{
		OldVal = Int(ConsoleCommand("get"@MATRIX_PATH));
//		log(self@"CheckMatrix Take 2: OldVal ="@OldVal@"Must match"@ref);
		return (OldVal == Ref);
	}
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
function bool MetoolEnabled()
{
	local int Ref;
	local int OldVal;

    ref = GetRefVal(InitVal, Metool_1, Metool_2, Metool_3);
//	log(self@"CheckMetool: MetoolAlloy ="@MetoolAlloy@"Must match"@ref);
	if (MetoolAlloy == Ref)
		return true;
	else
	{
		OldVal = Int(ConsoleCommand("get"@Metool_PATH));
//		log(self@"CheckMetool Take 2: OldVal ="@OldVal@"Must match"@ref);
		return (OldVal == Ref);
	}
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
function bool BustaEnabled()
{
	local int Ref;
	local int OldVal;

    ref = GetRefVal(InitVal, Busta_1, Busta_2, Busta_3);
//	log(self@"CheckBusta: BustaRhymes ="@BustaRhymes@"Must match"@ref);
	if (BustaRhymes == Ref)
		return true;
	else
	{
		OldVal = Int(ConsoleCommand("get"@Busta_PATH));
//		log(self@"CheckBusta Take 2: OldVal ="@OldVal@"Must match"@ref);
		return (OldVal == Ref);
	}
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
function bool FathomEnabled()
{
	local int Ref;
	local int OldVal;

    ref = GetRefVal(InitVal, Fathom_1, Fathom_2, Fathom_3);
//	log(self@"CheckFathom: UnFathomable ="@UnFathomable@"Must match"@ref);
	if (UnFathomable == Ref)
		return true;
	else
	{
		OldVal = Int(ConsoleCommand("get"@Fathom_PATH));
//		log(self@"CheckFathom Take 2: OldVal ="@OldVal@"Must match"@ref);
		return (OldVal == Ref);
	}
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
function bool AllEnabled()
{
	return (
		FinallyOver() &&
		MatrixEnabled() &&
		MetoolEnabled() &&
		BustaEnabled() &&
		FathomEnabled()
		);
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
function EnableSissy()
{
	UberSquire = GetRefVal(InitVal, UBER_1, UBER_2, UBER_3);
	SaveVals(,,UberSquire);
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
function DisableSissy()
{
	SaveVals(,,-1);
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
function RecordEnding()
{
//	FOver = FOVER_SUM - GameRefVal;
//	TimesBeatenGame++;
//	ConsoleCommand("set "@FOverPath@FOver);
//	ConsoleCommand("set "@TimesBeatenGamePath@TimesBeatenGame);
	Super.RecordEnding();

	CornHole = GetRefVal(InitVal, CORN_1, CORN_2, CORN_3);
	SaveVals(,CornHole);
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
function bool FinallyOver()
{
	local int Ref;
	local int OldVal;

    ref = GetRefVal(InitVal, Corn_1, Corn_2, Corn_3);
//	log(self@"CheckCorn: CornHole ="@CornHole@"Must match"@ref);
	if (CornHole == Ref)
		return true;
	else
	{
		OldVal = Int(ConsoleCommand("get"@Corn_PATH));
//		log(self@"CheckFOver Take 2: OldVal ="@OldVal@"Must match"@ref);
		return (OldVal == Ref);
	}
}

///////////////////////////////////////////////////////////////////////////////
// Check last code
///////////////////////////////////////////////////////////////////////////////
function bool VerifySeqTime(optional bool bUpdate)
{
	local bool bEStartCheck;
	local bool bverified;

	if(bUpdate)
		InfoSeqTime = int(ConsoleCommand("get "@InfoSeqPath));

	if (TheGameState.bEGameStart && FinallyOver())
		return true;
	else
		return false;
}

///////////////////////////////////////////////////////////////////////////////
// Reach into entry and put a new reference to every chameleon texture/meshes in this
// level.
///////////////////////////////////////////////////////////////////////////////
function StoreChams()
{
	local LevelInfo lev;
	local TextureLoader texl;
	local MeshLoader mesher;
	local Actor checkme;
	local PersonPawn checkpawn;

	lev = GetPlayer().GetEntryLevel();

//	log(self$" StoreChams "$lev);

	foreach lev.AllActors(class'Actor', checkme)
	{
		if(TextureLoader(checkme) != None)
			texl = TextureLoader(checkme);
		else if(MeshLoader(checkme) != None)
			mesher = MeshLoader(checkme);
	}

//	log(self$" checking texture loader "$mesher$" mesh loader "$mesher);
	if(texl != None
		&& mesher != None)
	{
		// Clear the lists
		texl.ClearArray();
		mesher.ClearArray();
		// Now go through each pawn and put their textures/meshes into the list
		foreach DynamicActors(class'PersonPawn', checkpawn)
		{
			texl.AddTexture(checkpawn.Skins[0]);
			mesher.AddMesh(checkpawn.Mesh);
			if(checkpawn.MyHead != None)
			{
				texl.AddTexture(checkpawn.MyHead.Skins[0]);
				mesher.AddMesh(checkpawn.MyHead.Mesh);
			}
		}
	}
}

///////////////////////////////////////////////////////////////////////////////
// Check to rain cats during the apocalypse
///////////////////////////////////////////////////////////////////////////////
function CheckCatRain(float DeltaTime)
{
	local P2Player p2p;
//	local CatRocket catr;
	local AWCatPawn catr;
	local vector dir, StartLoc;
	local float tempf;

	CatRainTime -= DeltaTime;

	// Check to toggle rain
	if(CatRainTime <= 0)
	{
		bRainingCats = !bRainingCats;
		if(bRainingCats)
			CatRainTime = RAIN_BASE_TIME + Rand(RAIN_RAND_TIME);
		else
			CatRainTime = Rand(RAIN_BASE_TIME);
	}

	if(bRainingCats)
	{
		NewCatTime -= DeltaTime;
		if(NewCatTime <= 0)
		{
			p2p = GetPlayer();
			if(p2p != None
				&& p2p.MyPawn != None)
			{
				NewCatTime = NEW_CAT_BASE_TIME + Rand(NEW_CAT_RAND_TIME);
				// Make it generally in front of the player
				// Move it in front of him
				StartLoc = vector(p2p.MyPawn.Rotation);
				StartLoc.z = 0;
				StartLoc = (FRand()*CAT_MAKE_RANGE_DIST + CAT_MAKE_BASE_DIST) * StartLoc;
				// Vaguely center the rain if he's dead
				if(p2p.MyPawn.Health <= 0)
				{
					tempf = 0.5*(CAT_MAKE_RANGE_DIST + CAT_MAKE_BASE_DIST);
					StartLoc.x += tempf;
					StartLoc.y += tempf;
				}
				// Now create a range around that point
				tempf = (0.5*CAT_MAKE_RANGE_XY);
				StartLoc.x = StartLoc.x + (FRand()*CAT_MAKE_RANGE_XY - tempf);
				StartLoc.y = StartLoc.y + (FRand()*CAT_MAKE_RANGE_XY - tempf);
				// Move it above him
				StartLoc.z += CAT_MAKE_Z;
				StartLoc = StartLoc + p2p.MyPawn.Location;
				dir = VRand();
				dir.z=-1;
//				catr = spawn(class'CatRocket',,,StartLoc,Rotator(dir));
				catr = spawn(ApocalypseCatClass,,, StartLoc, Rotator(Dir));
				if(catr != None)
				{
					AddController(CatR);
					catr.AmbientGlow=255;	// make them pulse so they're easier to see.
					// modify speed for a gentle rain
					Dir = vector(catr.Rotation);
					catr.Velocity = CAT_RAIN_SPEED * Dir;
					catr.Acceleration.z = CAT_RAIN_ACC;
				}
			}
		}
	}
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// Monitor things in the game, and rain cats
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
state RunningApocalypse
{
	function BeginState()
	{
		Super.BeginState();
		// Init new cat time, the first time the level starts
		NewCatTime = NEW_CAT_BASE_TIME + Rand(NEW_CAT_RAND_TIME);
	}
}

///////////////////////////////////////////////////////////////////////////////
// Copied from AWGameSPFinal, since we extend from AWGameSP and not
// AWGameSPFinal
///////////////////////////////////////////////////////////////////////////////
function bool PickupQuery( Pawn Other, Pickup item )
{
	if(P2WeaponPickup(item) == None
		|| AWDude(Other) == None
		|| AWDude(Other).GaryHeads <= 0
		|| AWDudePlayer(AWDude(Other).Controller).bNeverSwitchOnPickup)	// This variable gets stored and
		// switched to true on creation of a gary head, but if the user goes into the
		// options and switches it manually back and forth, it will come undone. We don't
		// want them doing this, so if they do, don't let them pick up the gun at all.
		// It'll all be back to normal when the heads are gone anyway.
		return Super.PickupQuery(Other, item);
	else
		return false;
}

///////////////////////////////////////////////////////////////////////////////
// Keep track of time elapsed for speed runs and whatnot
///////////////////////////////////////////////////////////////////////////////
event Tick(float Delta)
{
	local P2Player p2p;

	Super.Tick(Delta);

	p2p = GetPlayer();

	if(p2p != None
		&& p2p.Pawn != None
		&& p2p.Pawn.Health > 0
		&& p2p.Level.Pauser == None
		&& TheGameState != None)
		EAGameState(TheGameState).TimeElapsed += Delta;
}

///////////////////////////////////////////////////////////////////////////////
// Player calls this when it's ready for a save.  It will call this repeatedly
// until it returns true, which means we agree.
///////////////////////////////////////////////////////////////////////////////
function bool ReadyForSave(P2Player player)
	{
	// Set the all-critical flag, then make sure all the other conditions
	// are right.  If it works, then we're good to go.  Otherwise, we clear
	// the flag and wait for the next call.
	bSafeToSave = true;
	if (IsSaveAllowed(player))
		{
		// If an auto save is desired, now is the perfect time to do it
		if (bDoAutoSave)
			{
			// Clear flags before saving so that loaded games don't re-save
			bDoAutoSave = false;
			bForcedAutoSave = false;

			SaveGame(AutoSaveSlot, true);
			}
		}
	else
		bSafeToSave = false;

	return bSafeToSave;
	}

///////////////////////////////////////////////////////////////////////////////
// Save game to the specified slot
///////////////////////////////////////////////////////////////////////////////
function SaveGame(int Slot, bool bShowMessage)
	{
	Log(self$" SaveGame(): Slot="$Slot$" bAutoSave="$bDoAutoSave$" bForcedAutoSave="$bForcedAutoSave$" the game state difficulty "$TheGameState.GameDifficulty);

	if (bShowMessage)
		{
		if (Slot == QUICKSAVE_SLOT)
			GetPlayer().ClientMessage(EasySaveMessageString);
		else if (Slot == AUTOSAVE_SLOT && bForcedAutoSave)
			GetPlayer().ClientMessage(ForcedSaveMessageString);
		else if (Slot == AUTOSAVE_SLOT)
			GetPlayer().ClientMessage(AutoSaveMessageString);
		else
			GetPlayer().ClientMessage(NormalSaveMessageString);
		}

	// This slot is now the most recently used slot.  We update this BEFORE the
	// actual save so that if this game is loaded, it will refer to itself as
	// the most recently used.  In other words, whenever a game is loaded, the
	// slot it loaded from becomes the most recently used slot.
	MostRecentGameSlot = Slot;

	// Do the actual save
	ConsoleCommand("SaveGame "$Slot);

	// Save info about this slot
	MySlotInfoMgr.SetInfo(
		Slot,
		GetCurrentDayBase().Description@"-"@Level.Title@"-"@GetDiffName(TheGameState.GameDifficulty)@"-"@GameNameShort,
		GetCurrentDay(),
		Level.Year$"-"$Val2Str(level.Month, 2)$"-"$Val2Str(Level.Day, 2)@":"@Val2Str(Level.Hour, 2)$":"$Val2Str(Level.Minute, 2),
		int(ConsoleCommand("GETGMTIME")));
	}

///////////////////////////////////////////////////////////////////////////////
// Processes "red turtle day" selected at random.
// Actors who have 'RedTurtleDay' in the group will be excluded on all days
// except for the selected "red turtle day".
///////////////////////////////////////////////////////////////////////////////
function NeededForThisDay(Actor CheckA, out byte Needed, out byte SpecifiedDay)
{
	local int RTDay;
	local string GroupString;

	// Call super first. "Red turtle" actors might specify certain days as well,
	// making them even harder to find.
	// Super will return whether we're needed by default (w/o red turtle check)
	Super.NeededForThisDay(CheckA, Needed, SpecifiedDay);

	// Figure out what day the red turtle can be seen.
	RTDay = EAGameState(TheGameState).RedTurtleDay;
	GroupString = Caps(CheckA.Group);

	// If the actor is a "red turtle" actor then only show it on red turtle days.
	if (InStr(GroupString, REDTURTLE_DAYNAME) >= 0)
	{
		SpecifiedDay = 1;
		if (TheGameState.CurrentDay == RTDay && Needed == 1)
			Needed = 1;
		else
			Needed = 0;
	}
}


///////////////////////////////////////////////////////////////////////////////
// Default properties
///////////////////////////////////////////////////////////////////////////////

defaultproperties
{
     InitVal=124
     bNewWeaponOrder=True
     bDrawTime=True
     GameNameShort="7-day"
     WeekendStartURL="EA_WeekendIntro.fuk"
     ApocalypseCatClass=Class'EAPawns.DervishCatR'
     ZombieHeadClass=Class'EDPawns.EDZombieHead'
     EAMutatorClassName="EAGame.EAMutator"
     FireCorona=Texture'PB_textures.Special.PB_flare_1'
     GameStateClass=Class'EAFX.EAGameState'
     Begin Object Class=DayBase Name=DayBase107
         Description="Monday"
         UniqueName="7DAYA"
         ExcludeDays(0)="DEMO"
         MapTex="p2misc_full.map_day1"
         NewsTex="p2misc.newspaper_day_1"
         DudeNewsComment="DudeDialog.dude_news_monday"
         LoadTex="p2misc_full.loading1"
         DudeStartComment="DudeDialog.dude_map_exit1"
         Errands(0)=ErrandBase'EAGame.ErrandBase0'
         Errands(1)=ErrandBase'EAGame.ErrandBase1'
         Errands(2)=ErrandBase'EAGame.ErrandBase2'
         Errands(3)=ErrandBase'EAGame.ErrandBase_DeliverMotherboard'
         PlayerInvList(0)=(InvClassName="Inventory.MoneyInv",NeededAmount=20)
         PlayerInvList(1)=(InvClassName="Inventory.MapInv")
         PlayerInvList(2)=(InvClassName="Inventory.HandsWeapon")
         PlayerInvList(3)=(InvClassName="Bat.FistsWeapon")
         PlayerInvList(4)=(InvClassName="EAInventory.KarmaGun")
         TakeFromPlayerList(0)=(InvClassName="Inventory.NewspaperInv")
         TakeFromPlayerList(1)=(InvClassName="Inventory.GimpClothesInv")
         TakeFromPlayerList(2)=(InvClassName="Inventory.DudeClothesInv")
         TakeFromPlayerList(3)=(InvClassName="Inventory.MilkInv")
         TakeFromPlayerList(4)=(InvClassName="Inventory.PaycheckInv")
         TakeFromPlayerList(5)=(InvClassName="EAInventory.MotherboardInv")
         Name="DayBase107"
     End Object
     Days(0)=DayBase'EAGame.DayBase107'
     Begin Object Class=DayBase Name=DayBase108
         Description="Tuesday"
         UniqueName="7DAYB"
         ExcludeDays(0)="DEMO"
         MapTex="p2misc_full.map_day2"
         NewsTex="p2misc_full.newspaper_day_2"
         DudeNewsComment="DudeDialog.dude_news_tuesday"
         LoadTex="p2misc_full.loading2"
         DudeStartComment="DudeDialog.dude_map_exit2"
         Errands(0)=ErrandBase'EAGame.ErrandBase3'
         Errands(1)=ErrandBase'EAGame.ErrandBase4'
         Errands(2)=ErrandBase'EAGame.ErrandBase5'
         Errands(3)=ErrandBase'EAGame.ErrandBase6'
         PlayerInvList(0)=(InvClassName="Inventory.LibraryBookInv")
         PlayerInvList(1)=(InvClassName="Inventory.ClipboardWeapon")
         TakeFromPlayerList(0)=(InvClassName="Inventory.NewspaperInv")
         TakeFromPlayerList(1)=(InvClassName="Inventory.GimpClothesInv")
         TakeFromPlayerList(2)=(InvClassName="Inventory.DudeClothesInv")
         TakeFromPlayerList(3)=(InvClassName="Inventory.ClipboardWeapon")
         TakeFromPlayerList(4)=(InvClassName="Inventory.LibraryBookInv")
         Name="DayBase108"
     End Object
     Days(1)=DayBase'EAGame.DayBase108'
     Begin Object Class=DayBase Name=DayBase2
         Description="Wednesday"
         UniqueName="7DAYC"
         ExcludeDays(0)="DEMO"
         MapTex="p2misc_full.map_day3"
         NewsTex="p2misc_full.newspaper_day_3"
         DudeNewsComment="DudeDialog.dude_news_wednesday"
         LoadTex="p2misc_full.loading3"
         DudeStartComment="DudeDialog.dude_map_exit3"
         Errands(0)=ErrandBase'EAGame.ErrandBase7'
         Errands(1)=ErrandBase'EAGame.ErrandBase8'
         Errands(2)=ErrandBase'EAGame.ErrandBase9'
         Errands(3)=ErrandBase'EAGame.ErrandBase10'
         TakeFromPlayerList(0)=(InvClassName="Inventory.NewspaperInv")
         TakeFromPlayerList(1)=(InvClassName="Inventory.GimpClothesInv")
         TakeFromPlayerList(2)=(InvClassName="Inventory.DudeClothesInv")
         TakeFromPlayerList(3)=(InvClassName="Inventory.TreeInv")
         Name="DayBase2"
     End Object
     Days(2)=DayBase'EAGame.DayBase2'
     Begin Object Class=DayBase Name=DayBase3
         Description="Thursday"
         UniqueName="7DAYD"
         ExcludeDays(0)="DEMO"
         MapTex="p2misc_full.map_day4"
         NewsTex="p2misc_full.newspaper_day_4"
         DudeNewsComment="DudeDialog.dude_news_thursday"
         LoadTex="p2misc_full.loading4"
         DudeStartComment="DudeDialog.dude_map_exit4"
         Errands(0)=ErrandBase'EAGame.ErrandBase11'
         Errands(1)=ErrandBase'EAGame.ErrandBase12'
         Errands(2)=ErrandBase'EAGame.ErrandBase13'
         Errands(3)=ErrandBase'EAGame.ErrandBase14'
         PlayerInvList(0)=(InvClassName="Inventory.CitationInv")
         TakeFromPlayerList(0)=(InvClassName="Inventory.NewspaperInv")
         TakeFromPlayerList(1)=(InvClassName="Inventory.GimpClothesInv")
         TakeFromPlayerList(2)=(InvClassName="Inventory.DudeClothesInv")
         TakeFromPlayerList(3)=(InvClassName="Inventory.KrotchyInv")
         TakeFromPlayerList(4)=(InvClassName="Inventory.SteakInv")
         TakeFromPlayerList(5)=(InvClassName="Inventory.CitationInv")
         Name="DayBase3"
     End Object
     Days(3)=DayBase'EAGame.DayBase3'
     Begin Object Class=DayBase Name=DayBase4
         Description="Friday"
         UniqueName="7DAYE"
         ExcludeDays(0)="DEMO"
         MapTex="p2misc_full.map_day5"
         NewsTex="p2misc_full.newspaper_day_5"
         DudeNewsComment="DudeDialog.dude_news_friday"
         LoadTex="p2misc_full.loading5"
         DudeStartComment="DudeDialog.dude_map_exit5"
         Errands(0)=ErrandBase'EAGame.ErrandBase15'
         Errands(1)=ErrandBase'EAGame.ErrandBase16'
         Errands(2)=ErrandBase'EAGame.ErrandBase17'
         Errands(3)=ErrandBase'EAGame.ErrandBase18'
         PlayerInvList(0)=(InvClassName="Inventory.GiftInv")
         PlayerInvList(1)=(InvClassName="Inventory.GonorrheaAmmoInv")
         TakeFromPlayerList(0)=(InvClassName="Inventory.NewspaperInv")
         TakeFromPlayerList(1)=(InvClassName="Inventory.GimpClothesInv")
         TakeFromPlayerList(2)=(InvClassName="Inventory.DudeClothesInv")
         TakeFromPlayerList(3)=(InvClassName="Inventory.ParcelInv")
         TakeFromPlayerList(4)=(InvClassName="Inventory.AlternatorInv")
         TakeFromPlayerList(5)=(InvClassName="Inventory.PillsInv")
         TakeFromPlayerList(6)=(InvClassName="Inventory.GonorrheaAmmoInv")
         TakeFromPlayerList(7)=(InvClassName="Inventory.MapInv")
         TakeFromPlayerList(8)=(InvClassName="Inventory.GiftInv")
         TakeFromPlayerList(9)=(InvClassName="Inventory.GaryBookInv")
         Name="DayBase4"
     End Object
     Days(4)=DayBase'EAGame.DayBase4'
     Begin Object Class=DayBase Name=DayBase8
         Description="Saturday"
         UniqueName="DAY_A"
         ExcludeDays(0)="DEMO"
         LoadTex="aw_textures.loading_sat"
         DudeStartComment="DudeDialog.dude_map_exit1"
         Errands(0)=ErrandBase'AWGame.ErrandBase0'
         TakeFromPlayerList(0)=(InvClassName="Inventory.NewspaperInv")
         TakeFromPlayerList(1)=(InvClassName="Inventory.GimpClothesInv")
         TakeFromPlayerList(2)=(InvClassName="Inventory.DudeClothesInv")
         TakeFromPlayerList(3)=(InvClassName="Inventory.MapInv")
         Name="DayBase8"
     End Object
     Days(5)=DayBase'EAGame.DayBase8'
     Begin Object Class=DayBase Name=DayBase9
         Description="Sunday"
         UniqueName="DAY_B"
         ExcludeDays(0)="DEMO"
         LoadTex="aw_textures.loading_sun"
         DudeStartComment="DudeDialog.dude_map_exit1"
         Errands(0)=ErrandBase'AWGame.ErrandBase1'
         TakeFromPlayerList(0)=(InvClassName="Inventory.NewspaperInv")
         TakeFromPlayerList(1)=(InvClassName="Inventory.GimpClothesInv")
         TakeFromPlayerList(2)=(InvClassName="Inventory.DudeClothesInv")
         TakeFromPlayerList(3)=(InvClassName="Inventory.MapInv")
         Name="DayBase9"
     End Object
     Days(6)=DayBase'EAGame.DayBase9'
     IntroURL="EA_Intro.fuk"
     StartFirstDayURL="EA_Suburbs-3.fuk"
     StartNextDayURL="EA_Suburbs-3.fuk"
     FinishedDayURL="EA_HomeAtNight.fuk"
     JailURL="EA_Police.fuk#cell"
     ApocalypseTex="EATex.Misc.ApocalypseNewspaper"
     MainMenuURL="EA_Startup.fuk"
     ChameleonClass=Class'EAGame.EAChameleon'
     MatSwaps(0)=(OrigMat=Texture'Josh-textures.signs.game_banner_2',NewMat=Texture'Josh-textures.signs.game_banner_5')
     MatSwaps(1)=(OrigMat=Texture'Timb.arcade.Game_FagHunter',NewMat=Texture'Timb.arcade.Game_BastardFish')
     DefaultPlayerClassName="EAPawns.EAPostalDude"
//     HUDType="AW7Game.AW7HUD"
     HUDType="ED.EDHud"
     GameName="Eternal Apocalypse 5Day Game"
     PlayerControllerClassName="EAGame.EADudePlayer"
}
