///////////////////////////////////////////////////////////////////////////////
// NineProjectile.
// Copyright 2002 Running With Scissors, Inc.  All Rights Reserved.
//=============================================================
//   NineProjectile
//   Eternal Damnation
//   madjackal
//=============================================================
class AE50Projectile extends P2Projectile;

const FLYBY_MAX	=	7;

var Sound FlyBy[FLYBY_MAX];
var float FlyByRad;


const FLESH_HIT_MAX	=	8;
const WAIT_TIME = 0.05;
const MIN_SIZE = 10.0;

var class<TimedMarker> PawnHitMarkerMade;	// danger made when bullet hits a pawn 
var class<TimedMarker> BulletHitMarkerMade;	// danger made when bullet hits dirt
var Sound FleshHit[FLESH_HIT_MAX];
var float FleshRad;
var vector Direction;
//var vector OldLocation;

const FORCE_RAD_CHECK	=	50;			// Distance to the wall to check for attaching
										// the rocket force to a wall (instead of having the force
										// applied at the explosion epicenter--this is to make sure
										// things get thrown up and away more often.


///////////////////////////////////////////////////////////////////////////////
// Send it off in the right direction
///////////////////////////////////////////////////////////////////////////////
function SetVelocity(vector Dir)
{
	Direction = Dir;
	Velocity = Speed*Direction;
}

///////////////////////////////////////////////////////////////////////////////
// Blow up and generate effects (only in enhanced mode for the player)
///////////////////////////////////////////////////////////////////////////////
simulated function GenExplosion(vector HitLocation, vector HitNormal, Actor Other)
{
	local RifleExplosion exp;
	local vector WallHitPoint, OrigLoc;
	local Actor ViewThing;

	if(Other != None
		&& Other.bStatic)
	{
		// Make sure the force of this explosion is all the way against the wall that
		// we hit
		OrigLoc = HitLocation;
		WallHitPoint = HitLocation - FORCE_RAD_CHECK*HitNormal;
		if(Trace(HitLocation, HitNormal, WallHitPoint, HitLocation) == None)
		{
			HitLocation = OrigLoc;
			WallHitPoint = OrigLoc;
		}
	}
	else
		WallHitPoint = HitLocation;

	exp = spawn(class'RifleExplosion',GetMaker(),,HitLocation + ExploWallOut*HitNormal);
	exp.CheckForHitType(Other);
	exp.ShakeCamera(exp.ExplosionDamage);
	exp.ForceLocation = WallHitPoint;

 	Destroy();
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// Flying through the air
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
auto state Moving
{
	ignores HitWall, Explode, Blowup, ProcessTouch, Touch, EncroachingOn;

	///////////////////////////////////////////////////////////////////////////////
	// Make people notice a body dropping limply before them from a sniper hit
	///////////////////////////////////////////////////////////////////////////////

	function TellNPCs(Actor Other, vector HitLocation)
	{
	  
  	    PlaySound(FlyBy[Rand(ArrayCount(FlyBy))],SLOT_Pain,,,FlyByRad,GetRandPitch());

		if(Pawn(Other) != None)
		{
			// Set your enemy as the one you attacked.
			if(P2Player(Instigator.Controller) != None
				&& FPSPawn(Other) != None)
			{
				P2Player(Instigator.Controller).Enemy = FPSPawn(Other);
			}

			PawnHitMarkerMade.static.NotifyControllersStatic(
				Level,
				PawnHitMarkerMade,
				FPSPawn(Owner), 
				FPSPawn(Other), 
				PawnHitMarkerMade.default.CollisionRadius,
				HitLocation);
		}
		else	// wall hit.. still have them notice
		{
			BulletHitMarkerMade.static.NotifyControllersStatic(
				Level,
				BulletHitMarkerMade,
				FPSPawn(Owner), 
				None, 
				BulletHitMarkerMade.default.CollisionRadius,
				HitLocation);
		}
	}

	///////////////////////////////////////////////////////////////////////////////
	// False means you hit something you can't go past so die.
	///////////////////////////////////////////////////////////////////////////////
	function bool ProcessTraceHit(Actor Other, out vector HitLocation, Vector HitNormal)
	{
		local Rotator NewRot;
		local float usesize;

		if ( Other == None )
		{
			return true;
		}

		if(P2GameInfoSingle(Level.Game) != None
			&& P2GameInfoSingle(Level.Game).VerifySeqTime()
			&& P2Pawn(Instigator) != None
			&& P2Pawn(Instigator).bPlayer)
		{
			GenExplosion(HitLocation, HitNormal, Other);
			return false;
		}

		TellNPCs(Other, HitLocation);

		if(Other.bStatic)
		{
			spawn(class'EDGenericHitPack',Owner, ,HitLocation, Rotator(HitNormal));
			spawn(class'MaDBulletHitPack',Owner, ,HitLocation, Rotator(HitNormal));
			return false;
		}
		else 
		{
			Other.TakeDamage(class'AE50Ammunition'.default.DamageAmount, Pawn(Owner),
							HitLocation, Velocity, class'AE50Ammunition'.default.DamageTypeInflicted);

			// If we're a guy (krotchy) who doesn't take head shots at all, stop the bullet completely)
			if(P2Pawn(Other) != None
				&& P2Pawn(Other).TakesRifleHeadShot == 0)
			{
				spawn(class'EDGenericHitPack',Owner, ,HitLocation, Rotator(HitNormal));
				spawn(class'MaDBulletHitPack',Owner, ,HitLocation, Rotator(HitNormal));
				return false;
			}
			else if((Pawn(Other) != None 
					&& Other != Owner)
				|| PeoplePart(Other) != None
				|| CowheadProjectile(Other) != None)
			{
				Other.PlaySound(FleshHit[Rand(ArrayCount(FleshHit))],SLOT_Pain,,,FleshRad,GetRandPitch());
				spawn(class'EDSplatterPack',Owner, ,HitLocation,Rotator(HitNormal));
			}
			else if(Other.bBlockActors)
			{
				spawn(class'EDGenericHitPack',Owner, ,HitLocation, Rotator(HitNormal));
				spawn(class'MaDBulletHitPack',Owner, ,HitLocation, Rotator(HitNormal));
			}
			// Move to the other side of the thing you hit
			usesize = Other.CollisionRadius + Other.CollisionHeight;
			if(usesize < MIN_SIZE)
				usesize = MIN_SIZE;
			HitLocation += Direction*(usesize);
		}
		return true;
	}
	///////////////////////////////////////////////////////////////////////////////
	// Moves along next path and checks everything for hit damage. Goes through
	// pawns and windows, etc, till it hits static things.
	///////////////////////////////////////////////////////////////////////////////
	function TraceFire()
	{

		local vector HitLocation, HitNormal, StartTrace, EndTrace;
		local actor Other;
		
		//if(!SetLocation(OldLocation))
		//	Destroy();
		// Do collision test
		StartTrace = Location;
		EndTrace = StartTrace + Velocity*WAIT_TIME;

		Other = Trace(HitLocation,HitNormal,EndTrace,StartTrace,true);

		if ( Other == None )
			HitLocation = EndTrace;

		if(ProcessTraceHit(Other, HitLocation, HitNormal))
		{
			if(!SetLocation(HitLocation))
				Destroy();
			//else
			//{
			//	OldLocation = HitLocation;
			//}
		}
		else
			Destroy();
   
	}
	
	//function BeginState()
	//{
	//	OldLocation = Location;
	//}

Begin:
	TraceFire();
	Sleep(WAIT_TIME);
	Goto('Begin');
}

defaultproperties
{
     FlyBy(0)=Sound'MaD_EDSounds.Bfly_00'
     FlyBy(1)=Sound'MaD_EDSounds.Bfly_01'
     FlyBy(2)=Sound'MaD_EDSounds.Bfly_02'
     FlyBy(3)=Sound'MaD_EDSounds.Bfly_03'
     FlyBy(4)=Sound'MaD_EDSounds.Bfly_04'
     FlyBy(5)=Sound'MaD_EDSounds.Bfly_05'
     FlyBy(6)=Sound'MaD_EDSounds.Bfly_06'
     PawnHitMarkerMade=Class'Postal2Game.PawnShotMarker'
     BulletHitMarkerMade=Class'Postal2Game.BulletHitMarker'
     FleshHit(0)=Sound'ed_moresounds.Body.hit1'
     FleshHit(1)=Sound'ed_moresounds.Body.hit2'
     FleshHit(2)=Sound'ed_moresounds.Body.hit3'
     FleshHit(3)=Sound'ed_moresounds.Body.hit4'
     FleshHit(4)=Sound'WeaponSounds.bullet_hitflesh1'
     FleshHit(5)=Sound'WeaponSounds.bullet_hitflesh2'
     FleshHit(6)=Sound'WeaponSounds.bullet_hitflesh3'
     FleshHit(7)=Sound'WeaponSounds.bullet_hitflesh4'
     FleshRad=200.000000
     speed=10000.000000
     MaxSpeed=50000.000000
     Damage=100.000000
     MomentumTransfer=500000.000000
     DamageTypeInflicted=Class'MaD_EDDamage.BerettaDamage'
     MyDamageType=Class'MaD_EDDamage.BerettaDamage'
     bNetTemporary=False
     bUpdateSimulatedPosition=True
     StaticMesh=StaticMesh'MaD_EDMeshes.9mmBulletMesh'
     DrawType=DT_StaticMesh
     LifeSpan=4.000000
     AmbientGlow=0
     TransientSoundRadius=400.000000
     bCollideActors=False
     bCollideWorld=False
     bBlockZeroExtentTraces=False
     bBlockNonZeroExtentTraces=False
     bFixedRotationDir=True
     ForceType=FT_DragAlong
     ForceRadius=2.000000
     ForceScale=1.000000
}
