class AWPKatanaProjectile extends ScytheProjectile;

var class<P2WeaponPickup> AltPickupClass;

var array<Name> BoneList;

///////////////////////////////////////////////////////////////////////////////
// Attach the blurry effect
///////////////////////////////////////////////////////////////////////////////
simulated function PostBeginPlay()
{
	Super.PostBeginPlay();

	// If a cheat is used, don't make a pickup
	if(Instigator != None
		&& AWPKatanaWeapon(Instigator.Weapon) != None
		&& (AWPKatanaWeapon(Instigator.Weapon).bNinja || P2GameInfoSingle(Level.Game).VerifySeqTime()))
		bNoPickup=true;
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
simulated function AWPKatanaStuckInWall(Actor Wall, Vector HitNormal)
{
	local P2WeaponPickup newmac;
	const WALL_DIST = 5;

	if(!bNoPickup)
	{
		newmac = spawn(PickupClass, Owner,, Location, Rotation);
		// Turn into a pickup in that orientation
		if(newmac != None)
		{
			newmac.SetStaticMesh(StaticMesh);
			newmac.bRecordAfterPickup=false;
			//newmac.SetLocation(Location-(1*newmac.CollisionRadius*HitNormal));
			newmac.SetLocation(Location-(WALL_DIST*HitNormal));
			newmac.GotoState('Pickup');
		}
	}
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
simulated function AWPKatanaStuckInPawn(Actor Other, Vector HitLocation)
{
	local AWPKatanaPickupStabbed newmac;
	local P2MocapPawn Victim;
	local name attachb;
	local AWPStabbedKatana stabbed;
	local rotator newrot;
	local int StabBone;
	local coords UseCoords;
	local int i;
	local float BoneDist, ClosestDist;
	local name ClosestBone;

	const STAB_TIME = 'MALE01 spine2';

	if (P2MocapPawn(Other) == None)
	{
		MakePickup();
		return;
	}

	// tell the game info
	if (Instigator.Controller.bIsPlayer)
		EAGameState(P2GameinfoSingle(Level.Game).TheGameState).KatanaImpaleCount++;

	Victim = P2MocapPawn(Other);
	stabbed = spawn(class'AWPStabbedKatana', Owner,, HitLocation);
	if (stabbed != None)
	{
		ClosestDist=9999;
		for (i=0; i<BoneList.Length; i++)
		{
			UseCoords = victim.GetBoneCoords(BoneList[i]);
			BoneDist = VSize(UseCoords.Origin - HitLocation);
			if (BoneDist < ClosestDist)
			{
				ClosestDist = BoneDist;
				ClosestBone = BoneList[i];
			}
		}
		
		if (ClosestDist == 9999)
			attachb = STAB_TIME;
		else
			attachb = ClosestBone;

		Victim.AttachToBone(stabbed, attachb);
		newrot.Yaw = Rotation.Yaw - Other.Rotation.Yaw;
		stabbed.SetRelativeRotation(newrot);
	}

	if(!bNoPickup)
	{
		newmac = spawn(class'AWPKatanaPickupStabbed', Owner,,HitLocation);
		// Turn into a pickup in that orientation
		if(newmac != None && stabbed != None)
		{
			newmac.bRecordAfterPickup=false;
			newmac.victim = victim;
			newmac.stabbed = stabbed;
			newmac.GotoState('Pickup');
		}
		else
		{
			warn("either newmac or stabbed was none");
			newmac.Destroy();
			stabbed.Destroy();
		}
	}
}

///////////////////////////////////////////////////////////////////////////////
// Bounces off walls
///////////////////////////////////////////////////////////////////////////////
simulated function HitWall( vector HitNormal, actor Wall )
{
	local rotator userot;
	local SparkHit spark1;
	local SmokeHitPuffMelee smoke1;
	local ScissorsPickup sp;
	local bool bDoBounce;
	local vector mydir;
	local float hitdot;

	mydir = Normal(Velocity);
	hitdot = mydir dot HitNormal;
	// Die on doors, so they don't get stuck as much (only in MP games)
	if((Level.Game == None
			|| !FPSGameInfo(Level.Game).bIsSinglePlayer)
		&& DoorMover(Wall) != None)
	{
		if(Role == ROLE_Authority)
		{
			// Throw off some sparks from the stick
			spark1 = spawn(class'Fx.SparkHitProjectile',Owner,,Location,rotator(HitNormal));

			spark1.PlaySound(WallHitSound,,1.0,false,200.0,GetRandPitch());
		}
		MakePickup();
		Destroy();
	}
	else
	{
		// Hit hits a floor obliquely, then ricochet off and keep flying, otherwise, fall to ground
//		if(abs(HitNormal.z) < FLOOR_Z
//			&& abs(hitdot) < BOUNCE_WALL_DOT)
//			PerformBounce(HitNormal);
//		else
//		{
			if(Role == ROLE_Authority)
			{
//				MakePickup();
				AWPKatanaStuckInWall(Wall, HitNormal);

				// Throw off some sparks from the stick
				spark1 = spawn(class'Fx.SparkHitProjectile',Owner,,Location,rotator(HitNormal));

				spark1.PlaySound(WallHitSound,,1.0,false,200.0,GetRandPitch());

				smoke1 = Spawn(class'Fx.SmokeHitPuffMelee',Owner,,Location,rotator(HitNormal));
			}
			else
				PlaySound(WallHitSound,,1.0,false,200.0,GetRandPitch());

			// Destroy ourselves after we hit something, whether we made a pickup or not
			Destroy();
//		}
	}
}

///////////////////////////////////////////////////////////////////////////////
// Doesn't hurt anyone related to us, or ourselves, but will hurt others.
///////////////////////////////////////////////////////////////////////////////
simulated function ProcessTouch(Actor Other, Vector HitLocation)
{
	local name attachb;
	local AnimNotifyActor sp;
	local coords checkcoords;
	local P2Pawn hitpawn;
	local Rotator rot;
	local byte BlockedHit;
	local SparkHit spark1;
	local SmokeHitPuffMelee smoke1;
	local byte StateChange;

	if ( !RelatedToMe(Pawn(Other)) )
	{
		// AW dogs have special powers. Check to see if they hit it and want to do anything special with it
		if(InvDogPawn(Other) != None)
		{
			InvDogPawn(Other).CheckCatchProjectile(self, PickupClass, StateChange);
			// The dog caught it! We're done
			if(StateChange == 1)
			{
				Destroy();
				return;
			}
		}
		// If the dog didn't catch us, continue onward (and hurt the dog)
		if(StateChange == 0)
		{
			// Don't bounce off windows, break them
			if(Window(Other) != None)
				Other.Bump(self);
			// Don't bounce off things, but do fall afterwards, people, parts, bombs
			else if(PeoplePart(Other) != None
				|| Projectile(Other) != None
				|| Pawn(Other) != None)
			{
				// Make sure we're not hitting the last thing we've already hit, or something
				// owned by the last thing
				if(LastPawnHit == None
					|| (Other != LastPawnHit
						&& Other.Owner != LastPawnHit))
				{
					// Balance damage differently for MP vs SP
					if(Level.NetMode == NM_Standalone)
					{
						// People can possibly block a thrown projectile
						if(InvPerson(Other) == None)
							BlockedHit=0;
						else
							InvPerson(Other).CheckBlockMelee(Location, BlockedHit);

						if(BlockedHit == 0)
						{
							Other.TakeDamage( Damage, instigator, Location, MomentumTransfer * Normal(Velocity), MyDamageType);
							smoke1 = spawn(class'SmokeHitPuffMelee',Owner,,Location);
							if(smoke1 != None)
								smoke1.PlaySound(ScytheHitBody,,1.0,false,200.0,GetRandPitch());

							AWPKatanaStuckInPawn(Other, HitLocation);
							Destroy();
							return;
						}
						else // ricochet off the guy that is blocking it
						{
							Velocity = -Velocity;
							HitWall(Normal(Other.Location - Location), Level);
						}
					}
					else
					{
						Other.TakeDamage( DamageMP, instigator, Location, MomentumTransfer * Normal(Velocity), MyDamageType);
						spark1 = spawn(class'SparkHitProjectile',Owner,,Location);
						if(spark1 != None)
							spark1.PlaySound(WallHitSound,,1.0,false,200.0,GetRandPitch());

						AWPKatanaStuckInPawn(Other, HitLocation);
						Destroy();
						return;
					}
					// Save the last connection
					if(Pawn(Other) != None
						&& Other != Instigator)
						LastPawnHit=Other;
					ReduceFlight(FlyTimesLeft);
				}
			}
			// fall to the ground but hurt most things
			else
			{
				// Balance damage differently for MP vs SP
				if(Level.Game != None
					&& FPSGameInfo(Level.Game).bIsSinglePlayer)
					Other.TakeDamage( Damage, instigator, Location, MomentumTransfer * Normal(Velocity), MyDamageType);
				else
					Other.TakeDamage( DamageMP, instigator, Location, MomentumTransfer * Normal(Velocity), MyDamageType);
				PlaySound(WallHitSound,,,,1.0);
				// bounce off it
				Velocity = -Velocity/2;
				Velocity.z += FRand()*100;
				// then fall
				MakePickup();
				Destroy();
			}
		}
	}
}

defaultproperties
{
     AltPickupClass=Class'EAInventory.AWPKatanaPickupStabbed'
     BoneList(0)="MALE01 neck"
     BoneList(1)="MALE01 L Hand"
     BoneList(2)="MALE01 L Forearm"
     BoneList(3)="MALE01 L UpperArm"
     BoneList(4)="MALE01 L Clavicle"
     BoneList(5)="MALE01 L Thigh"
     BoneList(6)="MALE01 L Calf"
     BoneList(7)="MALE01 L Foot"
     BoneList(8)="MALE01 r hand"
     BoneList(9)="MALE01 R Forearm"
     BoneList(10)="MALE01 R UpperArm"
     BoneList(11)="MALE01 R Clavicle"
     BoneList(12)="MALE01 R Thigh"
     BoneList(13)="MALE01 r calf"
     BoneList(14)="MALE01 r foot"
     BoneList(15)="MALE01 pelvis"
     BoneList(16)="MALE01 spine"
     BoneList(17)="MALE01 spine1"
     BoneList(18)="MALE01 spine2"
     WallHitSound=Sound'MaDModsSounds.MaDKatanaBlock'
     FlyingSound=None
     PickupClass=Class'EAInventory.AWPKatanaPickup'
     WakeEffectClass=None
     ScytheHitBody=Sound'MaDModsSounds.MaDKatanaHit00'
     StartSpinMag=0.000000
     DamageMP=200.000000
     speed=1500.000000
     MaxSpeed=3000.000000
     Damage=200.000000
     MyDamageType=Class'BaseFX.CuttingDamage'
     StaticMesh=StaticMesh'EAMesh.Weapons.KatanaProj'
     bUseCylinderCollision=False
}
