//=============================================================================
// BOKUTOUWeapon.
//=============================================================================
class BOKUTOUWeapon extends EDWeapon;

var name Shoot1Anim;		// Based on where you first swing, it will decide from 2 downs, one left or right
							// and put the name in here to be used

const BLEND_TIME = 0.3;

const HEAD_OFFSET	=	10.0;
const PAWN_NECK		=	'MALE01 neck';

const SHOOTDOWN1	=	'Shoot1Down';
const SHOOTDOWN2	=	'Shoot1Front';
const SHOOTLEFT		=	'Shoot1Left';
const SHOOTRIGHT	=	'Shoot1Right';

var BOKUTOU BOKU;
var travel bool bShowHint1;
var bool bStopAtDoor;		// If this is set, then we care about hitting doors first and people
							// later. This is really for just the foot. Most of the time we kick doors
							// open. If we directly kick a door open, but a person was on the other side
							// we *don't* want them pissed off (attacking) because we couldn't know they	
							// we're on the other side and for the most part we didn't mean to do that.

var int BloodTextureIndex;			// index into following array
var array<Texture> BloodTextures;	// bloodier versions of this weapon skin


var float AlertRadius;		// how big an area to tell people i'm going to hit about me

var float saveTime, PlayEvery, WeaponSpeedEnd;							// we *don't* want them pissed off (attacking) because we couldn't know they
							// we're on the other side and for the most part we didn't mean to do that.

replication
{
	// Functions called by server on client
	reliable if(Role == ROLE_Authority)
		ClientShovelShake;
}

///////////////////////////////////////////////////////////////////////////////
// Set the texture that would handle the blood
///////////////////////////////////////////////////////////////////////////////
function SetBloodTexture(Texture NewTex)
{
	Skins[1] = NewTex;
}

///////////////////////////////////////////////////////////////////////////////
// Add more blood the weapon by incrementing into the blood texture array for
// skins
///////////////////////////////////////////////////////////////////////////////
function DrewBlood()
{
	// Can add more blood, so do
	if(BloodTextureIndex < BloodTextures.Length)
	{
		// update the texture
		SetBloodTexture(BloodTextures[BloodTextureIndex]);
		BloodTextureIndex++;
	}
	//log(self$" drew blood "$BloodTextureIndex$" new skin "$Skins[1]);
}

///////////////////////////////////////////////////////////////////////////////
// Remove all blood from blade
///////////////////////////////////////////////////////////////////////////////
function CleanWeapon()
{
	BloodTextureIndex = 0;
	SetBloodTexture(Texture(default.Skins[1]));
	//log(self$" clean weapon "$BloodTextureIndex$" new skin "$Skins[1]);
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// Weapon gets cleaned when you bring it back out each time
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
state Active
{
	function BeginState()
	{
		Super.BeginState();
		CleanWeapon();
	}
}

///////////////////////////////////////////////////////////////////////////////
// Tell them your about to smash things
///////////////////////////////////////////////////////////////////////////////
function SendSwingAlert(float UseRadius)
{
	local vector endpt;
	local AWPerson Victims;

	//log(self$" send swing alert ");
	if(Owner != None)
	{
		endpt = Owner.Location + UseMeleeDist*vector(Owner.Rotation);
	}
	else
	{
		endpt = Location;
	}

	foreach VisibleCollidingActors( class'AWPerson', Victims, UseRadius, endpt )
	{
		//log(self$" seeing "$victims);
		// If they're valid, tell them about your swing
		if(Victims != None
			&& !Victims.bDeleteMe
			&& Victims.Health > 0)
		{
			Victims.BigWeaponAlert(AWPerson(Owner));
		}
	}
}


///////////////////////////////////////////////////////////////////////////////
// Give hints about this item
///////////////////////////////////////////////////////////////////////////////
function bool GetHints(out String str1, out String str2, out String str3,
				out byte InfiniteHintTime)
{
	if(bAllowHints)
	{
		if(bShowHint1)
			str1=HudHint1;
		else
			str2=HudHint2;
		return true;
	}
	return false;
}

///////////////////////////////////////////////////////////////////////////////
// Allow hints again
///////////////////////////////////////////////////////////////////////////////
function RefreshHints()
{
	Super.RefreshHints();
	bShowHint1=true;
}

///////////////////////////////////////////////////////////////////////////////
// Based on about where you'll hit the pawn in front of us, pick an animation
// to play when swinging
///////////////////////////////////////////////////////////////////////////////
simulated function name PickFireAnim()
{
	local vector StartTrace, EndTrace, X, Y, Z, HitLoc, HitNormal;
	local vector usev;
	local coords usecoords;
	local Actor HitActor;
	local int i;
	local bool bSideways;

	GetAxes(Instigator.GetViewRotation(),X,Y,Z);
	StartTrace = GetFireStart(X,Y,Z);
	EndTrace = StartTrace + 2*X*UseMeleeDist;

	HitActor = Trace(HitLoc,HitNormal,EndTrace,StartTrace,true);
	if(PersonPawn(HitActor) != None
		&& PersonPawn(HitActor).Health > 0)
	{
		// approximate the best swing for the place you'll hit them
		usev = HitLoc - HitActor.Location;
		usecoords = HitActor.GetBoneCoords(PAWN_NECK);
		// Hit head
		if(hitloc.z > (usecoords.origin.z))
			bSideways=true;
		// hit legs
		else if(usev.z < 0)
			bSideways=true;
		if(bSideways)
		{
			if(Rand(2) != 0)
				return SHOOTLEFT;
			else
				return SHOOTRIGHT;
		}
		else // swing down
		{
			if(Rand(2) != 0)
				return SHOOTDOWN1;
			else
				return SHOOTDOWN2;
		}
	}
	else
	{
		// Hit a dead thing, or an animal, pick a random swing
		i = Rand(4);
		switch(i)
		{
			case 0:
				return SHOOTDOWN1;
			case 1:
				return SHOOTDOWN2;
			case 2:
				return SHOOTLEFT;
			case 3:
				return SHOOTRIGHT;
		}
	}
}

simulated function name GetAnimIdle()
{
	local int rnd;

	rnd = Rand(2);
	switch(rnd)
	{
	case 0:
		return 'Idle2';
		break;
	case 1:
		return 'Idle3';
		break;
	}
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
simulated function PlayFiring()
{
	// Determine hack anim to use based on where we're aiming now
	//Shoot1Anim = SHOOTDOWN1;
	Shoot1Anim = PickFireAnim();

	IncrementFlashCount();
	// Play MP sounds on everyone's computers
//	if(Level.Game == None
//		|| !FPSGameInfo(Level.Game).bIsSinglePlayer)
//		PlayOwnedSound(FireSound,SLOT_Interact,1.0,,,WeaponFirePitchStart + (FRand()*WeaponFirePitchRand),false);
//	else // just on yours in SP games
		Instigator.PlaySound(FireSound, SLOT_None, 1.0, true, , WeaponFirePitchStart + (FRand()*WeaponFirePitchRand));
	
	PlayAnim(Shoot1Anim, WeaponSpeedShoot1 + (WeaponSpeedShoot1Rand*FRand()), 0.05);

	if(bShowHint1)
	{
		bShowHint1=false;
		UpdateHudHints();
	}
	
	Super.PlayFiring();
}

simulated function PlayAltFiring()
{
    Instigator.PlaySound(AltFireSound, SLOT_None, 1.0, true, , WeaponFirePitchStart + (FRand()*WeaponFirePitchRand));
	Super.PlayAltFiring();

}

///////////////////////////////////////////////////////////////////////////////
// Play our proper idling animation
///////////////////////////////////////////////////////////////////////////////
simulated function PlayIdleAnim()
{
	PlayAnim('Idle1', WeaponSpeedIdle, 0.0);
}
     
state Idle
{
	simulated function BeginState()
		{
		// If instigator doesn't want to fire anymore then we can finally
		// end the whole pouring sequence.
		if (!Instigator.PressingFire())
			{
			if (Owner != None)
				ForceEndFire();
			}
		}




	        event Tick(float DeltaTime)
                {
                local int i;

	        i = frand() * 5;

                //dopamine if they have reached the amount of time in talk every
	        If (Level.TimeSeconds - SaveTime > PlayEvery){

                   //if they are not currently firing then play the animation
    	           if (!IsFiring()){
                        PlayAnim(GetAnimIdle(), WeaponSpeedEnd, 0.05);
                   }
	           SaveTime = Level.TimeSeconds;
                }
        }
}

///////////////////////////////////////////////////////////////////////////////
// Start hurting things
///////////////////////////////////////////////////////////////////////////////
function Notify_StartHit()
{
//	log(self$" notify start hit");
	DoHit( TraceAccuracy, 0, 0);
}

///////////////////////////////////////////////////////////////////////////////
// Stop hurting things
///////////////////////////////////////////////////////////////////////////////
simulated function Notify_StopHit()
{
}

///////////////////////////////////////////////////////////////////////////////
// We didn't hit anything when we fired/swung our weapon
///////////////////////////////////////////////////////////////////////////////
function HitNothing()
{
	// STUB
}

///////////////////////////////////////////////////////////////////////////////
// Normal trace fire, plus check where to make the danger marker to 
// alert people of the noise.
///////////////////////////////////////////////////////////////////////////////
function TraceFire( float Accuracy, float YOffset, float ZOffset )
{
	// EMPTY
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
simulated function ClientShovelShake(bool bHitSolid)
{
	local P2Player p2p;

	p2p = P2Player(Instigator.Controller);
	if (p2p!=None)
	{
		if(bHitSolid)
			p2p.ShakeView(ShakeRotMag, ShakeRotRate, ShakeRotTime, 
						ShakeOffsetMag, ShakeOffsetRate, ShakeOffsetTime);
		else
			// Don't shake it as hard if we hit something not so solid
			p2p.ShakeView(ShakeRotMag/2, ShakeRotRate, ShakeRotTime/2,
						ShakeOffsetMag/2, ShakeOffsetRate, ShakeOffsetTime/2);
	}
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
function DoHit( float Accuracy, float YOffset, float ZOffset )
{
	// We don't want this to do the work. We'll do it below

	// Do a sphere test around where the shovel projects, and try to hurt stuff there
	local vector markerpos, markerpos2;
	local bool secondary;
	
	// Weapon.TraceFire (modified to save where it hit inside LastHitLocation
	local vector HitLocation, HitNormal, StartTrace, EndTrace, StraightX, X,Y,Z, HitPoint;
	local actor FirstHit;
	local actor Victims;
	local FPSPawn LivePawnHit;
	local float dist, UseTime;
	local vector dir;
	local bool bDeliverDirectDamage;
	local bool bHitSomething, bHitSolid, bHitDoor;

//	log(self$" DoHit");
	Owner.MakeNoise(1.0);
	GetAxes(Instigator.GetViewRotation(),X,Y,Z);
	StraightX = X;
	StartTrace = GetFireStart(X,Y,Z); 
	AdjustedAim = Instigator.AdjustAim(AmmoType, StartTrace, 2*AimError);
	EndTrace = StartTrace + (YOffset + Accuracy * (FRand() - 0.5 ) ) * Y * 1000
		+ (ZOffset + Accuracy * (FRand() - 0.5 )) * Z * 1000;
	X = vector(AdjustedAim);
	HitPoint = EndTrace + (2*UseMeleeDist * X);

	// This performs the collision but also records where it hit and records it
	FirstHit = Trace(LastHitLocation,HitNormal,HitPoint,StartTrace,True);
//	log(self$" trace returns "$FirstHit);
	if(FirstHit != None)
	{
		bDeliverDirectDamage=true;
		if(FirstHit.bStatic)
			bHitSolid=true;
		// If we hit a live pawn, record that too
		else if(FPSPawn(FirstHit) != None
			&& FPSPawn(FirstHit).Health > 0)
			LivePawnHit = FPSPawn(FirstHit);
		// If we hit a door, don't let us hurt a person who may be on the FirstHit side
		// as we kick it open, if we are indeed kicking it open. 
		else if(DoorMover(FirstHit) != None
			&& bStopAtDoor)
			bHitDoor=true;

		// Process the damage here
		AmmoType.ProcessTraceHit(self, FirstHit, LastHitLocation, HitNormal, X,Y,Z);
		bHitSomething=true;
	}

	// Hurt things halfway to the end of the trace, for the radius of the trace,
	// so the sphere is halfway along the melee weapon, hurting everything in between
	// It's a hurting line with a hurting ball covering the same area.
	HitPoint = StartTrace + (UseMeleeDist * StraightX);

	// If we don't care about hitting doors or we didn't hit one, check the
	// area around us too for a possible hit
	if(!bHitDoor)
	{
//		log(self$" VisibleCollidingActors ");
		foreach VisibleCollidingActors( class 'Actor', Victims, UseMeleeDist, HitPoint )
		{
//			log(self$" VisibleCollidingActors loop returns "$Victims);
			if( (Victims != self)
				&& (!Victims.bStatic)
				&& (Victims != Instigator) 
				&& (Victims.Role == ROLE_Authority) 
				&& (FirstHit != Victims))	// handle the straight forward hits below
			{
				// If it's friendly pawn, don't let this 'wide area collision check' hurt
				// them. Only let the above, direct check hurt them. If it's any other pawn
				// or anything else in general, hurt it. This is so it's harder to accidentally
				// hurt your friends with the wide bludgeoning attacks.
				if(FPSPawn(Victims) == None
					|| !FPSPawn(Victims).bPlayerIsFriend)
				{
					// Save the first thing we hit, so we can alert other people about the hit
					if(!bDeliverDirectDamage
						&& FirstHit == None)
					{
						FirstHit = Victims;
						LastHitLocation = Victims.Location;
					}

					// If we hit something really hard, record it
					if(Victims.bStatic)
						bHitSolid=true;
					// If we hit a live pawn, record that too
					else if(LivePawnHit == None
						&& FPSPawn(Victims) != None
						&& FPSPawn(Victims).Health > 0)
						LivePawnHit = FPSPawn(Victims);
						DrewBlood();

					// Check to deliver damage
					dir = Victims.Location - HitLocation;
					dist = FMax(1,VSize(dir));
					dir = dir/dist;


//dopamine check this for how hard you are hitting
					AmmoType.ProcessTraceHit(self, Victims,
									Victims.Location - 0.5 * (Victims.CollisionHeight + Victims.CollisionRadius) * dir,
									HitNormal, X,Y,Z);
					bHitSomething=true;
				}
			} 
		}
	}

	// If we hit a pawn, don't say it was a solid hit, even if we hit other stuff
	if(LivePawnHit != None)
	        DrewBlood();
		bHitSolid=false;

	//log(self$" shovel hit something 0, inst "$Instigator$" hit something "$bHitSomething$" role "$Role);
	// If we hit something, only then (not when we fire) do we shake the view
	if(bHitSomething)
	{
		if ( Instigator != None)
		{
			ClientShovelShake(bHitSolid);
		}
	}
	// If we didn't hit anything, we may want to do something about it
	//  That is, we want to tell people we were swinging/kicking in mid-air
	else
		HitNothing();
	
	// Say we just fired
	ShotCount++;
	
	// Set your enemy as the one you attacked.
	if(P2Player(Instigator.Controller) != None
		&& LivePawnHit != None)
	{
		P2Player(Instigator.Controller).Enemy = LivePawnHit;
	}

	// Only make a new danger marker if the consecutive fires were as high
	// as the max
	if(ShotCount >= ShotCountMaxForNotify
		&& Instigator.Controller != None)
		{
		// tell it we know this just happened, by recording it.
		ShotCount -= ShotCountMaxForNotify;
		
		// This is if a pawn is hit by a bullet (or hurt bad), so it's really scary
		if(LivePawnHit != None
			&& PawnHitMarkerMade != None)
			{
			PawnHitMarkerMade.static.NotifyControllersStatic(
				Level,
				PawnHitMarkerMade,
				FPSPawn(Instigator), 
				FPSPawn(Instigator), // We want the creator to be instigator also
				// because this is much more like a gunfire sort of thing, rather than a pawn hit by
				// a bullet
				PawnHitMarkerMade.default.CollisionRadius,
				LivePawnHit.Location);
			}
		}
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
simulated function PostNetBeginPlay()
{
    local PlayerController PC;
	local Pawn P;

    Super.PostNetBeginPlay();

    P  = Pawn(Owner);
	PC = PlayerController(P.Controller);

   	if (P.IsLocallyControlled() && PC!=None && (!PC.bBehindView) )
	{
        BOKU = Spawn(Class'EAInventory.BOKUTOU');
        AttachToBone(BOKU, 'bokutou');
    }
   	else
	{
        if(Instigator != None)
	   {
		 if(Instigator.Mesh != None)

             BOKU.Destroyed();
             BOKU = None;
	   }
	}

}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
function DropFrom(vector StartLocation)
{

   Super.DropFrom(StartLocation);


     BOKU.Destroyed();

}

simulated function Destroyed()
{
   Super.Destroyed();

  if( BOKU != None )
  {
    DetachFromBone(BOKU);
    BOKU.Destroy();
    BOKU = None;
  }
}


///////////////////////////////////////////////////////////////////////////////
// Default properties
///////////////////////////////////////////////////////////////////////////////

defaultproperties
{
//     BloodTextures(0)=Texture'ED_WeaponSkins.Melee.katana2'
//     BloodTextures(1)=Texture'ED_WeaponSkins.Melee.katana3'
//     BloodTextures(2)=Texture'ED_WeaponSkins.Melee.katana4'
//     BloodTextures(3)=Texture'ED_WeaponSkins.Melee.katana5'
     AlertRadius=200.000000
     PlayEvery=12.000000
     WeaponSpeedEnd=1.000000
     bUsesAltFire=True
     ViolenceRank=3
     RecognitionDist=600.000000
//     HolsterSound=Sound'MaD_EDSounds.MaDKatanaSelect'
     PawnHitMarkerMade=Class'Postal2Game.PawnBeatenMarker'
//     NoHitMarkerMade=Class'Postal2Game.KickHitNothingMarker'
     ShotMarkerMade=Class'Postal2Game.KickHitNothingMarker'
     BulletHitMarkerMade=Class'Postal2Game.BulletHitMarker'
     holdstyle=WEAPONHOLDSTYLE_Melee
     switchstyle=WEAPONHOLDSTYLE_Melee
     firingstyle=WEAPONHOLDSTYLE_Melee
     bNoHudReticle=True
     ShakeOffsetTime=6.000000
     PlayerMeleeDist=140.000000
     NPCMeleeDist=140.000000
     HudHint1="Press Fire (Left Mouse Button) to swing."
     HudHint2="Alt Fire (Right Mouse Button) for special swing."
     CombatRating=1.500000
     FirstPersonMeshSuffix="MaD_LS_Katana"
     WeaponsPackageStr="MaD_EDAnims."
     WeaponSpeedHolster=1.500000
     WeaponSpeedShoot1=0.900000
     WeaponSpeedShoot1Rand=0.100000
     WeaponSpeedShoot2=1.100000
//     AltFireSound=Sound'MaD_EDSounds.MaDKatanaFireAlt'
     bCanThrowMP=False
     AmmoName=Class'EAInventory.BOKUTOUInv'
     FireOffset=(X=0.000000,Z=0.000000)
     ShakeRotTime=6.000000
     ShakeOffsetMag=(X=25.000000,Y=5.000000,Z=5.000000)
     TraceAccuracy=0.005000
     bMeleeWeapon=True
     aimerror=200.000000
     AIRating=0.110000
     MaxRange=95.000000
     PickupClass=Class'EAInventory.BOKUTOUPickup'
//     FireSound=Sound'MaD_EDSounds.MaDKatanaFire'
//     SelectSound=Sound'MaD_EDSounds.MaDKatanaSelect'
     GroupOffset=13
     PlayerViewOffset=(X=1.000000)
     BobDamping=0.970000
     AttachmentClass=Class'EAInventory.BOKUTOUAttachment'
     ItemName="Bokutou"
     Texture=Texture'EAWeaponsSkin.HUD.Icon_BOKUTOU'
     Mesh=SkeletalMesh'EA_Weapons.MaD_LS_Katana'
     Skins(0)=Texture'MP_FPArms.LS_arms.LS_hands_dude'
     Skins(1)=Shader'MaDModsTex.Effects.MaDGunFake'
     AmbientGlow=0
}
