///////////////////////////////////////////////////////////////////////////////
// Bong Weapon
// Created on: 10.09.2006 17:07:44
// Written by Eduard Klemens
// Mail to: madjackal_x@hotmail.com.
//  2006, MaD Inc. All Rights Reserved.
///////////////////////////////////////////////////////////////////////////////

class EABongWeapon extends P2Weapon;

var travel bool bShowHint1;

var float saveTime, PlayEvery, WeaponSpeedEnd;

var float WeedMaxHealthPercentage;
var bool bWorked;
var vector SmokeColor;
var Sound ComentSound;	                // sounds for smokin'!

var int BurntTextureIndex;		// index into following array
var array<Texture> BurntTextures;	// Burnt versions of this weapon skin

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
replication
{
	// Functions called by server on client
	reliable if(Role == ROLE_Authority)
		ClientBongShake;
}

///////////////////////////////////////////////////////////////////////////////
// Give hints about this item
///////////////////////////////////////////////////////////////////////////////
function bool GetHints(out String str1, out String str2, out String str3,
				out byte InfiniteHintTime)
{
	if(bAllowHints)
	{
		if(bShowHint1)
			str1=HudHint1;
		else
			str2=HudHint2;
		return true;
	}
	return false;
}


///////////////////////////////////////////////////////////////////////////////
// Allow hints again
///////////////////////////////////////////////////////////////////////////////
function RefreshHints()
{
	Super.RefreshHints();
	bShowHint1=true;
}

///////////////////////////////////////////////////////////////////////////////
// Play firing animation/sound/etc
// Set here that we want to reload after each throw
///////////////////////////////////////////////////////////////////////////////

simulated function PlayFiring()
{
	Super.PlayFiring();
	bForceReload=true;
	if(bShowHint1)
	{
		bShowHint1=false;
		UpdateHudHints();
	}
}

///////////////////////////////////////////////////////////////////////////////
// Set the texture that would handle the Burn
///////////////////////////////////////////////////////////////////////////////
function SetBurntTexture(Texture NewTex)
{
	Skins[4] = NewTex;
}

///////////////////////////////////////////////////////////////////////////////
// Add more burn the weapon by incrementing into the burnt texture array for
// skins
///////////////////////////////////////////////////////////////////////////////
function BurnIt()
{
	// Can add more Burn, so do
	if(BurntTextureIndex < BurntTextures.Length)
	{
		// update the texture
		SetBurntTexture(BurntTextures[BurntTextureIndex]);
		BurntTextureIndex++;
	}
	//log(self$" drew Burn "$BurntTextureIndex$" new skin "$Skins[4]);
}

///////////////////////////////////////////////////////////////////////////////
// Remove all Burn from Weapon
///////////////////////////////////////////////////////////////////////////////
function CleanWeapon()
{
	BurntTextureIndex = 0;
	SetBurntTexture(Texture(default.Skins[4]));
	//log(self$" clean weapon "$BurntTextureIndex$" new skin "$Skins[4]);
}

///////////////////////////////////////////////////////////////////////////////
// Start hurting things
///////////////////////////////////////////////////////////////////////////////
function Notify_Smoke()
{
       Activate();
}

///////////////////////////////////////////////////////////////////////////////
// Start hurting things
///////////////////////////////////////////////////////////////////////////////
function Notify_Burn()
{
       BurnIt();
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
simulated function BlowingSmoke()
{
	local P2MocapPawn CheckPawn;
	local P2Player p2p;

	CheckPawn = P2MocapPawn(Owner);
	p2p = P2Player(CheckPawn.Controller);

	p2p.BlowSmoke(SmokeColor);
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
function Activate()
{
   GoToState('Activated');
}

///////////////////////////////////////////////////////////////////////////////
// Active state: this Bong item is armed and ready to rock!
///////////////////////////////////////////////////////////////////////////////
state Activated
{
	ignores Activate;

	///////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////
	function bool SmokeIt()
	{
		local P2MocapPawn CheckPawn;
		local float HealingAmount;	// How much health you add

		CheckPawn = P2MocapPawn(Owner);

		// calc how much we need to get to our max health goal
		HealingAmount = (CheckPawn.HealthMax * WeedMaxHealthPercentage) - CheckPawn.Health;

		if(CheckPawn.AddHealth(HealingAmount,,,true, true))
		{
		        // Check now if we're out of these.
			P2AmmoInv(AmmoType).UseAmmoForShot();

			return true;
		}
		return false;
	}

Begin:
        SmokeIt();
	BlowingSmoke();
	Owner.PlaySound(ComentSound);
	Sleep(Owner.GetSoundDuration(ComentSound));
	ClientBongShake(true);

	GotoState('Reloading');
}

///////////////////////////////////////////////////////////////////////////////
// Normal trace fire, plus check where to make the danger marker to 
// alert people of the noise.
///////////////////////////////////////////////////////////////////////////////
function TraceFire( float Accuracy, float YOffset, float ZOffset )
{
	// EMPTY
}

///////////////////////////////////////////////////////////////////////////////
// Called on client's side to make the gun fire
// Check here to throw out danger markers to let people know the gun has gone
// off.
///////////////////////////////////////////////////////////////////////////////
simulated function LocalFire()
	{
	local P2Player P;

	PlayFiring();
	}		

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
simulated function ClientBongShake(bool bBongHit)
{
	local P2Player p2p;

	p2p = P2Player(Instigator.Controller);
	if (p2p!=None)
	{
		if(bBongHit)
			p2p.ShakeView(ShakeRotMag, ShakeRotRate, ShakeRotTime, 
						ShakeOffsetMag, ShakeOffsetRate, ShakeOffsetTime);
		else
			// Don't shake it as hard if we hit something not so solid
			p2p.ShakeView(ShakeRotMag/2, ShakeRotRate, ShakeRotTime/2,
						ShakeOffsetMag/2, ShakeOffsetRate, ShakeOffsetTime/2);
	}
}

///////////////////////////////////////////////////////////////////////////////
// Play our proper idling animation
///////////////////////////////////////////////////////////////////////////////
simulated function PlayIdleAnim()
{
	PlayAnim('Idle1', WeaponSpeedIdle, 0.0);
}
     
state Idle
{
	simulated function BeginState()
		{
		// If instigator doesn't want to fire anymore then we can finally
		// end the whole pouring sequence.
		if (!Instigator.PressingFire())
			{
			if (Owner != None)
				ForceEndFire();
			}
		}

	        event Tick(float DeltaTime)
                {
                local int i;

	        i = frand() * 5;

                //dopamine if they have reached the amount of time in talk every
	        If (Level.TimeSeconds - SaveTime > PlayEvery){

                   //if they are not currently firing then play the animation
    	           if (!IsFiring()){
                        PlayAnim('Idle2', WeaponSpeedEnd, 0.05);
                   }
	           SaveTime = Level.TimeSeconds;
                }
        }

}

///////////////////////////////////////////////////////////////////////////////
// Reload (state) I think desire Bug fix... orz
///////////////////////////////////////////////////////////////////////////////
state Reload
{
///////////////////////////////////////////////////////////////////////////////
// Play reloading
//////////////////////////////////////////////////////////////////////////////
simulated function PlayReloading()
	{
        P2MocapPawn(Instigator).PlayWeaponReload(self);
        PlayAnim('Reload', WeaponSpeedReload, 0.05);
	Instigator.PlaySound(ReloadSound, SLOT_Misc, 1.0);
	CleanWeapon();
	}
}

///////////////////////////////////////////////////////////////////////////////
// Default properties
///////////////////////////////////////////////////////////////////////////////

defaultproperties
{
     bShowHint1=True
     PlayEvery=12.000000
     WeaponSpeedEnd=1.000000
     WeedMaxHealthPercentage=2.00000
     smokecolor=(X=255.000000,Y=255.000000,Z=255.000000)
     ComentSound=Sound'WMaleDialog.wm_kickass'
     BurntTextures(0)=Texture'MaD_EDSkins.BongwaterTex'
     holdstyle=WEAPONHOLDSTYLE_Toss
     switchstyle=WEAPONHOLDSTYLE_Melee
     firingstyle=WEAPONHOLDSTYLE_Melee
     ShotMarkerMade=Class'Postal2Game.PawnBeatenMarker'
     bNoHudReticle=True
     bAllowHints=True
     bShowHints=True
     HudHint1="Press Fire (Left Mouse Button) to smoke."
     FirstPersonMeshSuffix="MaD_ED_Bong"
     WeaponsPackageStr="MaD_EDAnims."
     bCanThrowMP=True
     AmmoName=Class'EAInventory.EABongAmmunition'
     ReloadCount=1
     AutoSwitchPriority=0
     ShakeRotMag=(X=400.000000)
     ShakeRotTime=2.500000
     ShakeOffsetMag=(X=20.000000,Y=4.000000,Z=4.000000)
     InventoryGroup=10
     GroupOffset=99
     PickupClass=Class'EAInventory.EABongPickup'
     PlayerViewOffset=(X=1.000000,Y=0.300000,Z=-1.300000)
     BobDamping=0.975000
     AttachmentClass=Class'Bong.BongAttachment'
     ItemName="Bong!"
     Texture=Texture'MaD_EDSkins.Icon_Inv_MaDBong'
     Mesh=SkeletalMesh'MaD_EDAnims.MaD_ED_Bong'
     Skins(0)=Texture'MP_FPArms.LS_arms.LS_hands_dude'
     AmbientGlow=0
     SoundRadius=255.000000
}
