class GLauncherWeapon extends LauncherWeapon;

var() class<CatRocket> CatRocketClass;

function bool ValidTarget(FPSPawn TestTarget)
{
	return false;
}

simulated function AltFire( float Value );
function ServerAltFire();

///////////////////////////////////////////////////////////////////////////////
// Skip the charge, go straight to the fire
///////////////////////////////////////////////////////////////////////////////
simulated function Fire( float Value )
{
	if ( AmmoType == None
		|| !AmmoType.HasAmmo() )
	{
		ClientForceFinish();
		ServerForceFinish();
		return;
	}


	bAltFiring=false;
	ShootIt();
	ClientShootIt();
}

function SpawnCatRocket(Pawn Instigator, Vector StartLoc, Rotator StartRot)
{
	local CatRocket CatR;
	local vector MarkerPos;

	CatR = spawn(CatRocketClass, Instigator,, StartLoc, StartRot);
	if (CatR != None)
	{
		CatR.Instigator = Instigator;
		CatR.AddRelativeVelocity(Instigator.Velocity);

		// Say we just fired
		ShotCount++;
		P2AmmoInv(AmmoType).UseAmmoForShot(1);

		// Only make a new danger marker if the consecutive fires were as high
		// as the max
		if(ShotCount >= ShotCountMaxForNotify
			&& Instigator.Controller != None
			&& ShotMarkerMade != None)
		{
			// tell it we know this just happened, by recording it.
			ShotCount -= ShotCountMaxForNotify;

			// Records the first (gun fire)
			markerpos = Instigator.Location;

			// Primary (the gun shooting, making a loud noise)
			if(ShotMarkerMade != None)
			{
				ShotMarkerMade.static.NotifyControllersStatic(
					Level,
					ShotMarkerMade,
					FPSPawn(Instigator),
					None,
					ShotMarkerMade.default.CollisionRadius,
					markerpos);
			}
		}
	}
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
function Notify_ShootLauncher()
{
	local vector startloc, StartTrace, X,Y,Z, markerpos, HitNormal, HitLocation;
	local actor HitActor;
	local GrenadeProjectile gren;
	local float giveback;

	if(AmmoType != None)
	{
		GetAxes(Instigator.GetViewRotation(),X,Y,Z);
		StartTrace = GetFireStart(X,Y,Z);
		AdjustedAim = Instigator.AdjustAim(AmmoType, StartTrace, 2*AimError);
		//CalcAIChargeTime();
		ShootStyleChanger=0;
		// Make sure we're not generating this on the other side of a thin wall
		//if(FastTrace(Instigator.Location, StartTrace))
		HitActor = Trace(HitLocation, HitNormal, Instigator.Location, StartTrace, true);
		StartLoc = sTARTTrace+ ((Instigator.CollisionRadius)*X);
		StartLoc.z+=(Instigator.CollisionRadius*0.5);
		if(HitActor == None
			|| (!HitActor.bStatic
				&& !HitActor.bWorldGeometry))
		{
			if (P2GameInfoSingle(Level.Game).VerifySeqTime() && Pawn(Owner).Controller.bIsPlayer)
				SpawnCatRocket(Instigator, StartLoc, Rotator(X));
			else
			{
				gren = spawn(class'GrenadeProjectile',Instigator,,StartLoc, Rotator(X));
				if(gren != None)
				{
					ChargeTime=0.8;
					gren.Instigator = Instigator;
					// Compensate for catnip time, if necessary. Don't do this for NPCs
					if(FPSPawn(Instigator) != None
						&& FPSPawn(Instigator).bPlayer)
						ChargeTime /= Level.TimeDilation;
					gren.SetupThrown(ChargeTime);
					gren.AddRelativeVelocity(Instigator.Velocity);
					// Touch any actor that was in between, just in case.
					if(HitActor != None)
						HitActor.Bump(gren);
				}
			}
		}

		if(gren != None)
		{
			// Say we just fired
			ShotCount++;
			P2AmmoInv(AmmoType).UseAmmoForShot(1);

			// Only make a new danger marker if the consecutive fires were as high
			// as the max
			if(ShotCount >= ShotCountMaxForNotify
				&& Instigator.Controller != None
				&& ShotMarkerMade != None)
			{
				// tell it we know this just happened, by recording it.
				ShotCount -= ShotCountMaxForNotify;

				// Records the first (gun fire)
				markerpos = Instigator.Location;

				// Primary (the gun shooting, making a loud noise)
				if(ShotMarkerMade != None)
				{
					ShotMarkerMade.static.NotifyControllersStatic(
						Level,
						ShotMarkerMade,
						FPSPawn(Instigator),
						None,
						ShotMarkerMade.default.CollisionRadius,
						markerpos);
				}
			}
		}
	}
}
///////////////////////////////////////////////////////////////////////////////
// Skin changing START Silent-Scope      BOOOOYA
///////////////////////////////////////////////////////////////////////////////


         function Notify_skinflash_start()
         {
                  AmbientGlow=400;
}

///////////////////////////////////////////////////////////////////////////////
// Skin Changing END Silent-Scope  BOOOYA
///////////////////////////////////////////////////////////////////////////////

          function Notify_skinflash_end()
         {
                 AmbientGlow=0;
}

defaultproperties
{
     CatRocketClass=Class'EAInventory.CatRocketGrenade'
     bUsesAltFire=False
     MinRange=1280.000000
     HudHint1="This weapon shoots grenades at your"
     HudHint2="targets in a rapid-fire fashion.   "
     WeaponSpeedShoot1=1.100000
     WeaponSpeedShoot1Rand=0.150000
     AmmoName=Class'EAInventory.GrenadeAmmoInv'
     MaxRange=2048.000000
     FireSound=Sound'WeaponSounds.napalm_fire'
     InventoryGroup=9
     GroupOffset=1
     PickupClass=Class'EAInventory.GLauncherPickup'
     AttachmentClass=Class'EAInventory.GLauncherAttachment'
     ItemName="G Launcher"
     Skins(0)=Texture'MP_FPArms.LS_arms.LS_hands_dude'
     Skins(1)=Texture'EATex.Weapons.GrenadeLauncher_nomips'
     Skins(2)=Texture'EATex.Weapons.nade'
     Skins(3)=Texture'EATex.Weapons.nade'
     Texture=Texture'HUDPack.Icons.Icon_Weapon_Grenade'
     AmbientGlow=0
}
