class HolyPickup extends GrenadePickup;

var() class<TimedMarker> DangerMarker;
var() class<P2Emitter> BreakEffectClass;
var() sound BreakingSound;
var() float Health;

///////////////////////////////////////////////////////////////////////////////
// If DamageFilter is set,
// and bBlockFilter is false only allow this damage
// else don't allow only this damage
///////////////////////////////////////////////////////////////////////////////
function bool AcceptThisDamage(class<DamageType> damageType)
{
	if(ClassIsChildOf(DamageType, class'BulletDamage')
		|| ClassIsChildOf(DamageType, class'ExplodedDamage')
		|| ClassIsChildOf(DamageType, class'BurnedDamage'))
		return true;

	return false;
}

///////////////////////////////////////////////////////////////////////////////
// Set it to dead, trigger sounds and all, and blow it up, setting off the physics
///////////////////////////////////////////////////////////////////////////////
function BlowThisUp(int Damage, vector HitLocation, vector Momentum)
{
	local P2Emitter p2e;

	// set to dead
	Health=0;

	// Spawn effect so we don't have to record the hit values and
	// do it later in Broken beginstate or something. It's just
	// more efficient here
	p2e = spawn(BreakEffectClass, self,,Location);
//	if(bFitEffectToProp)
//		FitTheEffect(p2e, damage, HitLocation, momentum);

	// Play the breaking sound (code copied from mover)
	PlaySound( BreakingSound, SLOT_None, SoundVolume / 255.0, false, SoundRadius, 0.96 + FRand()*0.8);

	Destroy();
}

///////////////////////////////////////////////////////////////////////////////
// If strong enough, it breaks the prop
///////////////////////////////////////////////////////////////////////////////
function TakeDamage(int Damage, Pawn instigatedBy, Vector hitlocation,
						Vector momentum, class<DamageType> damageType)
{
	local TimedMarker ADanger;

	// Check first if we take this damage
	if(!AcceptThisDamage(damageType))
		return;

	// If so, check to make panic noises for people around us.
	// Only panic, if the damage before us *didn't* make some kind of
	// panic noise (like a bullet hitting this would, and so would an
	// explosion)
	if(DangerMarker != None)
	{
		ADanger = spawn(DangerMarker,,,HitLocation);
		ADanger.CreatorPawn = FPSPawn(InstigatedBy);
		ADanger.OriginActor = self;
		// This will cause people to see if they noticed and decide what to do
		ADanger.NotifyAndDie();
	}

	// Only remove health if the singular damage was strong enough
	Health -= damage;
	// if you run out of health, break
	if(Health <= 0)
	{
		BlowThisUp(Damage, HitLocation, Momentum);
	}
}

defaultproperties
{
     AmmoGiveCount=2
     BounceSound=Sound'EASounds.Holy.HOLYGRENADEIMPACT'
     MPAmmoGiveCount=2
    BreakEffectClass=Class'EAInventory.HolyExplosion'
     BreakingSound=Sound'EASounds.MiscWeapons.Big-Ass-Explosion'
     ShortSleeveType=Class'EAInventory.HolyWeaponSS'
     InventoryType=Class'EAInventory.HolyWeapon'
     PickupMessage="You picked up a Jihad Grenade."
     StaticMesh=StaticMesh'EAMesh.Weapons.HolyGrenadeProj'
}
