class KarmaGun extends P2Weapon;

#exec AUDIO IMPORT FILE="Sounds\Catch.wav" Name="CaughtIt"
#exec AUDIO IMPORT FILE="Sounds\object_throw.WAV" NAME="ChuckIt"

const DEF_HOLD_DIST = 80;
const MAX_CHARGE_TIME = 2.0;

var Actor OurHeldItem; // pointer to the box or whatever we're carrying
var float HoldDist; // how far away the dude holds it from his body
var float ThrowMomentum;
var float KThrowMomentum;
var vector ZeroVector;
var sound ErrorSound;
var bool OldbPawn;
var float ChargeTime;
var float ChargeStart;

var bool bShowMainHints;
var localized string HudHint3;
var localized string HudHint4;
var color ProgressBarColor;

replication
{
	reliable if (Role == ROLE_Authority)
		OurHeldItem, HoldDist, ClientGoHoldItem, PlayCaught, PlayAltFireSound;
	reliable if (Role < ROLE_Authority)
		TossOurItem;
}

///////////////////////////////////////////////////////////////////////////////
// Draw percentage bar
///////////////////////////////////////////////////////////////////////////////
simulated function DrawPercBar(Canvas Canvas, float ScreenX, float ScreenY, float Width, float Height, float Border, Color Fore, float Perc)
{
	local float InnerWidth;
	local float InnerHeight;

	Canvas.Style = ERenderStyle.STY_Alpha;

	Canvas.SetPos(ScreenX - (Width/2), ScreenY);
	Canvas.SetDrawColor(0,0,0,byte(float(Fore.A)*0.75));
	if (Canvas.DrawColor.A > 0)
		Canvas.DrawRect(Texture'engine.WhiteSquareTexture', Width, Height);

	InnerWidth = Width - 2 * Border;
	InnerHeight = Height - 2 * Border;

	Canvas.SetPos(ScreenX - (InnerWidth/2), ScreenY + Border);
	Canvas.DrawColor = Fore;
	if (Canvas.DrawColor.A > 0)
		Canvas.DrawRect(Texture'engine.WhiteSquareTexture', InnerWidth * Perc, InnerHeight);
}

///////////////////////////////////////////////////////////////////////////////
// Allow two sets of hud hints to explain primary, then alternative fire
///////////////////////////////////////////////////////////////////////////////
simulated function TurnOffHint()
{
	if(bShowMainHints)
	{
		bShowMainHints=false;
		// Swap out hints to show
		HudHint1 = HudHint3;
		HudHint2 = HudHint4;
		UpdateHudHints();
	}
	else
		Super.TurnOffHint();
}

function bool IsConstrained(KActor Other)
{
	local KConstraint KC;

	foreach DynamicActors(class'KConstraint', KC)
		if (KC.KConstraintActor1 == Other || KC.KConstraintActor2 == Other)
			return true;

	return false;
}

simulated function PlayFiring(); // stubbed out because we don't want animations
simulated function PlayAltFiring();
simulated function ClientIdleCheckFire(); // stubbed out because we don't automatically continue firing

simulated function PlayCaught()
{
	TurnOffHint();
	Instigator.PlaySound(FireSound, SLOT_None, 1.0, true, , WeaponFirePitchStart + (FRand()*WeaponFirePitchRand));
}

simulated function PlayAltFireSound()
{
	Instigator.PlaySound(AltFireSound, SLOT_None, 1.0, true, , WeaponFirePitchStart + (FRand()*WeaponFirePitchRand));
}

function DropOurItem() // function to get rid of whatever item we're carrying
{
	if (OurHeldItem != None)
	{
		if (IsSeveredHead(OurHeldItem))
		{
			OurHeldItem.SetPhysics(PHYS_Projectile);
			OurHeldItem.Acceleration.Z = Head(OurHeldItem).HEAD_GRAVITY;
		}
		else
		{
			OurHeldItem.SetPhysics(PHYS_Karma); // turn karma physics back on
			//if (!OurHeldItem.KIsAwake())
			//	OurHeldItem.KWake(); // give it a jolt so it'll fall to the ground
			OurHeldItem.KAddImpulse(ZeroVector, OurHeldItem.Location);
			KActor(OurHeldItem).bPawnMovesMe=OldbPawn;
		}
		OurHeldItem.bBlockPlayers=true; // have it block players again
		OurHeldItem = None; // reset our pointer
	}
}

function TossOurItem(float Charge) // function to get rid of whatever item we're carrying
{
	local vector UseMom;
	local vector KUseMom;
	local P2Pawn OPawn;
	local Controller OController;
	local float C;

	if (OurHeldItem != None)
	{
		if (Charge < 0.5)
			C = 0.5;
		else
			C = Charge;
		if (P2Pawn(Owner) != None)
		{
			OPawn = P2Pawn(Owner);
			OController = OPawn.Controller;
			UseMom = ThrowMomentum * Normal(Vector(OController.GetViewRotation())) * C;
			KUseMom = KThrowMomentum * Normal(Vector(OController.GetViewRotation())) * C;
			OurHeldItem.Instigator = OPawn;
		}
		if (IsSeveredHead(OurHeldItem))
		{
			OurHeldItem.SetPhysics(PHYS_Projectile);
			Head(OurHeldItem).GiveMomentum(UseMom);
			if (OPawn != None)
				Head(OurHeldItem).PlayerMovedMe = OPawn;
		}
		else
		{
			OurHeldItem.SetPhysics(PHYS_Karma); // turn karma physics back on
			//if (!OurHeldItem.KIsAwake())
			//	OurHeldItem.KWake(); // give it a jolt so it'll fall to the ground
			OurHeldItem.KAddImpulse(KUseMom, OurHeldItem.Location);
			KActor(OurHeldItem).bPawnMovesMe=OldbPawn;
		}
		PlayAltFireSound();
		OurHeldItem.bBlockPlayers=true; // have it block players again
		OurHeldItem = None; // reset our pointer
	}
}

function bool IsSeveredHead(actor Other)
{
	if (Head(Other) != None)
		if (Head(Other).MyBody == None)
			return true;

	return false;
}

simulated function Fire(float Value)
{
	ServerFire();
}

simulated function AltFire( float Value )
{
	bAltFiring=true;
	ServerAltFire();
	ChargeTime=0;

	if ( Role < ROLE_Authority && OurHeldItem != None )
	{
		GotoState('ChargingThrow');
	}
}

simulated function ClientGoHoldItem()
{
	GotoState('HoldingItem');
}

///////////////////////////////////////////////////////////////////////////////
// Normal trace fire, plus check where to make the danger marker to
// alert people of the noise.
///////////////////////////////////////////////////////////////////////////////
function TraceFire( float Accuracy, float YOffset, float ZOffset )
{
	local Actor BasedActor;
	local vector markerpos, markerpos2;
	local bool secondary;
	local BulletTracer bullt;
	local vector usev;
	local Rotator newrot;
	local vector HitNormal, StartTrace, EndTrace, X,Y,Z;
	local actor Other;

	//log("TraceFire called");

	if (OurHeldItem == None) // if not holding an item, look to see if we can pick one up
	{
		// Weapon.TraceFire (modified to save where it hit inside LastHitLocation
		GetAxes(Instigator.GetViewRotation(),X,Y,Z);
		StartTrace = GetFireStart(X,Y,Z);
		AdjustedAim = Instigator.AdjustAim(AmmoType, StartTrace, 2*AimError);
		EndTrace = StartTrace + (YOffset + Accuracy * (FRand() - 0.5 ) ) * Y * 1000
			+ (ZOffset + Accuracy * (FRand() - 0.5 )) * Z * 1000;
		X = vector(AdjustedAim);
		EndTrace += (TraceDist * X);
		// This performs the collision but also records where it hit
		Other = Trace(LastHitLocation,HitNormal,EndTrace,StartTrace,True);
		//log("Hit actor was" @ Other);
		if (KActor(Other) != None || IsSeveredHead(Other)) // this is a karma actor, so let's grab it
		{
			BasedActor = None;
			foreach Other.BasedActors(class'Actor', BasedActor)
				break;
			if (BasedActor == None)
			{
				foreach Other.TouchingActors(class'Actor', BasedActor)
					break;
			}
			
			if (KActor(Other) != None && IsConstrained(KActor(Other)))
				BasedActor = Other;

			// Crash prevention: Don't pick up karma objects with based actors
			if (BasedActor == None)
			{
				//log ("Karma physics detected, grabbing item");
				OurHeldItem = Other;
				ClientGoHoldItem();
				PlayCaught();
				GotoState ('HoldingItem');
			}
			else
				Other.PlaySound(ErrorSound, SLOT_MISC);
		}
		else // not a karma actor, ignore
		{
			//log ("No karma physics detected");
			//GotoState('NormalFire');
		}
	}
	else // if we're holding an item, drop it
	{
		//log ("Dropping held item");
		DropOurItem();
		//GotoState('Idle');
	}
}

///////////////////////////////////////////////////////////////////////////////
// Alt trace fire, plus check where to make the danger marker to
// alert people of the noise.
///////////////////////////////////////////////////////////////////////////////
function TraceAltFire( float Accuracy, float YOffset, float ZOffset )
{
	if (OurHeldItem != None)
	{
		GotoState('ChargingThrow');
	}
}

state ClientFiring
{
}

state NormalFire // thrown in to prevent the game from crashing
{
Begin:
	Sleep(0.25);
	GotoState('Idle');
}

// HoldingItem
// An extension of weapon idle, since we're not really firing
// Grab our item and drag it along with the dude wherever he goes
state HoldingItem extends Idle
{
	simulated function BeginState()
	{
		HoldDist = DEF_HOLD_DIST;
	}
	// LockToPawn
	// Misnamed function, all this really does is turn off karma physics and make it so that the dude
	// isn't blocked by the item we're carrying.
	function LockToPawn(Actor Other)
	{
		//Other.KAddImpulse(Other.Velocity, Other.Location);
		Other.SetPhysics(PHYS_None); // turn off physics
		//Other.KDisableKarma();
		if (ClassIsChildOf(Other.Class,class'KActor'))
			OldbPawn = KActor(Other).bPawnMovesMe;
		if (ClassIsChildOf(Other.Class,class'KActorExplodable'))
			KActorExplodable(Other).bReadyForImpact=true;
		Other.bBlockPlayers=false; // turn off player collision.
		// Leaves other collisions on, so we can be annoying and screw with people if we want
	}

	// UpdateItem
	// this function moves the object in front of the dude
	// Also rotates the object relative to the dude's view rotation
	simulated function UpdateItem(Actor Other)
	{
		local vector HitNormal, StartTrace, EndTrace, X,Y,Z;
		local rotator foo;

		foo = Pawn(Owner).Controller.GetViewRotation();
		Other.SetRotation(foo); // set object's rotation relative to the dude's view

		GetAxes(foo, X, Y, Z);
		// crazy vector math
		// basically says "hold this item out in front of the dude"
		// if we look up or down the item will move with the view
		StartTrace = Owner.Location + (HoldDist*X) + (HoldDist*Z/3);
		Other.SetLocation(StartTrace);
	}
	simulated event Tick(float Deltatime)
	{
		UpdateItem(OurHeldItem);
	}

// Beginning of state
Begin:
	LockToPawn(OurHeldItem); // Get our object ready for carrying
	if (OurHeldItem == None) // If our object goes away for some reason, go back to normal idling
		GotoState('Idle');
	//UpdateItem(OurHeldItem); // Position our object
}

state ChargingThrow extends HoldingItem
{
	ignores Fire, AltFire, ServerFire, ServerAltFire, TraceFire, TraceAltFire;

	simulated function RenderOverlays(Canvas Canvas)
	{
		local float ScreenX, ScreenY, Width, Height, Border, Perc;
		
		ScreenX = Canvas.SizeX/2;
		ScreenY = Canvas.SizeY*4/5;
		Width = Canvas.SizeX/4;
		Height = Canvas.SizeY/20;
		Border = 2;
		Perc = (Level.TimeSeconds-ChargeStart)/MAX_CHARGE_TIME;
		DrawPercBar(Canvas, ScreenX, ScreenY, Width, Height, Border, ProgressBarColor, Perc);
		Super.RenderOverlays(Canvas);
	}

	function LockToPawn(Actor Other);
	simulated event BeginState()
	{
		ChargeStart = Level.TimeSeconds;
	}
	simulated function FinishShot()
	{
		if (NotDedOnServer())
		{
			ChargeTime = Level.TimeSeconds - ChargeStart;
			TossOurItem(ChargeTime);
		}
		GotoState('Idle');
	}
	simulated event Tick(float DeltaTime)
	{
		UpdateItem(OurHeldItem);
		if (NotDedOnServer() && !Instigator.PressingAltFire() || Level.TimeSeconds-ChargeStart >= MAX_CHARGE_TIME)
			FinishShot();
	}
}

// DownWeapon
// Drop anything we're carrying before we switch to some other weapon, usually a firearm to handle attackers
state DownWeapon
{
	simulated function BeginState()
	{
		DropOurItem();
		Super.Beginstate();
	}
}

// Active
// tell the player we're working
state Active
{
	function BeginState()
	{
		Super.BeginState();
		//P2Player(Instigator.Controller).ClientMessage("Karma Gun selected");
	}
}

defaultproperties
{
     HoldDist=80.000000
     ThrowMomentum=100000.000000
     KThrowMomentum=250000.000000
     ErrorSound=Sound'MiscSounds.Bleeps.bleep1'
     bShowMainHints=True
     HudHint3="Press Fire to drop the object, or"
     HudHint4="hold Alt-Fire to charge a throw."
     ProgressBarColor=(R=255,A=192)
     bUsesAltFire=True
     bArrestableWeapon=True
     ShakeOffsetTime=0.000000
     HudHint1="Aim at a karma object and press"
     HudHint2="Fire to pick it up."
     DropWeaponHint1="They've seen your weapon!"
     DropWeaponHint2="Press ; to drop it."
     bBumpStartsFight=False
     CombatRating=0.500000
     FirstPersonMeshSuffix="Nothing"
     WeaponSpeedLoad=2.000000
     WeaponSpeedReload=1.500000
     WeaponSpeedHolster=5.000000
     WeaponSpeedShoot1=0.800000
     WeaponSpeedShoot2=0.800000
     AltFireSound=Sound'EASounds.KarmaGun.object_throw'
     AmmoName=Class'EAInventory.KarmaAmmoInv'
     bCanThrow=False
     AutoSwitchPriority=0
     ShakeRotMag=(X=0.000000,Y=0.000000,Z=0.000000)
     ShakeRotRate=(X=0.000000,Y=0.000000,Z=0.000000)
     ShakeRotTime=0.000000
     ShakeOffsetMag=(X=0.000000,Y=0.000000,Z=0.000000)
     ShakeOffsetRate=(X=0.000000,Y=0.000000,Z=0.000000)
     aimerror=0.000000
     AIRating=0.040000
     TraceDist=250.000000
     SelectSound=None
     GroupOffset=5
     BobDamping=0.975000
     AttachmentClass=None
     ItemName="Karmagun"
     Texture=Texture'EATex.Weapons.TheLetterK'
     Mesh=SkeletalMesh'MP_Weapons.MP_LS_Nothing'
     Skins(0)=Texture'MP_FPArms.LS_arms.LS_hands_dude'
     AmbientGlow=0
}Sound'WeaponSounds.weapon_pickup'
