///////////////////////////////////////////////////////////////////////////////
// RifleWeapon
// Copyright 2002 Running With Scissors, Inc.  All Rights Reserved.
//
// Rifle weapon (first and third person).
//
// Has powerful head shots. If you're zoomed in it has much better built-in aim.
//
// This uses by default, the mouse wheel to zoom in and out. Therefore, it turns
// off Next/Prev weapon when in zoomed mode. Two functions AllowNextWeapon and
// AllowPrevWeapon to determine if it can use the Next/PrevWeapon functions. 
// Normally we would have used states to ignore these functions, but because
// we need to be in NormalFire state zoomed and normal, we went with two
// extra functions to say if he's zoomed or not, instead of making a 'IsZoomed'
// state. This also followed Epics original method or their zoom mode.
//
///////////////////////////////////////////////////////////////////////////////

class M4A1Weapon extends MP5ReloaderWeapon;

///////////////////////////////////////////////////////////////////////////////
// Vars, structs, consts
///////////////////////////////////////////////////////////////////////////////
var bool bZoomed;
var Texture SniperReticleCorner;
var Texture SniperReticleCenter;
var Texture BlackBackground;
var float ZoomedTraceAccuracy;		// Better aim when you're zoomed in
var float BotTraceAccuracy;
var Sound SniperBreathing;	// breathing while in zoomed mode
var Sound SniperChangeZoomSound; // clicky noise as you zoom in and out
var Sound SniperFarting[2];
var float ZoomFOV;
var float FullScreenAlpha;
var float FadeSpeed;
var bool  bChangedZoom;
var int BreathCount;
var int BreathMax;
var int SoundIndex;
var float BreathWait;
var travel bool bShowHint1;
var bool bBlockZoomIn;
var float TestTraceDist;
var P2Pawn MyTarget;
var float TargetTime;
var float SPAccuracy;
var M4A1 M4;
var M4A1Magazine M4M;
var M4A1Trigger M4T;
var M4A1Selector M4S;

var sound SelectSound;


//dopamine
var float WeaponSpeedPrep, WeaponSpeedEnd;

var float myValue;

		// After every so often, reduce this again to 0. Use it to update
							// the client that they're still in range.


const SCALE_RAND =	0.3;
const OFFSET_X_BASE = 0.02;
const OFFSET_Y_BASE = 0.02;
const MAX_ZOOM_FOV		=	20;
const START_ZOOM_FOV	=	10;
const MIN_ZOOM_FOV		=	2;
const INC_ZOOM_FOV		=	1;
const BREATH_MAX		=	20;
const FADE_ZOOM_ENTRY_TIME=	1.4;
const CAN_SHOOT_FADE	=	100;
const BETWEEN_BREATH_WAIT= 1.0;
const BETWEEN_BREATH_WAIT_BEHIND= 5.0;

const MAX_TRACES	=	5;

const EXPECTED_START_RES_WIDTH_RIFLE	= 1024;

const WAIT_IDLE_TIME	=	0.2;

const TARGET_UPDATE_TIME	= 2.0;


///////////////////////////////////////////////////////////////////////////////
// Bots with a sniper rifle always use ZoomedTraceAccuracy
///////////////////////////////////////////////////////////////////////////////
function PostBeginPlay()
{
	Super.PostBeginPlay();
	
	// Bots with a sniper rifle use a higher accuracy
	if(FPSPawn(Instigator) != None
		&& !FPSPawn(Instigator).bPlayer)
//		TraceAccuracy = BotTraceAccuracy;
                TraceAccuracy = TraceAccuracy;
		}

simulated function PostNetBeginPlay()
{
    local PlayerController PC;
	local Pawn P;

    Super.PostNetBeginPlay();

    P  = Pawn(Owner);
	PC = PlayerController(P.Controller);

   	if (P.IsLocallyControlled() && PC!=None && (!PC.bBehindView) )
	{
	M4 = Spawn(Class'EAInventory.M4A1');
        	AttachToBone( M4, 'M4A1' );
        	M4M = Spawn(Class'EAInventory.M4A1Magazine');
        	AttachToBone( M4M, 'M4A1_Mag' );
	M4S = Spawn(Class'EAInventory.M4A1Selector');
        	AttachToBone( M4S, 'M4A1_Select' );
        	M4T = Spawn(Class'EAInventory.M4A1Trigger');
        	AttachToBone( M4T, 'M4A1_Trigg' );
    }
   	else
	{
        if(Instigator != None)
	   {
		 if(Instigator.Mesh != None)

             M4.Destroyed();
             M4 = None;
             M4M.Destroyed();
             M4M = None;
             M4S.Destroyed();
             M4S = None;
             M4T.Destroyed();
             M4T = None;
	   }
	}
}

///////////////////////////////////////////////////////////////////////////////
// Allow hints again
///////////////////////////////////////////////////////////////////////////////
function RefreshHints()
{
	Super.RefreshHints();
	bShowHint1=true;
}

///////////////////////////////////////////////////////////////////////////////
// Fire the weapon
///////////////////////////////////////////////////////////////////////////////
simulated function PlayFiring()
{
	if(!bZoomed){
//	Super.PlayFiring();
	PlayNewFiring();
	SetupMuzzleFlash();
	}
	else{
//	Super.PlayAltFiring();
	PlayNewAltFiring();
	SetupMuzzleFlash();
	}
}

simulated function camshake()
{
	local P2Player p2p;

	p2p = P2Player(Instigator.Controller);
	if (p2p!=None)


			// Don't shake it as hard if we hit something not so solid
			p2p.ShakeView(ShakeRotMag/2, ShakeRotRate, ShakeRotTime/2,
						ShakeOffsetMag/2, ShakeOffsetRate, ShakeOffsetTime/2);
	}


///////////////////////////////////////////////////////////////////////////////
// Play firing animation/sound/etc
///////////////////////////////////////////////////////////////////////////////
simulated function PlayNewFiring()
{
        camshake();
	IncrementFlashCount();
	// Play MP sounds on everyone's computers
	if(Level.Game == None
		|| !FPSGameInfo(Level.Game).bIsSinglePlayer)
		PlayOwnedSound(FireSound,SLOT_Interact,1.0,,,WeaponFirePitchStart + (FRand()*WeaponFirePitchRand),false);
		


	else // just on yours in SP games
		Instigator.PlaySound(FireSound, SLOT_None, 1.0, true, , WeaponFirePitchStart + (FRand()*WeaponFirePitchRand));
	PlayAnim('Shoot1_semi', WeaponSpeedShoot1 + (WeaponSpeedShoot1Rand*FRand()), 0.05);
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
simulated function PlayNewAltFiring()
{
        camshake();
	IncrementFlashCount();
	// Play MP sounds on everyone's computers
	if(Level.Game == None
		|| !FPSGameInfo(Level.Game).bIsSinglePlayer)
		PlayOwnedSound(AltFireSound,SLOT_Interact,1.0,,,WeaponFirePitchStart + (FRand()*WeaponFirePitchRand),false);
	else // just on yours in SP games
		Instigator.PlaySound(AltFireSound, SLOT_None, 1.0, true, , WeaponFirePitchStart + (FRand()*WeaponFirePitchRand));

	PlayAnim('Shoot1_auto', WeaponSpeedShoot2, 0.05);
}

simulated function PlayIdleAnim()
{
        if (!bZoomed)
        	PlayAnim('Idle_semi', WeaponSpeedIdle, 0.0);
        else
                PlayAnim('Idle_auto', WeaponSpeedIdle, 0.0);

}

simulated function PlayDownAnim()
{
	// If our instigator is on fire, then put away you're weapons super fast,
	// and don't even play a third-person anim, but only for NPC's
	if(FPSPawn(Instigator).MyBodyFire != None)
		//&& PlayerController(Instigator.Controller) == None)
	{
                //dopamine use the proper switch anim
		if (!bZoomed)
                  PlayAnim('Holster_Semi', 1000.0, 0.0);
                else
                  PlayAnim('Holster_Auto', 1000.0, 0.0);
	}
	else
	{
                //dopamine use the proper switch anim
		if (!bZoomed)
                  PlayAnim('Holster_semi', WeaponSpeedHolster, 0.05);
                else
                  PlayAnim('Holster_auto', WeaponSpeedHolster, 0.05);


		Instigator.PlaySound(HolsterSound, SLOT_Misc, 1.0);
		// Show the pawn putting away the weapon
		P2MoCapPawn(Instigator).PlayWeaponDown();
	}
}

simulated function name GetPlaySelectAnim()
{
	return 'Load_semi';
}
///////////////////////////////////////////////////////////////////////////////
// This will randomly change the color and the size of the dynamic
// light associate with the flash. Change in each weapon's file,
// but call each time you start up the flash again.
// This function is also used by the third-person muzzle flash, so the 
// colors will look the same
///////////////////////////////////////////////////////////////////////////////
simulated function PickLightValues()
{
	LightBrightness=150;
	LightSaturation=150;
	LightHue=12+FRand()*15;
	LightRadius=12+FRand()*6;
}

///////////////////////////////////////////////////////////////////////////////
// Make sure the muzzle flash renders, and move it around some
///////////////////////////////////////////////////////////////////////////////
simulated function SetupMuzzleFlash()
{

//dopamine add triple muzzleflash
        //if(!IsZoomed())
	//{
		bMuzzleFlash = true;
		bSetFlashTime = false;

		// Gets turned off in weapon, in RenderOverlays
		// Slightly change colors each time
		if(IsFirstPersonView()
			&& bDrawMuzzleFlash)
		{
			PickLightValues();
			bDynamicLight=bAllowDynamicLights;
		}
	//}
}


///////////////////////////////////////////////////////////////////////////////
// Don't allow them to shoot as the rifle is brightening up
///////////////////////////////////////////////////////////////////////////////
simulated function Fire( float Value )
{

myValue = Value;


        if (!bZoomed)
	//log(self$" FIRING");
//	if ( AmmoType == None
//		|| !AmmoType.HasAmmo() )
//	{
//		ForceFinish();
//		return;
//	}

	//log(self$" FIRING made it through ammo");
//	ServerFire();

//	if ( Role < ROLE_Authority )

        
        	Super.Fire(Value);
        
        else
	{
		PrepStreaming();
		GotoState('StartStream');

	}
	

}
simulated function AltFire( float Value )
{
	// Kamek don't use mode switching in multi
	if (P2GameInfoSingle(Level.Game) != None)
		LocalAltFire();
}

///////////////////////////////////////////////////////////////////////////////
// Called on client's side to make the gun fire
// Check here to throw out danger markers to let people know the gun has gone
// off.
///////////////////////////////////////////////////////////////////////////////
simulated function LocalFire()
{
	local P2Player P;
	
	bPointing = true;

	// Same as the one in P2Weapon, but we don't do the camera shake here
	// We make it shake when he throws

	if ( Affector != None )
		Affector.FireEffect();
	PlayFiring();

        if(Role < ROLE_Authority)
		GotoState('ClientFiring');
		
		
}
	
///////////////////////////////////////////////////////////////////////////////
// Same as above.. we don't want the shake here
///////////////////////////////////////////////////////////////////////////////
simulated function LocalAltFire()
{
	Zoom();
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
function ServerAltFire()
{
	Zoom();
}

///////////////////////////////////////////////////////////////////////////////
// Is zooming
///////////////////////////////////////////////////////////////////////////////
simulated function bool IsZoomed()
{
	return bZoomed;
}

///////////////////////////////////////////////////////////////////////////////
// Start the zoom
///////////////////////////////////////////////////////////////////////////////
simulated function Zoom()
{
	if(AmmoType.HasAmmo()
		&& Instigator.Physics == PHYS_Walking
		&& (P2Player(Instigator.Controller) == None
			|| P2Player(Instigator.Controller).IsInState('PlayerWalking')))
	{
		if ( !bZoomed )
		{
			//PlayAnim
			GotoState('ZoomingIn');

		}
		else
		{
			GotoState('ZoomingOut');
		}
	}
}

///////////////////////////////////////////////////////////////////////////////
// Do the work in the FOV to change the zoom mode.
///////////////////////////////////////////////////////////////////////////////
simulated function EnterZoomMode()
{
	bShowHints=false;
	if(bShowHint1)
		bShowHint1=false;


	//dopamine
        PlayAnim('Shoot2_auto2semi');
	bBlockZoomIn=true;
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
simulated function ExitZoomMode()
{
	if(bZoomed)
	{
		bBlockZoomIn=true;
		bZoomed = false;
	}
}

///////////////////////////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////////////////////////
// Skin changing START Silent-Scope      BOOOOYA
///////////////////////////////////////////////////////////////////////////////


         function Notify_skinflash_start()
         {
 //                 Skins[0]=Texture'ED_WeaponSkins.SMG.MP5a22'     ;
 //                 Skins[1]=Texture'ED_WeaponSkins.Misc.dude_arms2'       ;

          AmbientGlow=400;
}




///////////////////////////////////////////////////////////////////////////////
// Skin Changing END Silent-Scope  BOOOYA
///////////////////////////////////////////////////////////////////////////////


          function Notify_skinflash_end()
         {
//                  Skins[0]=Texture'ED_WeaponSkins.SMG.MP5a2'    ;
//                  Skins[1]=Texture'ED_WeaponSkins.Misc.dude_arms'       ;

      AmbientGlow=0;

}






///////////////////////////////////////////////////////////////////////////////


simulated function FinishZoomEntry()
{
	bBlockZoomIn=false;
	bZoomed=true;
	// Change over to second hint now
	bShowHints=true;
	UpdateHudHints();

	// Give it better accuracy when zoomed
	TraceAccuracy = ZoomedTraceAccuracy;
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
simulated function FinishZoomExit()
{
	// Reset the accuracy
	// Bots with a sniper rifle use a higher accuracy
	if(FPSPawn(Instigator) != None
		&& !FPSPawn(Instigator).bPlayer)
		TraceAccuracy = BotTraceAccuracy;
	else
		TraceAccuracy = default.TraceAccuracy;
	bShowHints=false;
	UpdateHudHints();
}


///////////////////////////////////////////////////////////////////////////////
// Give hints about this item
///////////////////////////////////////////////////////////////////////////////
function bool GetHints(out String str1, out String str2, out String str3,
				out byte InfiniteHintTime)
{
	if(bShowHints
		&& bAllowHints)
	{
		if(bShowHint1)
			str1=HudHint1;
		else
			str2=HudHint2;
		return true;
	}
	return false;
}

///////////////////////////////////////////////////////////////////////////////
// Make sure you're not zoomed
///////////////////////////////////////////////////////////////////////////////
simulated function bool ForceEndFire()
	{
	log(self$" sniper force end fire ");
	if(PlayerController(Instigator.Controller) != None)
		PlayerController(Instigator.Controller).ResetFOV();
	ExitZoomMode();
	FinishZoomExit();
	return Super.ForceEndFire();
	}

///////////////////////////////////////////////////////////////////////////////
// Send the rifle into a mode of fading the screen
///////////////////////////////////////////////////////////////////////////////
simulated function HandleZoomChange(float StartAlpha, float TimeSpent)
{
	FullScreenAlpha = StartAlpha;
	FadeSpeed = FullScreenAlpha/TimeSpent;
}

///////////////////////////////////////////////////////////////////////////////
// Next weapon like normal, unless you're in zoom mode then we don't want the
// player to use this
///////////////////////////////////////////////////////////////////////////////
simulated function bool AllowNextWeapon()
{
	//if(IsZoomed())
        //        PlayAnim('Selector_burst2semi', 1.0, 0.0);

        return Super.AllowNextWeapon();

//return true;
}

///////////////////////////////////////////////////////////////////////////////
// Prev weapon like normal, unless you're in zoom mode then we don't want the
// player to use this
///////////////////////////////////////////////////////////////////////////////
simulated function bool AllowPrevWeapon()
{
          //if(IsZoomed())
          //      PlayAnim('Selector_burst2semi', 1.0, 0.0);

          return Super.AllowNextWeapon();

}

///////////////////////////////////////////////////////////////////////////////
// Zooms scope in
///////////////////////////////////////////////////////////////////////////////
simulated function ZoomIn()
{
}

///////////////////////////////////////////////////////////////////////////////
// Zooms scope out
///////////////////////////////////////////////////////////////////////////////
simulated function ZoomOut()
{
}

//Function for each shot
function Notify_Fire()
{
   ProjectileFire();
   SetupMuzzleFlash();
   Instigator.PlaySound(FireSound, SLOT_Misc, 1.0);
   AmbientGlow=400;
}
///////////////////////////////////////////////////////////////////////////////
// Normal projectile fire, plus check where to make the danger marker to
// alert people of the noise.
///////////////////////////////////////////////////////////////////////////////
function ProjectileFire()
{

	local vector markerpos, EndTrace;
	local vector StartTrace, X,Y,Z;



	// Moves like a projectile, but set up like a trace
	Owner.MakeNoise(1.0);
	GetAxes(Instigator.GetViewRotation(),X,Y,Z);
	StartTrace = GetFireStart(X,Y,Z);
	AdjustedAim = Instigator.AdjustAim(AmmoType, StartTrace, 2*AimError);
	EndTrace = StartTrace + (TraceAccuracy * (FRand() - 0.5 ) ) * Y * 1000
		+ (TraceAccuracy * (FRand() - 0.5 )) * Z * 1000;
	X = vector(AdjustedAim);
	EndTrace += (TraceDist * X);

        // Make it
	EA556mmAmmunation(AmmoType).SpawnEA556mmProjectile(StartTrace,Normal(EndTrace - StartTrace));

	// Only make a new danger marker if the consecutive fires were as high
	// as the max
	if(ShotCount >= ShotCountMaxForNotify
		&& Instigator.Controller != None
		&& ShotMarkerMade != None)
	{
		// tell it we know this just happened, by recording it.
		ShotCount -= ShotCountMaxForNotify;

		// Records the first (gun fire)
		markerpos = Instigator.Location;

		// Primary (the gun shooting, making a loud noise)
		if(ShotMarkerMade != None)
		{
			ShotMarkerMade.static.NotifyControllersStatic(
				Level,
				ShotMarkerMade,
				FPSPawn(Instigator),
				None,
				ShotMarkerMade.default.CollisionRadius,
				markerpos);
		}
	}
	



}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
function DropFrom(vector StartLocation)
{
	if(bZoomed)
	{
		ExitZoomMode();
		FinishZoomExit();
	}
	Super.DropFrom(StartLocation);

	Super.DropFrom(StartLocation);

             M4.Destroyed();
             M4M.Destroyed();
             M4S.Destroyed();
             M4T.Destroyed();

}

simulated function Destroyed()
{
   Super.Destroyed();

  if( M4 != None )
  {
    DetachFromBone(M4);
    M4.Destroy();
    M4 = None;
  }

  if( M4M != None )
  {
    DetachFromBone(M4M);
    M4M.Destroy();
    M4M = None;
  }

  if( M4S != None )
  {
    DetachFromBone(M4S);
    M4S.Destroy();
    M4S = None;
  }

  if( M4T != None )
  {
    DetachFromBone(M4T);
    M4T.Destroy();
    M4T = None;
  }
}

///////////////////////////////////////////////////////////////////////////////
// When necessary, handle the full screen alpha change
///////////////////////////////////////////////////////////////////////////////
simulated function FadeTick(float DeltaTime)
{
	if (Level.NetMode != NM_DedicatedServer )
	{
		if(FullScreenAlpha != 0)
		{
			FullScreenAlpha -= DeltaTime*FadeSpeed;

			if(FullScreenAlpha < 0)
			{
				FullScreenAlpha = 0;
			}
		}
	}
}

simulated function bool PutDown()
{
	bChangeWeapon = true;
	ExitZoomMode();
	FinishZoomExit();
	return true;
}

///////////////////////////////////////////////////////////////////////////////
// Make glare so he'll be easier to spot in MP games
///////////////////////////////////////////////////////////////////////////////
function MakeGlare()
{
}




simulated function PlayReloading()
	{
         P2MocapPawn(Instigator).PlayWeaponReload(self);
	if (!bZoomed)
        	PlayAnim('Reload_Semi', WeaponSpeedReload, 0.05);
        else
        	PlayAnim('Reload_Auto', WeaponSpeedReload, 0.05);


	Instigator.PlaySound(ReloadSound, SLOT_Misc, 1.0);
	}



///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// Zooming
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
state Zooming
{
	ignores ForceReload, NextWeapon, PrevWeapon,
		ZoomIn, ZoomOut, WeaponChange;

	simulated function Fire(float F)
	{
		if ( AmmoType == None
			|| !AmmoType.HasAmmo() )
		{
			ClientForceFinish();
			ServerForceFinish();
			return;
		}
	}

	simulated function AltFire(float F)
	{
		Fire(F);
	}

	simulated function bool IsZoomed()
	{
		return false;
	}

	simulated function bool PutDown()
	{
		bChangeWeapon = true;
		ExitZoomMode();
		FinishZoomExit();
		return True;
	}
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// ZoomingIn/Out
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
state ZoomingIn extends Zooming
{
	simulated function AnimEnd(int Channel)
	{
		FinishZoomEntry();
		GotoState('Idle');
	}

	simulated function BeginState()
	{
		// START zoom mode
		EnterZoomMode();
		PlayAnim('Shoot2_semi2auto');
		Instigator.PlaySound(SelectSound, SLOT_Misc, 1.0);
	}
}

state ZoomingOut extends Zooming
{
	simulated function AnimEnd(int Channel)
	{
		FinishZoomExit();
		if(Level.NetMode == NM_Client)
			PlayIdleAnim();
		GotoState('Idle');
	}

	simulated function BeginState()
	{
		// END zoom mode
		ExitZoomMode();
		//dopamine
                PlayAnim('Shoot2_Auto2semi');
                Instigator.PlaySound(SelectSound, SLOT_Misc, 1.0);
	}
}





///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// Idle
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
state Idle
{
	ignores ClientIdleCheckFire;

	///////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////
	simulated function bool PutDown()
	{
		GotoState('DownWeapon');
		return True;
	}

	///////////////////////////////////////////////////////////////////////////////
	// Don't let them zoom in/out till this is called
	///////////////////////////////////////////////////////////////////////////////
	simulated function Timer()
	{
		bBlockZoomIn=false;
	}

	///////////////////////////////////////////////////////////////////////////////
	// When necessary, handle the full screen alpha change
	///////////////////////////////////////////////////////////////////////////////
	simulated function Tick(float DeltaTime)
	{
		local vector HitLocation, HitNormal, StartTrace, EndTrace, X,Y,Z;
		local Actor TestTarget;

		FadeTick(DeltaTime);

		// If it's the client, or a normal, single player game
		if (P2GameInfoSingle(Level.Game) == None)
		{
			// The client can't check a trace/collision, only the server
			if(Level.NetMode != NM_Client)
			{
				// If you're zoomed and in an MP game, trace forward and see if you'll
				// hit a pawn or not. Then we can alert the person playing and tell them
				// they're in range to get shot in the head.
				if(bZoomed
					&& PlayerController(Instigator.Controller) != None)
				{
					GetAxes(Instigator.GetViewRotation(),X,Y,Z);
					StartTrace = Instigator.Location + Instigator.EyePosition();
					EndTrace = StartTrace + TestTraceDist*X;
					TestTarget = Trace(HitLocation, HitNormal, EndTrace, StartTrace, true);

					// Undo view from last target, because we have a new/none target
					if(MyTarget != None
						&& TestTarget != MyTarget)
					{
						MyTarget.ClientSniperViewingMe(P2Pawn(Owner), false);
						MyTarget = None;
					}

					// Keep track of how long we've have this target.
					if(MyTarget != None)
						TargetTime+=DeltaTime;

					// Tell our new target he's in our sites!
					if(TestTarget != MyTarget
						&& P2Pawn(TestTarget) != None
						&& PlayerController(Pawn(TestTarget).Controller) != None)
					{
						MyTarget = P2Pawn(TestTarget);
						MyTarget.ClientSniperViewingMe(P2Pawn(Owner), true);
						TargetTime=DeltaTime;
					}
					else if(MyTarget != None
						&& TargetTime > TARGET_UPDATE_TIME)
						// We've got the same old target, just update him
					{
						TargetTime =0;
						MyTarget.ClientSniperViewingMe(P2Pawn(Owner), true);
					}
				}
			}
		}
	}

	///////////////////////////////////////////////////////////////////////////////
	// If we're zoomed, make sure to update the new zone's zone fog, and change
	// the old one's before you leave.
	///////////////////////////////////////////////////////////////////////////////
	event ZoneChange( ZoneInfo NewZone )
	{
		if(IsZoomed())
		{
			if(P2GameInfoSingle(Level.Game) != None)
			{
				// Reset old zone's fog
				P2GameInfo(Level.Game).SetGeneralZoneFog(Owner);
				// Set new one
				P2GameInfo(Level.Game).SetNewSniperZoneFog(NewZone);
			}
		}
	}

	///////////////////////////////////////////////////////////////////////////////
	// If it's the dude and he farted, apologize
	///////////////////////////////////////////////////////////////////////////////
	function float DudeCommentsOnFarting()
	{
		if(P2Player(Instigator.Controller) != None)
			return P2Player(Instigator.Controller).CommentOnFarting();
		return 0.0;
	}


	///////////////////////////////////////////////////////////////////////////////
	// Reset shot count if you're not still firing
	///////////////////////////////////////////////////////////////////////////////
	simulated function BeginState()
	{
		// Always reset the count with each shot
		ShotCountReset();
		bBlockZoomIn=true;
		SetTimer(WAIT_IDLE_TIME, false);
		// Make sure to make a glare right off the bat
		if(IsZoomed())
		{
			MakeGlare();
		}
	}

	simulated function EndState()
	{
		bBlockZoomIn=false;
		// Undo view from last target because we're leaving this state
		if(MyTarget != None)
		{
			MyTarget.ClientSniperViewingMe(P2Pawn(Owner), false);
			MyTarget = None;
			TargetTime =0;
		}

		//Instigator.PlaySound(SniperBreathing,SLOT_Interface,0.01);
		Super.EndState();
	}
Begin:
	bPointing=False;
	if ( NeedsToReload() && AmmoType.HasAmmo() )
		GotoState('Reloading');
	if ( !AmmoType.HasAmmo() ) 
		Instigator.Controller.SwitchToBestWeapon();  //Goto Weapon that has Ammo
	if ( Instigator.PressingFire() ) Fire(0.0);
	//if ( Instigator.PressingAltFire() ) AltFire(0.0);	
	PlayIdleAnim();
	Sleep(WAIT_IDLE_TIME);	// Don't let them switch during this time

}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// NormalFire
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
state NormalFire
{

	///////////////////////////////////////////////////////////////////////////////
	// When necessary, handle the full screen alpha change
	///////////////////////////////////////////////////////////////////////////////
	simulated function Tick(float DeltaTime)
	{
		FadeTick(DeltaTime);
	}
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// ClientFiring
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
state ClientFiring
{
	///////////////////////////////////////////////////////////////////////////////
	// When necessary, handle the full screen alpha change
	///////////////////////////////////////////////////////////////////////////////
	simulated function Tick(float DeltaTime)
	{
		FadeTick(DeltaTime);
	}
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// Downweapon
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
state DownWeapon
{
	ignores Zoom;

	simulated function BeginState()
	{
		Super.BeginState();
		ExitZoomMode();
		FinishZoomExit();
	}
}




  










/////////////////////////////////
///////////////////////////
////////////////////////DOPAMINE

simulated function PrepStreaming()
{
	// play our intro stream anim
	PlayStreamStart();
	if(!bDelayedStartSound)
		Instigator.PlaySound(soundStart, SLOT_None, 1.0, false, 1000.0, 1.0);
}

simulated function PlayStreamStart()
	{

        if (bZoomed)
                PlayAnim('Shoot1_autoPrep', WeaponSpeedPrep, 0.0);

	}
simulated function PlayStreamEnd()
	{

        if (bZoomed)
             PlayAnim('Shoot1_autoEnd', WeaponSpeedEnd, 0.0);

	}








state StartStream
{
	ignores Fire, ServerFire, AltFire;

	///////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////
	simulated function AnimEnd(int Channel)
	{
		//log(self$" going to streaming");
		GotoState('Streaming');
	}

	///////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////
	simulated function BeginState()
	{
	

		//log(self$" startstream");
	}
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// Streaming
// 'Fire' this weapon in a looping fashion, till it's done
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
state Streaming
{
	ignores PrevWeapon, NextWeapon, WeaponChange, Fire, ServerFire, AltFire;
//	ignores PrevWeapon, NextWeapon, WeaponChange;

	///////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////
	simulated function bool PutDown()
	{
		bChangeWeapon = true;
		return True;
	}

	///////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////
	simulated function AnimEnd(int Channel)
	{
		// If it's the guy playing that hosts the listen server
		// or it's a typical client/stand alone game.
		if(NotDedOnServer())
		{
			if(Instigator.PressingFire()
				&& AmmoType.HasAmmo())
			{



                                PlayFiring();

                                //dopamine - Not only play the animation, but fire a fucking Bullet for Gods sake
                                ProjectileFire();

                                //dopamine, have to call this for the semi fire to count reloads
                                Serverfire();
                                


                                reloadcount--;

                                if (reloadcount <= 0)
                                {
				     EndStreaming();
				     ServerEndStreaming();
			        }

			}
			else
			{
				EndStreaming();
				ServerEndStreaming();
			}
		}
	}

	///////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////
	simulated function EndState()
	{
		Instigator.AmbientSound = None;
		//log(self$" end streaming");
	}

	///////////////////////////////////////////////////////////////////////////////
	// Set to seeking when you get to this state
	///////////////////////////////////////////////////////////////////////////////
	simulated function BeginState()
	{
		PlayFiring();
		
		                                //dopamine - Not only play the animation, but fire a fucking bullet for Gods sake
                                ProjectileFire();

                                //dopamine, have to call this for the semi fire to count reloads
                                Serverfire();
                                
                                                                reloadcount--;

                                if (reloadcount <= 0)
                                {
				     EndStreaming();
				     ServerEndStreaming();
			        }
		

	}
Begin:
	if(!bDelayedStartSound)
		Instigator.AmbientSound = soundLoop1;
	else
	{
		Instigator.PlaySound(soundStart, SLOT_None, 1.0, false, 1000.0, 1.0);
		Sleep(Instigator.GetSoundDuration(soundStart));
		Instigator.AmbientSound = soundLoop1;
	}
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// EndStream
//////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
state EndStream
{
	ignores Fire;

	///////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////
	simulated function AnimEnd(int Channel)
	{
		GotoState('Idle');
	}

	///////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////
	simulated function BeginState()
	{
		// play our intro stream anim
		PlayStreamEnd();
		Instigator.PlaySound(soundEnd, SLOT_None, 1.0, false, 1000.0, 1.0);
	}
}










///////////////////////////////////////////////////////////////////////////////
// Default properties
///////////////////////////////////////////////////////////////////////////////

defaultproperties
{
     SniperReticleCorner=none
     SniperReticleCenter=none
     BlackBackground=none
     ZoomedTraceAccuracy=2.000000
     BotTraceAccuracy=0.500000
     SniperBreathing=none
     SniperChangeZoomSound=none
     SniperFarting(0)=none
     SniperFarting(1)=none
     bShowHint1=True
     TestTraceDist=10.000000
     SelectSound=Sound'MiscSounds.Menu.MenuClick'
     WeaponSpeedPrep=1.500000
     WeaponSpeedEnd=1.500000
     bUsesAltFire=True
     ViolenceRank=3
     RecognitionDist=1100.000000
     ShotMarkerMade=Class'Postal2Game.GunfireMarker'
     BulletHitMarkerMade=Class'Postal2Game.BulletHitMarker'
     holdstyle=WEAPONHOLDSTYLE_Both
     switchstyle=WEAPONHOLDSTYLE_Both
     firingstyle=WEAPONHOLDSTYLE_Both
     MinRange=100.000000
     ShakeOffsetTime=2.500000
     bAllowHints=True
     bShowHints=True
     HudHint1="Secondary Fire(Right Mouse Button) Selects Semi-auto or Full-auto"
     HudHint2="Secondary Fire(Right Mouse Button) Selects Semi-auto or Full-auto"
     CombatRating=2.000000
     FirstPersonMeshSuffix="ED_MP5A2_NEW"
     WeaponsPackageStr="EA_Weapons."
     WeaponSpeedLoad=0.700000
     WeaponSpeedHolster=0.900000
     WeaponSpeedShoot1=0.800000
     WeaponSpeedShoot1Rand=0.150000
     WeaponSpeedShoot2=55.00000
     AltFireSound=Sound'EAWeaponsSounds.WeaponsFire.M4A1_Fire'
     AmmoName=Class'EAInventory.M4A1AmmoInv'
     ReloadCount=30
     AutoSwitchPriority=9
     ShakeRotMag=(X=400.000000)
     ShakeRotTime=2.500000
     ShakeOffsetMag=(X=40.000000,Y=4.000000,Z=4.000000)
     ShakeOffsetRate=(X=800.000000,Y=800.000000,Z=800.000000)
     TraceAccuracy=0.100000
     aimerror=0.900000
     AIRating=0.800000
     MaxRange=100.000000
     FireSound=Sound'EAWeaponsSounds.WeaponsFire.M4A1_Fire'
     FlashOffsetY=-0.090000
     FlashOffsetX=0.095000
     MuzzleFlashSize=96.000000
     MFTexture=Texture'Timb.muzzle_flash.machine_gun_corona'
     MuzzleFlashScale=2.400000
     MuzzleFlashStyle=STY_Normal
     InventoryGroup=4
     GroupOffset=22
     PickupClass=Class'EAInventory.M4A1Pickup'
     PlayerViewOffset=(X=1.000000,Y=0.300000,Z=-1.300000)
     BobDamping=0.975000
     AttachmentClass=Class'EAInventory.M4A1Attachment'
     ItemName="Colt M4A1"
     Mesh=SkeletalMesh'EA_Weapons.ED_MP5A2_NEW'
     Skins(0)=Shader'MaD_ED_Skins.AttachTex.FakeShad'
     Skins(1)=Texture'ED_WeaponSkins.Pistol.beretta'
     Skins(2)=Texture'MP_FPArms.LS_arms.LS_hands_dude'
     Texture=Texture'EAWeaponsSkin.HUD.Icon_M4A1'
     AmbientGlow=0
     SoundRadius=255.000000
}
