//=============================================================================
// ShotGunWeapon
// Copyright 2001 Running With Scissors, Inc.  All Rights Reserved.
//
// Shotgun weapon (first and third person).
// This can have a cat put on the end on the of the weapon.
//
// For AI characters, this weapon scales the number of times it's shot continuously
// with the difficulty level. Higher difficulty--more they shoot in one burst.
//
//	History:
//		01/29/02 MJR	Started history, probably won't be updated again until
//							the pace of change slows down.
//
//=============================================================================

class M870Weapon extends M870ReloaderWeapon;

///////////////////////////////////////////////////////////////////////////////
// Vars, structs, consts
///////////////////////////////////////////////////////////////////////////////

var float SPAccuracy;				// How accurate we are in single player games.

//const BASE_FLASH_OFFSET_X = 0.06;
//const BASE_FLASH_OFFSET_Y = 0.015;
const BASE_FLASH_OFFSET_X = 0.055;
const BASE_FLASH_OFFSET_Y = 0.01;
const RAND_OFFSET = 0.01;

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
function PostBeginPlay()
{
	local float diffoffset;

	Super.PostBeginPlay();

	// If it's a single player game, make our accuracy much worse
	if(Level.Game != None
		&& FPSGameInfo(Level.Game).bIsSinglePlayer)
	{
		TraceAccuracy = SPAccuracy;

		// Based on the difficulty, bump up the number of times AI will continuously
		// shoot this weapon
		diffoffset = P2GameInfo(Level.Game).GetDifficultyOffset();
		if(diffoffset > 0)
		{
			AI_BurstCountExtra+=(diffoffset/2);
		}
	}
}

///////////////////////////////////////////////////////////////////////////////
// This will randomly change the color and the size of the dynamic
// light associate with the flash. Change in each weapon's file,
// but call each time you start up the flash again.
// This function is also used by the third-person muzzle flash, so the
// colors will look the same
///////////////////////////////////////////////////////////////////////////////
simulated function PickLightValues()
{
	LightBrightness=150;
	LightSaturation=150;
	LightHue=12+FRand()*15;
	LightRadius=12+FRand()*6;
}

///////////////////////////////////////////////////////////////////////////////
// Make sure the muzzle flash renders, and move it around some
///////////////////////////////////////////////////////////////////////////////
simulated function SetupMuzzleFlash()
{
	bMuzzleFlash = true;
	bSetFlashTime = false;
	// Gets turned off in weapon, in RenderOverlays
	// Slightly change colors each time
	if(IsFirstPersonView()
		&& bDrawMuzzleFlash)
	{
		PickLightValues();
		bDynamicLight=bAllowDynamicLights;
	}
}

///////////////////////////////////////////////////////////////////////////////
// DrawMuzzleFlash()
//Default implementation assumes that flash texture can be drawn inverted in X and/or Y direction to increase apparent
//number of flash variations
// We might need to draw a dark blood splat
///////////////////////////////////////////////////////////////////////////////
simulated function DrawMuzzleFlash(Canvas Canvas)
{
	local float Scale, ULength, VLength, UStart, VStart;

//	FlashOffsetX = BASE_FLASH_OFFSET_X + FRand()*RAND_OFFSET;
//	FlashOffsetY = BASE_FLASH_OFFSET_Y + FRand()*RAND_OFFSET;

	if(CatOnGun == 0
		|| !class'P2Player'.static.BloodMode())
		Super.DrawMuzzleFlash(Canvas);
	else	// if a cat, then draw it alpha blended with a gross blood splat
	{
		Scale = MuzzleScale * Canvas.ClipX/640.0;
		Canvas.Style = ERenderStyle.STY_Alpha;
		ULength = MFTexture.USize;
		if ( FRand() <= 0.5 )
		{
			UStart = ULength;
			ULength = -1 * ULength;
		}
		VLength = MFTexture.VSize;
		if ( FRand() <= 0.5 )
		{
			VStart = VLength;
			VLength = -1 * VLength;
		}

		Canvas.DrawTile(MFTexture, Scale * MFTexture.USize, Scale * MFTexture.VSize, 
					UStart, VStart, ULength, VLength);
		Canvas.Style = ERenderStyle.STY_Normal;
	}
}

///////////////////////////////////////////////////////////////////////////////
// See what we hit
///////////////////////////////////////////////////////////////////////////////
function TraceFire( float Accuracy, float YOffset, float ZOffset )
{
	local int i;
	
	// Reduce the ammo only by 1 here, for the shotgun, but shoot
	// ShotCountMaxForNotify number of pellets each time.
	P2AmmoInv(AmmoType).UseAmmoForShot();

	// Reduce the cat ammo if we're using one
	if(CatOnGun == 1)
		CatAmmoLeft--;

	for(i=0; i<ShotCountMaxForNotify; i++)
	{
		Super.TraceFire(Accuracy, YOffset, ZOffset);
	}
}
state ClientFiring
{
}
state NormalFire
{
}

//////////////////////Silent-Scope Cocking action add/////////////////////////////////////////////////////////
simulated function Cock()
{
	PlayAnim('Cock', WeaponSpeedReload, 0.05);
	
	P2MocapPawn(Instigator).PlayWeaponSwitch(self);
}

///////////////////////////////////////////////////////////////////////////////
// Skin changing START Silent-Scope      BOOOOYA
///////////////////////////////////////////////////////////////////////////////


         function Notify_skinflash_start()
         {
                  AmbientGlow=400;
}

///////////////////////////////////////////////////////////////////////////////
// Skin Changing END Silent-Scope  BOOOYA
///////////////////////////////////////////////////////////////////////////////

          function Notify_skinflash_end()
         {
                 AmbientGlow=0;
}

///////////////////////////////////////////////////////////////////////////////
// Default properties
///////////////////////////////////////////////////////////////////////////////

defaultproperties
{
     SPAccuracy=1.400000
     CatMesh=SkeletalMesh'EA_ClassicWeapons.FP_Dude_ShotgunCat'
     MFBloodTexture=Texture'nathans.muzzleflashes.bloodmuzzleflash'
     StartShotsWithCat=3
     CatFireSound=Sound'WeaponSounds.shotgun_catfire'
     CatSkinIndex=2
     ViolenceRank=6
     RecognitionDist=1100.000000
     ShotCountMaxForNotify=4
     AI_BurstCountMin=4
     holdstyle=WEAPONHOLDSTYLE_Double
     switchstyle=WEAPONHOLDSTYLE_Double
     firingstyle=WEAPONHOLDSTYLE_Double
     MinRange=200.000000
     ShakeOffsetTime=2.500000
     CombatRating=4.000000
     FirstPersonMeshSuffix="FP_Dude_Shotgun"
     WeaponsPackageStr="EA_ClassicWeapons."
     WeaponSpeedLoad=1.000000
     WeaponSpeedHolster=1.000000
     WeaponSpeedShoot1=0.900000
     WeaponSpeedShoot1Rand=0.300000
     WeaponSpeedIdle=0.800000
     AmmoName=Class'EAInventory.Gauge12Ammunition'
     ReloadCount=8
     AutoSwitchPriority=6
     ShakeRotMag=(X=400.000000)
     ShakeRotTime=2.500000
     ShakeOffsetMag=(X=30.000000,Y=4.000000,Z=4.000000)
     TraceAccuracy=1.500000
     aimerror=400.000000
     AIRating=0.300000
     MaxRange=512.000000
     FireSound=Sound'WeaponSounds.shotgun_fire'
     MessageNoAmmo="12Gauges has no ammo."
     FlashOffsetY=-0.050000
     FlashOffsetX=0.060000
     FlashLength=0.050000
     MuzzleFlashSize=128.000000
     MFTexture=Texture'Timb.muzzle_flash.shotgun_corona'
     bDrawMuzzleFlash=True
     MuzzleFlashScale=2.400000
     MuzzleFlashStyle=STY_Normal
     InventoryGroup=3
     GroupOffset=1
     PickupClass=Class'EAInventory.M870Pickup'
     BobDamping=0.975000
     AttachmentClass=Class'Inventory.ShotgunAttachment'
     ItemName="Remington M870 FS"
     Mesh=SkeletalMesh'EA_ClassicWeapons.FP_Dude_Shotgun'
     Skins(0)=Texture'WeaponSkins.Dude_Hands'
     Skins(1)=Texture'WeaponSkins.shotgun_timb'
     Skins(2)=Texture'AnimalSkins.Cat_Orange'
     Skins(3)=Texture'AnimalSkins.Cat_Orange'
     SoundRadius=255.000000
     Texture=Texture'HUDPack.Icons.Icon_Weapon_Shotgun'
     AmbientGlow=0
}
