//=============================================================
//   Beretta Pickup
//   Eternal Damnation
//   Silent-Scope|Nial
//=============================================================

class P230Weapon extends BerettaReloaderWeapon;

var bool bCanFireAgain;		// If you're the player and this is set to true, then you
							// shoot again if even you're still in NormalFire

var float WeaponSpeedShoot1MP;	// Allow faster shooting in multiplayer.
var float SPAccuracy;				// How accurate we are in single player games.

var P230 P2;
var P230Magazine P2M;
var P230Barrel P2B;
var P230Trigger P2T;
var P230Hammer P2H;

const FIRING_WAIT_TIME = 0.4;


///////////////////////////////////////////////////////////////////////////////
simulated function PostBeginPlay()
{
	local float diffoffset;

	Super.PostBeginPlay();

	// If it's a single player game, make our accuracy much worse
	if(Level.Game != None
		&& FPSGameInfo(Level.Game).bIsSinglePlayer)
	{
		TraceAccuracy = SPAccuracy;

		// Based on the difficulty, bump up the number of times AI will continuously
		// shoot this weapon
		diffoffset = P2GameInfo(Level.Game).GetDifficultyOffset();
		if(diffoffset > 0)
		{
			AI_BurstCountExtra+=(diffoffset/1);
		}
	}
}

simulated function PostNetBeginPlay()
{
    local PlayerController PC;
	local Pawn P;

    Super.PostNetBeginPlay();

    P  = Pawn(Owner);
	PC = PlayerController(P.Controller);

   	if (P.IsLocallyControlled() && PC!=None && (!PC.bBehindView) )
	{
	P2 = Spawn(Class'EAInventory.P230');
        	AttachToBone( P2, 'P230' );
        	P2M = Spawn(Class'EAInventory.P230Magazine');
        	AttachToBone( P2M, 'P230_Magazine' );
	P2B = Spawn(Class'EAInventory.P230Barrel');
        	AttachToBone( P2B, 'P230_Barrel' );
	P2H = Spawn(Class'EAInventory.P230Hammer');
        	AttachToBone( P2H, 'P230_Hammer' );
	P2T = Spawn(Class'EAInventory.P230Trigger');
        	AttachToBone( P2T, 'P230_Trigger' );
    }
   	else
	{
        if(Instigator != None)
	   {
		 if(Instigator.Mesh != None)

             P2.Destroyed();
             P2 = None;
             P2M.Destroyed();
             P2M = None;
             P2B.Destroyed();
             P2B = None;
             P2H.Destroyed();
             P2H = None;
             P2T.Destroyed();
             P2T = None;
	   }
	}

}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
function DropFrom(vector StartLocation)
{

   Super.DropFrom(StartLocation);


             P2.Destroyed();
             P2M.Destroyed();
             P2B.Destroyed();
             P2H.Destroyed();
             P2T.Destroyed();

}

simulated function Destroyed()
{
   Super.Destroyed();

  if( P2 != None )
  {
    DetachFromBone(P2);
    P2.Destroy();
    P2 = None;
  }

  if( P2M != None )
  {
    DetachFromBone(P2M);
    P2M.Destroy();
    P2M = None;
  }

  if( P2B != None )
  {
    DetachFromBone(P2B);
    P2B.Destroy();
    P2B = None;
  }

  if( P2H != None )
  {
    DetachFromBone(P2H);
    P2H.Destroy();
    P2H = None;
  }

  if( P2T != None )
  {
    DetachFromBone(P2T);
    P2T.Destroy();
    P2T = None;
  }
}


///////////////////////////////////////////////////////////////////////////////
// Fire the weapon
///////////////////////////////////////////////////////////////////////////////
simulated function PlayFiring()
	{
	// Basics
	IncrementFlashCount();
	if(Level.Game == None
		|| !FPSGameInfo(Level.Game).bIsSinglePlayer)
	{
		// Play your sound in MP in your attachment to save bandwidth
		PlayAnim('Shoot1', WeaponSpeedShoot1MP + (WeaponSpeedShoot1Rand*FRand()), 0.05);
	}
	else
	{
		Instigator.PlaySound(FireSound, SLOT_None, 1.0, true, , WeaponFirePitchStart + (FRand()*WeaponFirePitchRand));
		PlayAnim('Shoot1', WeaponSpeedShoot1 + (WeaponSpeedShoot1Rand*FRand()), 0.05);
	}

	// Adv.
	SetupMuzzleFlash();
	//dopamine - Not only play the animation, but fire a fucking bullet for Gods sake
//        ProjectileFire();
	}

///////////////////////////////////////////////////////////////////////////////
// This will randomly change the color and the size of the dynamic
// light associate with the flash. Change in each weapon's file,
// but call each time you start up the flash again.
// This function is also used by the third-person muzzle flash, so the
// colors will look the same
///////////////////////////////////////////////////////////////////////////////
simulated function PickLightValues()
{
	LightBrightness=150;
	LightSaturation=150;
	LightHue=12+FRand()*15;
	LightRadius=12+FRand()*6;
}

///////////////////////////////////////////////////////////////////////////////
// Make sure the muzzle flash renders, and move it around some
///////////////////////////////////////////////////////////////////////////////
simulated function SetupMuzzleFlash()
{
	bMuzzleFlash = true;
	bSetFlashTime = false;
	// Gets turned off in weapon, in RenderOverlays
	// Slightly change colors each time
	if(IsFirstPersonView()
		&& bDrawMuzzleFlash)
	{
		PickLightValues();
		bDynamicLight=bAllowDynamicLights;
	}
}

///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Skin changing START Silent-Scope      BOOOOYA
///////////////////////////////////////////////////////////////////////////////


         function Notify_skinflash_start()
         {
                  AmbientGlow=400;
}




///////////////////////////////////////////////////////////////////////////////
// Skin Changing END Silent-Scope  BOOOYA
///////////////////////////////////////////////////////////////////////////////


          function Notify_skinflash_end()
         {
                 AmbientGlow=0;
}






///////////////////////////////////////////////////////////////////////////////




///////////////////////////////////////////////////////////////////////////////
// Normal projectile fire, plus check where to make the danger marker to
// alert people of the noise.
///////////////////////////////////////////////////////////////////////////////
function ProjectileFire()
{

	local vector markerpos, EndTrace;
	local vector StartTrace, X,Y,Z;



	// Moves like a projectile, but set up like a trace
	Owner.MakeNoise(1.0);
	GetAxes(Instigator.GetViewRotation(),X,Y,Z);
	StartTrace = GetFireStart(X,Y,Z);
//	AdjustedAim = Instigator.AdjustAim(AmmoType, StartTrace, 1*AimError);
	AdjustedAim = Instigator.AdjustAim(AmmoType, StartTrace, 1*AimError);
	EndTrace = StartTrace + (TraceAccuracy * (FRand() - 0.5 ) ) * Y * 1000
		+ (TraceAccuracy * (FRand() - 0.5 )) * Z * 1000;
	X = vector(AdjustedAim);
	EndTrace += (TraceDist * X);

        // Make it
	EA9mmAmmunation(AmmoType).SpawnEA9mmProjectile(StartTrace,Normal(EndTrace - StartTrace));

	// Only make a new danger marker if the consecutive fires were as high
	// as the max
	if(ShotCount >= ShotCountMaxForNotify
		&& Instigator.Controller != None
		&& ShotMarkerMade != None)
	{
		// tell it we know this just happened, by recording it.
		ShotCount -= ShotCountMaxForNotify;

		// Records the first (gun fire)
		markerpos = Instigator.Location;

		// Primary (the gun shooting, making a loud noise)
		if(ShotMarkerMade != None)
		{
			ShotMarkerMade.static.NotifyControllersStatic(
				Level,
				ShotMarkerMade,
				FPSPawn(Instigator),
				None,
				ShotMarkerMade.default.CollisionRadius,
				markerpos);
		}
	}
	



}

state ClientFiring
{
}

simulated function PlayReloading()
	{
        P2MocapPawn(Instigator).PlayWeaponReload(self);
        	
        PlayAnim('Reload', WeaponSpeedReload, 0.05);

	Instigator.PlaySound(ReloadSound, SLOT_Misc, 1.0);
	}

///////////////////////////////////////////////////////////////////////////////
// Default properties
///////////////////////////////////////////////////////////////////////////////

defaultproperties
{
     bCanFireAgain=True
     WeaponSpeedShoot1MP=1.000000
     SPAccuracy=0.010000
     ViolenceRank=4
     RecognitionDist=4000.000000
     AI_BurstCountExtra=3
     AI_BurstTime=0.800000
     ShotMarkerMade=Class'Postal2Game.GunfireMarker'
     BulletHitMarkerMade=Class'Postal2Game.BulletHitMarker'
     holdstyle=WEAPONHOLDSTYLE_Single
     switchstyle=WEAPONHOLDSTYLE_Single
     firingstyle=WEAPONHOLDSTYLE_Single
//     bNoHudReticle=True
     MinRange=5000.000000
     ShakeOffsetTime=2.200000
     CombatRating=3.000000
     FirstPersonMeshSuffix="EA_Beretta_NEW"
     WeaponsPackageStr="EA_Weapons."
     WeaponSpeedLoad=1.000000
     WeaponSpeedReload=1.000000
     WeaponSpeedShoot1=0.800000
     AmmoName=Class'EAInventory.P230AmmoInv'
     ReloadCount=7
     AutoSwitchPriority=6
     TraceAccuracy=0.010000
     aimerror=4.000000
     AIRating=0.200000
     MaxRange=2024.000000
     FireSound=Sound'EAWeaponsSounds.WeaponsFire.P230_Fire'
     FlashOffsetY=0.150000
     FlashOffsetX=0.215000
     FlashLength=0.010000
     MuzzleFlashSize=128.000000
     MFTexture=Texture'Timb.muzzle_flash.pistol_corona'
     MuzzleFlashScale=2.400000
     MuzzleFlashStyle=STY_Normal
     InventoryGroup=2
     GroupOffset=25
     PickupClass=Class'EAInventory.P230Pickup'
     PlayerViewOffset=(X=0.000000,Z=-1.300000)
     BobDamping=0.975000
     AttachmentClass=Class'EAInventory.P230Attachment'
     ItemName="Walther P230 (9mm)"
     Texture=Texture'EAWeaponsSkin.HUD.Icon_P230'
     Mesh=SkeletalMesh'EA_Weapons.EA_Beretta_NEW'
     Skins(0)=Shader'MaD_ED_Skins.AttachTex.FakeShad'
     Skins(1)=Shader'MaD_ED_Skins.AttachTex.FakeShad'
     Skins(2)=Texture'MP_FPArms.LS_arms.LS_hands_dude'
     AmbientGlow=0
     SoundRadius=255.000000
}
