///////////////////////////////////////////////////////////////////////////////
// ChainSaw Weapon
//
// Requires some special coding since we use gascan ammo.
///////////////////////////////////////////////////////////////////////////////
class EAChainsawWeapon extends P2WeaponStreaming;

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
var() sound IdleSound;								// sound made during idle
var() sound HitWallSound;
var() array<sound> BodySound;
var bool bForceReload;
var int ReloadCount;	       // Amount of ammo depletion before reloading. 0 if no reloading is done.
var() class<DamageType> DamageTypeInflicted;		// damage type
//var() class<DamageType> AltDamageTypeInflicted;
var() class<DamageType> BodyDamage;					// body damage
var() float DamageAmount;							// damage amount
var() float SeverMag;								// sever momentum
var() float MomentumHitMag;							// hit momentum

var float CurrentDelta;
var float MaxDelta;

var int BloodTextureIndex;			// index into following array
var int BloodTextureIndex2;			// index into following array
var array<Texture> BloodTextures;	// bloodier versions of this weapon skin
var array<TexPanner> BloodTextures2;	// bloodier versions of this weapon skin

var name Shoot1Anim;		// Based on where you first swing, it will decide from 2 downs, one left or right
							// and put the name in here to be used

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
const HEAD_OFFSET	=	10.0;
const PAWN_NECK		=	'MALE01 neck';

const SHOOTDOWN1	=	'Shoot1Down1';
const SHOOTDOWN2	=	'Shoot1Down2';
const SHOOTLEFT		=	'Shoot1Left';
const SHOOTRIGHT	=	'Shoot1Right';

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
function ProcessTraceHit(Actor Other, Vector HitLocation, Vector HitNormal, Vector X, Vector Y, Vector Z)
{
	local SmokeHitPuff smoke1;
	local DirtClodsMachineGun dirt1;
	local SparkHitMachineGun spark1;
	local Rotator NewRot;
	local vector Momentum;
	local byte BlockedHit;

//	log(self@"processtracehit other"@other@"hitloc"@hitlocation@"normal"@hitnormal@"x"@x@"y"@y@"z"@z);

	if ( Other == None )
		return;

	// Check if they're allowed to hit the person they did, if not, None out Other
	// so it's like a wall hit
	if(InvPerson(Other) != None)
	{
		// Instead of using hit location, ensure the block knows it comes from the
		// attacker originally, so use the weapon's owner
		InvPerson(Other).CheckBlockMelee(Owner.Location, BlockedHit);
		if(BlockedHit == 1)
			Other = None;	// don't let them hit Other!
	}

	if(Pawn(Other) == None
		&& PeoplePart(Other) == None)
	{
		// randomly orient the splat on the wall (rotate around the normal)
		NewRot = Rotator(-HitNormal);
		NewRot.Roll=(65536*FRand());

	        spawn(class'SliceSplatMaker',Owner, ,HitLocation, Rotator(HitNormal));

		smoke1 = Spawn(class'Fx.SmokeHitPuffMelee',Owner,,HitLocation, Rotator(HitNormal));
		if(FRand()<0.3)
		{
			dirt1 = Spawn(class'Fx.DirtClodsMachineGun',Owner,,HitLocation, Rotator(HitNormal));
		}
		if(FRand()<0.15)
		{
			spark1 = Spawn(class'Fx.SparkHitMachineGun',Owner,,HitLocation, Rotator(HitNormal));
		}
	}


	if ( (Other != self) && (Other != Owner) )
	{
		// Sever limbs first (sever people is picked in the machete weapon itself)
		if(PeoplePart(Other) != None)
		{
			Momentum = SeverMag*(-X + VRand()/2);
			if(Momentum.z<0)
				Momentum.z=-Momentum.z;
			Other.TakeDamage(DamageAmount, Pawn(Owner), HitLocation, Momentum, DamageTypeInflicted);
		}
		else
		{
			if(FPSPawn(Other) == None
				|| HurtingAttacker(FPSPawn(Other)))
			{
				Momentum = -MomentumHitMag*(Z/2);

				Other.TakeDamage(DamageAmount, Pawn(Owner), HitLocation, Momentum, BodyDamage);
			}
		}

		if(FPSPawn(Other) != None
			|| PeoplePart(Other) != None)
		{
			Instigator.PlayOwnedSound(BodySound[Rand(BodySound.Length)], SLOT_None, 1.0,,TransientSoundRadius,GetRandPitch());
		}
		else
		{
			if(smoke1 != None
				&& Level.Game != None
				&& FPSGameInfo(Level.Game).bIsSinglePlayer)
				smoke1.PlaySound(HitWallSound, SLOT_None, 1.0,,TransientSoundRadius,GetRandPitch());
			else
				Instigator.PlayOwnedSound(HitWallSound, SLOT_None, 1.0,,TransientSoundRadius,GetRandPitch());
		}
	}
}

///////////////////////////////////////////////////////////////////////////////
// Always notify controllers
///////////////////////////////////////////////////////////////////////////////
event Tick(float Delta)
{
	if(IsInState('Idle') || IsInState('Streaming') || IsInState('StartStream') || IsInState('EndStream'))
	{
		CurrentDelta += Delta;
		if (CurrentDelta >= MaxDelta)
		{
			CurrentDelta = CurrentDelta - MaxDelta;
			if (ShotMarkerMade != None)
				ShotMarkerMade.static.NotifyControllersStatic(
					Level,
					ShotMarkerMade,
					FPSPawn(Instigator),
					FPSPawn(Instigator),
					ShotMarkerMade.default.CollisionRadius,
					Instigator.Location);
		}
	}
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// Weapon is up and ready to fire, but not firing.
// extends original, to keep track of shot count
// Uses the same code below it's Begin:, except we use HasAmmoFinished for
// special weapons (like the shocker) that never want to switch, but need
// to recharge eventually.
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
state Idle
{
	simulated event BeginState()
	{
		Super.BeginState();
		Instigator.AmbientSound = IdleSound;
	}

	simulated event EndState()
	{
		Super.EndState();
		Instigator.AmbientSound = None;
	}
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// Streaming
// 'Fire' this weapon in a looping fashion, till it's done
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
state Streaming
{
	///////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////
//	function Notify_DoHit()
//	{
//		DoHit(TraceAccuracy, 0, 0);
//	}

	///////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////
	event Tick(float Delta)
	{
		Global.Tick(Delta);
		ReduceAmmo(Delta);
		DoHit(TraceAccuracy, 0, 0);
		ReloadCount--;
	}
}


///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
function DoHit( float Accuracy, float YOffset, float ZOffset )
{
	// Do a sphere test around where the weapon projects, and try to hurt stuff there
	local vector markerpos, markerpos2;
	local bool secondary;

	local vector HitLocation, HitNormal, StartTrace, EndTrace, StraightX, X,Y,Z, ExtendTrace;
	local actor FirstHit;
	local actor Victims;
	local FPSPawn PawnHit, CheckPawn;
	local float dist, UseTime, checkdist, fardist;
	local vector dir;
	local bool bDeliverDirectDamage;
	local bool bHitSomething, bHitSolid, bHitDoor;
	local byte SeverHit;

//	log(self@"dohit accuracy"@accuracy@"yoffset"@yoffset@"zoffset"@zoffset);

	Owner.MakeNoise(1.0);
	GetAxes(Instigator.GetViewRotation(),X,Y,Z);

//	log(self@"instigator viewrot"@Instigator.GetViewRotation()@"axes x"@X@"Y"@Y@"Z"@Z@"melee dist"@usemeleedist);

	StartTrace = GetFireStart(X,Y,Z);
	EndTrace = StartTrace + X*UseMeleeDist;
	ExtendTrace = EndTrace + X*UseMeleeDist;

//	log(self@"trace start"@StartTrace@"end trace"@EndTrace@"extend trace"@ExtendTrace);

	// This performs the collision but also records where it hit and records it
	FirstHit = Trace(LastHitLocation,HitNormal,ExtendTrace,StartTrace,true);
	if(FirstHit != None)
	{
		bDeliverDirectDamage=true;
		if(FirstHit.bStatic)
			bHitSolid=true;
		// If we hit a pawn, record that too
		else if(FPSPawn(FirstHit) != None)
			//&& FPSPawn(FirstHit).Health > 0)
			PawnHit = FPSPawn(FirstHit);
		// If we hit a door, don't let us hurt a person who may be on the FirstHit side
		// as we kick it open, if we are indeed kicking it open.
//		else if(DoorMover(FirstHit) != None
//			&& bStopAtDoor)
//			bHitDoor=true;

		// Handle hitting pawns later down the function
		if(PawnHit == None)
		{
			// Process the damage here
			ProcessTraceHit(FirstHit, LastHitLocation, HitNormal, X,Y,Z);
			bHitSomething=true;
		}
	}

	// If we didn't hit a door first, or a pawn, try to hit more
	if(!bHitDoor)
	{
		fardist = 2*UseMeleeDist;
		foreach VisibleCollidingActors( class 'Actor', Victims, UseMeleeDist, EndTrace )
		{
			if( (Victims != self)
				&& (!Victims.bStatic)
				&& (Victims.bCollideWorld)
				&& (Victims != Instigator)
				&& (Victims.Role == ROLE_Authority)
				&& (FirstHit != Victims))	// handle the straight forward hits below
			{
				// If it's friendly pawn, don't let this 'wide area collision check' hurt
				// them. Only let the above, direct check hurt them. If it's any other pawn
				// or anything else in general, hurt it. This is so it's harder to accidentally
				// hurt your friends with the wide bludgeoning attacks.
				if(FPSPawn(Victims) == None
					|| !FPSPawn(Victims).bPlayerIsFriend)
				{
					// Save the first thing we hit, so we can alert other people about the hit
					if(!bDeliverDirectDamage
						&& FirstHit == None)
					{
						FirstHit = Victims;
						LastHitLocation = Victims.Location;
					}

					if(FPSPawn(Victims) != None)
					// If we hit a pawn, record that too
					{
						checkdist = VSize(Victims.Location - EndTrace);
						if(checkdist < fardist
							&& FPSPawn(Victims) != None)
							//&& FPSPawn(Victims).Health > 0)
						{
							fardist = checkdist;
							CheckPawn = FPSPawn(Victims);
						}
					}
					else // if we hit anything else, say so
					{
						if(Victims.bStatic)
							bHitSolid=true;
						bHitSomething=true;
						// Check to deliver damage
						dir = Victims.Location - EndTrace;
						dist = FMax(1,VSize(dir));
						dir = dir/dist;

						ProcessTraceHit(Victims, 
										EndTrace,
										HitNormal, X,Y,Z);
					}
				}
			}
		}
	}
	if(CheckPawn != None)
		PawnHit = CheckPawn;

	// If we hit a pawn, don't say it was a solid hit, even if we hit other stuff
	if(PawnHit != None)
	{
		bHitSolid=false;
		bHitSomething=true;

		HitLocation = GetHitLocation(X, StartTrace, EndTrace, PawnHit, ExtendTrace, SeverHit);

		// Process the damage here
		if(SeverHit == 1
			&& PersonPawn(PawnHit) != None)
			ProcessSeverHit(PawnHit, HitLocation, HitNormal, X,Y,Z);
		else
			ProcessTraceHit(PawnHit, HitLocation, HitNormal, X,Y,Z);
	}

	// If we hit something, only then (not when we fire) do we shake the view
	if(bHitSomething)
	{
		if ( Instigator != None)
		{
			ClientMacheteShake(bHitSolid);
		}
	}
	// If we didn't hit anything, we may want to do something about it
	//  That is, we want to tell people we were swinging/kicking in mid-air
//	else
//		HitNothing();

	// Say we just fired
	ShotCount++;
	
	// Set your enemy as the one you attacked.
	if(P2Player(Instigator.Controller) != None
		&& PawnHit != None
		&& PawnHit.Health > 0)
	{
		P2Player(Instigator.Controller).Enemy = PawnHit;
	}

	// Only make a new danger marker if the consecutive fires were as high
	// as the max
	if(ShotCount >= ShotCountMaxForNotify
		&& Instigator.Controller != None)
		{
		// tell it we know this just happened, by recording it.
		ShotCount -= ShotCountMaxForNotify;
		
		// This is if a pawn is hit by a bullet (or hurt bad), so it's really scary
		if(PawnHit != None
			&& PawnHit.Health > 0
			&& PawnHitMarkerMade != None)
			{
			PawnHitMarkerMade.static.NotifyControllersStatic(
				Level,
				PawnHitMarkerMade,
				FPSPawn(Instigator),
				FPSPawn(Instigator), // We want the creator to be instigator also
				// because this is much more like a gunfire sort of thing, rather than a pawn hit by
				// a bullet
				PawnHitMarkerMade.default.CollisionRadius,
				PawnHit.Location);
			}
		}
}

///////////////////////////////////////////////////////////////////////////////
// Play reloading
///////////////////////////////////////////////////////////////////////////////
simulated function PlayReloading()
	{
        P2MocapPawn(Instigator).PlayWeaponSwitch(self);

        PlayAnim('load', WeaponSpeedReload, 0.05);

	Instigator.PlaySound(ReloadSound, SLOT_Misc, 1.0);
	}

/* Force reloading even though clip isn't empty.  Called by player controller exec function,
and implemented in idle state */
simulated function ForceReload();

function ServerForceReload()
{
	bForceReload = true;
}

simulated function ClientForceReload()
{
	bForceReload = true;
	GotoState('Reloading');
}

simulated function bool NeedsToReload()
{
	return ( bForceReload || (Default.ReloadCount > 0) && (ReloadCount == 0) );
}

state Reloading
{
	function ServerForceReload() {}
	function ClientForceReload() {}
	function Fire( float Value ) {}
	function AltFire( float Value ) {}

	function ServerFire()
	{
		bForceFire = true;
	}

	function ServerAltFire()
	{
		bForceAltFire = False;
	}

	simulated function bool PutDown()
	{
		bChangeWeapon = true;
		return True;
	}

	simulated function BeginState()
	{
		if ( !bForceReload )
		{
			if ( Role < ROLE_Authority )
				ServerForceReload();
			else
				ClientForceReload();
		}
		bForceReload = false;
		PlayReloading();
	}

	simulated function AnimEnd(int Channel)
	{
		ReloadCount = Default.ReloadCount;
		if ( Role < ROLE_Authority )
			ClientFinish();
		else
			Finish();
		CheckAnimating();
	}

	function CheckAnimating()
	{
		if ( !IsAnimating() )
			warn(self$" stuck in Reloading and not animating!");
	}
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

defaultproperties
{
     HitWallSound=Sound'ed_moresounds.Weapons.bladehitwall'
     BodySound(0)=Sound'Slawterhaus.BoneRip01'
     BodySound(1)=Sound'Slawterhaus.BoneRip02'
     BodySound(2)=Sound'Slawterhaus.BoneRip03'
     ReloadCount=40
     DamageTypeInflicted=Class'AWEffects.FlyingScytheDamage'
     BodyDamage=Class'AWEffects.FlyingScytheDamage'
     DamageAmount=30.000000
     SeverMag=40000.000000
     MomentumHitMag=50000.000000
     CurrentDelta=1.000000
     MaxDelta=1.000000
     BloodTextures(0)=Texture'ED_WeaponSkins.Melee.chainsawskin'
     BloodTextures(1)=Texture'ED_WeaponSkins.Melee.chainsawskin2'
     BloodTextures(2)=Texture'ED_WeaponSkins.Melee.chainsawskin3'
     BloodTextures(3)=Texture'ED_WeaponSkins.Melee.chainsawskin4'
     BloodTextures2(0)=TexPanner'ED_WeaponSkins.Melee.chainblur'
     BloodTextures2(1)=TexPanner'ED_WeaponSkins.Melee.chainblur2'
     BloodTextures2(2)=TexPanner'ED_WeaponSkins.Melee.chainblur3'
     soundStart=Sound'ed_moresounds.Chainsaw.cs_prepstart'
     soundLoop1=None
//     soundLoop1=Sound'ed_moresounds.Chainsaw.cs_shoot1'
     soundEnd=Sound'ed_moresounds.Chainsaw.cs_prepend'
     AmmoUseRate=1.000000
     bDelayedStartSound=True
     bUsesAltFire=False
     ViolenceRank=8
     RecognitionDist=700.000000
     HolsterSound=Sound'ed_moresounds.Chainsaw.cs_holster'
     ShotCountMaxForNotify=10
     ShotMarkerMade=Class'Postal2Game.GunfireMarker'
     holdstyle=WEAPONHOLDSTYLE_Pour
     switchstyle=WEAPONHOLDSTYLE_Carry
     firingstyle=WEAPONHOLDSTYLE_Pour
     bNoHudReticle=True
     PlayerMeleeDist=100.000000
     NPCMeleeDist=100.000000
     CombatRating=2.000000
     WeaponsPackageStr="ED_Weapons."
     WeaponSpeedLoad=0.850000
     WeaponSpeedHolster=0.550000
     WeaponSpeedShoot1Rand=0.010000
     WeaponSpeedShoot2Rand=0.100000
     AltFireSound=Sound'ed_moresounds.Chainsaw.cs_startmiddle_end'
     AmmoName=Class'Inventory.GasCanBulletAmmoInv'
     PickupAmmoCount=10
     TraceAccuracy=0.005000
     bMeleeWeapon=True
     aimerror=200.000000
     AIRating=0.110000
     MaxRange=95.000000
     InventoryGroup=5
     GroupOffset=1
     PickupClass=Class'EAInventory.EAChainsawPickup'
     AttachmentClass=Class'EAInventory.EAChainsawAttachment'
     Texture=Texture'ED_Hud.HUDchainsaw'
     Mesh=SkeletalMesh'ED_Weapons.ED_Chainsaw_NEW'
     Skins(0)=Texture'ED_WeaponSkins.Melee.chainsawskin'
     Skins(1)=TexPanner'ED_WeaponSkins.Melee.chainblur'
     Skins(2)=Texture'MP_FPArms.LS_arms.LS_hands_dude'
     AmbientGlow=0
}
