///////////////////////////////////////////////////////////////////////////////
// Dervish Cat Rocket Controller
//
// Ensures that the cat screams while falling, then start dervishing forever
///////////////////////////////////////////////////////////////////////////////
class DervishCatRController extends AWCatController;

///////////////////////////////////////////////////////////////////////////////
// Vars
///////////////////////////////////////////////////////////////////////////////
var() sound CatFlying;
var() sound CatStartFlying;
var() sound CatStopFlying;

var bool bFirstMeow;

///////////////////////////////////////////////////////////////////////////////
// FallingStartDervishRocket
// Play screeching sound while falling.
///////////////////////////////////////////////////////////////////////////////
auto state FallingStartDervishRocket extends FallingStartDervish
{
	event Touch(actor Other)
	{
		if (FPSPawn(Other) != None)
			DervishThisPawn(FPSPawn(Other));
		else
			Super.Touch(Other);
	}

	///////////////////////////////////////////////////////////////////////////////
	// Play your movement noise again
	///////////////////////////////////////////////////////////////////////////////
	simulated event Timer()
	{
		if (!bFirstMeow)
		{
			Pawn.PlaySound(CatStartFlying, SLOT_Talk, 1.0, false, 100.0, 1.0);
			SetTimer(GetSoundDuration(CatStartFlying), false);
			bFirstMeow=True;
		}
		else
		{
			Pawn.PlaySound(CatFlying, SLOT_Talk, 1.0, false, 100.0, 1.0);
			SetTimer(GetSoundDuration(CatFlying), false);
		}
	}

	event BeginState()
	{
		Super.BeginState();
		// Play initial flying sound
		Pawn.PlaySound(CatStartFlying, SLOT_Talk, 1.0, false, 100.0, 1.0);
		SetTimer(GetSoundDuration(CatStartFlying), false);
	}

	event EndState()
	{
		Super.EndState();
		// Turn off meowing sound
		Pawn.PlaySound(CatStopFlying, SLOT_Talk, 1.0, false, 100.0, 1.0);
	}
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// Coming out of dervish,
// allow them to be picked up, but not much else
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
state DervishFinish
{
	///////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////
	function BeginState()
	{
		TurnOffDervish();
		Pawn.ChunkUp(0);
	}
}

///////////////////////////////////////////////////////////////////////////////
// Default properties
///////////////////////////////////////////////////////////////////////////////

defaultproperties
{
     CatFlying=Sound'AnimalSounds.Cat.CatScream_loop1'
     CatStartFlying=Sound'AnimalSounds.Cat.CatScream_fire1'
     CatStopFlying=Sound'AnimalSounds.CatScream'
}
