// EAAuthorityFigure
// Base class for all authority figures in EA.
class EAAuthorityFigure extends AWAuthorityFigure
	notplaceable
	Abstract;

///////////////////////////////////////////////////////////////////////////////
// Vars
///////////////////////////////////////////////////////////////////////////////
var(Cell) class<P2Weapon> CellPhoneClass;
var(Cell) bool bCellUser;
var(Cell) sound CellBeep;
var(Cell) sound CellRing;
var(Police) class<P2PowerupInv> CopRadioClass;
var(Footsteps) Sound FootStepSounds[5];
var(Footsteps) bool bDoFootsteps;
var KevlarMesh ArmorMesh;		// Model of kevlar armor you're wearing on your person

const REMOVE_DEAD_START_TIME = 30.00;

var(Character) array<MeshAnimation> EDMeshAnims; // Mesh animations for ED
//dopamine Bali sound declarations
var Sound BaliHitHead;
var Sound BaliCleaveHead;
var Sound BaliHitBody;

var bool bHadThroatSlit;

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
var float StuckTime;				// How long the player has been 'stuck'. Determined below. If they are not
									// moving in Z for too long (MAX_STUCK_TIME),
									// then warp them to the nearest good pathnode.
var float StuckCheckRadius;			// Size around which you check for pathnodes to warp to
var PathNode LastUnstuckPoint;		// Save the last point we warped to, after being unstuck. Don't use it
									// the very next time, to keep us from infinite unsticking--still stuck
									// scenarios.
var bool bPlayingFootstepSound;	// Set to true when you play a step, false after timer is
								// called again

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
const MAX_STUCK_TIME				=	2.0;
const STUCK_RADIUS					=	2048;

const RUN_FOOTSTEP_TIME		=	0.35;
const WALK_FOOTSTEP_TIME	=	0.60;

const WALK_VOL				= 0.07;
//=	0.07;
const RUN_VOL				= 0.2;
//=	0.2;

const FOOTSTEP_RADIUS			=	100;
const FOOTSTEP_RADIUS_MP		=	200;
const FOOTSTEP_RADIUS_LOCAL_MP	=	20;

const MELEE_BLOCK_MIN_HESTON = 0.50;
const MELEE_BLOCK_MAX_HESTON = 0.75;

const MELEE_BLOCK_MIN_INSANE = 1.00;
const MELEE_BLOCK_MAX_INSANE = 1.00;

const INSANE_MEDGUN_PCT			= 0.5;
const INSANE_BIGGUN_PCT			= 0.25;
const INSANE_UBERGUN_PCT		= 0.005;
const INSANE_MELEE_PCT			= 0.25;


///////////////////////////////////////////////////////////////////////////////
// Link this pawn to the anims it needs
///////////////////////////////////////////////////////////////////////////////
simulated function LinkAnims()
{
	local int i;

	// Link up ED anims
	for (i=0; i<EDMeshAnims.Length; i++)
		LinkSkelAnim(EDMeshAnims[i]);

	// Also put in more AW single player anims.
	LinkSkelAnim(AW_SPMeshAnim);

	// Links to the special specified mesh for SP games.
	LinkSkelAnim(GetDefaultAnim(SkeletalMesh(Mesh)));

	// Always link to the core anims, too, because some characters use a mixture
	// of their own anims plus some core anims.  Linking to core anims twice,
	// which can happen if default anims happen to match core anims, is safe.
	LinkSkelAnim(CoreMeshAnim);
}


// ED BLOOD CODE
///////////////////////////////////////////////////////////////////////////////
//	Tries to make a small blood splat on the ground based on hit velocity
///////////////////////////////////////////////////////////////////////////////
function DripBloodOnGround(vector Momentum)
{
	local Rotator NewRot;
	local Actor HitActor;
	local vector checkpoint, HitNormal, HitLocation;

	//log("before mom "$Momentum);
	Momentum.x*=FRand();
	Momentum.y*=FRand();
	Momentum = Normal(Momentum);
	Momentum.z=BLOOD_DRIP_GRAVITY;
	//log("after mom "$Momentum);
	checkpoint = Location + DRIP_FLOOR_Z_CHECK*Momentum;
	HitActor = Trace(HitLocation, HitNormal, checkpoint, Location, true);
	if ( HitActor != None
		&& HitActor.bWorldGeometry 
		&& HitNormal.z > 0.9)
	{
		NewRot = Rotator(-HitNormal);
		NewRot.Roll=(65536*FRand());
		spawn(class'EDBloodDripSplat',self,,HitLocation,NewRot);
	}
}

///////////////////////////////////////////////////////////////////////////////
//	A blood splash here
///////////////////////////////////////////////////////////////////////////////
function BloodHit(vector BloodHitLocation, vector Momentum)
{
	local vector BloodOffset, dir, HitLocation, HitNormal, checkpoint;
	local float tempf;
	local EDBloodImpactMachineGun bloodhitvar;
	local Rotator NewRot;
	local Actor HitActor;
	//, Mo;
	//local class<Effects> DesiredEffect;
//		class<P2Damage>(damageType).static.GetPawnDamageEffect(HitLocation, Damage, Momentum, self, (Level.bDropDetail || !Level.bHighDetailMode));
//		DesiredEffect = DamageType.static.GetPawnDamageEffect(HitLocation, Damage, Momentum, self, (Level.bDropDetail || !Level.bHighDetailMode));

//		if ( DesiredEffect != None )
//		{

	// Find direction to center
	dir = BloodHitLocation - Location;
	dir = Normal(BloodHitLocation - Location);
	// push it away from the his center
	BloodOffset = 0.2 * CollisionRadius * dir;
	// pull it up some from the bottom and pull it down from the top
	BloodOffset.Z = BloodOffset.Z * 0.75;

	//Mo = Momentum;
	//if ( Mo.Z > 0 )
		//Mo.Z *= 0.5;

//			spawn(DesiredEffect,self,,BloodHitLocation + BloodOffset);//, rotator(Mo));
//		}
	////////////////
	// Blood that squirts in the air
	bloodhitvar = spawn(class'EDBloodImpactMachineGun',self,,BloodHitLocation+BloodOffset);
	// This makes the blood shoot off at cool angles

	////////////////
	// Blood that shoots onto the wall 
	// Check to see if you're close enough to the wall, to squirt blood on it.
	// Do this by coming out of the actor where we hit and continue along the path
	// that goes from the original hit point, toward the player. (So look 
	// behind the player)
	checkpoint = BloodHitLocation + DIST_TO_WALL_FOR_BLOODSPLAT*Normal(Momentum);
	//log("momentum "$Momentum);
	HitActor = Trace(HitLocation, HitNormal, checkpoint, BloodHitLocation, true);

	if ( HitActor != None
		&& HitActor.bStatic ) 
//	if(LevelInfo(HitActor) != None
//		|| TerrainInfo(HitActor) != None
//		|| StaticMeshActor(HitActor) != None
//		|| Brush(HitActor) != None)
	{
		NewRot = Rotator(-HitNormal);
		/*
		tempf = atan(Momentum.x/Momentum.y);
			log("tempf "$tempf);
		NewRot.Roll = 65536*(atan(Momentum.y/Momentum.x)/(2*Pi));
		if(Momentum.x < 0)
			NewRot.Roll -= 32768;

  maybe something with the wall normal
		NewRot.Roll = NewRot.Roll & 65535;
		spawn(class'BloodSplatOblique',self,,HitLocation,NewRot);
		*/
		NewRot.Roll=(65536*FRand());
		spawn(class'EDBloodMachineGunSplat',self,,HitLocation,NewRot);
	}

	////////////////
	// Drips of blood on the ground around you (smaller)
	if(FRand() <= 0.7)
	{
		DripBloodOnGround(Momentum);
	}
}
// END ED BLOOD CODE

///////////////////////////////////////////////////////////////////////////////
// Record pawn dead, if player killed them
///////////////////////////////////////////////////////////////////////////////
function Died(Controller Killer, class<DamageType> damageType, vector HitLocation)
{
	if(P2GameInfoSingle(Level.Game) != None
		&& P2GameInfoSingle(Level.Game).TheGameState != None
		&& EAGameState(P2GameInfoSingle(Level.Game).TheGameState) != None
		&& Killer != None
		&& Killer.bIsPlayer)
	{
		// Record the pawn as an innocent, if we don't hate the player.
		if (bInnocent)
			EAGameState(P2GameInfoSingle(Level.Game).TheGameState).InnocentsKilled++;
		// If he was a cop, record that too
		if (Self.IsA('EAPolice'))
			P2GameInfoSingle(Level.Game).TheGameState.CopsKilled++;
	}

	Super.Died(Killer, damageType, HitLocation);
}

function bool HandleSpecialShots(int Damage, vector HitLocation, vector Momentum, out class<DamageType> ThisDamage,
							vector XYdir, Pawn InstigatedBy, out int returndamage, out byte HeadShot)
{
	local float PercentUpBody, ZDist, DistToMe;

	// Only let the player get special head shots
	// Projectile weapons
	if(FPSPawn(InstigatedBy).bPlayer)
	{
		if(Health > 0)
		{
			if(ThisDamage == class'ShotgunDamage'
				|| ThisDamage == class'RifleDamage'
				|| ThisDamage == class'BulletDamage')
			{
				// For if no damage is done
				if(TakesShotgunHeadShot == 0.0
					|| TakesShotgunHeadShot == 0.0
					|| TakesPistolHeadShot == 0.0)
				{
					// Make a ricochet sound and puff out some smoke and sparks
					SparkHit(HitLocation, Momentum, 1);//Rand(2));
					DustHit(HitLocation, Momentum);
					returndamage = 0;
					return true;
				}

				PercentUpBody = (hitlocation.z - Location.z)/CollisionHeight;
				//log("dist to head for explode try "$VSize(XYDir));
				//log("percent up body "$PercentUpBody);
				// Check to see if we're in fake head shot range
				if(PercentUpBody > HEAD_RATIO_OF_FULL_HEIGHT)
				{
					DistToMe = VSize(XYdir);

					if((DistToMe < DISTANCE_TO_PUNCTURE_HEAD
							|| (P2GameInfo(Level.Game) != None
								&& P2GameInfo(Level.Game).PlayerGetsHeadShots()))
						&& ThisDamage == class'BulletDamage')
					// Is close enough with a pistol and behind them to puncture head with a pistol
					{
						// Check a little more accurately, if you actually hit the head or not
						// And check to make sure the guy got shot from behind, before we allow it.
						if(((Momentum dot vector(Rotation)) > 0
								&& CheckHeadForHit(HitLocation, ZDist))
							|| (P2GameInfo(Level.Game) != None
								&& P2GameInfo(Level.Game).PlayerGetsHeadShots()))
						{
							// We've hit the head, now reduce the damage, if necessary
							if(!(P2GameInfo(Level.Game) != None
								&& P2GameInfo(Level.Game).PlayerGetsHeadShots()))
								returndamage = TakesPistolHeadShot*HealthMax;
							else
								returndamage = HealthMax;
							// if this kills them, puncture the head
							if(returndamage >= Health
								&& bHeadCanComeOff
								&& !(P2GameInfo(Level.Game) != None
									&& P2GameInfo(Level.Game).PlayerGetsHeadShots()))
							{
								// record special kill
								if(P2GameInfoSingle(Level.Game) != None
									&& P2GameInfoSingle(Level.Game).TheGameState != None
									&& P2Pawn(InstigatedBy) != None
									&& P2Pawn(InstigatedBy).bPlayer)
								{
									P2GameInfoSingle(Level.Game).TheGameState.PistolHeadShot++;
								}

								if(class'P2Player'.static.BloodMode())
									PunctureHead(HitLocation, Momentum);
							}
							HeadShot = 1;
							return true;
						}
						// Over the head but not hitting the head means this guy won't take damage
						// If we had hit the head, the above would have returned already
						if(ZDist > 0) 
							return false;
					}
					else if(DistToMe < DISTANCE_TO_EXPLODE_HEAD
						&& ThisDamage == class'ShotgunDamage')
					// Is close enough with a shotgun to explode the head
					{
						// Check a little more accurately, if you actually hit the head or not
						if(CheckHeadForHit(HitLocation, ZDist))
						{
							// We've hit the head, now reduce the damage, if necessary
							if(!(P2GameInfo(Level.Game) != None
								&& P2GameInfo(Level.Game).PlayerGetsHeadShots()))
								returndamage = TakesShotgunHeadShot*HealthMax;
							else
								returndamage = HealthMax;

							// if this kills them, blow their head up
							if(returndamage >= Health
								&& bHeadCanComeOff)
							{
								// record special kill
								if(P2GameInfoSingle(Level.Game) != None
									&& P2GameInfoSingle(Level.Game).TheGameState != None
									&& P2Pawn(InstigatedBy) != None
									&& P2Pawn(InstigatedBy).bPlayer)
								{
									P2GameInfoSingle(Level.Game).TheGameState.ShotgunHeadShot++;
								}

								if(class'P2Player'.static.BloodMode())
								{
									ExplodeHead(HitLocation, Momentum);
								}
							}
							HeadShot = 1;
							return true;
						}
						// Over the head but not hitting the head means this guy won't take damage
						// If we had hit the head, the above would have returned already
						if(ZDist > 0)
							return false;
					}
					else if(ThisDamage == class'RifleDamage')
					// Sniper rifle round to the head punctures your head
					{
						// We've hit the head, now reduce the damage, if necessary
						//returndamage = TakesRifleHeadShot*HealthMax;
						returndamage = HealthMax;
						// if this kills them, puncture the head
						if(returndamage >= Health
								&& bHeadCanComeOff)
						{
							// record special kill
							if(P2GameInfoSingle(Level.Game) != None
								&& P2GameInfoSingle(Level.Game).TheGameState != None
								&& P2Pawn(InstigatedBy) != None
								&& P2Pawn(InstigatedBy).bPlayer)
							{
								P2GameInfoSingle(Level.Game).TheGameState.RifleHeadShot++;
							}

							if(class'P2Player'.static.BloodMode())
								PunctureHead(HitLocation, Momentum);
						}
						HeadShot = 1;
						return true;
					}
				}
				// continue on, if this didn't take
			}
			else if((P2GameInfo(Level.Game) != None
						&& P2GameInfo(Level.Game).PlayerGetsHeadShots())
					&& ThisDamage == class'MachinegunDamage')
				// Just for the silly head shots cheat
			{
				PercentUpBody = (hitlocation.z - Location.z)/CollisionHeight;
				if(PercentUpBody > HEAD_RATIO_OF_FULL_HEIGHT)
				{
					// We've hit the head, now make it take two machine gun bullets
					// to down a guy when hit in the head.
					returndamage = 0.5*HealthMax;
					HeadShot = 1;
					return true;
				}
				// Over the head but not hitting the head means this guy won't take damage
				// If we had hit the head, the above would have returned already
				if(ZDist > 0)
					return false;
			}
			
			if(ClassIsChildOf(ThisDamage, class'RifleDamage'))
	                {
	                    PercentUpBody = (hitlocation.z - Location.z)/CollisionHeight;
                            if(PercentUpBody > HEAD_RATIO_OF_FULL_HEIGHT)
                            {
                                // Beretta round to the head blows your head
			        if(ThisDamage == class'BerettaDamage')
			        {
			                 if(IsMPHeadshot(hitlocation))
                                           {
						// We've hit the head, now reduce the damage, if necessary
						if(!(P2GameInfo(Level.Game) != None
								&& P2GameInfo(Level.Game).PlayerGetsHeadShots()))
								returndamage = TakesRifleHeadShot*HealthMax;
						else
								returndamage = HealthMax;
						// if this kills them, puncture the head
						if(returndamage >= Health
								&& bHeadCanComeOff)
						{
							// record special kill
							if(P2GameInfoSingle(Level.Game) != None
								&& P2GameInfoSingle(Level.Game).TheGameState != None
								&& P2Pawn(InstigatedBy) != None
								&& P2Pawn(InstigatedBy).bPlayer)
							{
								P2GameInfoSingle(Level.Game).TheGameState.PistolHeadShot++;
							}

							if(class'P2Player'.static.BloodMode())
								ExplodeHead(HitLocation, Momentum);
						}
						HeadShot = 1;
						return true;
                                           }
                                }
                            }
                       }
		}
	}


	// Melee
	if(ClassIsChildOf(ThisDamage, class'BludgeonDamage'))
	{
		if(CheckHeadForHit(HitLocation, ZDist, true))
		{
			// shovel's knock heads off
            if(ThisDamage == class'BatDamage')
			{
				// Decide randomly to knock the head off. If you're closer to
				// death, then be more likely to make the head pop off
				// Let the player take off heads, and let NPCs take off each
				// others heads
				if(bHeadCanComeOff
					&& class'P2Player'.static.BloodMode()
					&& FRand() >= Health/HealthMax
					&& (P2Player(InstigatedBy.Controller) != None
						|| P2Player(Controller) == None))
				{
					// We've hit the head, now compound the damage, if necessary
					returndamage = TakesShovelHeadShot*HealthMax;
				}
				else
				{
					returndamage = Damage;
				}
				//Dopamine Give headshot only when alive
				if(returndamage >= Health)
				{
	                //Dopamine Take off head even if dead
					PopOffHead(HitLocation, Momentum);
                    HeadShot = 1;
				}
			}

			else if(ThisDamage == class'ShovelDamage')
			{
				// Decide randomly to knock the head off. If you're closer to
				// death, then be more likely to make the head pop off
				// Let the player take off heads, and let NPCs take off each
				// others heads
				if(bHeadCanComeOff
					&& class'P2Player'.static.BloodMode()
					&& FRand() >= Health/HealthMax
					&& (P2Player(InstigatedBy.Controller) != None
						|| P2Player(Controller) == None))
				{
					// We've hit the head, now compound the damage, if necessary
					returndamage = TakesShovelHeadShot*HealthMax;
				}
				else
				{
					returndamage = Damage;
				}
				// If this kills them, pop off the head
				if(returndamage >= Health)
				{
                                        PopOffHead(HitLocation, Momentum);
					HeadShot = 1;
					PlaySound(ShovelCleaveHead,,,,,GetRandPitch());
				}
				else // Otherwise, they just get hit in the head, hard
					PlaySound(ShovelHitHead,,,,,GetRandPitch());
			}

                        //dopamine

			else if(ThisDamage == class'BatonDamage')
				// batons incapacitate people.
			{
				if(P2GameInfoSingle(Level.Game) != None
					&& P2GameInfoSingle(Level.Game).VerifySeqTime()
					&& P2Pawn(InstigatedBy) != None
					&& P2Pawn(InstigatedBy).bPlayer)
				{
					// We've hit the head, now reduce the damage, if necessary
					// use same version as shotgun explodes
					returndamage = TakesShotgunHeadShot*HealthMax;
					// if this kills them, explodes the head
					if(returndamage >= Health
							&& bHeadCanComeOff)
					{
						if(class'P2Player'.static.BloodMode())
						{
							ExplodeHead(HitLocation, Momentum);
						}
					}
					HeadShot = 1;
					return true;
				}
				else
					returndamage = Damage;
			}
			else if(ThisDamage == class'KickingDamage')
			{
				PlaySound(FootKickHead,,,,,GetRandPitch());
				// kicking to the face just draws blood
				returndamage = Damage;
			}


                        //Dopamine bali damage special shot
                        else if(ThisDamage == class'BaliDamage')
			{
				// Decide randomly to knock the head off. If you're closer to
				// death, then be more likely to make the head pop off
				// Let the player take off heads, and let NPCs take off each
				// others heads

                                returndamage = Damage;

				//dopamine
                                // If this almost kills them, slit throat
				if(returndamage >= Health-20)
				{

                                        //dopamine slit throat only once
                                        if (!bHadThroatSlit){
                                           //dopamine Slit their throat
                                           SlitThroat(HitLocation, Momentum);

                                           //dopamine remember the throat was slit
                                           bHadThroatSlit = true;

	   		                   HeadShot = 1;

					   //dopamine Bali Sound
					   PlaySound(BaliCleaveHead,,,,,GetRandPitch());
					}
                                           
				}
				else // Otherwise, they just get hit in the head, hard
				
				        //Dopamine bali sound
					PlaySound(BaliHitHead,,,,,GetRandPitch());
			}
			else
			{
				returndamage = Damage;
			}
			return true;
		}
		else // if it was a bludgeon attack, but didn't hit the
			// face, then don't draw blood
		{
			if(ThisDamage == class'ShovelDamage')
			{
				PlaySound(ShovelHitBody,,,,,GetRandPitch());
			}
			else if(ThisDamage == class'KickingDamage')
			{
				PlaySound(FootKickBody,,,,,GetRandPitch());
			}
			
			//dopamine Bali sound for hit body
                        else if(ThisDamage == class'BaliDamage')
			{
				PlaySound(BaliHitBody,,,,,GetRandPitch());
			}


			// Cutting attacks always draw blood, but if not, at this point
			// we only want a dust hit, so change the damage type.
			if(!ClassIsChildOf(ThisDamage, class'CuttingDamage'))
				ThisDamage = class'BodyDamage';
		}
	}

	return false;
}


//Make the stream now shoot forwards instead of straight up
function SnapSpoutThroat(optional bool bInitArc)
{
	local vector startpos, X,Y,Z;
	local vector forward;
	local coords checkcoords;

	checkcoords = GetBoneCoords(BONE_NECK);
	FluidSpout.SetLocation(checkcoords.Origin);
	
	//dopamine REMINDER - blood direction is controled in ThroatBloodSpoutFeeder
	FluidSpout.SetDir(checkcoords.Origin, checkcoords.XAxis,,bInitArc);

}


//Used so that the head does not detach but the PourerFeeder still activates
function SlitThroat(vector HitLocation, vector Momentum)
{
	local PoppedHeadEffects headeffects;
	local P2Emitter HeadBloodTrail;			// Blood trail I drip if I'm detached.

        //DOPAMINE DELETED
	//Super.PopOffHead(HitLocation, Momentum);



	// Create blood from neck hole
	if(FluidSpout == None
		&& P2GameInfo(Level.Game).AllowBloodSpouts())
	{
		// If we're puking at the time our head goes away, then
		// keep puke going out the neckhole

                /*DOPAMINE - this is unneeded I think
                if(Head(MyHead).PukeStream != None)
		{
			FluidSpout = spawn(class'PukePourFeeder',self,,Location);
			// Make it the same type of puke
			FluidSpout.SetFluidType(Head(MyHead).PukeStream.MyType);
		}
                */

                //dopamine try for our own bloodspout class
		//else// If our head is removed while not puking, then make blood squirt out
			FluidSpout = spawn(class'ThroatBloodSpoutFeeder',self,,Location);


		FluidSpout.MyOwner = self;
		FluidSpout.SetStartSpeeds(100, 10.0);
		AttachToBone(FluidSpout, BONE_HEAD);

		//Dopamine above function overrides spout direction
		SnapSpoutThroat(true);
	}




	//Dopamine Removed all because we don't want the head to come off
	/**********************************************************************

	// Pop off the head
	//dopamine DELETED DetachFromBone(MyHead); AND REPLACED WITH
	//Super.DetachFromBone(MyHead);





	// Get it ready to fly
	Head(MyHead).StopPuking();
	Head(MyHead).StopDripping();
	MyHead.SetupAfterDetach();
	// Make a blood drip effect come out of the head
	HeadBloodTrail = Spawn(class'BloodChunksDripping ',self);
	HeadBloodTrail.Emitters[0].RespawnDeadParticles=false;
	HeadBloodTrail.SetBase(self);

	MyHead.GotoState('Dead');

	// Send it flying
	MyHead.GiveMomentum(Momentum);

	// Make some blood mist where it hit
	headeffects = spawn(class'PoppedHeadEffects',,,HitLocation);
	headeffects.SetRelativeMotion(Momentum, Velocity);

	//Remove connection to head but don't destroy it
	DissociateHead(false);
	
	**********************************************************************/
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
function PostBeginPlay()
{
	if (P2GameInfoSingle(Level.Game).bIsDemo)
	{
		Destroy();
		return;
	}

	Super.PostBeginPlay();

	Cowardice=0.0;

	StuckCheckRadius = STUCK_RADIUS;
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
function SetupBoltons()
{
	local int i;

	Super.SetupBoltons();

	for (i=0; i<ArrayCount(Boltons); i++)
	{
		if (Boltons[i].Part != None && Boltons[i].bAttachToHead)
			Boltons[i].Part.SetDrawScale3d(MyHead.DrawScale3D);
	}
}


///////////////////////////////////////////////////////////////////////////////
// Decide on a prize for a quick kill
///////////////////////////////////////////////////////////////////////////////
function PickQuickKillPrize(int KillIndex)
{
	local class<Inventory> invclass;
	local Inventory newone;

	switch(KillIndex)
	{
		case 0:
			invclass = class'DonutInv';
		break;
		case 1:
			invclass = class'MoneyInv';
		break;
		case 2:
			invclass = class'FastFoodInv';
		break;
		case 4:
			invclass = class'CrackInv';
		break;
		case 6:
			invclass = class'CatnipInv';
		break;
	}

	newone = spawn(invclass, self);
	TossThisInventory(vector(Rotation), newone);
}

///////////////////////////////////////////////////////////////////////////////
// toss out the inventory passed in
///////////////////////////////////////////////////////////////////////////////
function bool TossThisInventory(vector TossVel, Inventory ThisInv)
{
	local vector X,Y,Z;
	local Rotator userot;
	local P2Weapon ThisWeap;

	if(ThisInv == None)
		return false;

	// Really make it stop doing it's stuff
	// And don't drop things that can't be dropped
	ThisWeap = P2Weapon(ThisInv);
	if(ThisWeap != None)
	{
		if(!ThisWeap.bCanThrow
			|| (!ThisWeap.bCanThrowMP
				&& !Level.Game.bIsSinglePlayer))
			return false;

		ThisWeap.ForceEndFire();
	}

	// Comment out the 1409 fix
/*
	if(P2PowerupInv(ThisInv) != None)
	{
		if(!P2PowerupInv(ThisInv).bCanThrow
			// 1409 fix, make sure you have stuff to throw!
			// Before this, you could use a crack pipe, then throw it quickly as the
			// smoke was coming out, and you'd still get the health and the crack pipe back--fixed!
			|| P2PowerupInv(ThisInv).Amount <= 0)
			return false;
	}
*/

	ThisInv.velocity = TossVel;
	userot = Rotation;
	userot.Yaw = FRAnd()*65535;
	GetAxes(userot,X,Y,Z);
	// And make the ThisWeap align with the direction of the throw
	ThisInv.SetRotation(userot);
	ThisInv.DropFrom(Location + 0.3 * CollisionRadius * X + CollisionRadius * Z);
	return true;
}

///////////////////////////////////////////////////////////////////////////////
// Adds all the required equipment and picks out the urethra.
// Do this here so we access to the inventory package for specific things
// like HandsWeapon and UrethraWeapon.
///////////////////////////////////////////////////////////////////////////////
function AddDefaultInventory()
{
	local int i, j, randpick;
	local Inventory thisinv;
	local P2PowerupInv pinv;
	local P2Weapon p2weap;
	local class<P2Weapon> p2weapclass;
	local int usedam;
	local int Count;
	local bool bWeaponsGranted;

	// Only let this be called once
	if (!bGotDefaultInventory)
	{
		//Log(self$" PersonPawn.AddDefaultInventory() is adding inventory");
		// Everyone gets hands, so add that explicitly here
		thisinv = CreateInventoryByClass(class'Inventory.HandsWeapon');

		// Give them a cell phone
		if (bCellUser && CellPhoneClass != None)
			CreateInventoryByClass(CellPhoneClass);
		else // No cell phone class means no calls
			bCellUser = False;

		//Log(self$" PersonPawn.AddDefaultInventory() is adding inventory");
		// Everyone gets hands, so add that explicitly here
		thisinv = CreateInventoryByClass(class'Inventory.HandsWeapon');

		// Give them a cop radio, if they have one.
		if (bHasRadio && CopRadioClass != None)
			CreateInventoryByClass(CopRadioClass);
	
		// Handle money seperately
		if(MaxMoneyToStart > 0)
		{
			pinv = P2PowerupInv(CreateInventoryByClass(class'Inventory.MoneyInv'));
			pinv.SetAmount(MaxMoneyToStart);
		}

		// Add in the extra stuff now
		for ( i=0; i<BaseEquipment.Length; i++ )
		{
			if (BaseEquipment[i].weaponclass != None)
			{
				//log(self$" base equipment "$i$" weap "$BaseEquipment[i].weaponclass);
				p2weapclass = class<P2Weapon>(BaseEquipment[i].weaponclass);

				// If you're in Liebermode and this is a violent weapon, change
				// it to a weak one (but don't bother the player)
				if(p2weapclass != None
					&& P2GameInfo(Level.Game).InLiebermode()
					&& p2weapclass.default.ViolenceRank >= class'PPKSilenceWeapon'.default.ViolenceRank
					&& !bPlayer)
				{
					// If you're a cop, don't let them have their pistols/machine guns
					// (leave them with batons)
					if(class == class'Police')
						BaseEquipment[i].weaponclass = None;
					else // Other people get random melee weapons in their inventories
					{
						randpick = Rand(3);
						switch(randpick)
						{
							case 0:
								BaseEquipment[i].weaponclass = class'BatonWeapon';
								break;
							case 1:
								BaseEquipment[i].weaponclass = class'ShovelWeapon';
								break;
							case 2:
								BaseEquipment[i].weaponclass = class'ShockerWeapon';
								break;
						}
					}
				}

				// If you have anything at all
				if(BaseEquipment[i].weaponclass != None)
				{
					// Create the weapon from this base equipment index
					thisinv = CreateInventoryByClass(BaseEquipment[i].weaponclass);

					// Special weapon handling:
					// Keep track of the foot now, because we can't do it below (like the urethra)
					// --it's not added to the inventory list
					// Link up your foot
					if(FootWeapon(thisinv) != None)
					{
						MyFoot = P2Weapon(thisinv);
						MyFoot.GotoState('Idle');
						MyFoot.bJustMade=false;
						if(Controller != None)
							Controller.NotifyAddInventory(MyFoot);
						ClientSetFoot(P2Weapon(thisinv));
					}
				}
			}
		}

		// If you're in heston mode, no matter what, always give them some more random guns
		// (unless you're the player)
		if(P2GameInfo(Level.Game).InHestonMode()
			&& !bPlayer)
		{
			randpick = Rand(6);
			switch(randpick)
			{
				case 0:
					CreateInventoryByClass(class'DesertEagleWeapon');
					break;
				case 1:
					CreateInventoryByClass(class'M870Weapon');
					break;
				case 2:
					CreateInventoryByClass(class'M16A2Weapon');
					break;
				case 3:
					CreateInventoryByClass(class'SMEGWeapon');
					break;
			}
			// And bump them up to be ready to fight
			if(PainThreshold < 1.0)
				PainThreshold = 1.0;

			if(Cajones < 1.0)
				Cajones = 1.0;

			// Make them block melee attacks more often
			if (BlockMeleeFreq < MELEE_BLOCK_MIN_HESTON)
				BlockMeleeFreq = frand() * (MELEE_BLOCK_MAX_HESTON - MELEE_BLOCK_MIN_HESTON) + MELEE_BLOCK_MIN_HESTON;
		}

		// If you're in Insane-o mode, no matter what, always give them some more random guns
		// (unless you're the player) (but give them more than heston)
		if(P2GameInfo(Level.Game).InInsaneMode()
			&& !bPlayer)
		{
			// In EA, people get lots of guns in Insane Mode.
			if (FRand() <= INSANE_UBERGUN_PCT)
			{
				// Give them an "uber" gun
				bWeaponsGranted = true;
				randpick = Rand(3);
				switch(randpick)
				{
					case 0:
						CreateInventoryByClass(class'NukeWeapon');
						break;
					case 1:
						CreateInventoryByClass(class'HolyWeapon');
						break;
					case 2:
						CreateInventoryByClass(class'PlagueWeapon');
						break;
				}
			}

			if (FRand() <= INSANE_BIGGUN_PCT)
			{
				// Give them a bigger gun
				bWeaponsGranted = true;
				randpick = Rand(5);
				switch(randpick)
				{
					case 0:
						CreateInventoryByClass(class'GLauncherWeapon');
						break;
					case 1:
						CreateInventoryByClass(class'LauncherWeapon');
						break;
					case 2:
						CreateInventoryByClass(class'NapalmWeapon');
						break;
				}
			}

			if (FRand() <= INSANE_MEDGUN_PCT)
			{
				// Give them a medium-sized gun
		//		log(self@"granting medium gun");
				bWeaponsGranted = true;
				randpick = Rand(9);
				switch(randpick)
				{
					case 0:
						CreateInventoryByClass(class'CowHeadWeapon');
						break;
					case 1:
						CreateInventoryByClass(class'GrenadeWeapon');
						break;
					case 2:
						CreateInventoryByClass(class'MolotovWeapon');
						break;
					case 3:
						CreateInventoryByClass(class'SMEGWeapon');
						break;
					case 4:
						CreateInventoryByClass(class'MaDBombWeapon');
						break;
					case 5:
						CreateInventoryByClass(class'EAMaDColtWeapon');
						break;
					case 7:
						CreateInventoryByClass(class'DoubleBarrelWeapon');
						break;
					case 8:
						CreateInventoryByClass(Class'FlameWeapon');
						break;
				}
			}

			if (FRand() <= INSANE_MELEE_PCT)
			{
				// Give them a cool melee weapon
		//		log(self@"granting melee weapon");
				bWeaponsGranted = true;
				randpick = Rand(7);
				switch(randpick)
				{
					case 0:
						CreateInventoryByClass(class'MacheteWeapon');
						break;
					case 1:
						CreateInventoryByClass(class'SledgeWeapon');
						break;
					case 2:
						CreateInventoryByClass(class'ScytheWeapon');
						break;
					case 3:
						CreateInventoryByClass(class'PitchforkWeapon');
						break;
					case 4:
						CreateInventoryByClass(class'KatanaWeapon');
						break;
				}
			}

			if (!bWeaponsGranted)
			{
				// Give them the standard
		//		log(self@"granting small gun");
				randpick = Rand(6);
				switch(randpick)
				{
					case 0:
						CreateInventoryByClass(class'DesertEagleWeapon');
						break;
					case 1:
						CreateInventoryByClass(class'M870Weapon');
						break;
					case 2:
						CreateInventoryByClass(class'M16A2Weapon');
						break;
					case 3:
						CreateInventoryByClass(class'ScissorsWeapon');
						break;
					case 4:
						CreateInventoryByClass(class'M4A1Weapon');
						break;
					case 5:
						CreateInventoryByClass(class'UZIWeapon');
						break;
				}
			}

			// And bump them up to be ready to fight
			if(PainThreshold < 1.0)
				PainThreshold = 1.0;

			if(Cajones < 1.0)
				Cajones = 1.0;

			// Make them block melee attacks more often
			if (BlockMeleeFreq < MELEE_BLOCK_MIN_INSANE)
				BlockMeleeFreq = frand() * (MELEE_BLOCK_MAX_INSANE - MELEE_BLOCK_MIN_INSANE) + MELEE_BLOCK_MIN_INSANE;
		}

		// tell the weapons they've been made now
		thisinv = Inventory;
		while(thisinv != None)
		{
			if(P2Weapon(thisinv) != None)
			{
				// After the creation of the weapon, while  normal weapons link up their ammo,
				// we wait and finally link up the urethra. Don't do it above, during the creation
				// of the weapon, it must wait till here.
				if(UrethraWeapon(thisinv) != None)
				{
					MyUrethra = Weapon(thisinv);
					MyUrethra.GiveAmmo(self);
					ClientSetUrethra(P2Weapon(thisinv));
				}

				P2Weapon(thisinv).bJustMade=false;
			}
			thisinv = thisinv.inventory;
			Count++;
			if (Count > 5000)
				break;
		}

		// Switch them as a default, to use their hands
		if(PersonController(Controller) != None)
			PersonController(Controller).SwitchToThisWeapon(StartWeapon_Group, StartWeapon_Offset);

		// If you're using distance-based weapon changing, then save the weapons to use
		if(WeapChangeDist > 0)
		{
			if(BaseEquipment.Length >= 2)
			{
				if(BaseEquipment[CloseWeaponIndex].weaponclass != None)
					CloseWeap = class<P2Weapon>(BaseEquipment[CloseWeaponIndex].weaponclass);
				if(CloseWeap == None)
					log(self$" ERROR: WeapChangeDist and Close weapon is invalid!");

				if(BaseEquipment[FarWeaponIndex].weaponclass != None)
					FarWeap = class<P2Weapon>(BaseEquipment[FarWeaponIndex].weaponclass);
				if(FarWeap == None)
					log(self$" ERROR: WeapChangeDist and Far weapon is invalid!");
			}
			else
			{
				log(self$" ERROR: WeapChangeDist set with only 1 weapon!");
				WeapChangeDist = 0;
			}
		}

		// Note that we got our default inventory
		bGotDefaultInventory = true;

		//log(self$" in they hate me: "$P2GameInfo(Level.Game).TheyHateMeMode()$" game diff "$P2GameInfo(Level.Game).GameDifficulty);
		// If you're in They Hate Me mode, make everyone hate the player.
		// Do this at the end, so we don't retry to do this whole function again in
		// the new Possess, should redo our controller (below)
		if(P2GameInfo(Level.Game).TheyHateMeMode()
			&& !bPlayer)
		{
			// Leave innocents and cashiers alone. Hostiles get
			// made more hostile.
			if(bHasViolentWeapon)
			{
				bPlayerIsEnemy=true;
				bPlayerIsFriend=false;
				// And bump them up to be ready to fight, if they have a weapon
				if(PainThreshold < 1.0)
					PainThreshold = 1.0;
				if(Cajones < 1.0)
					Cajones = 1.0;

				if(CashierController(Controller) == None)
				{
					// If you were a Cop/Military, then turn into a cop-like bystander
					// so you don't try arrests or anything like that.
					Controller.Destroy();
					Controller = None;
					Controller = spawn(class'EACivilianController');
					if(Controller != None )
					{
						Controller.Possess(self);
						AIController(Controller).Skill += SkillModifier;
						CheckForAIScript();
					}
				}
			}
		}

		// Make sure *un-armed* bystanders take one pistol bullet if you hit them exactly in the middle
		if(!bHasViolentWeapon
			&& bInnocent)
		{
			// Calculate pistol damage for one hit kills.
			usedam = class'PistolBulletAmmoInv'.default.DamageAmount*class'Dude'.default.DamageMult - 1;
			//log(self$" new health "$usedam);
			HealthMax = usedam;
			Health = HealthMax;
		}
		//log(self$" initting for new level, game info diff "$P2GameInfoSingle(Level.Game).GameDifficulty$" game state diff "$P2GameInfoSingle(Level.Game).TheGameState.GameDifficulty$" GET game state diff "$P2GameInfoSingle(Level.Game).GetGameDifficulty());
	}
	//else
	//{
	//	Log(self$" PersonPawn.AddDefaultInventory() is being bypassed because inventory was added already");
	//}
}

///////////////////////////////////////////////////////////////////////////////
// Decide if he's stuck or not
///////////////////////////////////////////////////////////////////////////////
function bool DetectStuckPlayer(float DeltaTime)
{
	// If you're falling and not moving for too long, you're stuck
	if(Physics == PHYS_Falling
		&& !bIsCrouched)
	{
		// If you're not moving in z for too long
		if(Velocity.z == 0)
		{
			StuckTime += DeltaTime;

			// Now that he's really stuck, move him
			if(StuckTime > MAX_STUCK_TIME)
			{
				StuckTime = 0;
				return true;
			}
		}
		else
			StuckTime = 0;
	}
	return false;
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
simulated function PlayCellIn()
{
	PersonController(Controller).SwitchToThisWeapon(CellPhoneClass.default.InventoryGroup, CellPhoneClass.default.GroupOffset);
//	PlayAnim('s_idle_cellin', 1.0, 0.2);
}

simulated function PlayCellOut()
{
	PlayAnim('s_idle_cellout', 1.0, 0.2);
	PlaySound(CellBeep, SLOT_TALK);
	PersonController(Controller).SwitchToHands();
}

simulated function PlayCellOn()
{
	PlayAnim('s_idle_cellon2', 1.1, 0.2);
	PlaySound(CellBeep, SLOT_TALK);
}

simulated function PlayCellLoop()
{
	PlayAnim('s_idle_cell', 0.8);
}

simulated function float PlayIncomingCall()
{
	PlaySound(CellRing, SLOT_TALK);
	return GetSoundDuration(CellRing);
}

///////////////////////////////////////////////////////////////////////////////
// Find the nearest free pathnode to the stuck player, and put him there
// Warps him to each possible spot to make sure it will work. But it doesn't
// warp him back, it just uses the oldloc to test from. The last one you warped
// him to will have to be the closest to oldloc.
///////////////////////////////////////////////////////////////////////////////
function HandleStuckPlayer()
{
	local PathNode pn, savepn;
	local float closedist, usedist;
	local vector useloc, oldloc;

	Warn(self$" ERROR! NPC WAS STUCK here: "$Location);

	oldloc = Location;

	closedist = StuckCheckRadius;
	// Check pathnodes in a given radius
	foreach RadiusActors(class'Pathnode', pn, StuckCheckRadius, oldloc)
	{
		usedist = VSize(pn.Location - oldloc);
		if(LastUnstuckPoint	!= pn		// Not where we last unstuck ourself from
			&& usedist < closedist		// closest one
			&& !pn.bBlocked)			// not blocked
		{
			// Warp the player there now, to make sure the spot was
			// okay for him to be there. If this happens, it will only
			// work with the closest node, so this will also be the final move
			useloc = pn.Location;
			// Add just enough buffer to pathnode, to make sure he somehow doesn't
			// get warped to a point below the floor, and then fall through the floor.
			useloc.z += (CollisionHeight/2);
			if(SetLocation(useloc))
			{
				closedist = usedist;
				savepn = pn;
			}
		}
	}

	if(savepn != None)
	{
		//mypawnfix
		StuckCheckRadius = STUCK_RADIUS;
		LastUnstuckPoint = savepn;	// Save where we last unstuck from so as to not use it again, next time
		Warn(self$" UNsticking NPC--warping him to "$savepn);
	}
	else
	{
		Warn(self$" UNsticking warp failed");
		// You've failed, then increase you're check area
		StuckCheckRadius += STUCK_RADIUS;
	}
}

///////////////////////////////////////////////////////////////////////////////
// Prep for climbing a ladder
///////////////////////////////////////////////////////////////////////////////
function ClimbLadder(LadderVolume L)
{
	//log(Self$" climb ladder "$L);
	Super.ClimbLadder(L);
	if(EAPoliceController(Controller) != None)
		EAPoliceController(Controller).StartClimbLadder();
}

///////////////////////////////////////////////////////////////////////////////
// Flying machete coming our way--check to block it!
///////////////////////////////////////////////////////////////////////////////
function BlockMelee(Actor MeleeAttacker)
{
	local vector Rot, dir;
	local float dot1;

	//log(self$" BlockMelee "$MeleeAttacker$" weapon "$weapon$" firing style "$P2Weapon(Weapon).GetFiringStyle());
	if(Health > 0					// alive
		&& !bDeleteMe				// not deleted
		&& bHasViolentWeapon		// can block with the weapon
		&& MyBodyFire == None		// not on fire
		&& !bIsCrouched				// not crouching
		&& !bIsDeathCrawling		// not crawling
		&& FRand() < BlockMeleeFreq	// randomly decide to block or not
		&& P2Weapon(Weapon) != None	// has a weapon
		&& (P2Weapon(Weapon).GetFiringStyle() == WEAPONHOLDSTYLE_Both			// has a two-handed weapon
			|| P2Weapon(Weapon).GetFiringStyle() == WEAPONHOLDSTYLE_Double))
	{
		// Check now if you can see the actor to block against
		Rot = vector(Rotation);
		dir = MeleeAttacker.Location - Location;
		dir.z=0;
		dir = Normal(dir);
		dot1 = Rot Dot dir;
		//log(self$" check melee, dot "$dot1$" hit loc "$MeleeAttacker.Location);

		if(dot1 > BLOCK_MELEE_DOT)
		{
			if(EAPoliceController(Controller) != None)
				EAPoliceController(Controller).PerfomBlockMelee(MeleeAttacker);
		}
	}
}

///////////////////////////////////////////////////////////////////////////////
//  If we can block this melee attack
///////////////////////////////////////////////////////////////////////////////
function CheckBlockMelee(vector HitLocation, out byte StateChange)
{
	local vector Rot, dir;
	local float dot1;

	//log(self$" check block melee "$controller$" state "$controller.getstatename());
	// Check to complete block the melee attack
	if(Controller != None
		&& !Controller.bDeleteMe
		&& Controller.IsInState('BlockMelee'))
	{
		//log(Self$" doing block ");
		// See if they are facing our direction enough to block the attack
		Rot = vector(Rotation);
		dir = HitLocation - Location;
		dir.z=0;
		dir = Normal(dir);
		dot1 = Rot Dot dir;
		//log(self$" block melee, dot "$dot1$" hit loc "$HitLocation);

		if(dot1 > BLOCK_MELEE_DOT)
		{
			if(EAPoliceController(Controller) != None)
				EAPoliceController(Controller).DidBlockMelee(StateChange);
		}
		//log(self$" after block state "$statechange);
	}
}

///////////////////////////////////////////////////////////////////////////////
// Set up how you start without legs
///////////////////////////////////////////////////////////////////////////////
function bool StartMissingLegs()
{
	if(bStartMissingLegs)
	{
		bMissingBottomHalf=true;
		RemoveLimbsAfterLoad(self);
		BoneArr[LEFT_LEG]=0;
		BoneArr[RIGHT_LEG]=0;
		bMissingLegParts=true;
		bMissingLimbs=true;
		if(EAPoliceController(Controller) != None)
		{
			EAPoliceController(Controller).StartInHalf();
			return true;
		}
	}
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// State Living
// Do footsteps and other crap
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
auto simulated state Living
{
	event Tick(float DeltaTime)
	{
		Super.Tick(DeltaTime);

		// Unsticking logic for NPCs.
        if (!Controller.bIsPlayer && DetectStuckPlayer(DeltaTime))
			HandleStuckPlayer();
	}

	///////////////////////////////////////////////////////////////////////////////
	// Check to change footstep speed
	///////////////////////////////////////////////////////////////////////////////
	simulated function SetWalking(bool bNewIsWalking)
	{
		local bool OldWalking;

		OldWalking = bIsWalking;

		Super.SetWalking(bNewIsWalking);

		if ( bNewIsWalking != OldWalking )
		{
			Timer();
		}
	}

	///////////////////////////////////////////////////////////////////////////////
	//  Play sound
	///////////////////////////////////////////////////////////////////////////////
	simulated function Timer()
	{
		local float vol, usepitch, userad;
		local vector usevel;
		local bool bOtherMp; // someone other than you

//		log(self$" timer, phys "$Physics$" walking "$bIsWalking$" velocity "$Velocity$" role "$Role$" viewport "$ViewPort(PlayerController(Controller).Player));

		// only on client or stand alone
		if(bDoFootsteps && !P2GameInfo(Level.Game).IsCinematic())
		{
			bPlayingFootstepSound=false;

			if ( Physics == PHYS_Walking )
			{
				usepitch=1.0;
				if(bIsWalking
					|| bIsCrouched)
				{
					vol = WALK_VOL;
					SetTimer(WALK_FOOTSTEP_TIME, false);
				}
				else
				{
					vol = RUN_VOL;
					SetTimer(RUN_FOOTSTEP_TIME, false);
				}

				usepitch=1.0;
				userad = FOOTSTEP_RADIUS;

				if(VSize(Velocity) > 0)
				{
					PlaySound(FootStepSounds[Rand(ArrayCount(FootStepSounds))],SLOT_Interact,vol,,userad,usepitch);
					bPlayingFootstepSound=true;
				}
			}
			else
				SetTimer(WALK_FOOTSTEP_TIME, false);
		}
	}
	simulated function BeginState()
	{
//		log(self$" beginstate Living, role "$Role$" remote "$RemoteRole);
		SetTimer(RUN_FOOTSTEP_TIME, false);
	}
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// State Dying
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
state Dying
{
	///////////////////////////////////////////////////////////////////////////////
	// Get hurt, plus some ragdoll
	//
	// Only let you hurt the dead guys in single player, because in MP, the characters
	// won't match up with other people's computers and if you set someone on fire
	// they will be on fire in two different incorrect places (blood would shoot
	// out of weird places too.. ) It just makes a big mess.
	///////////////////////////////////////////////////////////////////////////////
	function TakeDamage( int Damage, Pawn InstigatedBy, Vector Hitlocation,
							Vector Momentum, class<DamageType> DamageType)
	{
		local bool bUseSuper;
		
		if (MyBodyFire == None)
			MyBodyFire = None;

		if(Level.Game != None
			&& Level.Game.bIsSinglePlayer)
		{
			if(class'P2Player'.static.BloodMode()
				// can't cut off player limbs
				&& P2Player(Controller) == None)
			{
				if(ClassIsChildOf(damageType, class'SuperShotgunDamage'))
					bUseSuper = !(HandleScythe(InstigatedBy, momentum, damagetype, Damage, HitLocation));
				else
					bUseSuper=true;
			}

			if(bUseSuper)
			{
				// Check to cut off limbs, and pass it along to original TakeDamage
				// for momentum transfer
				if(class'P2Player'.static.BloodMode()
					&& ClassIsChildOf(damageType, class'ExplodedDamage'))
					HandleExplosionDead(Damage, InstigatedBy, HitLocation, momentum, damageType);

				Super.TakeDamage(Damage, InstigatedBy, HitLocation, Momentum, DamageType);
			}
		}
	}
}



defaultproperties
{
     FootStepSounds(0)=Sound'MoreMiscSounds.loudfootsteps.footstep1h'
     FootStepSounds(1)=Sound'MoreMiscSounds.loudfootsteps.footstep2h'
     FootStepSounds(2)=Sound'MoreMiscSounds.loudfootsteps.footstep4h'
     FootStepSounds(3)=Sound'MoreMiscSounds.loudfootsteps.footstep5h'
     FootStepSounds(4)=Sound'MoreMiscSounds.loudfootsteps.footstep6h'
     bDoFootsteps=True
     EDMeshAnims(0)=MeshAnimation'ED_Characters.avgMental'
     EDMeshAnims(1)=MeshAnimation'ED_Characters.ED_deadBody'
     EDMeshAnims(2)=MeshAnimation'ED_Characters.rws_aw_anims_zomb'
     EDMeshAnims(3)=MeshAnimation'ED_Characters.smockingtostand'
     EDMeshAnims(4)=MeshAnimation'ED_Characters.SYS_HEDGETRIMMER'
     EDMeshAnims(5)=MeshAnimation'ED_Characters.ED_Scope_Slit_throat'
     EDMeshAnims(6)=MeshAnimation'ED_Characters.SyS_fisting'
     EDMeshAnims(7)=MeshAnimation'ED_Characters.GroundExit'
     EDMeshAnims(8)=MeshAnimation'ED_Characters.returntolifeFloaty'
     EDMeshAnims(9)=MeshAnimation'MP_Characters.anim_MP'
     EDMeshAnims(10)=MeshAnimation'ED_Characters.FLIGHTPACK'
     EDMeshAnims(11)=MeshAnimation'ED_Characters.animMaD'
     EDMeshAnims(12)=MeshAnimation'ED_ScopeNewCharacters.ScopesZombie1'
     EDMeshAnims(13)=MeshAnimation'MaD_ED_MP_Strippers.anim_StripperMP'
     EDMeshAnims(14)=MeshAnimation'Characters.animBig'
     EDMeshAnims(15)=MeshAnimation'AWCharacters.animAvg_AW'
     CoreMeshAnim=MeshAnimation'ED_Characters.animAvg'
     BaliHitHead=Sound'WeaponSounds.shovel_stab'
     BaliCleaveHead=Sound'WeaponSounds.shovel_stab'
     BaliHitBody=Sound'WeaponSounds.shovel_stab'
     SeverBone(0)="MALE01 L UpperArm"
     SeverBone(1)="MALE01 L Forearm"
     SeverBone(2)="MALE01 R UpperArm"
     SeverBone(3)="MALE01 R Forearm"
     SeverBone(4)="MALE01 L Thigh"
     SeverBone(5)="MALE01 L Calf"
     SeverBone(6)="MALE01 R Thigh"
     SeverBone(7)="MALE01 r calf"
     BoneArr(0)=1
     BoneArr(1)=1
     BoneArr(2)=1
     BoneArr(3)=1
     BoneArr(4)=1
     BoneArr(5)=1
     BoneArr(6)=1
     BoneArr(7)=1
     LimbClass=Class'EAPawns.EALimb'
     GutsEmitterClass1=Class'EDPawnsBase.TorsoGutsIntestine'
     GutsEmitterClass2=None
     HeadClass=Class'EAFX.EAHead'
     StumpBloodClass=Class'EDPawnsBase.EDStumpBlood'
     BodyHitSounds(0)=Sound'MiscSounds.People.bodyhitground1'
     BodyHitSounds(1)=Sound'MiscSounds.People.bodyhitground2'
     ChameleonMeshPkgs(0)="Characters"
     ChamelHeadSkins(0)="ChamelHeadSkins.MWA__001__AvgMale"
     ChamelHeadSkins(1)="ChamelHeadSkins.MBA__013__AvgBrotha"
     ChamelHeadSkins(2)="ChamelHeadSkins.MBA__014__AvgBrotha"
     ChamelHeadSkins(3)="ChamelHeadSkins.MMA__016__AvgMale"
     ChamelHeadSkins(4)="ChamelHeadSkins.MMF__024__FatMale"
     ChamelHeadSkins(5)="ChamelHeadSkins.MWA__002__AvgMale"
     ChamelHeadSkins(6)="ChamelHeadSkins.MMA__003__AvgMale"
     ChamelHeadSkins(7)="ChamelHeadSkins.MWA__004__AvgMale"
     ChamelHeadSkins(8)="ChamelHeadSkins.MWA__005__AvgMale"
     ChamelHeadSkins(9)="ChamelHeadSkins.MWA__006__AvgMale"
     ChamelHeadSkins(10)="ChamelHeadSkins.MWA__007__AvgMale"
     ChamelHeadSkins(11)="ChamelHeadSkins.MWA__008__AvgMale"
     ChamelHeadSkins(12)="ChamelHeadSkins.MWA__009__AvgMale"
     ChamelHeadSkins(13)="ChamelHeadSkins.MWA__010__AvgMale"
     ChamelHeadSkins(14)="ChamelHeadSkins.MWA__011__AvgMale"
     ChamelHeadSkins(15)="ChamelHeadSkins.MWA__015__AvgMale"
     ChamelHeadSkins(16)="ChamelHeadSkins.MWA__021__AvgMaleBig"
     ChamelHeadSkins(17)="ChamelHeadSkins.MWA__035__AvgMale"
     ChamelHeadSkins(18)="ChamelHeadSkins.MWF__025__FatMale"
     ChamelHeadSkins(19)="ChamelHeadSkins.MWA__022__AvgMaleBig"
     ChamelHeadSkins(20)="ChamelHeadSkins.FBA__033__FemSH"
     ChamelHeadSkins(21)="ChamelHeadSkins.FMA__028__FemSH"
     ChamelHeadSkins(22)="ChamelHeadSkins.FMA__034__FemSH"
     ChamelHeadSkins(23)="ChamelHeadSkins.FWA__026__FemLH"
     ChamelHeadSkins(24)="ChamelHeadSkins.FWA__027__FemLH"
     ChamelHeadSkins(25)="ChamelHeadSkins.FWA__029__FemSH"
     ChamelHeadSkins(26)="ChamelHeadSkins.FWA__032__FemSH"
     ChamelHeadSkins(27)="ChamelHeadSkins.FWF__023__FatFem"
     ChamelHeadSkins(28)="ChamelHeadSkins.FWA__037__FemSHcropped"
     ChamelHeadSkins(29)="ChamelHeadSkins.FMA__038__FemSHcropped"
     ChamelHeadSkins(30)="ChamelHeadSkins.FMA__039__FemSHcropped"
     ChamelHeadSkins(31)="ChamelHeadSkins.FWA__040__FemSHcropped"
     ChamelHeadSkins(32)="ChamelHeadSkins.MBF__042__FatMale"
     ChamelHeadSkins(33)="ChamelHeadSkins.FBF__043__FatFem"
     ChamelHeadSkins(34)="ChamelHeadSkins.FMF__044__FatFem"
     ChamelHeadSkins(35)="ChamelHeadSkins.FWA__031__FemSH"
     ChamelHeadSkins(36)="ChamelHeadSkins.FBA__063__FemSH"
     ChamelHeadSkins(37)="ED_HeadsAndSkins.Tim_Wambolts.FWA__030__FemSH"
     ChamelHeadSkins(38)="ED_HeadsAndSkins.biohazardous_snarks.MBA__021__AvgBrotha"
     ChamelHeadSkins(39)="ED_HeadsAndSkins.biohazardous_snarks.MBA__022__AvgBrotha"
     ChamelHeadSkins(40)="ED_HeadsAndSkins.biohazardous_snarks.MBA__023__AvgBrotha"
     ChamelHeadSkins(41)="ED_HeadsAndSkins.Tim_Wambolts.FWA__030__FemLH"
     ChamelHeadSkins(42)="ED_HeadsAndSkins.biohazardous_snarks.MBA__024__AvgBrotha"
     ChamelHeadSkins(43)="ED_HeadsAndSkins.biohazardous_snarks.MWA__001__AvgMale"
     ChamelHeadSkins(44)="ED_HeadsAndSkins.biohazardous_snarks.MWA__002__AvgMale"
     ChamelHeadSkins(45)="ED_HeadsAndSkins.biohazardous_snarks.MWA__003__AvgMale"
     ChamelHeadSkins(46)="ED_HeadsAndSkins.biohazardous_snarks.MWA__004__AvgMale"
     ChamelHeadSkins(47)="ED_HeadsAndSkins.biohazardous_snarks.MWA__005__AvgMale"
     ChamelHeadSkins(48)="ED_HeadsAndSkins.biohazardous_snarks.FWA__244__FemSH"
     ChamelHeadSkins(49)="ED_HeadsAndSkins.Mimics.MMA__007__AvgMale"
     ChamelHeadSkins(50)="ED_HeadsAndSkins.Steevees.MWA__409__AvgMale"
     ChamelHeadSkins(51)="ED_HeadsAndSkins.Steevees.MWA__410__AvgMale"
     ChamelHeadSkins(52)="ED_HeadsAndSkins.Steevees.MWA__411__AvgMale"
     ChamelHeadSkins(53)="ED_HeadsAndSkins.Steevees.MWA__012__AvgMale"
     ChamelHeadSkins(54)="ED_HeadsAndSkins.Steevees.MWA__013__AvgMale"
     ChamelHeadSkins(55)="ED_HeadsAndSkins.Steevees.MWA__014__AvgMale"
     ChamelHeadSkins(56)="ED_HeadsAndSkins.Steevees.MWA__015__AvgMale"
     ChamelHeadSkins(57)="ED_HeadsAndSkins.Steevees.MWA__016__AvgMale"
     ChamelHeadSkins(58)="ED_HeadsAndSkins.Steevees.MWA__017__AvgMale"
     ChamelHeadSkins(59)="ED_HeadsAndSkins.Tim_Wambolts.MWA__018__AvgMale"
     ChamelHeadSkins(60)="ED_HeadsAndSkins.Tim_Wambolts.MWA__019__AvgMale"
     ChamelHeadSkins(61)="ED_HeadsAndSkins.Tim_Wambolts.MWA__020__AvgMale"
     ChamelHeadSkins(62)="ED_HeadsAndSkins.SniperX.MWA__008__AvgMale"
     ChamelHeadSkins(63)="ED_HeadsAndSkins.biohazardous_snarks.MWA__006__AvgMale"
     ChamelHeadSkins(64)="ED_HeadsAndSkins.biohazardous_snarks.MWA__007__AvgMale"
     ChamelHeadSkins(65)="ED_HeadsAndSkins.biohazardous_snarks.MWA__008__AvgMale"
     ChamelHeadSkins(66)="ED_HeadsAndSkins.biohazardous_snarks.MMA__009__AvgMale"
     ChamelHeadSkins(67)="ED_HeadsAndSkins.biohazardous_snarks.MWA__010__AvgMale"
     ChamelHeadSkins(68)="ED_HeadsAndSkins.biohazardous_snarks.MWA__011__AvgMale"
     ChamelHeadSkins(69)="ED_HeadsAndSkins.Steevees.FWA__025__FemSH"
     ChamelHeadSkins(70)="ED_HeadsAndSkins.Tim_Wambolts.FWA__026__FemSH"
     ChamelHeadSkins(71)="ED_HeadsAndSkins.Tim_Wambolts.FWA__027__FemSH"
     ChamelHeadSkins(72)="ED_HeadsAndSkins.Tim_Wambolts.FWA__028__FemLH"
     ChamelHeadSkins(73)="ED_HeadsAndSkins.Tim_Wambolts.FWA__029__FemSH"
     ChamelHeadSkins(74)="ED_HeadsAndSkins.Tim_Wambolts.FWA__237__FemSHcropped"
     ChamelHeadSkins(75)="ED_HeadsAndSkins.Tim_Wambolts.FWA__238__FemSHcropped"
     ChamelHeadSkins(76)="ED_HeadsAndSkins.Tim_Wambolts.FWA__031__FemSH"
     ChamelHeadSkins(77)="ED_HeadsAndSkins.Tim_Wambolts.FWA__242__FemSH"
     ChamelHeadSkins(78)="ED_HeadsAndSkins.Tim_Wambolts.FWA__243__FemSH"
     ChamelHeadSkins(79)="ED_HeadsAndSkins.biohazardous_snarks.FWA__239"
     ChamelHeadSkins(80)="ED_HeadsAndSkins.biohazardous_snarks.MWA__010__AvgMale"
     ChamelHeadSkins(81)="ED_HeadsAndSkins.biohazardous_snarks.MWA__011__AvgMale"
     ChamelHeadSkins(82)="AW7PawnSkins.heads.FWA__218__FemSH"
     ChamelHeadSkins(83)="AW7PawnSkins.heads.FWA__228__FemLH"
     ChamelHeadSkins(84)="AW7PawnSkins.heads.FWA__229__FemSH"
     ChamelHeadSkins(85)="AW7PawnSkins.heads.FWA__230__FemSH"
     ChamelHeadSkins(86)="AW7PawnSkins.heads.FWA__231__FemSHCropped"
     ChamelHeadSkins(87)="AW7PawnSkins.heads.FWA__232__FemSHCropped"
     ChamelHeadSkins(88)="AW7PawnSkins.heads.FWA__233__FemSHcropped"
     ChamelHeadSkins(89)="AW7PawnSkins.heads.FWA__234__FemSH"
     ChamelHeadSkins(90)="AW7PawnSkins.heads.FWA__235__FemSH"
     ChamelHeadSkins(91)="AW7PawnSkins.heads.FWA__242__FemSH"
     ChamelHeadSkins(92)="AW7PawnSkins.heads.MBA__201__AvgBrotha"
     ChamelHeadSkins(93)="AW7PawnSkins.heads.MBA__203__AvgBrotha"
     ChamelHeadSkins(94)="AW7PawnSkins.heads.MMA__204__AvgMale"
     ChamelHeadSkins(95)="AW7PawnSkins.heads.MMA__216__AvgMale"
     ChamelHeadSkins(96)="AW7PawnSkins.heads.MWA__200__AvgMaleBig"
     ChamelHeadSkins(97)="AW7PawnSkins.heads.MWA__202__AvgMale"
     ChamelHeadSkins(98)="AW7PawnSkins.heads.MWA__205__AvgMale"
     ChamelHeadSkins(99)="AW7PawnSkins.heads.MWA__206__AvgMale"
     ChamelHeadSkins(100)="AW7PawnSkins.heads.MWA__207__AvgMale"
     ChamelHeadSkins(101)="AW7PawnSkins.heads.MWA__208__AvgMale"
     ChamelHeadSkins(102)="AW7PawnSkins.heads.MWA__209__AvgMale"
     ChamelHeadSkins(103)="AW7PawnSkins.heads.MWA__210__AvgMale"
     ChamelHeadSkins(104)="AW7PawnSkins.heads.MWA__211__AvgMale"
     ChamelHeadSkins(105)="AW7PawnSkins.heads.MWA__212__AvgMale"
     ChamelHeadSkins(106)="AW7PawnSkins.heads.MWA__213__AvgMale"
     ChamelHeadSkins(107)="AW7PawnSkins.heads.MWA__215__AvgMale"
     ChamelHeadSkins(108)="AW7PawnSkins.heads.MWA__217__AvgMale"
     ChamelHeadSkins(109)="AW7PawnSkins.heads.MWA__219__AvgMale"
     ChamelHeadSkins(110)="AW7PawnSkins.heads.MWA__220__AvgMale"
     ChamelHeadSkins(111)="AW7PawnSkins.heads.MWA__221__AvgMale"
     ChamelHeadSkins(112)="AW7PawnSkins.heads.MWA__222__AvgMale"
     ChamelHeadSkins(113)="AW7PawnSkins.heads.MWA__223__AvgMale"
     ChamelHeadSkins(114)="AW7PawnSkins.heads.MWA__224__AvgMale"
     ChamelHeadSkins(115)="AW7PawnSkins.heads.MWA__225__AvgMale"
     ChamelHeadSkins(116)="AW7PawnSkins.heads.MWA__226__AvgMale"
     ChamelHeadSkins(117)="AW7PawnSkins.heads.MWA__227__AvgMale"
     ChamelHeadSkins(118)="AW7PawnSkins.heads.MWA__236__AvgMale"
     ChamelHeadSkins(119)="AW7PawnSkins.heads.MWA__237__AvgMale"
     ChamelHeadSkins(120)="AW7PawnSkins.heads.MWA__238__AvgMale"
     ChamelHeadSkins(121)="AW7PawnSkins.heads.MWA__239__AvgMaleBig"
     ChamelHeadSkins(122)="AW7PawnSkins.heads.MWA__240__AvgMale"
     ChamelHeadSkins(123)="AW7PawnSkins.heads.MWA__241__AvgMale"
     ChamelHeadSkins(124)="AW7PawnSkins.heads.MWA__243__AvgMale"
     ChamelHeadSkins(125)="AW7PawnSkins.heads.MWA__244__AvgMaleBig"
     ChamelHeadSkins(126)="AW7PawnSkins.heads.MWA__245__AvgMaleBig"
     ChamelHeadSkins(127)="AW7PawnSkins.heads.MWA__247__AvgMale"
     ChamelHeadSkins(128)="End"
     ChamelHeadMeshPkgs(0)="heads"
     AmbientGlow=0
     DamageMult=1.000000
     CollisionRadius=22.000000
     CollisionHeight=72.000000
     TakesMachinegunDamage=1.000000
     TakesPistolHeadShot=1.000000
     TakesShotgunHeadShot=1.000000
     TakesRifleHeadShot=1.000000
     TakesShovelHeadShot=1.000000
     TakesOnFireDamage=1.000000
     TakesAnthraxDamage=1.000000
     TakesChemDamage=1.000000
     TakesShockerDamage=0.300000
     TakesSledgeDamage=1.000000
     TakesMacheteDamage=0.750000
     TakesScytheDamage=1.000000
     TakesDervishDamage=0.750000
     TakesZombieSmashDamage=1.000000
}
