///////////////////////////////////////////////////////////////////////////////
// EABystandersNoReport
// Bystanders for the bonus stage. These don't report to the GameState that we
// died, so the dude can shoot people here all he wants but it's not "real"
///////////////////////////////////////////////////////////////////////////////
class EABystandersNoReport extends EABystanders;

var(Events) bool bDeathTriggerPlayerOnly;		// if true, will only trigger Event when killed by a player.

///////////////////////////////////////////////////////////////////////////////
// DON'T record pawn dead if we killed them
///////////////////////////////////////////////////////////////////////////////
function Died(Controller Killer, class<DamageType> damageType, vector HitLocation)
{
	local FPSGameInfo checkg;

	if ( bDeleteMe )
		return; //already destroyed

	// If I'm used for an errand, check to see if I did anything important
	CheckForErrandCompleteOnDeath(Killer);

	// mutator hook to prevent deaths
	// WARNING - don't prevent bot suicides - they suicide when really needed
	if ( Level.Game.PreventDeath(self, Killer, damageType, HitLocation) )
	{
		Health = max(Health, 1); //mutator should set this higher
		return;
	}
	Health = Min(0, Health);

	Level.Game.Killed(Killer, Controller, self, damageType);

	if ( Killer != None )
	{
		if (!bDeathTriggerPlayerOnly || Killer.bIsPlayer)
			TriggerEvent(Event, self, Killer.Pawn);
	}
	else if (!bDeathTriggerPlayerOnly)
		TriggerEvent(Event, self, None);

	// This was from the engine. I changed it to a constant at least.
	// It apparently bumps up a person who's moving when they die, to make it 
	// a little more dramatic.
	Velocity.Z *= DIE_Z_MULT;

	if ( IsHumanControlled() )
		PlayerController(Controller).ForceDeathUpdate();

	PlayDying(DamageType, HitLocation);
	
	if ( Level.Game.bGameEnded )
		return;
	if ( !bPhysicsAnimUpdate && !IsLocallyControlled() )
		ClientDying(DamageType, HitLocation);
}

///////////////////////////////////////////////////////////////////////////////
// Same as super, but don't record anything to the GameState
///////////////////////////////////////////////////////////////////////////////
function bool HandleSpecialShots(int Damage, vector HitLocation, vector Momentum, out class<DamageType> ThisDamage,
							vector XYdir, Pawn InstigatedBy, out int returndamage, out byte HeadShot)
{
	local float PercentUpBody, ZDist, DistToMe;

	// Only let the player get special head shots
	// Projectile weapons
	if(FPSPawn(InstigatedBy).bPlayer)
	{
		if(Health > 0)
		{
			if(ThisDamage == class'ShotgunDamage'
				|| ThisDamage == class'RifleDamage'
				|| ThisDamage == class'BulletDamage')
			{
				// For if no damage is done
				if(TakesShotgunHeadShot == 0.0
					|| TakesShotgunHeadShot == 0.0
					|| TakesPistolHeadShot == 0.0)
				{
					// Make a ricochet sound and puff out some smoke and sparks
					SparkHit(HitLocation, Momentum, 1);//Rand(2));
					DustHit(HitLocation, Momentum);
					returndamage = 0;
					return true;
				}

				PercentUpBody = (hitlocation.z - Location.z)/CollisionHeight;
				//log("dist to head for explode try "$VSize(XYDir));
				//log("percent up body "$PercentUpBody);
				// Check to see if we're in fake head shot range
				if(PercentUpBody > HEAD_RATIO_OF_FULL_HEIGHT)
				{
					DistToMe = VSize(XYdir);
					
					if((DistToMe < DISTANCE_TO_PUNCTURE_HEAD
							|| (P2GameInfo(Level.Game) != None
								&& P2GameInfo(Level.Game).PlayerGetsHeadShots()))
						&& ThisDamage == class'BulletDamage')
					// Is close enough with a pistol and behind them to puncture head with a pistol
					{
						// Check a little more accurately, if you actually hit the head or not
						// And check to make sure the guy got shot from behind, before we allow it.
						if(((Momentum dot vector(Rotation)) > 0
								&& CheckHeadForHit(HitLocation, ZDist))
							|| (P2GameInfo(Level.Game) != None
								&& P2GameInfo(Level.Game).PlayerGetsHeadShots()))
						{
							// We've hit the head, now reduce the damage, if necessary
							if(!(P2GameInfo(Level.Game) != None
								&& P2GameInfo(Level.Game).PlayerGetsHeadShots()))
								returndamage = TakesPistolHeadShot*HealthMax;
							else
								returndamage = HealthMax;
							// if this kills them, puncture the head
							if(returndamage >= Health
								&& bHeadCanComeOff
								&& !(P2GameInfo(Level.Game) != None
									&& P2GameInfo(Level.Game).PlayerGetsHeadShots()))
							{
								if(class'P2Player'.static.BloodMode())
									PunctureHead(HitLocation, Momentum);
							}
							HeadShot = 1;
							return true;
						}
						// Over the head but not hitting the head means this guy won't take damage
						// If we had hit the head, the above would have returned already
						if(ZDist > 0) 
							return false;
					}
					else if(DistToMe < DISTANCE_TO_EXPLODE_HEAD
						&& ThisDamage == class'ShotgunDamage')
					// Is close enough with a shotgun to explode the head
					{
						// Check a little more accurately, if you actually hit the head or not
						if(CheckHeadForHit(HitLocation, ZDist))
						{
							// We've hit the head, now reduce the damage, if necessary
							if(!(P2GameInfo(Level.Game) != None
								&& P2GameInfo(Level.Game).PlayerGetsHeadShots()))
								returndamage = TakesShotgunHeadShot*HealthMax;
							else
								returndamage = HealthMax;

							// if this kills them, blow their head up
							if(returndamage >= Health
								&& bHeadCanComeOff)
							{
								if(class'P2Player'.static.BloodMode())
								{
									ExplodeHead(HitLocation, Momentum);
								}
							}
							HeadShot = 1;
							return true;
						}
						// Over the head but not hitting the head means this guy won't take damage
						// If we had hit the head, the above would have returned already
						if(ZDist > 0)
							return false;
					}
					else if(ThisDamage == class'RifleDamage')
					// Sniper rifle round to the head punctures your head
					{
						// We've hit the head, now reduce the damage, if necessary
						//returndamage = TakesRifleHeadShot*HealthMax;
						returndamage = HealthMax;
						// if this kills them, puncture the head
						if(returndamage >= Health
								&& bHeadCanComeOff)
						{
							if(class'P2Player'.static.BloodMode())
								PunctureHead(HitLocation, Momentum);
						}
						HeadShot = 1;
						return true;
					}
				}
				// continue on, if this didn't take
			}
			else if((P2GameInfo(Level.Game) != None
						&& P2GameInfo(Level.Game).PlayerGetsHeadShots())
					&& ThisDamage == class'MachinegunDamage')
				// Just for the silly head shots cheat
			{
				PercentUpBody = (hitlocation.z - Location.z)/CollisionHeight;
				if(PercentUpBody > HEAD_RATIO_OF_FULL_HEIGHT)
				{
					// We've hit the head, now make it take two machine gun bullets
					// to down a guy when hit in the head.
					returndamage = 0.5*HealthMax;
					HeadShot = 1;
					return true;
				}
				// Over the head but not hitting the head means this guy won't take damage
				// If we had hit the head, the above would have returned already
				if(ZDist > 0) 
					return false;
			}
		}
	}

	// Melee
	if(ClassIsChildOf(ThisDamage, class'BludgeonDamage'))
	{
		if(CheckHeadForHit(HitLocation, ZDist, true))
		{
			// shovel's knock heads off
			if(ThisDamage == class'ShovelDamage')
			{
				// Decide randomly to knock the head off. If you're closer to
				// death, then be more likely to make the head pop off
				// Let the player take off heads, and let NPCs take off each
				// others heads
				if(bHeadCanComeOff
					&& class'P2Player'.static.BloodMode()
					&& FRand() >= Health/HealthMax
					&& (P2Player(InstigatedBy.Controller) != None
						|| P2Player(Controller) == None))
				{
					// We've hit the head, now compound the damage, if necessary
					returndamage = TakesShovelHeadShot*HealthMax;
				}
				else
				{
					returndamage = Damage;
				}
				// If this kills them, pop off the head
				if(returndamage >= Health)
				{
					PopOffHead(HitLocation, Momentum);
					HeadShot = 1;
					PlaySound(ShovelCleaveHead,,,,,GetRandPitch());
				}
				else // Otherwise, they just get hit in the head, hard
					PlaySound(ShovelHitHead,,,,,GetRandPitch());
			}
			else if(ThisDamage == class'BatonDamage')
				// batons incapacitate people.
			{
				if(P2GameInfoSingle(Level.Game) != None
					&& P2GameInfoSingle(Level.Game).VerifySeqTime()
					&& P2Pawn(InstigatedBy) != None
					&& P2Pawn(InstigatedBy).bPlayer)
				{
					// We've hit the head, now reduce the damage, if necessary
					// use same version as shotgun explodes
					returndamage = TakesShotgunHeadShot*HealthMax;
					// if this kills them, explodes the head
					if(returndamage >= Health
							&& bHeadCanComeOff)
					{
						if(class'P2Player'.static.BloodMode())
						{
							ExplodeHead(HitLocation, Momentum);
						}
					}
					HeadShot = 1;
					return true;
				}
				else
					returndamage = Damage;
			}
			else if(ThisDamage == class'KickingDamage')
			{
				PlaySound(FootKickHead,,,,,GetRandPitch());
				// kicking to the face just draws blood
				returndamage = Damage;
			}
			else
			{
				returndamage = Damage;
			}
			return true;
		}
		else // if it was a bludgeon attack, but didn't hit the
			// face, then don't draw blood
		{
			if(ThisDamage == class'ShovelDamage')
			{
				PlaySound(ShovelHitBody,,,,,GetRandPitch());
			}
			else if(ThisDamage == class'KickingDamage')
			{
				PlaySound(FootKickBody,,,,,GetRandPitch());
			}
			// Cutting attacks always draw blood, but if not, at this point
			// we only want a dust hit, so change the damage type.
			if(!ClassIsChildOf(ThisDamage, class'CuttingDamage'))
				ThisDamage = class'BodyDamage';
		}
	}

	return false;
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// Dying
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
state Dying
{
	///////////////////////////////////////////////////////////////////////////////
	// Same as super, but don't record anything to the gamestate
	///////////////////////////////////////////////////////////////////////////////
	function TakeDamage( int Damage, Pawn InstigatedBy, Vector Hitlocation,
							Vector Momentum, class<DamageType> DamageType)
	{
		local vector shotDir, hitNormal;
		local actor tmpActor;
		local float maxDim;
		local bool bReInitted;
		local vector KMomentum;
		local bool bUseSuper;

		if(Level.Game != None
			&& Level.Game.bIsSinglePlayer)
		{
			if(class'P2Player'.static.BloodMode()
				// can't cut off player limbs
				&& P2Player(Controller) == None)
			{
				if(ClassIsChildOf(damageType, class'MacheteDamage'))
					bUseSuper = !(HandleSever(InstigatedBy, momentum, damagetype, INVALID_LIMB, Damage, HitLocation));
				else if(ClassIsChildOf(damageType, class'SledgeDamage'))
					bUseSuper = !(HandleSledge(InstigatedBy, momentum, damagetype, Damage, HitLocation));
				else if(ClassIsChildOf(damageType, class'ScytheDamage'))
					bUseSuper = !(HandleScythe(InstigatedBy, momentum, damagetype, Damage, HitLocation));
				else
					bUseSuper=true;
				if(ClassIsChildOf(damageType, class'CuttingDamage'))
					PlaySound(FleshHit,,,,,GetRandPitch());
			}

			if(bUseSuper)
			{
				// Check to cut off limbs, and pass it along to original TakeDamage
				// for momentum transfer
				if(class'P2Player'.static.BloodMode()
					&& ClassIsChildOf(damageType, class'ExplodedDamage'))
					HandleExplosionDead(Damage, InstigatedBy, HitLocation, momentum, damageType);

				// Save who last hit us
				Instigator = InstigatedBy;
	
				// Keep a karma momentum, and cap it from going through walls and floor too much
				KMomentum = Momentum;
	
				CapKarmaMomentum(KMomentum, damageType, 1.0, 1.0, HitLocation);

				// If fire hit you, even dead, catch on fire for sure
				if(ClassIsChildOf(damageType, class'BurnedDamage'))
					SetOnFire(FPSPawn(instigatedBy), (damageType==class'NapalmDamage'));
				// If you're infected, be infected even in death
				if(damageType == class'ChemDamage')
					SetInfected(FPSPawn(instigatedBy));
	
				// If fire has killed me,
				// or we're on fire and we died,
				// then swap to my burn victim mesh
				if(damageType == class'OnFireDamage'
					|| ClassIsChildOf(damageType, class'BurnedDamage')
					|| MyBodyFire != None)
					SwapToBurnVictim();
	
				if(MyHead != None
					&& bHeadCanComeOff)
				{
					// Check to remove their heads off, even when dead, with the shotgun or shovel
					// If the thing hit the head, and you're close enough to take it off
					if(VSize(MyHead.Location - HitLocation) < CollisionRadius
						&& DamageType == class'ShotgunDamage')
					{
						if(VSize(Location - InstigatedBy.Location) < DISTANCE_TO_EXPLODE_HEAD)
						{
							if(class'P2Player'.static.BloodMode())
							{
								ExplodeHead(HitLocation, Momentum);
							}
						}
					}
					// Because the collision on the shovel is so large and general, just take their
					// head off anyway, half the time.
					else if(DamageType == class'ShovelDamage'
						&& FRand() > 0.5)
					{
						if(class'P2Player'.static.BloodMode())
						{
							PopOffHead(HitLocation, Momentum);
							PlaySound(ShovelCleaveHead,,,,,GetRandPitch());
						}
					}
				}
	
				// If his skeleton has been taken away from him, reserve it
				// for some more time, if one's available
				if(KParams == None
					&& AllowRagdoll(DamageType))
				{
					GetKarmaSkeleton();
					if(KParams != None)
					{
						// Don't crouch or crawl anymore
						ShouldCrouch(false);
						ShouldDeathCrawl(false);
	
						StopAnimating();
						
						bPhysicsAnimUpdate = false;
	
						//log(self$" setting ragdoll "$GetStateName());
						SetPhysics(PHYS_KarmaRagDoll);
	
						// Get things going first, for sure
						KWake();
						KSetSkelVel( (DeathVelMag * Normal(KMomentum)) );
						bReInitted=true;
					}
				}
	
				if(Level.Game != None
					&& FPSGameInfo(Level.Game).bIsSinglePlayer
					&& (KarmaParamsSkel(KParams) != None) 
					&& Physics == PHYS_KarmaRagDoll)
				{
					// Reset timer to ensure use of ragdoll
					RagDollStartTime = Level.TimeSeconds;
	
					// Again, (as in p2pawn, takedamage) modify explosion momentum for funnier effects
					if(ClassIsChildOf(damageType,class'ExplodedDamage'))
					{
						// Move the hit point around the ragdoll to get a better tumble
						HitLocation = RAND_MOVE_AROUND_RAGDOLL_HIT*VRand() + HitLocation;
	
						// Make sure to throw it a good ways, if we can
						if(KMomentum.z > 0
							&& KMomentum.z < DeathVelMag*Mass)
						{
							KMomentum.z *= 8;
							KMomentum = MAX_EXPL_MOMENTUM*Normal(KMomentum);
						}
	
						//log(self$" hit "$KMomentum$" size "$VSize(KMomentum)$" loc "$Location$" hit "$HitLocation);
						/*
						if(!bReInitted)
							KSetSkelVel(THROW_VEL_RATIO_EXPL*KMomentum);
						else
						*/
						if(bReInitted)
						{
							//log(self$" setting ragdoll "$GetStateName());
							// Ensure each time we set the physics
							SetPhysics(PHYS_KarmaRagDoll);
							bReInitted=false;
						}
						KAddImpulse(KMomentum, HitLocation, BONE_PELVIS);
					}
					else
					{
						// If you don't take any of these damages, then block them here
						if((TakesMachinegunDamage == 0.0
								&& DamageType == class'MachinegunDamage')
							|| (TakesShotgunHeadShot == 0.0
								&& DamageType == class'ShotgunDamage')
							|| (TakesPistolHeadShot == 0.0
								&& DamageType == class'BulletDamage'))
						{
							// Make a ricochet sound and puff out some smoke and sparks
							SparkHit(HitLocation, Momentum, 1);
							DustHit(HitLocation, Momentum);
						}
						else
							// Throw out some effects of the hit
							PlayHit(Damage, hitLocation, damageType, Momentum);
	
						if(VSize(KMomentum) > 0)
						{
							// Move the body
							if(!bReInitted)

								KSetSkelVel(THROW_VEL_RATIO*KMomentum);
							else
							{
								//log(self$" setting ragdoll "$GetStateName());
								// Ensure each time we set the physics
								SetPhysics(PHYS_KarmaRagDoll);
								bReInitted=false;
							}
							//log(self$" hit "$KMomentum$" size "$VSize(KMomentum)$" loc "$Location$" hit "$HitLocation);
							KAddImpulse(KMomentum, HitLocation, BONE_PELVIS);
						}
					}
				}
			}
		}
	}
}

defaultproperties
{
     SeverBone(0)="MALE01 L UpperArm"
     SeverBone(1)="MALE01 L Forearm"
     SeverBone(2)="MALE01 R UpperArm"
     SeverBone(3)="MALE01 R Forearm"
     SeverBone(4)="MALE01 L Thigh"
     SeverBone(5)="MALE01 L Calf"
     SeverBone(6)="MALE01 R Thigh"
     SeverBone(7)="MALE01 r calf"
     BoneArr(0)=1
     BoneArr(1)=1
     BoneArr(2)=1
     BoneArr(3)=1
     BoneArr(4)=1
     BoneArr(5)=1
     BoneArr(6)=1
     BoneArr(7)=1
     BodyHitSounds(0)=Sound'MiscSounds.People.bodyhitground1'
     BodyHitSounds(1)=Sound'MiscSounds.People.bodyhitground2'
     ChameleonSkins(0)="ChameleonSkins.FB__094__Fem_SS_Shorts"
     ChameleonSkins(1)="ChameleonSkins.MW__022__Avg_M_SS_Pants"
     ChameleonSkins(2)="ChameleonSkins.FB__112__Fat_F_SS_Pants"
     ChameleonSkins(3)="ChameleonSkins.MB__015__Avg_M_Jacket_Pants"
     ChameleonSkins(4)="ChameleonSkins.FB__120__Fem_LS_Pants"
     ChameleonSkins(5)="ChameleonSkins.MB__026__Avg_M_SS_Pants"
     ChameleonSkins(6)="ChameleonSkins.FM__095__Fem_SS_Shorts"
     ChameleonSkins(7)="ChameleonSkins.MB__048__Avg_M_SS_Pants"
     ChameleonSkins(8)="ChameleonSkins.FM__118__Fem_LS_Pants"
     ChameleonSkins(9)="ChameleonSkins.MB__060__Avg_M_SS_Shorts"
     ChameleonSkins(10)="ChameleonSkins.FW__083__Fem_LS_Skirt"
     ChameleonSkins(11)="ChameleonSkins.MB__104__Fat_M_SS_Pants"
     ChameleonSkins(12)="ChameleonSkins.FW__084__Fem_LS_Skirt"
     ChameleonSkins(13)="ChameleonSkins.MM__018__Avg_M_Jacket_Pants"
     ChameleonSkins(14)="ChameleonSkins.FW__085__Fem_LS_Skirt"
     ChameleonSkins(15)="ChameleonSkins.MM__027__Avg_M_SS_Pants"
     ChameleonSkins(16)="ChameleonSkins.FW__092__Fem_LS_Skirt"
     ChameleonSkins(17)="ChameleonSkins.MM__102__Fat_M_SS_Pants"
     ChameleonSkins(18)="ChameleonSkins.FW__096__Fem_SS_Shorts"
     ChameleonSkins(19)="ChameleonSkins.MM__061__Avg_M_SS_Shorts"
     ChameleonSkins(20)="ChameleonSkins.FW__113__Fat_F_SS_Pants"
     ChameleonSkins(21)="ChameleonSkins.MM__105__Fat_M_SS_Pants"
     ChameleonSkins(22)="ChameleonSkins.FW__119__Fem_LS_Pants"
     ChameleonSkins(23)="ChameleonSkins.MM__107__Fat_M_SS_Pants"
     ChameleonSkins(24)="ChameleonSkins.FW__124__Fem_LS_Pants"
     ChameleonSkins(25)="ChameleonSkins.MW__016__Avg_M_Jacket_Pants"
     ChameleonSkins(26)="ChameleonSkins.MW__017__Avg_M_Jacket_Pants"
     ChameleonSkins(27)="ChameleonSkins.MW__019__Avg_M_Jacket_Pants"
     ChameleonSkins(28)="ChameleonSkins.MW__021__Avg_M_Jacket_Pants"
     ChameleonSkins(29)="ChameleonSkins.MW__024__Avg_M_SS_Pants"
     ChameleonSkins(30)="ChameleonSkins.MW__025__Avg_M_SS_Pants"
     ChameleonSkins(31)="ChameleonSkins.MW__028__Avg_M_SS_Pants"
     ChameleonSkins(32)="ChameleonSkins.MW__062__Avg_M_SS_Shorts"
     ChameleonSkins(33)="ChameleonSkins.MW__098__Fat_M_Jacket_Pants"
     ChameleonSkins(34)="ChameleonSkins.MW__106__Fat_M_SS_Pants"
     ChameleonSkins(35)="ChameleonSkins.MW__160__Avg_M_SS_Pants"
     ChameleonSkins(36)="ChameleonSkins.FM__082__Fem_LS_Skirt"
     ChameleonSkins(37)="ED_HeadsAndSkins.Bodies.FW__010__Fem_SS_Shorts"
     ChameleonSkins(38)="ED_HeadsAndSkins.Bodies.FW__007__Fem_LS_Skirt"
     ChameleonSkins(39)="ED_HeadsAndSkins.Bodies.FW__007__Fem_LS_Skirt"
     ChameleonSkins(40)="ED_HeadsAndSkins.Bodies.FW__008__Fem_SS_Shorts"
     ChameleonSkins(41)="ED_HeadsAndSkins.Bodies.FW__006__Fem_SS_Skirt"
     ChameleonSkins(42)="ED_HeadsAndSkins.Bodies.FW__007__Fem_SS_Skirt"
     ChameleonSkins(43)="ED_HeadsAndSkins.Bodies.FW__009__Fem_LS_Pants"
     ChameleonSkins(44)="ED_HeadsAndSkins.Bodies.FW__007__Fem_LS_Skirt"
     ChameleonSkins(45)="ED_HeadsAndSkins.Bodies.MB__001__Avg_M_LS_Pants"
     ChameleonSkins(46)="ED_HeadsAndSkins.Bodies.MB__002__Avg_M_LS_Pants"
     ChameleonSkins(47)="ED_HeadsAndSkins.Bodies.MB__003__Avg_M_LS_Pants"
     ChameleonSkins(48)="ED_HeadsAndSkins.Bodies.MW__004__Avg_M_SS_Pants"
     ChameleonSkins(49)="ED_HeadsAndSkins.Bodies.FW__005__Fem_LS_Skirt"
     ChameleonSkins(50)="ED_HeadsAndSkins.Bodies.FW__236__Fem_LS_Skirt"
     ChameleonSkins(51)="ED_HeadsAndSkins.Bodies.FW__236__Fem_LS_Skirt"
     ChameleonSkins(52)="ED_HeadsAndSkins.Bodies.MB__107__Avg_M_SS_Pants"
     ChameleonSkins(53)="AW7PawnSkins.Skins.MW__225__Avg_M_SS_Pants"
     ChameleonSkins(54)="AW7PawnSkins.PaletteSwap.FB__200__Fem_SS_Shorts"
     ChameleonSkins(55)="AW7PawnSkins.PaletteSwap.FB__201__Fat_F_SS_Pants"
     ChameleonSkins(56)="AW7PawnSkins.PaletteSwap.FB__202__Fem_LS_Pants"
     ChameleonSkins(57)="AW7PawnSkins.PaletteSwap.FM__203__Fem_LS_Pants"
     ChameleonSkins(58)="AW7PawnSkins.PaletteSwap.FW__204__Fem_LS_Skirt"
     ChameleonSkins(59)="AW7PawnSkins.PaletteSwap.FW__205__Fem_LS_Skirt"
     ChameleonSkins(60)="AW7PawnSkins.PaletteSwap.FW__206__Fem_LS_Skirt"
     ChameleonSkins(61)="AW7PawnSkins.PaletteSwap.FW__207__Fat_F_SS_Pants"
     ChameleonSkins(62)="AW7PawnSkins.PaletteSwap.FW__208__Fem_LS_Pants"
     ChameleonSkins(63)="AW7PawnSkins.PaletteSwap.FW__209__Fem_LS_Pants"
     ChameleonSkins(64)="AW7PawnSkins.PaletteSwap.MM__210__Avg_M_SS_Pants"
     ChameleonSkins(65)="AW7PawnSkins.PaletteSwap.MM__211__Avg_M_SS_Shorts"
     ChameleonSkins(66)="AW7PawnSkins.PaletteSwap.MW__212__Avg_M_Jacket_Pants"
     ChameleonSkins(67)="AW7PawnSkins.PaletteSwap.MW__213__Avg_M_SS_Pants"
     ChameleonSkins(68)="AW7PawnSkins.PaletteSwap.MW__214__Avg_M_SS_Shorts"
     ChameleonSkins(69)="AW7PawnSkins.PaletteSwap.MW__215__Fat_M_SS_Pants"
     ChameleonSkins(70)="AW7PawnSkins.PaletteSwap.MW__216__Avg_M_SS_Pants"
     ChameleonSkins(71)="AW7PawnSkins.Skins.FW__402__Fem_LS_LongSkirt"
     ChameleonSkins(72)="AW7PawnSkins.Skins.FW__403__Fem_LS_LongSkirt"
     ChameleonSkins(73)="AW7PawnSkins.Skins.FM__404__Fem_LS_LongSkirt"
     ChameleonSkins(74)="AW7PawnSkins.Skins.FW__405__Fem_LS_LongSkirt"
     ChameleonSkins(75)="AW7PawnSkins.Skins.FM__400__Fem_LS_LongSkirt"
     ChameleonSkins(76)="AW7PawnSkins.Skins.FW__217__Fem_LS_Skirt"
     ChameleonSkins(77)="AW7PawnSkins.Skins.FW__218__Fem_SS_Shorts"
     ChameleonSkins(78)="AW7PawnSkins.Skins.FW__219__Fem_SS_Shorts"
     ChameleonSkins(79)="AW7PawnSkins.Skins.FW__220__Fem_LS_Skirt"
     ChameleonSkins(80)="AW7PawnSkins.Skins.MM__226__Avg_M_SS_Pants"
     ChameleonSkins(81)="End"
     ChameleonMeshPkgs(0)="Characters"
     ChamelHeadSkins(0)="ChamelHeadSkins.MWA__001__AvgMale"
     ChamelHeadSkins(1)="ChamelHeadSkins.MBA__013__AvgBrotha"
     ChamelHeadSkins(2)="ChamelHeadSkins.MBA__014__AvgBrotha"
     ChamelHeadSkins(3)="ChamelHeadSkins.MMA__016__AvgMale"
     ChamelHeadSkins(4)="ChamelHeadSkins.MMF__024__FatMale"
     ChamelHeadSkins(5)="ChamelHeadSkins.MWA__002__AvgMale"
     ChamelHeadSkins(6)="ChamelHeadSkins.MMA__003__AvgMale"
     ChamelHeadSkins(7)="ChamelHeadSkins.MWA__004__AvgMale"
     ChamelHeadSkins(8)="ChamelHeadSkins.MWA__005__AvgMale"
     ChamelHeadSkins(9)="ChamelHeadSkins.MWA__006__AvgMale"
     ChamelHeadSkins(10)="ChamelHeadSkins.MWA__007__AvgMale"
     ChamelHeadSkins(11)="ChamelHeadSkins.MWA__008__AvgMale"
     ChamelHeadSkins(12)="ChamelHeadSkins.MWA__009__AvgMale"
     ChamelHeadSkins(13)="ChamelHeadSkins.MWA__010__AvgMale"
     ChamelHeadSkins(14)="ChamelHeadSkins.MWA__011__AvgMale"
     ChamelHeadSkins(15)="ChamelHeadSkins.MWA__015__AvgMale"
     ChamelHeadSkins(16)="ChamelHeadSkins.MWA__021__AvgMaleBig"
     ChamelHeadSkins(17)="ChamelHeadSkins.MWA__035__AvgMale"
     ChamelHeadSkins(18)="ChamelHeadSkins.MWF__025__FatMale"
     ChamelHeadSkins(19)="ChamelHeadSkins.MWA__022__AvgMaleBig"
     ChamelHeadSkins(20)="ChamelHeadSkins.FBA__033__FemSH"
     ChamelHeadSkins(21)="ChamelHeadSkins.FMA__028__FemSH"
     ChamelHeadSkins(22)="ChamelHeadSkins.FMA__034__FemSH"
     ChamelHeadSkins(23)="ChamelHeadSkins.FWA__026__FemLH"
     ChamelHeadSkins(24)="ChamelHeadSkins.FWA__027__FemLH"
     ChamelHeadSkins(25)="ChamelHeadSkins.FWA__029__FemSH"
     ChamelHeadSkins(26)="ChamelHeadSkins.FWA__032__FemSH"
     ChamelHeadSkins(27)="ChamelHeadSkins.FWF__023__FatFem"
     ChamelHeadSkins(28)="ChamelHeadSkins.FWA__037__FemSHcropped"
     ChamelHeadSkins(29)="ChamelHeadSkins.FMA__038__FemSHcropped"
     ChamelHeadSkins(30)="ChamelHeadSkins.FMA__039__FemSHcropped"
     ChamelHeadSkins(31)="ChamelHeadSkins.FWA__040__FemSHcropped"
     ChamelHeadSkins(32)="ChamelHeadSkins.MBF__042__FatMale"
     ChamelHeadSkins(33)="ChamelHeadSkins.FBF__043__FatFem"
     ChamelHeadSkins(34)="ChamelHeadSkins.FMF__044__FatFem"
     ChamelHeadSkins(35)="ChamelHeadSkins.FWA__031__FemSH"
     ChamelHeadSkins(36)="ChamelHeadSkins.FBA__063__FemSH"
     ChamelHeadSkins(37)="ED_HeadsAndSkins.Tim_Wambolts.FWA__030__FemSH"
     ChamelHeadSkins(38)="ED_HeadsAndSkins.biohazardous_snarks.MBA__021__AvgBrotha"
     ChamelHeadSkins(39)="ED_HeadsAndSkins.biohazardous_snarks.MBA__022__AvgBrotha"
     ChamelHeadSkins(40)="ED_HeadsAndSkins.biohazardous_snarks.MBA__023__AvgBrotha"
     ChamelHeadSkins(41)="ED_HeadsAndSkins.Tim_Wambolts.FWA__030__FemLH"
     ChamelHeadSkins(42)="ED_HeadsAndSkins.biohazardous_snarks.MBA__024__AvgBrotha"
     ChamelHeadSkins(43)="ED_HeadsAndSkins.biohazardous_snarks.MWA__001__AvgMale"
     ChamelHeadSkins(44)="ED_HeadsAndSkins.biohazardous_snarks.MWA__002__AvgMale"
     ChamelHeadSkins(45)="ED_HeadsAndSkins.biohazardous_snarks.MWA__003__AvgMale"
     ChamelHeadSkins(46)="ED_HeadsAndSkins.biohazardous_snarks.MWA__004__AvgMale"
     ChamelHeadSkins(47)="ED_HeadsAndSkins.biohazardous_snarks.MWA__005__AvgMale"
     ChamelHeadSkins(48)="ED_HeadsAndSkins.biohazardous_snarks.FWA__244__FemSH"
     ChamelHeadSkins(49)="ED_HeadsAndSkins.Mimics.MMA__007__AvgMale"
     ChamelHeadSkins(50)="ED_HeadsAndSkins.Steevees.MWA__409__AvgMale"
     ChamelHeadSkins(51)="ED_HeadsAndSkins.Steevees.MWA__410__AvgMale"
     ChamelHeadSkins(52)="ED_HeadsAndSkins.Steevees.MWA__411__AvgMale"
     ChamelHeadSkins(53)="ED_HeadsAndSkins.Steevees.MWA__012__AvgMale"
     ChamelHeadSkins(54)="ED_HeadsAndSkins.Steevees.MWA__013__AvgMale"
     ChamelHeadSkins(55)="ED_HeadsAndSkins.Steevees.MWA__014__AvgMale"
     ChamelHeadSkins(56)="ED_HeadsAndSkins.Steevees.MWA__015__AvgMale"
     ChamelHeadSkins(57)="ED_HeadsAndSkins.Steevees.MWA__016__AvgMale"
     ChamelHeadSkins(58)="ED_HeadsAndSkins.Steevees.MWA__017__AvgMale"
     ChamelHeadSkins(59)="ED_HeadsAndSkins.Tim_Wambolts.MWA__018__AvgMale"
     ChamelHeadSkins(60)="ED_HeadsAndSkins.Tim_Wambolts.MWA__019__AvgMale"
     ChamelHeadSkins(61)="ED_HeadsAndSkins.Tim_Wambolts.MWA__020__AvgMale"
     ChamelHeadSkins(62)="ED_HeadsAndSkins.SniperX.MWA__008__AvgMale"
     ChamelHeadSkins(63)="ED_HeadsAndSkins.biohazardous_snarks.MWA__006__AvgMale"
     ChamelHeadSkins(64)="ED_HeadsAndSkins.biohazardous_snarks.MWA__007__AvgMale"
     ChamelHeadSkins(65)="ED_HeadsAndSkins.biohazardous_snarks.MWA__008__AvgMale"
     ChamelHeadSkins(66)="ED_HeadsAndSkins.biohazardous_snarks.MMA__009__AvgMale"
     ChamelHeadSkins(67)="ED_HeadsAndSkins.biohazardous_snarks.MWA__010__AvgMale"
     ChamelHeadSkins(68)="ED_HeadsAndSkins.biohazardous_snarks.MWA__011__AvgMale"
     ChamelHeadSkins(69)="ED_HeadsAndSkins.Steevees.FWA__025__FemSH"
     ChamelHeadSkins(70)="ED_HeadsAndSkins.Tim_Wambolts.FWA__026__FemSH"
     ChamelHeadSkins(71)="ED_HeadsAndSkins.Tim_Wambolts.FWA__027__FemSH"
     ChamelHeadSkins(72)="ED_HeadsAndSkins.Tim_Wambolts.FWA__028__FemLH"
     ChamelHeadSkins(73)="ED_HeadsAndSkins.Tim_Wambolts.FWA__029__FemSH"
     ChamelHeadSkins(74)="ED_HeadsAndSkins.Tim_Wambolts.FWA__237__FemSHcropped"
     ChamelHeadSkins(75)="ED_HeadsAndSkins.Tim_Wambolts.FWA__238__FemSHcropped"
     ChamelHeadSkins(76)="ED_HeadsAndSkins.Tim_Wambolts.FWA__031__FemSH"
     ChamelHeadSkins(77)="ED_HeadsAndSkins.Tim_Wambolts.FWA__242__FemSH"
     ChamelHeadSkins(78)="ED_HeadsAndSkins.Tim_Wambolts.FWA__243__FemSH"
     ChamelHeadSkins(79)="ED_HeadsAndSkins.biohazardous_snarks.FWA__239"
     ChamelHeadSkins(80)="ED_HeadsAndSkins.biohazardous_snarks.MWA__010__AvgMale"
     ChamelHeadSkins(81)="ED_HeadsAndSkins.biohazardous_snarks.MWA__011__AvgMale"
     ChamelHeadSkins(82)="AW7PawnSkins.heads.FWA__218__FemSH"
     ChamelHeadSkins(83)="AW7PawnSkins.heads.FWA__228__FemLH"
     ChamelHeadSkins(84)="AW7PawnSkins.heads.FWA__229__FemSH"
     ChamelHeadSkins(85)="AW7PawnSkins.heads.FWA__230__FemSH"
     ChamelHeadSkins(86)="AW7PawnSkins.heads.FWA__231__FemSHCropped"
     ChamelHeadSkins(87)="AW7PawnSkins.heads.FWA__232__FemSHCropped"
     ChamelHeadSkins(88)="AW7PawnSkins.heads.FWA__233__FemSHcropped"
     ChamelHeadSkins(89)="AW7PawnSkins.heads.FWA__234__FemSH"
     ChamelHeadSkins(90)="AW7PawnSkins.heads.FWA__235__FemSH"
     ChamelHeadSkins(91)="AW7PawnSkins.heads.FWA__242__FemSH"
     ChamelHeadSkins(92)="AW7PawnSkins.heads.MBA__201__AvgBrotha"
     ChamelHeadSkins(93)="AW7PawnSkins.heads.MBA__203__AvgBrotha"
     ChamelHeadSkins(94)="AW7PawnSkins.heads.MMA__204__AvgMale"
     ChamelHeadSkins(95)="AW7PawnSkins.heads.MMA__216__AvgMale"
     ChamelHeadSkins(96)="AW7PawnSkins.heads.MWA__200__AvgMaleBig"
     ChamelHeadSkins(97)="AW7PawnSkins.heads.MWA__202__AvgMale"
     ChamelHeadSkins(98)="AW7PawnSkins.heads.MWA__205__AvgMale"
     ChamelHeadSkins(99)="AW7PawnSkins.heads.MWA__206__AvgMale"
     ChamelHeadSkins(100)="AW7PawnSkins.heads.MWA__207__AvgMale"
     ChamelHeadSkins(101)="AW7PawnSkins.heads.MWA__208__AvgMale"
     ChamelHeadSkins(102)="AW7PawnSkins.heads.MWA__209__AvgMale"
     ChamelHeadSkins(103)="AW7PawnSkins.heads.MWA__210__AvgMale"
     ChamelHeadSkins(104)="AW7PawnSkins.heads.MWA__211__AvgMale"
     ChamelHeadSkins(105)="AW7PawnSkins.heads.MWA__212__AvgMale"
     ChamelHeadSkins(106)="AW7PawnSkins.heads.MWA__213__AvgMale"
     ChamelHeadSkins(107)="AW7PawnSkins.heads.MWA__215__AvgMale"
     ChamelHeadSkins(108)="AW7PawnSkins.heads.MWA__217__AvgMale"
     ChamelHeadSkins(109)="AW7PawnSkins.heads.MWA__219__AvgMale"
     ChamelHeadSkins(110)="AW7PawnSkins.heads.MWA__220__AvgMale"
     ChamelHeadSkins(111)="AW7PawnSkins.heads.MWA__221__AvgMale"
     ChamelHeadSkins(112)="AW7PawnSkins.heads.MWA__222__AvgMale"
     ChamelHeadSkins(113)="AW7PawnSkins.heads.MWA__223__AvgMale"
     ChamelHeadSkins(114)="AW7PawnSkins.heads.MWA__224__AvgMale"
     ChamelHeadSkins(115)="AW7PawnSkins.heads.MWA__225__AvgMale"
     ChamelHeadSkins(116)="AW7PawnSkins.heads.MWA__226__AvgMale"
     ChamelHeadSkins(117)="AW7PawnSkins.heads.MWA__227__AvgMale"
     ChamelHeadSkins(118)="AW7PawnSkins.heads.MWA__236__AvgMale"
     ChamelHeadSkins(119)="AW7PawnSkins.heads.MWA__237__AvgMale"
     ChamelHeadSkins(120)="AW7PawnSkins.heads.MWA__238__AvgMale"
     ChamelHeadSkins(121)="AW7PawnSkins.heads.MWA__239__AvgMaleBig"
     ChamelHeadSkins(122)="AW7PawnSkins.heads.MWA__240__AvgMale"
     ChamelHeadSkins(123)="AW7PawnSkins.heads.MWA__241__AvgMale"
     ChamelHeadSkins(124)="AW7PawnSkins.heads.MWA__243__AvgMale"
     ChamelHeadSkins(125)="AW7PawnSkins.heads.MWA__244__AvgMaleBig"
     ChamelHeadSkins(126)="AW7PawnSkins.heads.MWA__245__AvgMaleBig"
     ChamelHeadSkins(127)="AW7PawnSkins.heads.MWA__247__AvgMale"
     ChamelHeadSkins(128)="ED_HeadSkins.Male.mimic__AVgMaleBig"
     ChamelHeadSkins(129)="End"
     ChamelHeadMeshPkgs(0)="heads"
     Skins(0)=Texture'ChameleonSkins.BystandersF.XX__142__Fem_SS_Shorts'
}
