///////////////////////////////////////////////////////////////////////////////
// EACivilianController
// Normal AW bystanders but we can use this class if we need to make any major
// changes to civilian characters.
// This class for civilians only -- use EAPolice to alter police officers'
// behavior, and military doesn't care about anything that we plan to add so
// just use normal AWMilitaryController.
///////////////////////////////////////////////////////////////////////////////

class EACivilianController extends EDBystanderController;

///////////////////////////////////////////////////////////////////////////////
// Consts 'n' stuff
///////////////////////////////////////////////////////////////////////////////
const KMASS_DAMAGEMULT			= 5.00;

const RIOT_MEDGUN_PCT			= 0.2;
const RIOT_BIGGUN_PCT			= 0.1;
const RIOT_UBERGUN_PCT			= 0.01;
const RIOT_MELEE_PCT			= 0.1;

const RIOT_HATE_PLAYER			= 1.0;
const RIOT_HEALTH_INCREASE		= 4.0;
const RIOT_DAMAGE_INCREASE		= 4.0;

const IDLE_CELL_FREQ			= 0.001;
const CELL_TALK_LOOP			= 5.0;
const CELL_TALK_WAIT			= 2.0;
const CELL_TALK_RING			= 3.0;

var bool bLoopCell;

var int TimesKickedDeadThing;
const MAX_KICK_DEAD_THING		= 3;		// max number of times we wanna kick a dead thing

const MAX_ROT_DIFF = 1024;

///////////////////////////////////////////////////////////////////////////////
// Bystanders have view pitching now
///////////////////////////////////////////////////////////////////////////////
simulated event Tick(float Delta)
{
	local rotator ORot;
	local vector VDiff;
	local float PDiff;

	Super.Tick(Delta);

	if (Focus != None && !Focus.bStatic)
	{
		VDiff = Focus.Location - Pawn.Location;
		ORot = Rotator(VDiff);
		// Only look up/down at focus if we're looking directly at them, otherwise it looks stupid
		if (CanSeePoint(Pawn, Focus.Location)
			&& !IsInState('ThrownThroughAir')
			&& !IsInState('DoPuking')
			&& !IsInState('BeingShocked')
			&& !IsInState('RestFromAttacker')
			&& !IsInState('DanceHere')
			&& !IsInState('PlayArcadeGame')
			&& !IsInState('AttackedByChompy')
			&& !IsInState('WipeFace')
			&& !IsInState('PrepDeathCrawl')
			&& !IsInState('RunOnFire')
			&& !IsInState('DeathCrawlFromAttacker')
			&& !IsInState('CowerInABall')
			&& !IsInState('ImOnFire')
			)
			Pawn.ViewPitch = Clamp(ORot.Pitch / 256, 0, 255);
		else
			Pawn.ViewPitch = 0;
//		log(self@"focus is"@Focus@"VDiff"@VDiff@"ORot"@ORot@"Rotation"@Pawn.Rotation);
	}
	else
		Pawn.ViewPitch = 0;
}

///////////////////////////////////////////////////////////////////////////////
// Face the direction we are and be ready to kill someone
///////////////////////////////////////////////////////////////////////////////
function SetToTurret()
{
	local FPSPawn keepp;
	local Inventory inv;

	SwitchToBestWeapon();

	MyPawn.SetMood(MOOD_Combat, 1.0);
	// Make sure he never tries for cover
	MyPawn.WillUseCover=0.0;

	// turrets are stronger in the weekend levels
	if (EAEarlyGameInfo(Level.Game).IsWeekend())
	{
		// Set all his bullet weapons to full trace dist,
		// so they can shoot from where they are, when they get
		// a line of site
		inv = MyPawn.Inventory;
		while(inv != None)
		{
			if(P2Weapon(inv) != None
				&& !P2Weapon(inv).bMeleeWeapon)
				P2Weapon(inv).TraceDist = TURRET_TRACE_DIST;

			inv = inv.Inventory;
		}

		// If he hates the player, start attacking now
		if(MyPawn.bPlayerIsEnemy)
		{
			keepp = GetRandomPlayer().MyPawn;

			// check for some one to attack
			if(keepp != None)
			{
				SetAttacker(keepp);
				MyPawn.DropBoltons(Velocity);
				SetNextState('ShootAtAttacker');
			}
			else
				SetNextState('ActAsTurret');
		}
		else
			SetNextState('ActAsTurret');
	}
	else
		SetNextState('ActAsTurret');
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
function GrantExtraWeapons()
{
}

///////////////////////////////////////////////////////////////////////////////
// Get ready for the Apocalypse. Usually, you get more weapons and get meaner
// and crazier.
///////////////////////////////////////////////////////////////////////////////
function ConvertToRiotMode()
{
	local int randpick;
	local bool bWeaponsGranted;

//	log(self@"prepare for Apocalypse");

	// In EA, people get lots of guns during the Apocalypse.
	if (FRand() <= RIOT_UBERGUN_PCT)
	{
		// Give them an "uber" gun
//		log(self@"granting uber gun");
		bWeaponsGranted = true;
		randpick = Rand(3);
		switch(randpick)
		{
			case 0:
				MyPawn.CreateInventoryByClass(class'GLauncherWeapon');
				break;
			case 1:
				MyPawn.CreateInventoryByClass(class'HolyWeapon');
				break;
			case 2:
				MyPawn.CreateInventoryByClass(class'PlagueWeapon');
				break;
		}
	}

	if (FRand() <= RIOT_BIGGUN_PCT)
	{
		// Give them a bigger gun
//		log(self@"granting big gun");
		bWeaponsGranted = true;
		randpick = Rand(5);
		switch(randpick)
		{
					case 0:
						MyPawn.CreateInventoryByClass(class'CowHeadWeapon');
						break;
					case 1:
						MyPawn.CreateInventoryByClass(class'GrenadeWeapon');
						break;
					case 2:
						MyPawn.CreateInventoryByClass(class'MolotovWeapon');
						break;
					case 3:
						MyPawn.CreateInventoryByClass(class'SMEGWeapon');
						break;
					case 4:
						MyPawn.CreateInventoryByClass(class'MaDBombWeapon');
						break;
					case 6:
						MyPawn.CreateInventoryByClass(class'M870Weapon');
						break;
					case 7:
						MyPawn.CreateInventoryByClass(class'DoubleBarrelWeapon');
						break;
					case 8:
						MyPawn.CreateInventoryByClass(Class'FlameWeapon');
						break;
					case 9:
						MyPawn.CreateInventoryByClass(class'ScissorsWeapon');
						break;
					case 10:
						MyPawn.CreateInventoryByClass(class'CrossbowWeapon');
						break;
		}
	}

	if (FRand() <= RIOT_MEDGUN_PCT)
	{
		// Give them a medium-sized gun
//		log(self@"granting medium gun");
		bWeaponsGranted = true;
		randpick = Rand(9);
		switch(randpick)
		{
					case 0:
						MyPawn.CreateInventoryByClass(class'MacheteWeapon');
						break;
					case 1:
						MyPawn.CreateInventoryByClass(class'SledgeWeapon');
						break;
					case 2:
						MyPawn.CreateInventoryByClass(class'ScytheWeapon');
						break;
					case 3:
						MyPawn.CreateInventoryByClass(class'PitchforkWeapon');
						break;
					case 4:
						MyPawn.CreateInventoryByClass(class'KatanaWeapon');
						break;
					case 5:
						MyPawn.CreateInventoryByClass(class'FrypanWeapon');
						break;
					case 6:
						MyPawn.CreateInventoryByClass(class'BatWeapon');
						break;
					case 7:
						MyPawn.CreateInventoryByClass(Class'ChainsawWeapon');
						break;
					case 8:
						MyPawn.CreateInventoryByClass(Class'WoodenBatWeapon');
						break;
					case 9:
						MyPawn.CreateInventoryByClass(Class'BatWeapon');
						break;
					case 10:
						MyPawn.CreateInventoryByClass(Class'BatWeapon');
						break;
					case 11:
						MyPawn.CreateInventoryByClass(Class'BaliWeapon');
						break;
					case 12:
						MyPawn.CreateInventoryByClass(Class'AxeWeapon');
						break;
					case 13:
						MyPawn.CreateInventoryByClass(Class'FistsWeapon');
						break;
					case 14:
						MyPawn.CreateInventoryByClass(Class'DrillWeapon');
						break;
					case 15:
						MyPawn.CreateInventoryByClass(Class'DustersWeapon');
						break;
					case 16:
						MyPawn.CreateInventoryByClass(Class'HammerWeapon');
						break;
					case 17:
						MyPawn.CreateInventoryByClass(Class'BatonWeapon');
						break;
					case 18:
						MyPawn.CreateInventoryByClass(Class'ShovelWeapon');
						break;
					case 19:
						MyPawn.CreateInventoryByClass(Class'FistsWeapon');
						break;
					case 20:
						MyPawn.CreateInventoryByClass(Class'ShovelWeapon');
						break;
					case 21:
						MyPawn.CreateInventoryByClass(Class'FistsWeapon');
						break;
					case 22:
						MyPawn.CreateInventoryByClass(Class'MeatCleaverWeapon');
						break;
					case 23:
						MyPawn.CreateInventoryByClass(Class'ShearsWeapon');
						break;
					case 24:
						MyPawn.CreateInventoryByClass(Class'SickleWeapon');
						break;
					case 25:
						MyPawn.CreateInventoryByClass(Class'FootWeapon');
						break;
					case 26:
						MyPawn.CreateInventoryByClass(Class'AWPKatanaWeapon');
						break;
		}
	}

	if (FRand() <= RIOT_MELEE_PCT)
	{
		// Give them a cool melee weapon
//		log(self@"granting melee weapon");
		bWeaponsGranted = true;
		randpick = Rand(7);
		switch(randpick)
		{
			case 0:
				MyPawn.CreateInventoryByClass(class'MacheteWeapon');
				break;
			case 1:
				MyPawn.CreateInventoryByClass(class'SledgeWeapon');
				break;
			case 2:
				MyPawn.CreateInventoryByClass(class'ScytheWeapon');
				break;
			case 3:
				MyPawn.CreateInventoryByClass(class'PitchforkWeapon');
				break;
			case 4:
				MyPawn.CreateInventoryByClass(class'KatanaWeapon');
				break;	
		}
	}

	if (!bWeaponsGranted)
	{
		// Give them the standard
//		log(self@"granting small gun");
		randpick = Rand(6);
		switch(randpick)
		{
					case 0:
						MyPawn.CreateInventoryByClass(class'PPKWeapon');
						break;
					case 1:
						MyPawn.CreateInventoryByClass(class'M870Weapon');
						break;
					case 2:
						MyPawn.CreateInventoryByClass(class'M16A2Weapon');
						break;
					case 3:
						MyPawn.CreateInventoryByClass(class'M4A1Weapon');
						break;
					case 4:
						MyPawn.CreateInventoryByClass(class'UZIWeapon');
						break;
					case 5:
						MyPawn.CreateInventoryByClass(class'GSelectWeapon');
						break;
					case 6:
						MyPawn.CreateInventoryByClass(class'GlockAkimboWeapon');
						break;
					case 7:
						MyPawn.CreateInventoryByClass(class'BerettaWeapon');
						break;
					case 8:
						MyPawn.CreateInventoryByClass(class'MP5Weapon');
						break;
					case 9:
						MyPawn.CreateInventoryByClass(class'IngramWeapon');
						break;
		}
	}

	// Make them meaner and stronger
	if(MyPawn.Cajones < 1.0)
		MyPawn.Cajones = 1.0;
	if(MyPawn.PainThreshold < 1.0)
		MyPawn.PainThreshold = 1.0;
	MyPawn.Cowardice = 1.0-MyPawn.PainThreshold;

	// Up their health and the damage they do
	MyPawn.HealthMax = RIOT_HEALTH_INCREASE*MyPawn.default.HealthMax;
	MyPawn.Health = MyPawn.default.HealthMax;
	MyPawn.DamageMult = RIOT_DAMAGE_INCREASE*MyPawn.default.DamageMult;

	// And much crazier...(if you're not the dude's friend)
	if(!MyPawn.bPlayerIsFriend)
	{
		if(FRand() < RIOT_HATE_PLAYER)
			MyPawn.bPlayerIsEnemy=true;

		MyPawn.bGunCrazy=true;
	}
}

///////////////////////////////////////////////////////////////////////////////
// Some karma actors hurt us
///////////////////////////////////////////////////////////////////////////////
function CheckForKarmaDamage(Actor Other)
{
	local int DamageTaken;

	// See if it's a kactor. Ignore carexplodables, they hurt us themselves
	if (ClassIsChildOf(Other.Class,class'KActor') && Other.Instigator != None && !Other.IsA('CarExplodable'))
	{
		// Below a certain mass, get pissed and flip off the player
		if (Other.KGetMass() <= 2)
			InterestIsAnnoyingUs(Other.Instigator, False);
		else // Take damage; possibly retaliate
		{
			DamageTaken = Other.KGetMass() * KMASS_DAMAGEMULT;
			Pawn.TakeDamage(DamageTaken, Other.Instigator, Location, Other.Velocity, class'SmashDamage');
			DamageAttitudeTo(Other.Instigator, DamageTaken);
		}
	}
}

///////////////////////////////////////////////////////////////////////////////
// Some karma actors hurt us
///////////////////////////////////////////////////////////////////////////////
event Bump(actor Other)
{
	CheckForKarmaDamage(Other);
	Super.Bump(Other);
}

///////////////////////////////////////////////////////////////////////////////
// Some karma actors hurt us
///////////////////////////////////////////////////////////////////////////////
event Touch(actor Other)
{
	CheckForKarmaDamage(Other);
	Super.Touch(Other);
}

///////////////////////////////////////////////////////////////////////////////
// Returns true if the pawn is a cop, false if not
///////////////////////////////////////////////////////////////////////////////
function bool IsACop(P2Pawn P)
{
	return (P.bAuthorityFigure || DudeDressedAsCop(P));
}

///////////////////////////////////////////////////////////////////////////////
// We've seen the pawn, now decide if we care
// Returns true if there state changes at some point
///////////////////////////////////////////////////////////////////////////////
function ActOnPawnLooks(FPSPawn LookAtMe, optional out byte StateChange)
{
	// CopKilla cheat
	if (EAGameState(P2GameInfoSingle(Level.Game).TheGameState).bCopKilla && IsACop(P2Pawn(LookAtMe)))
	{
		// decide to fight if you have a weapon, or run if you don't
		if(MyPawn.bHasViolentWeapon)
		{
			// if I'm not attacking anyone already--esp. not the dude
			// then decide to recognize him
			if(Attacker == None)
			{
				InterestPawn = LookAtMe;
				SetAttacker(LookAtMe);
				MakeMoreAlert();
				GotoStateSave('SightedHatedGuy');
				StateChange=1;
			}
			else if(!IsInState('SightedHatedGuy'))
				// we already know we hate him (if we're not trying to decide already)
			{
				// Drop things if you had them in your hands
				MyPawn.DropBoltons(MyPawn.Velocity);
				SetAttacker(LookAtMe);
				MakeMoreAlert();
				SaveAttackerData(LookAtMe);
				GotoStateSave('RecognizeAttacker');
				StateChange=1;
			}
			return;
		}
	}

	Super.ActOnPawnLooks(LookAtMe, StateChange);
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// Thinking
// Same as super, but adjust for charisma/playahata cheats
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
state Thinking
{
	///////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////
	event BeginState()
	{
		Super.BeginState();

		if (EAGameState(P2GameInfoSingle(Level.Game).TheGameState).bCharisma && !MyPawn.bPlayerIsEnemy)
			MyPawn.bPlayerIsFriend = True;

		if (EAGameState(P2GameInfoSingle(Level.Game).TheGameState).bPlayaHata && !MyPawn.bPlayerIsFriend)
			MyPawn.bPlayerIsEnemy = True;
	}
//Begin:
//	GotoState('PerformCellIdle');
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// SightedHatedGuy
// Same as super, but adjusted for CopKilla cheat.
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
state SightedHatedGuy
{
	///////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////
	function bool RecognizeHim()
	{
		local float dist;

		dist = VSize(Attacker.Location - MyPawn.Location);

		// Turrets say things, but don't wait to attack, and they
		// recognize instantly.
		if(MyPawn.PawnInitialState == MyPawn.EPawnInitialState.EP_Turret)
		{
			SayTime=0;
			// Don't record a time, so you'll attack quicker
			if(MyPawn.bPlayerHater || 
			(IsACop(P2Pawn(Focus)) && EAGameState(P2GameInfoSingle(Level.Game).TheGameState).bCopKilla)
			)
			{
				PrintDialogue("I hate you!");
				Say(MyPawn.myDialog.lLynchMob);
			}
			else
			{
				PrintDialogue("Hey, it's the player enemy!");
				Say(MyPawn.myDialog.lSeesEnemy);	// Don't record a time, so you'll attack quicker
			}
			return true;
		}

		// Closer the are, the more likely he will recogize you (unless you're
		// in riot mode, then you just attack all the time)
		else if(FRand() > (2*dist)/(P2Pawn(Attacker).ReportLooksRadius)
			|| MyPawn.bRiotMode)
		{
			// Kamek: This tricky statement checks to see if the person we hate is a cop, and we have copkiller on.
			// It's dumb when they say stuff like "There's the killer!" to the cops.
			if(MyPawn.bPlayerHater || 
			(IsACop(P2Pawn(Focus)) && EAGameState(P2GameInfoSingle(Level.Game).TheGameState).bCopKilla)
			)
			{
				PrintDialogue("I hate you!");
				SayTime = Say(MyPawn.myDialog.lLynchMob);
			}
			else
			{
				PrintDialogue("Hey, it's the player enemy!");
				SayTime = Say(MyPawn.myDialog.lSeesEnemy);
			}
			return true;
		}
		else // if not, go back to watching
		{
			return false;
		}
	}
}

///////////////////////////////////////////////////////////////////////////////
// Ignore intruders in stasis (this is a problem for some reason)
///////////////////////////////////////////////////////////////////////////////
state StasisState
{
	function CheckForIntruder(FPSPawn LookAtMe, out byte StateChange);
	function HandleIntruder(FPSPawn LookAtMe);
	event Trigger( Actor Other, Pawn EventInstigator );
}

///////////////////////////////////////////////////////////////////////////////
// Ignore intruders in stasis (this is a problem for some reason)
///////////////////////////////////////////////////////////////////////////////
state SliderStasis
{
	function CheckForIntruder(FPSPawn LookAtMe, out byte StateChange);
	function HandleIntruder(FPSPawn LookAtMe);
	event Trigger( Actor Other, Pawn EventInstigator );
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// DecideToGetDown
// Same as super, but respond quicker.
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
state DecideToGetDown
{
Begin:
	// Wait for how naturally slow they react, plus how rebellious they are
	// Also, look to see who shouted it InterestPawn and decide to believe
	// them or not
	if (UseReactivity + MyPawn.Rebel >= 1.0)
		Sleep(1.0 - (UseReactivity + MyPawn.Rebel));

	// If he has a weak weapon, then do nothing, or tell him off
	if(FPSPawn(Focus) != None
		&& !FreakedAboutWeapon(P2Weapon(FPSPawn(Focus).Weapon)))
	{
		ToldGetDownCount++;	// record how many times it didn't matter--crying wolf

		if(FRand() <= MyPawn.Rebel)
			Goto('ScrewYou');
		// go back to what you were doing
		else
			GotoState(MyOldState);
	}
	// if he has a weapon and we're still a rebel and we're strong, then tell him off
	else if(FRand() <= MyPawn.Rebel
		&& FRand() <= MyPawn.PainThreshold)
		Goto('ScrewYou');
	else // otherwise, crouch like told
	{
		DangerPos = MyPawn.Location; // Just consider where we are a dangerous position
		// now set the interest pawn back to the focus
		InterestPawn = FPSPawn(Focus);
		GotoState('PerformGetDown');
	}

ScrewYou:
	// Flip them off
	MyPawn.PlayTellOffAnim();
	PrintDialogue("Go screw yourself!");
	SayTime = Say(MyPawn.myDialog.lDefiant);
	Sleep(SayTime);
	GotoState(MyOldState);
}

///////////////////////////////////////////////////////////////////////////////
// Chance of going into cell phone idle sequence.
///////////////////////////////////////////////////////////////////////////////
function CheckForCellIdle()
{
	local float f;

	f = frand();
	if (FreeToSeekPlayer() && f <= IDLE_CELL_FREQ && EAPerson(Pawn).bCellUser)
		GotoStateSave('PerformCellIdle');
}

///////////////////////////////////////////////////////////////////////////////
// LegMotionToTarget
// Chance of going into cell phone idling
///////////////////////////////////////////////////////////////////////////////
state LegMotionToTarget
{
	function BeginState()
	{
		Super.BeginState();
		CheckForCellIdle();
	}
}

///////////////////////////////////////////////////////////////////////////////
// WalkToTarget
// Chance of going into cell phone idling
///////////////////////////////////////////////////////////////////////////////
state WalkToTarget
{
	///////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////
	function InterimChecks()
	{
		Super.InterimChecks();
		CheckForCellIdle();
	}

	///////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////
	function IntermediateGoalReached()
	{
		Super.IntermediateGoalReached();
		CheckForCellIdle();
	}
}

///////////////////////////////////////////////////////////////////////////////
// PerformIdle
// Chance of going into cell phone idling
///////////////////////////////////////////////////////////////////////////////
state PerformIdle
{
	function BeginState()
	{
		Super.BeginState();
		CheckForCellIdle();
	}
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// PerformCellIdle
// We're idling around and someone calls us.
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
state PerformCellIdle extends PerformIdle
{
	ignores CanBeMugged;

    ///////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////
	function AnimEnd(int channel)
	{
		MyPawn.AnimEnd(channel);
		if (bLoopCell)
			EAPerson(Pawn).PlayCellLoop();
//		bFinishedAnim = true;
	}

    ///////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////
	function EndState()
	{
		Super.EndState();
		PersonController(Pawn.Controller).SwitchToHands();
	}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
Begin:
	bLoopCell=false;

	// Stop what we're doing
	MyPawn.StopAcc();

	// for debug
//	GetRandomPlayer().ClientMessage(self@"entering cell phone talk");
//	spawn(class'CellBeacon',,, Location);

	// Begin cell phone idling. Phone rings a couple times
	CurrentFloat = Rand(CELL_TALK_RING) + 1;
ObnoxiousRing:
	// Phone rings
	SayTime = EAPerson(Pawn).PlayIncomingCall();
	Sleep(SayTime*2);

	// See if we care to answer it yet
	CurrentFloat = CurrentFloat - 1;
	if (CurrentFloat >= 0)
		Goto('ObnoxiousRing');

	// Finally decide to answer phone
AnswerPhone:
	// Switch to cell phone weapon
	EAPerson(Pawn).PlayCellIn();
	FinishAnim();

	// Answer the phone
	EAPerson(Pawn).PlayCellOn();
	Sleep(1.0);
	bLoopCell=True;
	EAPerson(Pawn).PlayCellLoop();

	// Say hello to the caller
	PrintDialogue("hello?");
	SayTime = Say(MyPawn.myDialog.lGreeting);
	Sleep(SayTime + FRand());

	// Enter a loop of random bullshitting with the caller
    CurrentFloat = Rand(CELL_TALK_LOOP) + 1;

	// Wait for the guy on the other end to "say" something
	Sleep(CELL_TALK_WAIT + 2*(FRand()-0.5));
RandomBullshitting:

	// Say something back
	PrintDialogue("cell phone bullshitting");
	SayTime = Say(MyPawn.myDialog.lCellPhoneTalk);
	Sleep(SayTime + FRand());

	// Wait for the guy on the other end to "say" something
	Sleep(CELL_TALK_WAIT + 2*(FRand()-0.5));
	CurrentFloat = CurrentFloat - 1;

	if (CurrentFloat > 0)
		Goto('RandomBullshitting');

	// Done bullshitting, hang up the phone
HangUp:
	bLoopCell=false;
	EAPerson(Pawn).PlayCellOut();
	FinishAnim();

	// Go back to whatever we were doing
	if (MyOldState != '')
		GotoState(MyOldState);
	else
		GotoState('Thinking');
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// GettingPickPocketed
// Same as super but with added dialogue.
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
state GettingPickPocketed
{
Begin:
	Sleep(2.5);
	Focus = InterestActor;	// look away
	MyPawn.SetMood(MOOD_Angry, 1.0);
	Sleep(1.5);
	Focus = InterestPawn;	// look back again as you realize what happened
	SayTime = Say(MyPawn.myDialog.lwhatthe, true);
	Sleep(SayTime);
	SayTime = Say(MyPawn.MyDialog.lGettingRobbed);
	Sleep(SayTime);
	// You're a wimp, so complain, then get a cop
	if(!MyPawn.bHasViolentWeapon)
	{
		bDontSetFocus=false;
		GotoStateSave('LookForCop');
	}
	else
	{
		GotoStateSave('AssessAttacker');
	}
}

///////////////////////////////////////////////////////////////////////////////
// Some danger occurred
///////////////////////////////////////////////////////////////////////////////
function MarkerIsHere(class<TimedMarker> bliphere,
					  FPSPawn CreatorPawn,
					  Actor OriginActor,
					  vector blipLoc)
{
	local byte Reacted;

	if(!MyPawn.bIgnoresSenses)
	{
		if(bliphere == class'ParadeMarker')
		{
			SetupWatchParade(OriginActor, Reacted);
		}
		else if(bliphere == class'HeadExplodeMarker')
		{
			WatchHeadExplode(P2Pawn(OriginActor), Reacted);
		}
		else if(bliphere == class'DeadCatHitGuyMarker')
		{
			WatchDeadCatHitGuy(P2Pawn(OriginActor), P2Pawn(CreatorPawn));
		}
		else if(bliphere == class'DeadHeadMarker' || bliphere == class'DeadLimbMarker')
		// Check for dismembered stuff on the ground
		{
			CheckDeadHead(OriginActor);
		}
		else if(ClassIsChildOf(bliphere, class'DeadBodyMarker'))
		// Check for dead NPCs or a dead dude (head is checked above)
		{
			CheckDeadBody(P2Pawn(OriginActor));
		}
		else if(ClassIsChildOf(bliphere, class'FunnyThingMarker'))
		{
			WatchFunnyThing(P2Pawn(OriginActor));
		}
		else if(ClassIsChildOf(bliphere, class'DesiredThingMarker'))
		{
			CheckDesiredThing(OriginActor, bliphere);
		}
		else if(blipLoc != Pawn.Location)
		{
			GetReadyToReactToDanger(bliphere, CreatorPawn, OriginActor, blipLoc, Reacted);
			if(Reacted==1)
			{
				// If we went to a new panicked state drop things if you had them in your hands
				MyPawn.DropBoltons(MyPawn.Velocity);
			}
		}
	}
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// You see a dead body or a disembodied head off in the distance, decide what to do
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
state InvestigateDeadThing
{
StareALongTime:
	Sleep(Rand(STARE_AT_DEAD_THING) + 1);
Begin:
	TimesKickedDeadThing = 0;	// reset # of times we've kicked the dead thing
	Focus = InterestActor;
	Sleep(2.0 - 2*MyPawn.Reactivity);
	MyPawn.StopAcc();
	Sleep(2.0 - 2*MyPawn.Reactivity);
ReadyToMove:
	MoveCloser();

CloseEnough:
	// Tough guy here is strong enough not to puke and probably wants
	// to kick the body/head
	if(MyPawn.bHasViolentWeapon
		|| (InterestPawn != None
			&& InterestPawn.bPlayer)
		|| OldAttacker != None)
	// tough guy choices
	{
		SetEndPoint(InterestActor.Location, UseSafeRangeMin/2);
		SetNextState('InvestigateDeadThing', 'StareAtBody');
		GotoStateSave('WalkToDeadThing');
StareAtBody:
		Focus = InterestActor;
		Sleep(MyPawn.Curiosity + 1);	// stare a while
		if(InterestPawn != None
			&& InterestPawn.bPlayer)
		{
			Sleep(Rand(STARE_AT_DEAD_DUDE));// possibly stare a long while
			// Say something mean about him
			SayTime = Say(MyPawn.myDialog.lDudeDead);
			PrintDialogue("The dude sure was bad...");
			Sleep(Rand(STARE_AT_DEAD_DUDE) + 1);	// stare a long while
		}

		// Sick ones will kick the body
		if(FRand() > MyPawn.Conscience
			|| FRand() > MyPawn.Curiosity
			|| (InterestPawn != None
				&& InterestPawn.bPlayer
				&& FRand() > KICK_DEAD_DUDE))
		{
TryToKick:
			// If you're close enough, just kick
			if(VSize(InterestActor.Location - MyPawn.Location) <= DEFAULT_END_RADIUS + 2*MyPawn.CollisionRadius)
			{
				// Don't sit around and kick the same body all day
				TimesKickedDeadThing++;
				if (TimesKickedDeadThing < MAX_KICK_DEAD_THING)
					SetNextState('InvestigateDeadThing','StareAtBody');
				else
					SetNextState('Thinking');
				GotoStateSave('DoKicking');
			}
			else // not close.. then try to get closer
			{
				SetEndPoint(InterestActor.Location, DEFAULT_END_RADIUS + MyPawn.CollisionRadius);
				SetNextState('InvestigateDeadThing', 'TryToKick');
				GotoStateSave('WalkToDeadThing');
			}
		}
		// Normal-ish people will just walk up and look
		// or stop if they're bored
		// Be more likley to kick the dead dude
		else
		{
			GotoStateSave('Thinking');
		}

	}
	else // scared choices
	{
		// See if you will puke
		CheckToPukeBody();

		// Didn't puke, so run away from this point now
		DangerPos = InterestActor.Location;
		InterestActor = None;
		InterestPawn = None;
		UseSafeRangeMin = MyPawn.SafeRangeMin;

		PrintDialogue("the horror!!");
		SayTime = Say(MyPawn.myDialog.lcarnageoccurred);
		Sleep(SayTime + FRand());

		GotoStateSave('FleeFromDanger');
	}
}

defaultproperties
{
}
