//=============================================================================
// EAPostalDude.
//=============================================================================
class EAPostalDude extends JohnPawn
	placeable;

//class JohnPawn extends Dude
const PEE_BONE = 'MALE01 pelvis';

var(Character) array<MeshAnimation> EDMeshAnims; // Mesh animations for ED
var() Material PreInjuryHeadSkin;
var() Mesh ExpectedHeadMesh;
var Mesh OldHeadMesh;
var KevlarMesh ArmorMesh;		// Model of kevlar armor you're wearing on your person

///////////////////////////////////////////////////////////////////////////////
// Const
///////////////////////////////////////////////////////////////////////////////
const DAY_FRIDAY = 4;

///////////////////////////////////////////////////////////////////////////////
// Make sure the censored bar for the
///////////////////////////////////////////////////////////////////////////////
function name GetWeaponBoneFor(Inventory I)
	{
	if(UrethraWeapon(I) != None)
		return PEE_BONE;
	else
		return Super.GetWeaponBoneFor(I);
	}

///////////////////////////////////////////////////////////////////////////////
// Link this pawn to the anims it needs
///////////////////////////////////////////////////////////////////////////////
simulated function LinkAnims()
{
	local int i;

	// Link up ED anims
	for (i=0; i<EDMeshAnims.Length; i++)
		LinkSkelAnim(EDMeshAnims[i]);

	// Also put in more AW single player anims.
	//LinkSkelAnim(AW_SPMeshAnim);

	// Links to the special specified mesh for SP games.
	LinkSkelAnim(GetDefaultAnim(SkeletalMesh(Mesh)));

	// Always link to the core anims, too, because some characters use a mixture
	// of their own anims plus some core anims.  Linking to core anims twice,
	// which can happen if default anims happen to match core anims, is safe.
	LinkSkelAnim(CoreMeshAnim);
}





///////////////////////////////////////////////////////////////////////////////
// Don't let NPCs telefrag us when they get stuck -- just gib the NPC
///////////////////////////////////////////////////////////////////////////////
event EncroachedBy(Actor Other)
{
	if (Pawn(Other) != None && !Pawn(Other).Controller.bIsPlayer)
		Other.EncroachedBy(self);
	else
		Super.EncroachedBy(Other);
}

///////////////////////////////////////////////////////////////////////////////
// Called by EAGameInfo
///////////////////////////////////////////////////////////////////////////////
function DudeCheckHeadSkin()
{
	log(self@"current day:"@P2GameInfoSingle(Level.Game).TheGameState.CurrentDay);
	log(self@"next day:"@P2GameInfoSingle(Level.Game).TheGameState.NextDay);

	// If it's not the weekend yet, use the normal (non-injured) dude head.
	if (!EAEarlyGameInfo(Level.Game).IsWeekend() && MyHead.DrawType == DT_Mesh && MyHead.Mesh == ExpectedHeadMesh)
	{
		log(self@"swapping to pre-injury head skin");
		MyHead.Skins[1] = PreInjuryHeadSkin;
	}
}


///////////////////////////////////////////////////////////////////////////////
// Switch to this new mesh
///////////////////////////////////////////////////////////////////////////////
function SwitchToNewMesh(Mesh NewMesh,
						 Material NewSkin,
						 Mesh NewHeadMesh,
						 Material NewHeadSkin,
						 optional Mesh NewCoreMesh)
{
	local bool bSwappedFromOldDude;

	// Setup body (true means "keep anim state")
	SetMyMesh(NewMesh, NewCoreMesh, true);
	SetMySkin(NewSkin);
	LinkAnims();
	PlayWaiting();

	// The clothes are wonky, so if they specify using the old dude head, use the new head instead and set
	// the bandage skin accordingly.
	if (NewHeadMesh == class'DudeClothesInv'.Default.HeadMesh && NewHeadSkin == class'DudeClothesInv'.Default.HeadSkin)
	{
		NewHeadMesh = ExpectedHeadMesh;
		NewHeadSkin = PreInjuryHeadSkin;
		bSwappedFromOldDude = True;
	}

	// Setup head
	MyHead.LinkMesh(NewHeadMesh, true);
	// Because our headmesh is screwey, ask the head what it wants to set
	if(AWDudeHead(MyHead) != None && NewHeadMesh == ExpectedHeadMesh)
		AWDudeHead(MyHead).SetMainSkin(NewHeadSkin);
	else
		MyHead.Skins[0] = NewHeadSkin;

	if (bSwappedFromOldDude)
		DudeCheckHeadSkin();
}


///////////////////////////////////////////////////////////////////////////////
// Default properties
///////////////////////////////////////////////////////////////////////////////

defaultproperties
{
     EDMeshAnims(0)=MeshAnimation'ED_Characters.avgMental'
     EDMeshAnims(1)=MeshAnimation'ED_Characters.ED_deadBody'
     EDMeshAnims(2)=MeshAnimation'ED_Characters.rws_aw_anims_zomb'
     EDMeshAnims(3)=MeshAnimation'ED_Characters.smockingtostand'
     EDMeshAnims(4)=MeshAnimation'ED_Characters.SYS_HEDGETRIMMER'
     EDMeshAnims(5)=MeshAnimation'ED_Characters.ED_Scope_Slit_throat'
     EDMeshAnims(6)=MeshAnimation'ED_Characters.SyS_fisting'
     EDMeshAnims(7)=MeshAnimation'ED_Characters.GroundExit'
     EDMeshAnims(8)=MeshAnimation'ED_Characters.returntolifeFloaty'
     EDMeshAnims(9)=MeshAnimation'MP_Characters.anim_MP'
     EDMeshAnims(10)=MeshAnimation'ED_Characters.FLIGHTPACK'
     EDMeshAnims(11)=MeshAnimation'ED_Characters.animMaD'
     EDMeshAnims(12)=MeshAnimation'ED_ScopeNewCharacters.ScopesZombie1'
     EDMeshAnims(13)=MeshAnimation'MaD_ED_MP_Strippers.anim_StripperMP'
     EDMeshAnims(14)=MeshAnimation'Characters.animBig'
     EDMeshAnims(15)=MeshAnimation'AWCharacters.animAvg_AW'
     EDMeshAnims(16)=MeshAnimation'Characters.animFem'
     CoreMPMeshAnim=MeshAnimation'MP_Characters.anim_MP'
     CoreSPMesh=SkeletalMesh'Characters.Avg_Dude'
     SeverBone(0)="MALE01 L UpperArm"
     SeverBone(1)="MALE01 L Forearm"
     SeverBone(2)="MALE01 R UpperArm"
     SeverBone(3)="MALE01 R Forearm"
     SeverBone(4)="MALE01 L Thigh"
     SeverBone(5)="MALE01 L Calf"
     SeverBone(6)="MALE01 R Thigh"
     SeverBone(7)="MALE01 r calf"
     BoneArr(0)=1
     BoneArr(1)=1
     BoneArr(2)=1
     BoneArr(3)=1
     BoneArr(4)=1
     BoneArr(5)=1
     BoneArr(6)=1
     BoneArr(7)=1
     TakesMachinegunDamage=0.600000
     TakesPistolHeadShot=0.600000
     TakesShotgunHeadShot=0.600000
     TakesRifleHeadShot=0.600000
     TakesShovelHeadShot=0.600000
     TakesOnFireDamage=1.00000
     TakesAnthraxDamage=1.000000
     TakesChemDamage=1.000000
     TakesShockerDamage=1.000000
     TakesSledgeDamage=1.000000
     TakesMacheteDamage=0.650000
     TakesScytheDamage=1.000000
     TakesDervishDamage=0.650000
     TakesZombieSmashDamage=1.000000
     WillDodge=1.000000
     WillKneel=1.000000
     WillUseCover=1.00000
     DamageMult=1.000000
     HeadClass=Class'AWPawns.AWDudeHead'
     HeadSkin=Texture'AW_Characters.Special.Dude_AW'
     HeadMesh=SkeletalMesh'AW_Heads.AW_Dude'
     BodyHitSounds(0)=Sound'MiscSounds.People.bodyhitground1'
     BodyHitSounds(1)=Sound'MiscSounds.People.bodyhitground2'
     ChameleonMeshPkgs(0)="AWCharacters"
     ChamelHeadMeshPkgs(0)="heads"
     ChamelHeadMeshPkgs(1)="AW_Heads"
     dialogclass=Class'EADialog.EADudeDialog'
     FootTexture=Texture'ChameleonSkins.Special.Dude'
     BaseEquipment(0)=(WeaponClass=Class'UrethraWeapon')
     BaseEquipment(1)=(WeaponClass=Class'FootWeapon')
     BaseEquipment(2)=(WeaponClass=Class'MatchesWeapon')
     BaseEquipment(3)=(WeaponClass=Class'StrongFistsWeapon')
     BaseEquipment(4)=(WeaponClass=Class'AWInventory.CellPhoneWeapon')
     Mesh=SkeletalMesh'AWCharacters.Avg_Dude'
     FootTexture=Texture'ChameleonSkins.Special.Dude'
     Skins(0)=Texture'ChameleonSkins.Special.Dude'
     MenuName="Postal Dude"
     AmbientGlow=0
     HealthMax=300.000000
     Mass=120.000000
}
