///////////////////////////////////////////////////////////////////////////////
// MenuControlsEditActions.uc
// Copyright 2002 Running With Scissors, Inc.  All Rights Reserved.
//
//	02/01/03 JMI	Moved all control definitions into the base class.  The idea
//					is to be able to know all the mappable controls from all the
//					menus.  GetControls() still returns the controls this menu
//					will edit.
//
// 12/17/02 NPF Renamed from Character to Actions menu
//
///////////////////////////////////////////////////////////////////////////////
class EAMenuControlsEditActions extends MenuControlsEditActions;

var Control BulletControl;

///////////////////////////////////////////////////////////////////////////////
// Get the controls to be edited.  Tells the base class what to edit.
///////////////////////////////////////////////////////////////////////////////
function array<Control> GetControls()
{
	local array<Control> ThisArr;
	local P2GameInfoSingle ThisGame;

	ThisArr = aActionControls;

	ThisGame = GetGameSingle();
	if (EAGameInfo(ThisGame) != None && EAGameInfo(ThisGame).AllEnabled())
	{
		aActionControls.Insert(aActionControls.Length, 1);
		aActionControls[aActionControls.Length - 1] = BulletControl;
	}

	return aActionControls;
}

///////////////////////////////////////////////////////////////////////////////
// Default properties
///////////////////////////////////////////////////////////////////////////////

defaultproperties
{
     BulletControl=(strAlias="Bullet",strLabel="Bullet Time")
     Defaults(0)=(alias="MoveForward",Key="W",Cat="New")
     Defaults(1)=(alias="MoveForward",Key="Up",Cat="Old")
     Defaults(2)=(alias="MoveBackward",Key="S",Cat="New")
     Defaults(3)=(alias="MoveBackward",Key="Down",Cat="Old")
     Defaults(4)=(alias="StrafeLeft",Key="A",Cat="New")
     Defaults(5)=(alias="StrafeLeft",Key="Left",Cat="Old")
     Defaults(6)=(alias="StrafeRight",Key="D",Cat="New")
     Defaults(7)=(alias="StrafeRight",Key="Right",Cat="Old")
     Defaults(8)=(alias="Walking",Key="Shift",Cat="New")
     Defaults(9)=(alias="TurnLeft",Key="Comma",Cat="Old")
     Defaults(10)=(alias="TurnRight",Key="Period",Cat="Old")
     Defaults(11)=(alias="LookToggle",Key="CapsLock",Cat="Ancient")
     Defaults(12)=(alias="LookUp",Key="PageUp",Cat="Ancient")
     Defaults(13)=(alias="LookDown",Key="Insert",Cat="Ancient")
     Defaults(14)=(alias="CenterView",Key="Home",Cat="Ancient")
     Defaults(15)=(alias="Strafe",Key="M",Cat="Ancient")
     Defaults(16)=(alias="ToggleToHands",Key="E",Cat="New")
     Defaults(17)=(alias="ToggleToHands",Key="End",Cat="Old")
     Defaults(18)=(alias="Duck",Key="C",Cat="New")
     Defaults(19)=(alias="Duck",Key="NumPad0",Cat="Old")
     Defaults(20)=(alias="Jump",Key="Space",Cat="New")
     Defaults(21)=(alias="Jump",Key="Ctrl",Cat="Old")
     Defaults(22)=(alias="DoKick",Key="Q",Cat="New")
     Defaults(23)=(alias="DoKick",Key="Delete",Cat="Old")
     Defaults(24)=(alias="UseZipper",Key="R",Cat="New")
     Defaults(25)=(alias="UseZipper",Key="PageDown",Cat="Old")
     Defaults(26)=(alias="Suicide",Key="K",Cat="New")
     Defaults(27)=(alias="GetDown",Key="G",Cat="New")
     Defaults(28)=(alias="Fire",Key="LeftMouse",Cat="New")
     Defaults(29)=(alias="AltFire",Key="RightMouse",Cat="New")
     Defaults(30)=(alias="SwitchWeapon 1",Key="1",Cat="New")
     Defaults(31)=(alias="SwitchWeapon 2",Key="2",Cat="New")
     Defaults(32)=(alias="SwitchWeapon 3",Key="3",Cat="New")
     Defaults(33)=(alias="SwitchWeapon 4",Key="4",Cat="New")
     Defaults(34)=(alias="SwitchWeapon 5",Key="5",Cat="New")
     Defaults(35)=(alias="SwitchWeapon 6",Key="6",Cat="New")
     Defaults(36)=(alias="SwitchWeapon 7",Key="7",Cat="New")
     Defaults(37)=(alias="SwitchWeapon 8",Key="8",Cat="New")
     Defaults(38)=(alias="SwitchWeapon 9",Key="9",Cat="New")
     Defaults(39)=(alias="SwitchWeapon 10",Key="0",Cat="New")
     Defaults(40)=(alias="PrevWeapon",Key="MouseWheelDown",Cat="New")
     Defaults(41)=(alias="NextWeapon",Key="MouseWheelUp",Cat="New")
     Defaults(42)=(alias="NextWeapon",Key="Slash",Cat="Old")
     Defaults(43)=(alias="ThrowWeapon",Key="Z",Cat="New")
     Defaults(44)=(alias="ThrowWeapon",Key="Semicolon",Cat="Old")
     Defaults(45)=(alias="InventoryPrevious",Key="LeftBracket",Cat="New")
     Defaults(46)=(alias="InventoryNext",Key="RightBracket",Cat="New")
     Defaults(47)=(alias="InventoryActivate",Key="Enter",Cat="New")
     Defaults(48)=(alias="ThrowPowerup",Key="X",Cat="New")
     Defaults(49)=(alias="ThrowPowerup",Key="SingleQuote",Cat="Old")
     Defaults(50)=(alias="QuickUseMap",Key="F",Cat="New")
     Defaults(51)=(alias="QuickUseMap",Key="NumPad2",Cat="Old")
     Defaults(52)=(alias="QuickHealth",Key="T",Cat="New")
     Defaults(53)=(alias="QuickHealth",Key="NumPad1",Cat="Old")
     Defaults(54)=(alias="WantsToSkip",Key="Enter",Cat="New")
     Defaults(55)=(alias="WantsToSkip",Key="Space",Cat="New")
     Defaults(56)=(alias="Type",Key="Tab",Cat="New")
     Defaults(57)=(alias="ConsoleToggle",Key="Tilde",Cat="New")
     Defaults(58)=(alias="shot",Key="F9",Cat="New")
     Defaults(59)=(alias="Talk",Key="B",Cat="New")
     Defaults(60)=(alias="TeamTalk",Key="V",Cat="New")
     Defaults(61)=(alias="ShowScores",Key="F1",Cat="New")
     Defaults(62)=(alias="GammaUp",Key="F10",Cat="New")
     Defaults(63)=(alias="BrightnessUp",Key="F11",Cat="New")
     Defaults(64)=(alias="GrowHUD",Key="Equals",Cat="New")
     Defaults(65)=(alias="ShrinkHUD",Key="Minus",Cat="New")
     Defaults(66)=(alias="ToggleInvHints",Key="J",Cat="New")
     Defaults(67)=(alias="ViewTeam",Key="F6",Cat="New")
     Defaults(68)=(alias="Bullet",Key="Alt",Cat="New")
     Categories(0)="New"
     Categories(1)="Old"
     Categories(2)="Ancient"
     aActionControls(0)=(strAlias="ToggleToHands",strLabel="Empty Hands")
     aActionControls(1)=(strAlias="Duck",strLabel="Crouch")
     aActionControls(2)=(strAlias="Jump",strLabel="Jump")
     aActionControls(3)=(strAlias="DoKick",strLabel="Kick")
     aActionControls(4)=(strAlias="UseZipper",strLabel="Unzip/Zip Pants")
     aActionControls(5)=(strAlias="Suicide",strLabel="Commit Suicide")
     aActionControls(6)=(strAlias="GetDown",strLabel="Yell 'Get Down'")
     aActionControls(7)=(strAlias="ReloadWeapon",strLabel="Reload Weapon")
     aDisplayControls(0)=(strAlias="GammaUp",strLabel="Gamma")
     aDisplayControls(1)=(strAlias="BrightnessUp",strLabel="Brightness")
     aDisplayControls(2)=(strAlias="GrowHUD",strLabel="More HUD")
     aDisplayControls(3)=(strAlias="ShrinkHUD",strLabel="Less HUD")
     aDisplayControls(4)=(strAlias="ToggleInvHints",strLabel="Toggle Weapon/Inv Hints")
     aInvControls(0)=(strAlias="InventoryActivate",strLabel="Use Item")
     aInvControls(1)=(strAlias="ThrowPowerup",strLabel="Drop Item")
     aInvControls(2)=(strAlias="InventoryNext",strLabel="Next Item")
     aInvControls(3)=(strAlias="InventoryPrevious",strLabel="Previous Item")
     aInvControls(4)=(strAlias="QuickUseMap",strLabel="Show Map")
     aInvControls(5)=(strAlias="QuickHealth",strLabel="Quick Health")
     aMiscControls(0)=(strAlias="shot",strLabel="Screenshot")
     aMiscControls(1)=(strAlias="ShrinkHUD",strLabel="Shrink HUD")
     aMiscControls(2)=(strAlias="GrowHUD",strLabel="Grow HUD")
     aMiscControls(3)=(strAlias="Type",strLabel="Type")
     aMiscControls(4)=(strAlias="ConsoleToggle",strLabel="Console")
     aMiscControls(5)=(strAlias="Talk",strLabel="Talk (Say)")
     aMiscControls(6)=(strAlias="TeamTalk",strLabel="Team Talk (Team Say)")
     aMiscControls(7)=(strAlias="ShowScores",strLabel="Scoreboard")
     aMiscControls(8)=(strAlias="ScoreToggle",strLabel="Scoreboard toggle")
     aMovementControls(0)=(strAlias="MoveForward",strLabel="Forward")
     aMovementControls(1)=(strAlias="MoveBackward",strLabel="Backward")
     aMovementControls(2)=(strAlias="StrafeLeft",strLabel="Strafe Left")
     aMovementControls(3)=(strAlias="StrafeRight",strLabel="Strafe Right")
     aMovementControls(4)=(strAlias="Walking",strLabel="Walk")
     aMovementControls(5)=(strAlias="Strafe",strLabel="Strafe Toggle")
     aMovementControls(6)=(strAlias="TurnLeft",strLabel="Turn Right")
     aMovementControls(7)=(strAlias="TurnRight",strLabel="Turn Left")
     aMovementControls(8)=(strAlias="LookUp",strLabel="Look Up")
     aMovementControls(9)=(strAlias="LookDown",strLabel="Look Down")
     aMovementControls(10)=(strAlias="CenterView",strLabel="Center View")
     aWeaponControls(0)=(strAlias="Fire",strLabel="Primary Fire")
     aWeaponControls(1)=(strAlias="AltFire",strLabel="Secondary Fire")
     aWeaponControls(2)=(strAlias="SwitchWeapon 1",strLabel="Shocker,Baton,Shovel")
     aWeaponControls(3)=(strAlias="SwitchWeapon 2",strLabel="Pistol")
     aWeaponControls(4)=(strAlias="SwitchWeapon 3",strLabel="Shotgun")
     aWeaponControls(5)=(strAlias="SwitchWeapon 4",strLabel="Machine Gun")
     aWeaponControls(6)=(strAlias="SwitchWeapon 5",strLabel="Gas Can,Matches")
     aWeaponControls(7)=(strAlias="SwitchWeapon 6",strLabel="Grenade,Molotov,Scissors")
     aWeaponControls(8)=(strAlias="SwitchWeapon 7",strLabel="Cow Head")
     aWeaponControls(9)=(strAlias="SwitchWeapon 8",strLabel="Hunting Rifle")
     aWeaponControls(10)=(strAlias="SwitchWeapon 9",strLabel="Rocket Launcher")
     aWeaponControls(11)=(strAlias="SwitchWeapon 10",strLabel="Napalm Launcher")
     aWeaponControls(12)=(strAlias="PrevWeapon",strLabel="Previous Weapon")
     aWeaponControls(13)=(strAlias="NextWeapon",strLabel="Next Weapon")
     aWeaponControls(14)=(strAlias="ThrowWeapon",strLabel="Drop Weapon")
     aSayControls(0)=(strAlias="ExtraSay 1",strLabel="Pre-set Say 1")
     aSayControls(1)=(strAlias="ExtraSay 2",strLabel="Pre-set Say 2")
     aSayControls(2)=(strAlias="ExtraSay 3",strLabel="Pre-set Say 3")
     aSayControls(3)=(strAlias="ExtraSay 4",strLabel="Pre-set Say 4")
     aSayControls(4)=(strAlias="ExtraSay 5",strLabel="Pre-set Say 5")
     aSayControls(5)=(strAlias="ExtraSay 6",strLabel="Pre-set Say 6")
     aSayControls(6)=(strAlias="ExtraSay 7",strLabel="Pre-set Say 7")
     aSayControls(7)=(strAlias="ExtraSay 8",strLabel="Pre-set Say 8")
     aTeamSayControls(0)=(strAlias="ExtraTeamSay 1",strLabel="Pre-set TeamSay 1")
     aTeamSayControls(1)=(strAlias="ExtraTeamSay 2",strLabel="Pre-set TeamSay 2")
     aTeamSayControls(2)=(strAlias="ExtraTeamSay 3",strLabel="Pre-set TeamSay 3")
     aTeamSayControls(3)=(strAlias="ExtraTeamSay 4",strLabel="Pre-set TeamSay 4")
     aTeamSayControls(4)=(strAlias="ExtraTeamSay 5",strLabel="Pre-set TeamSay 5")
     aTeamSayControls(5)=(strAlias="ExtraTeamSay 6",strLabel="Pre-set TeamSay 6")
     aTeamSayControls(6)=(strAlias="ExtraTeamSay 7",strLabel="Pre-set TeamSay 7")
     aTeamSayControls(7)=(strAlias="ExtraTeamSay 8",strLabel="Pre-set TeamSay 8")
     astrTextureDetailNames(0)="UltraLow"
     astrTextureDetailNames(1)="Low"
     astrTextureDetailNames(2)="Medium"
     astrTextureDetailNames(3)="High"
}
