///////////////////////////////////////////////////////////////////////////////
// AWMenuStart.uc
// Copyright 2003 Running With Scissors, Inc.  All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////
class EAMenuStart extends AWMenuStart;

var string StartGameEA;
var string StartGameWeekend;
var string StartGameMF;
var string StartGameOrig;
var string StartGameP2;

var UWindowCheckBox EnhancedCheckbox;
var ShellMenuChoice StartEA;
var ShellMenuChoice StartMF;
var ShellMenuChoice StartWeekend;
var ShellMenuChoice StartOrig;
var ShellMenuChoice StartP2;

var localized string EnhancedText;
var localized string StartEAText;
var localized string StartMFText;
var localized string StartWeekendText;
var localized string StartOrigText;
var localized string StartP2Text;

var localized string EnhancedHelp;
var localized string StartEAHelp;
var localized string StartMFHelp;
var localized string StartWeekendHelp;
var localized string StartOrigHelp;
var localized string StartP2Help;

///////////////////////////////////////////////////////////////////////////////
// Create menu contents
///////////////////////////////////////////////////////////////////////////////
function CreateMenuContents()
	{
	Super(ShellMenuCW).CreateMenuContents();

	AddTitle(TitleText, F_FancyXL, TA_Left);

	ItemFont = F_FancyL;
	DifficultyCombo = AddComboBox(DifficultyText, DifficultyHelp, ItemFont);
	DifficultyCombo.List.MaxVisible = ArrayCount(P2GameInfoSingle(GetPlayerOwner().Level.Game).DifficultyNames);

	if (GetGameSingle().FinallyOver())
	{
		EnhancedCheckbox = AddCheckbox(EnhancedText, EnhancedHelp, ItemFont);
		EnhancedCheckbox.SetValue(False);
	}

	StartEA =		AddChoice(StartEAText,		StartEAHelp,		ItemFont,	TA_Left);
	StartMF =		AddChoice(StartMFText,		StartMFHelp,		ItemFont,	TA_Left);
	StartWeekend =	AddChoice(StartWeekendText,	StartWeekendHelp,	ItemFont,	TA_Left);
	StartOrig =		AddChoice(StartOrigText,	StartOrigHelp,		ItemFont,	TA_Left);
	StartP2 =		AddChoice(StartP2Text,		StartP2Help,		ItemFont,	TA_Left);

	BackChoice =	AddChoice(BackText,   	"",				ItemFont,	TA_Left, true);

	LoadValues();
	}

///////////////////////////////////////////////////////////////////////////////
// Allows for use of enhanced mode
///////////////////////////////////////////////////////////////////////////////
function StartGame2(bool bEnhanced)
{
	local P2Player p2p;
	local P2GameInfoSingle usegame;

	usegame = GetGameSingle();
	p2p = usegame.GetPlayer();
	P2RootWindow(Root).StartingGame();

	// Stop any active SceneManager so player will have a pawn
	usegame.StopSceneManagers();

	usegame.PrepIniStartVals();
	usegame.TheGameState.bEGameStart = bEnhanced;

	// Get the difficulty ready for this game state.
	usegame.SetupDifficultyOnce();

	// Get rid of any things in his inventory before a new game starts
	P2Pawn(p2p.pawn).DestroyAllInventory();

	// Game doesn't actually start until player is sent to first day,
	// which *should* happen at the end of the intro sequence.
	usegame.TheGameState.bChangeDayPostTravel = true;
	usegame.TheGameState.NextDay = 0;
	// Actually start the game with the first level
	usegame.SendPlayerTo(p2p, StartGameURL);
}

///////////////////////////////////////////////////////////////////////////////
// Handle notifications from controls
///////////////////////////////////////////////////////////////////////////////
function Notify(UWindowDialogControl C, byte E)
	{
	local int val;

	Super(ShellMenuCW).Notify(C, E);
	switch(E)
		{
		case DE_Change:
			switch (C)
			{
				case DifficultyCombo:
					DiffChanged(bUpdate);
					break;
			}
			break;
		case DE_Click:
			switch (C)
			{
				case BackChoice:
					GoBack();
					break;
				case StartEA:
					StartGameURL = StartGameEA;
					StartGame2(EnhancedCheckbox.GetValue());
					ShellRootWindow(Root).bLaunchedMultiplayer = false;
					break;
				case StartMF:
					StartGameURL = StartGameMF;
					StartGame2(EnhancedCheckbox.GetValue());
					ShellRootWindow(Root).bLaunchedMultiplayer = false;
					break;
				case StartWeekend:
					StartGameURL = StartGameWeekend;
					GetGameSingle().TheGameState.CurrentDay = 6; // fool the loading screen into thinking it's Saturday
					StartGame2(EnhancedCheckbox.GetValue());
					ShellRootWindow(Root).bLaunchedMultiplayer = false;
					break;
				case StartOrig:
					StartGameURL = StartGameOrig;
					GetGameSingle().TheGameState.CurrentDay = 6; // fool the loading screen into thinking it's Saturday
					StartGame2(EnhancedCheckbox.GetValue());
					ShellRootWindow(Root).bLaunchedMultiplayer = false;
					break;
				case StartP2:
					StartGameURL = StartGameP2;
					StartGame2(EnhancedCheckbox.GetValue());
					ShellRootWindow(Root).bLaunchedMultiplayer = false;
					break;
			}
			break;
		}
}

defaultproperties
{
     StartGameEA="EA_Intro.fuk?Game=EAGame.EAGameInfo"
     StartGameWeekend="EA_WeekendIntro.fuk?Game=EAGame.EAGameWeekend"
     StartGameMF="EA_Intro.fuk?Game=EAGame.EAGameMF"
     StartGameOrig="ed_ss_basement.fuk?Game=EAFX.AW7GameSPFinal"
     StartGameP2="Intro.fuk?Game=EAGame.EAP2GameSingle"
     EnhancedText="Enhanced Game"
     StartEAText="Eternal apocalypse start"
     StartMFText="M-F Start"
     StartWeekendText="Weekend Start"
     StartOrigText="Original AW Start"
     StartP2Text="Original Postal 2 Start"
     EnhancedHelp="Play the Enhanced Game"
     StartEAHelp="Play all seven days"
     StartMFHelp="Play Monday through Friday only"
     StartWeekendHelp="Play Weekend only"
     StartOrigHelp="Play original ED game (non EA)"
     StartP2Help="Play the original Postal 2 (non EA)"
     MenuHeight=400.000000
     ItemSpacingY=10.000000
     astrTextureDetailNames(0)="UltraLow"
     astrTextureDetailNames(1)="Low"
     astrTextureDetailNames(2)="Medium"
     astrTextureDetailNames(3)="High"
}
